@@ -21,6 +21,7 @@
 
 #include "qtractorAbout.h"
 #include "qtractorAudioBuffer.h"
+#include "qtractorAudioPeak.h"
 
 #include "qtractorTimeStretcher.h"
 
@@ -65,7 +66,6 @@ private:
 };
 
 
-
 //----------------------------------------------------------------------
 // class qtractorAudioBuffer -- Ring buffer/cache method implementation.
 //
@@ -122,6 +122,8 @@ qtractorAudioBuffer::qtractorAudioBuffer ( unsigned short iChannels,
 	m_ppOutBuffer    = NULL;
 	m_ppSrcState     = NULL;
 #endif
+
+	m_pPeak          = NULL;
 }
 
 // Default destructor.
@@ -159,6 +161,13 @@ unsigned int qtractorAudioBuffer::sampleRate (void) const
 }
 
 
+// Operational properties.
+unsigned int qtractorAudioBuffer::bufferSize (void) const
+{
+	return m_iBufferSize;
+}
+
+
 // Working resample ratio.
 float qtractorAudioBuffer::resampleRatio (void) const
 {
@@ -194,6 +203,8 @@ bool qtractorAudioBuffer::open ( const QString& sFilename, int iMode )
 	// Just one more sanity check...
 	if (iChannels < 1 || m_pFile->sampleRate() < 1) {
 		m_pFile->close();
+		delete m_pFile;
+		m_pFile = NULL;
 		return false;
 	}
 
@@ -285,22 +296,26 @@ void qtractorAudioBuffer::close (void)
 	}
 
 	// Write-behind any remains, if applicable...
-	if (m_pFile->mode() & qtractorAudioFile::Write)
+	if (m_pFile->mode() & qtractorAudioFile::Write) {
 		writeSync();
+		if (m_pPeak) {
+			m_pPeak->closeWrite();
+			m_pPeak = NULL;
+		}
+	}
 
 	// Time to close it good.
 	m_pFile->close();
 
-	// Release internal I/O buffers.
-	deleteIOBuffers();
-
 	// Deallocate any buffer stuff...
 	if (m_pTimeStretcher) {
 		delete m_pTimeStretcher;
 		m_pTimeStretcher = NULL;
 	}
 
+	// Release internal I/O buffers.
 	if (m_pRingBuffer) {
+		deleteIOBuffers();
 		delete m_pRingBuffer;
 		m_pRingBuffer = NULL;
 	}
@@ -326,6 +341,8 @@ void qtractorAudioBuffer::close (void)
 
 	m_fPrevGain = 0.0f;
 	m_fNextGain = 0.0f;
+
+	m_pPeak = NULL;
 }
 
 
@@ -343,60 +360,61 @@ int qtractorAudioBuffer::read ( float **ppFrames, unsigned int iFrames,
 #endif
 
 	// Check for logical EOF...
-	if (m_iReadOffset + iFrames > m_iFileLength)
-		iFrames = m_iFileLength - m_iReadOffset;
+	unsigned long ro = m_iReadOffset;
+	if (iFrames + ro > m_iFileLength)
+		iFrames = m_iFileLength - ro;
+
+	int nread;
 
 	// Are we self-contained (ie. got integral file in buffer) and looping?
-	unsigned int nframes = iFrames;
 	unsigned long ls = m_iLoopStart;
 	unsigned long le = m_iLoopEnd;
 	// Are we in the middle of the loop range ?
 	if (ls < le) {
 		if (m_bIntegral) {
 			unsigned int ri = m_pRingBuffer->readIndex();
-			while (ri < le && ri + nframes >= le) {
-				unsigned int nread = le - ri;
-				m_pRingBuffer->read(ppFrames, nread, iOffset);
-				nframes -= nread;
+			while (ri < le && ri + iFrames >= le) {
+				nread = m_pRingBuffer->read(ppFrames, le - ri, iOffset);
+				iFrames -= nread;
 				iOffset += nread;
+				ro = m_iOffset + ls;
 				m_pRingBuffer->setReadIndex(ls);
-				m_iReadOffset = m_iOffset + ls;
 			}
-			iFrames = nframes;
 		} else {
 			ls += m_iOffset;
 			le += m_iOffset;
-			while (le >= m_iReadOffset && m_iReadOffset + nframes >= le) {
-				nframes -= (le - m_iReadOffset);
-				m_iReadOffset = ls;
+			while (ro < le && ro + iFrames >= le) {
+				nread = m_pRingBuffer->read(ppFrames, le - ro, iOffset);
+				iFrames -= nread;
+				iOffset += nread;
+				ro = ls;
 			}
 		}
 	}
 
 	// Move the (remaining) data around...
-	m_pRingBuffer->read(ppFrames, iFrames, iOffset);
-
-	m_iReadOffset += nframes;
+	nread = m_pRingBuffer->read(ppFrames, iFrames, iOffset);
+	m_iReadOffset = (ro + nread);
 	if (m_iReadOffset >= m_iOffset + m_iLength) {
 		// Force out-of-sync...
 		m_bReadSync = false;
 	}
 
 #ifdef DEBUG_0
-	dump_state(QString("-read(%1)").arg(iFrames));
+	dump_state(QString("-read(%1)").arg(nread));
 #endif
 
 	// Time to sync()?
 	if (m_pSyncThread && m_pRingBuffer->writable() > m_iThreshold)
 		m_pSyncThread->sync();
 
-	return iFrames;
+	return nread;
 }
 
 
 // Buffer data write.
 int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
-	unsigned short iChannels )
+	unsigned short iChannels, unsigned int iOffset )
 {
 	if (m_pRingBuffer == NULL)
 		return -1;
@@ -415,7 +433,7 @@ int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
 
 	if (iChannels == iBuffers) {
 		// Direct write...
-		nwrite = m_pRingBuffer->write(ppFrames, nwrite);
+		nwrite = m_pRingBuffer->write(ppFrames, nwrite, iOffset);
 	} else {
 		// Multiplexed write...
 		unsigned int ws = m_pRingBuffer->writable();
@@ -437,15 +455,15 @@ int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
 			if (iChannels > iBuffers) {
 				for (i = 0, j = 0; i < iBuffers; ++i, ++j) {
 					for (n = 0; n < n1; ++n)
-						ppBuffer[j][n + w] = ppFrames[i][n];
+						ppBuffer[j][n + w] = ppFrames[i][n] + iOffset;
 					for (n = 0; n < n2; ++n)
-						ppBuffer[j][n] = ppFrames[i][n + n1];
+						ppBuffer[j][n] = ppFrames[i][n + n1] + iOffset;
 				}
 				for (j = 0; i < iChannels; ++i) {
 					for (n = 0; n < n1; ++n)
-						ppBuffer[j][n + w] += ppFrames[i][n];
+						ppBuffer[j][n + w] += ppFrames[i][n] + iOffset;
 					for (n = 0; n < n2; ++n)
-						ppBuffer[j][n] += ppFrames[i][n + n1];
+						ppBuffer[j][n] += ppFrames[i][n + n1] + iOffset;
 					if (++j >= iBuffers)
 						j = 0;
 				}
@@ -453,9 +471,9 @@ int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
 				i = 0;
 				for (j = 0; j < iBuffers; ++j) {
 					for (n = 0; n < n1; ++n)
-						ppBuffer[j][n + w] = ppFrames[i][n];
+						ppBuffer[j][n + w] = ppFrames[i][n] + iOffset;
 					for (n = 0; n < n2; ++n)
-						ppBuffer[j][n] = ppFrames[i][n + n1];
+						ppBuffer[j][n] = ppFrames[i][n + n1] + iOffset;
 					if (++i >= iChannels)
 						i = 0;
 				}
@@ -468,7 +486,7 @@ int qtractorAudioBuffer::write ( float **ppFrames, unsigned int iFrames,
 	m_iWriteOffset += nwrite;
 
 #ifdef DEBUG_0
-	dump_state(QString("-write(%1)").arg(iFrames));
+	dump_state(QString("-write(%1)").arg(nwrite));
 #endif
 
 	// Time to sync()?
@@ -493,56 +511,57 @@ int qtractorAudioBuffer::readMix ( float **ppFrames, unsigned int iFrames,
 #endif
 
 	// Check for logical EOF...
-	if (m_iReadOffset + iFrames > m_iFileLength)
-		iFrames = m_iFileLength - m_iReadOffset;
+	unsigned long ro = m_iReadOffset;
+	if (iFrames + ro > m_iFileLength)
+		iFrames = m_iFileLength - ro;
+
+	int nread;
 
 	// Are we self-contained (ie. got integral file in buffer) and looping?
-	unsigned int nframes = iFrames;
 	unsigned long ls = m_iLoopStart;
 	unsigned long le = m_iLoopEnd;
 	// Are we in the middle of the loop range ?
 	if (ls < le) {
 		if (m_bIntegral) {
 			unsigned int ri = m_pRingBuffer->readIndex();
-			while (ri < le && ri + nframes >= le) {
-				unsigned int nread = le - ri;
-				readMixFrames(ppFrames, nread, iChannels, iOffset, fGain);
-				nframes -= nread;
-				iOffset += nread;
-				m_pRingBuffer->setReadIndex(ls);
-				m_iReadOffset = m_iOffset + ls;
+			while (ri < le && ri + iFrames >= le) {
 				m_fNextGain = 0.0f;
+				nread = readMixFrames(ppFrames, le - ri, iChannels, iOffset, fGain);
+				iFrames -= nread;
+				iOffset += nread;
+				ro = m_iOffset + ls;
+				m_pRingBuffer->setReadIndex(ls);
 			}
-			iFrames = nframes;
 		} else {
 			ls += m_iOffset;
 			le += m_iOffset;
-			while (le >= m_iReadOffset && m_iReadOffset + nframes >= le) {
-				nframes -= (le - m_iReadOffset);
-				m_iReadOffset = ls;
+			while (le >= ro && ro + iFrames >= le) {
 				m_fNextGain = 0.0f;
+				nread = readMixFrames(ppFrames, le - ro, iChannels, iOffset, fGain);
+				iFrames -= nread;
+				iOffset += nread;
+				ro = ls;
 			}
 		}
 	}
 
 	// Mix the (remaining) data around...
-	readMixFrames(ppFrames, iFrames, iChannels, iOffset, fGain);
-
-	m_iReadOffset += nframes;
+	nread = readMixFrames(ppFrames, iFrames, iChannels, iOffset, fGain);
+	m_iReadOffset = (ro + nread);
 	if (m_iReadOffset >= m_iOffset + m_iLength) {
 		// Force out-of-sync...
 		m_bReadSync = false;
 	}
 
 #ifdef DEBUG_0
-	dump_state(QString("-readMix(%1)").arg(iFrames));
+	dump_state(QString("-readMix(%1)").arg(nread));
 #endif
 
 	// Time to sync()?
 	if (m_pSyncThread && m_pRingBuffer->writable() > m_iThreshold)
 		m_pSyncThread->sync();
 
-	return iFrames;
+	return nread;
 }
 
 
@@ -554,6 +573,10 @@ bool qtractorAudioBuffer::seek ( unsigned long iFrame )
 	if (m_pFile == NULL)
 		return false;
 
+	// Must not break while on initial sync...
+	if (!m_bInitSync)
+		return false;
+
 	// Seek is only valid on read-only mode.
 	if (m_pFile->mode() & qtractorAudioFile::Write)
 		return false;
@@ -581,13 +604,11 @@ bool qtractorAudioBuffer::seek ( unsigned long iFrame )
 	dump_state(QString(">seek(%1)").arg(iFrame));
 #endif
 
+	// Check if target is already cached...
 	unsigned int  rs = m_pRingBuffer->readable();
 	unsigned int  ri = m_pRingBuffer->readIndex();
 	unsigned long ro = m_iReadOffset;
-
-	// Check if target is already cached...
 	if (iFrame >= ro && ro + rs >= iFrame) {
-	//	if (m_bReadSync) // WTF?
 		m_pRingBuffer->setReadIndex(ri + iFrame - ro);
 	//	m_iWriteOffset += iFrame - ro;
 		m_iReadOffset  += iFrame - ro;
@@ -605,6 +626,7 @@ bool qtractorAudioBuffer::seek ( unsigned long iFrame )
 	//		return false;
 	// Force out-of-sync...
 	m_bReadSync   = false;
+	m_iReadOffset = m_iOffset + m_iLength + 1; // An unlikely offset!
 	m_iSeekOffset = iFrame;
 
 	ATOMIC_INC(&m_seekPending);
@@ -652,9 +674,17 @@ bool qtractorAudioBuffer::initSync (void)
 			m_bIntegral = true;
 			deleteIOBuffers();
 		}
+		else // Re-sync if loop falls short in initial area... 
+		if (m_iLoopStart < m_iLoopEnd
+			&& m_iWriteOffset >= m_iOffset + m_iLoopEnd) {
+			// Will do it again, but now we're
+			// sure we aren't fit integral...
+			m_iSeekOffset = m_iOffset;
+			ATOMIC_INC(&m_seekPending);
+		}
 	}
 
-	// We're done with this initialization.
+	// We're mostly done with initialization...
 	m_bInitSync = true;
 
 	// Just carry on, if not integrally cached...
@@ -717,7 +747,7 @@ void qtractorAudioBuffer::readSync (void)
 {
 	// Keep seeking...
 	do {
-	
+
 		// Check whether we have some hard-seek pending...
 		if (ATOMIC_TAZ(&m_seekPending)) {
 			// Do it...
@@ -755,7 +785,7 @@ void qtractorAudioBuffer::readSyncIn (void)
 		// Take looping into account, if any...
 		unsigned long ls = m_iOffset + m_iLoopStart;
 		unsigned long le = m_iOffset + m_iLoopEnd;
-		bool bLooping = (ls < le && m_iWriteOffset < le);
+		bool bLooping = (ls < le && m_iWriteOffset < le && m_bInitSync);
 		// Adjust request for sane size...
 		if (nahead > m_iBufferSize)
 			nahead = m_iBufferSize;
@@ -1006,8 +1036,11 @@ int qtractorAudioBuffer::readBuffer ( unsigned int iFrames )
 int qtractorAudioBuffer::writeBuffer ( unsigned int iFrames )
 {
 	int nwrite = m_pRingBuffer->read(m_ppFrames, iFrames, 0);
-	if (nwrite > 0)
+	if (nwrite > 0) {
 		nwrite = m_pFile->write(m_ppFrames, nwrite);
+		if (m_pPeak)
+			m_pPeak->write(m_ppFrames, nwrite);
+	}
 
 	return nwrite;
 }
@@ -1153,24 +1186,7 @@ void qtractorAudioBuffer::reset ( bool bLooping )
 	if (bLooping && m_iLoopStart < m_iLoopEnd)
 		iFrame = m_iLoopStart;
 
-#if 0
-	if (m_bIntegral) {
-		m_iReadOffset = m_iOffset + iFrame;
-		m_pRingBuffer->setReadIndex(iFrame);
-	//	m_pRingBuffer->setWriteIndex(m_iLength);
-	} else {
-		// Force out-of-sync...
-		m_bReadSync = false;
-		m_iSeekOffset = m_iOffset + iFrame;
-		m_iSeekPending++;
-		if (m_pSyncThread)
-			m_pSyncThread->sync();
-	}
-	m_fPrevGain = 0.0f;
-	m_fNextGain = 0.0f;
-#else
 	seek(iFrame);
-#endif
 
 #ifdef DEBUG
 	dump_state("-reset()");
@@ -1254,6 +1270,7 @@ void qtractorAudioBuffer::setLoop ( unsigned long iLoopStart,
 
 	// Force out-of-sync...
 	m_bReadSync = false;
+	m_iReadOffset = m_iOffset + m_iLength + 1; // An unlikely offset!
 }
 
 unsigned long qtractorAudioBuffer::loopStart (void) const
@@ -1309,6 +1326,22 @@ bool qtractorAudioBuffer::isPitchShift (void) const
 }
 
 
+// Internal peak descriptor accessors.
+void qtractorAudioBuffer::setPeak ( qtractorAudioPeak *pPeak )
+{
+	m_pPeak = pPeak;
+
+ 	// Reset for building the peak file on-the-fly...
+	if (!m_pPeak->openWrite(m_iChannels, m_iSampleRate))
+		m_pPeak = NULL;
+}
+
+qtractorAudioPeak *qtractorAudioBuffer::peak (void) const
+{
+	return m_pPeak;
+}
+
+
 // Sample-rate converter type (global option).
 int qtractorAudioBuffer::g_iResampleType = 2;	// SRC_SINC_FASTEST;
 
@@ -1383,6 +1416,8 @@ qtractorAudioBufferThread::qtractorAudioBufferThread (
 // Run state accessor.
 void qtractorAudioBufferThread::setRunState ( bool bRunState )
 {
+	QMutexLocker locker(&m_mutex);
+
 	m_bRunState = bRunState;
 }
 
@@ -1408,9 +1443,9 @@ void qtractorAudioBufferThread::sync (void)
 // Bypass executive wait condition (non RT-safe).
 void qtractorAudioBufferThread::syncExport (void)
 {
-	m_mutex.lock();
+	QMutexLocker locker(&m_mutex);
+
 	m_pAudioBuffer->sync();
-	m_mutex.unlock();
 }
 
 
@@ -1425,7 +1460,9 @@ void qtractorAudioBufferThread::run (void)
 	m_bRunState = m_pAudioBuffer->initSync();
 	while (m_bRunState) {
 		// Do whatever we must, then wait for more...
+		m_mutex.unlock();
 		m_pAudioBuffer->sync();
+		m_mutex.lock();
 		m_cond.wait(&m_mutex);
 	}
 	m_mutex.unlock();
