@@ -37,6 +37,7 @@
 
 
 // Forward declarations.
+class qtractorAudioPeak;
 class qtractorAudioBufferThread;
 class qtractorTimeStretcher;
 
@@ -62,6 +63,9 @@ public:
 	unsigned long frames() const;
 	unsigned int sampleRate() const;
 
+	// Operational properties.
+	unsigned int bufferSize() const;
+
 	// Resample ratio accessor.
 	float resampleRatio() const;
 
@@ -73,7 +77,7 @@ public:
 	int read(float **ppFrames, unsigned int iFrames,
 		unsigned int iOffset = 0);
 	int write(float **ppFrames, unsigned int iFrames,
-		unsigned short iChannels = 0);
+		unsigned short iChannels = 0, unsigned int iOffset = 0);
 
 	// Special kind of super-read/channel-mix.
 	int readMix(float **ppFrames, unsigned int iFrames,
@@ -124,6 +128,10 @@ public:
 	// Export-mode sync executive.
 	void syncExport();
 
+	// Internal peak descriptor accessors.
+	void setPeak(qtractorAudioPeak *pPeak);
+	qtractorAudioPeak *peak() const;
+
 	// Sample-rate converter type accessor (global option).
 	static void setResampleType(int iResampleType);
 	static int resampleType();
@@ -224,6 +232,8 @@ private:
 	SRC_STATE    **m_ppSrcState;
 #endif
 
+	qtractorAudioPeak *m_pPeak;
+
 	// Sample-rate converter type global option.
 	static int     g_iResampleType;
 
