@@ -62,7 +62,7 @@ public:
 
 	// Direct write method.
 	void write(float **ppBuffer, unsigned int iFrames,
-		unsigned short iChannels = 0);
+		unsigned short iChannels = 0, unsigned int iOffset = 0);
 
 	// Export-mode sync method.
 	void syncExport();
@@ -83,12 +83,18 @@ public:
 	void process(unsigned long iFrameStart, unsigned long iFrameEnd);
 
 	// Clip paint method.
-	void drawClip(QPainter *pPainter, const QRect& rect,
+	void draw(QPainter *pPainter, const QRect& clipRect,
 	    unsigned long iClipOffset);
 
 	// Audio clip tool-tip.
 	QString toolTip() const;
 
+	// Audio clip export method.
+	typedef void (*ClipExport)(float **, unsigned int, void *);
+
+	bool clipExport(ClipExport pfnClipExport, void *pvArg,
+		unsigned long iOffset = 0, unsigned long iLength = 0) const;
+
 protected:
 
 	// Virtual document element methods.
