@@ -29,6 +29,7 @@
 #include "qtractorSessionCursor.h"
 #include "qtractorSessionDocument.h"
 #include "qtractorMidiEngine.h"
+#include "qtractorMidiBuffer.h"
 #include "qtractorPlugin.h"
 #include "qtractorClip.h"
 
@@ -46,10 +47,13 @@ static inline bool sse_enabled (void)
 	bool bSSE = false;
 #if defined(__GNUC__)
 #if defined(__i386__) || defined(__x86_64__)
-	unsigned int a, b, c, d;
+	unsigned int eax, ebx, ecx, edx;
 	__asm__ __volatile__ (
-		"cpuid": "=a" (a), "=b" (b), "=c" (c), "=d" (d) : "a" (1));
-	bSSE = (d & (1 << 25));
+		"movl %%ebx, %%esi\n\t" \
+		"cpuid\n\t" \
+		"xchgl %%ebx, %%esi" \
+		: "=a" (eax), "=S" (ebx), "=c" (ecx), "=d" (edx) \
+		: "0" (1));
 #endif
 #endif
 	return bSSE;
@@ -64,7 +68,7 @@ static inline void sse_buffer_add (
 
 	for (unsigned short i = 0; i < iChannels; ++i) {
 		float *pBuffer = ppBuffer[j] + iOffset;
-		float *pFrames = ppFrames[i];
+		float *pFrames = ppFrames[i] + iOffset;
 		unsigned int nframes = iFrames;
 		for (; (long(pBuffer) & 15) && (nframes > 0); --nframes)
 			*pBuffer++ += *pFrames++;
@@ -95,7 +99,7 @@ static inline void std_buffer_add (
 
 	for (unsigned short i = 0; i < iChannels; ++i) {
 		float *pBuffer = ppBuffer[j] + iOffset;
-		float *pFrames = ppFrames[i];
+		float *pFrames = ppFrames[i] + iOffset;
 		for (unsigned int n = 0; n < iFrames; ++n)
 			*pBuffer++ += *pFrames++;
 		if (++j >= iBuffers)
@@ -167,8 +171,8 @@ static void qtractorAudioEngine_shutdown ( void *pvArg )
 
 	pAudioEngine->shutdown();
 
-	if (pAudioEngine->notifyWidget()) {
-		QApplication::postEvent(pAudioEngine->notifyWidget(),
+	if (pAudioEngine->notifyObject()) {
+		QApplication::postEvent(pAudioEngine->notifyObject(),
 			new QEvent(pAudioEngine->notifyShutdownType()));
 	}
 }
@@ -183,8 +187,8 @@ static int qtractorAudioEngine_xrun ( void *pvArg )
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (pvArg);
 
-	if (pAudioEngine->notifyWidget()) {
-		QApplication::postEvent(pAudioEngine->notifyWidget(),
+	if (pAudioEngine->notifyObject()) {
+		QApplication::postEvent(pAudioEngine->notifyObject(),
 			new QEvent(pAudioEngine->notifyXrunType()));
 	}
 
@@ -201,8 +205,8 @@ static int qtractorAudioEngine_graph_order ( void *pvArg )
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (pvArg);
 
-	if (pAudioEngine->notifyWidget()) {
-		QApplication::postEvent(pAudioEngine->notifyWidget(),
+	if (pAudioEngine->notifyObject()) {
+		QApplication::postEvent(pAudioEngine->notifyObject(),
 			new QEvent(pAudioEngine->notifyPortType()));
 	}
 
@@ -219,8 +223,8 @@ static void qtractorAudioEngine_graph_port ( jack_port_id_t, int, void *pvArg )
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (pvArg);
 
-	if (pAudioEngine->notifyWidget()) {
-		QApplication::postEvent(pAudioEngine->notifyWidget(),
+	if (pAudioEngine->notifyObject()) {
+		QApplication::postEvent(pAudioEngine->notifyObject(),
 			new QEvent(pAudioEngine->notifyPortType()));
 	}
 }
@@ -235,8 +239,8 @@ static int qtractorAudioEngine_buffer_size ( jack_nframes_t, void *pvArg )
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (pvArg);
 
-	if (pAudioEngine->notifyWidget()) {
-		QApplication::postEvent(pAudioEngine->notifyWidget(),
+	if (pAudioEngine->notifyObject()) {
+		QApplication::postEvent(pAudioEngine->notifyObject(),
 			new QEvent(pAudioEngine->notifyBufferType()));
 	}
 
@@ -267,7 +271,7 @@ qtractorAudioEngine::qtractorAudioEngine ( qtractorSession *pSession )
 {
 	m_pJackClient = NULL;
 
-	m_pNotifyWidget       = NULL;
+	m_pNotifyObject       = NULL;
 	m_eNotifyShutdownType = QEvent::None;
 	m_eNotifyXrunType     = QEvent::None;
 	m_eNotifyPortType     = QEvent::None;
@@ -284,7 +288,6 @@ qtractorAudioEngine::qtractorAudioEngine ( qtractorSession *pSession )
 	m_pExportFile  = NULL;
 	m_iExportStart = 0;
 	m_iExportEnd   = 0;
-	m_iExportSync  = 0;
 	m_bExportDone  = true;
 
 	// Audio metronome stuff.
@@ -296,7 +299,7 @@ qtractorAudioEngine::qtractorAudioEngine ( qtractorSession *pSession )
 	m_iMetroBeatStart = 0;
 	m_iMetroBeat      = 0;
 
-	// Audition/pre-listening player stuff. 
+	// Audition/pre-listening player stuff.
 	ATOMIC_SET(&m_playerLock, 0);
 	m_bPlayerOpen  = false;
 	m_bPlayerBus   = false;
@@ -314,9 +317,9 @@ jack_client_t *qtractorAudioEngine::jackClient (void) const
 
 
 // Event notifier widget settings.
-void qtractorAudioEngine::setNotifyWidget ( QWidget *pNotifyWidget )
+void qtractorAudioEngine::setNotifyObject ( QObject *pNotifyObject )
 {
-	m_pNotifyWidget = pNotifyWidget;
+	m_pNotifyObject = pNotifyObject;
 }
 
 void qtractorAudioEngine::setNotifyShutdownType ( QEvent::Type eNotifyShutdownType )
@@ -340,9 +343,9 @@ void qtractorAudioEngine::setNotifyBufferType ( QEvent::Type eNotifyBufferType )
 }
 
 
-QWidget *qtractorAudioEngine::notifyWidget (void) const
+QObject *qtractorAudioEngine::notifyObject (void) const
 {
-	return m_pNotifyWidget;
+	return m_pNotifyObject;
 }
 
 QEvent::Type qtractorAudioEngine::notifyShutdownType (void) const
@@ -409,6 +412,16 @@ bool qtractorAudioEngine::init ( const QString& sClientName )
 	openPlayerBus();
 	openMetroBus();
 
+	// MIDI plugin managers output buses...
+	qtractorMidiManager *pMidiManager
+		= session()->midiManagers().first();
+	while (pMidiManager) {
+		qtractorAudioBus *pAudioBus = pMidiManager->audioOutputBus();
+		if (pAudioBus && pMidiManager->isAudioOutputBus())
+			pAudioBus->open();
+		pMidiManager = pMidiManager->next();
+	}
+
 	return true;
 }
 
@@ -458,6 +471,16 @@ bool qtractorAudioEngine::activate (void)
 			pAudioBus->autoConnect();
 	}
 
+	// MIDI plugin managers output buses...
+	qtractorMidiManager *pMidiManager
+		= session()->midiManagers().first();
+	while (pMidiManager) {
+		qtractorAudioBus *pAudioBus = pMidiManager->audioOutputBus();
+		if (pAudioBus && pMidiManager->isAudioOutputBus())
+			pAudioBus->autoConnect();
+		pMidiManager = pMidiManager->next();
+	}
+
 	// We're now ready and running...
 	return true;
 }
@@ -487,6 +510,14 @@ void qtractorAudioEngine::stop (void)
 	    return;
 
 	jack_transport_stop(m_pJackClient);
+
+	// MIDI plugin managers reset...
+	qtractorMidiManager *pMidiManager
+		= session()->midiManagers().first();
+	while (pMidiManager) {
+		pMidiManager->reset();
+		pMidiManager = pMidiManager->next();
+	}
 }
 
 
@@ -540,17 +571,16 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 	if (pAudioCursor == NULL)
 		return 0;
 
+	// Reset buffer offset.
+	m_iBufferOffset = 0;
+
 	// Are we actually freewheeling for export?...
 	// notice that freewheeling has no RT requirements.
 	if (m_bFreewheel) {
 		// Make sure we're in a valid state...
 		if (m_pExportFile && m_pExportBus && !m_bExportDone) {
 			// Force/sync every audio clip approaching...
-			m_iExportSync += nframes;
-			if (m_iExportSync > (sampleRate() >> 3)) {
-				m_iExportSync = 0;
-				syncExport();
-			}
+			syncExport();
 			// This the legal process cycle frame range...
 			unsigned long iFrameStart = pAudioCursor->frame();
 			unsigned long iFrameEnd   = iFrameStart + nframes;
@@ -575,6 +605,14 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 		return 0;
 	}
 
+	// Session RT-safeness lock...
+	if (!pSession->acquire())
+		return 0;
+
+	// Track whether audio output buses
+	// buses needs monitoring while idle...
+	int iOutputBus = 0;
+
 	// Prepare all current audio buses...
 	for (qtractorBus *pBus = buses().first();
 			pBus; pBus = pBus->next()) {
@@ -583,19 +621,13 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 		if (pAudioBus)
 			pAudioBus->process_prepare(nframes);
 	}
+
 	// The owned buses too, if any...
 	if (m_bMetroBus)
 		m_pMetroBus->process_prepare(nframes);
 	if (m_bPlayerBus)
 		m_pPlayerBus->process_prepare(nframes);
 
-	// Session RT-safeness lock...
-	if (!pSession->acquire())
-		return 0;
-
-	// Reset buffer offset.
-	m_iBufferOffset = 0;
-
 	// Process audition/pre-listening...
 	if (m_bPlayerOpen && ATOMIC_TAS(&m_playerLock)) {
 		m_pPlayerBuff->readMix(m_pPlayerBus->out(), nframes,
@@ -604,54 +636,71 @@ int qtractorAudioEngine::process ( unsigned int nframes )
 		m_iPlayerFrame += nframes;
 		if (m_bPlayerBus)
 			m_pPlayerBus->process_commit(nframes);
+		else
+			iOutputBus++;
 		ATOMIC_SET(&m_playerLock, 0);
 	}
 
+	// MIDI plugin manager processing...
+	qtractorMidiManager *pMidiManager
+		= pSession->midiManagers().first();
+	if (pMidiManager) {
+		unsigned long iTimeStart = pAudioCursor->frameTime();
+		unsigned long iTimeEnd   = iTimeStart + nframes;
+		while (pMidiManager) {
+			pMidiManager->process(iTimeStart, iTimeEnd);
+			if (!pMidiManager->isAudioOutputBus())
+				iOutputBus++;
+			pMidiManager = pMidiManager->next();
+		}
+	}
+
 	// Don't go any further, if not playing.
 	if (!isPlaying()) {
 		// Do the idle processing...
-		bool bRecord = (pSession->recordTracks() > 0);
-		if (bRecord) {
-			for (qtractorTrack *pTrack = pSession->tracks().first();
-					pTrack; pTrack = pTrack->next()) {
-				// Audio-buffers needs some preparation...
-				if (pTrack->isRecord()
-					&& pTrack->trackType() == qtractorTrack::Audio) {
-					qtractorAudioBus *pInputBus
-						= static_cast<qtractorAudioBus *> (pTrack->inputBus());
-					qtractorAudioMonitor *pAudioMonitor
-						= static_cast<qtractorAudioMonitor *> (pTrack->monitor());
-					// Pre-monitoring...
-					if (pInputBus && pAudioMonitor) {
+		for (qtractorTrack *pTrack = pSession->tracks().first();
+				pTrack; pTrack = pTrack->next()) {
+			// Audio-buffers needs some preparation...
+			if (pTrack->trackType() == qtractorTrack::Audio) {
+				qtractorAudioBus *pInputBus
+					= static_cast<qtractorAudioBus *> (pTrack->inputBus());
+				qtractorAudioMonitor *pAudioMonitor
+					= static_cast<qtractorAudioMonitor *> (pTrack->monitor());
+				// Pre-monitoring...
+				if (pAudioMonitor && pInputBus) {
+					// Record non-passthru metering...
+					if (pTrack->isRecord()) {
+						pAudioMonitor->process_meter(
+							pInputBus->in(), nframes, pInputBus->channels());
+					}
+					// Monitor passthru processing...
+					if (pSession->isTrackMonitor(pTrack)) {
 						pAudioMonitor->process(
 							pInputBus->in(), nframes, pInputBus->channels());
-						// Post-passthru monitoring...
-						if (pTrack->isMonitor() && !pTrack->isMute()
-							&& (!pSession->soloTracks() || pTrack->isSolo())) {
-							qtractorAudioBus *pOutputBus
-								= static_cast<qtractorAudioBus *> (pTrack->outputBus());
-							if (pOutputBus) {
-								// Plugin chain thru-processing...
-								pOutputBus->buffer_prepare_in(pInputBus, nframes);
-								qtractorPluginList *pPluginList = pTrack->pluginList();
-								if (pPluginList && pPluginList->activated())
-									pPluginList->process(pOutputBus->buffer(), nframes);
-								pOutputBus->buffer_commit(nframes);
-							}
+						// Plugin-chain processing...
+						qtractorAudioBus *pOutputBus
+							= static_cast<qtractorAudioBus *> (pTrack->outputBus());
+						if (pOutputBus) {
+							pOutputBus->buffer_prepare(nframes, pInputBus);
+							if ((pTrack->pluginList())->activated() > 0)
+								(pTrack->pluginList())->process(
+									pOutputBus->buffer(), nframes);
+							pOutputBus->buffer_commit(nframes);
+							iOutputBus++;
 						}
 					}
 				}
 			}
 		}
 		// Process audition/pre-listening bus...
-		if (m_pPlayerBus && !m_bPlayerBus && !m_pPlayerBus->isPassthru())
+		if (m_pPlayerBus && m_bPlayerBus)
 			m_pPlayerBus->process_commit(nframes);
 		// Pass-thru current audio buses...
 		for (qtractorBus *pBus = buses().first();
 			pBus; pBus = pBus->next()) {
 			qtractorAudioBus *pAudioBus
 				= static_cast<qtractorAudioBus *> (pBus);
-			if (pAudioBus && (bRecord || pAudioBus->isPassthru()))
+			if (pAudioBus && (iOutputBus > 0 || pAudioBus->isPassthru()))
 				pAudioBus->process_commit(nframes);
 		}
 		// Done as idle...
@@ -862,7 +911,7 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	unsigned long iExportStart, unsigned long iExportEnd,
 	qtractorAudioBus *pExportBus )
 {
-	// No similtaneous or foul exports...
+	// No simultaneous or foul exports...
 	if (!isActivated() || isPlaying() || isExporting())
 		return false;
 
@@ -903,7 +952,6 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	m_pExportFile  = pExportFile;
 	m_iExportStart = iExportStart;
 	m_iExportEnd   = iExportEnd;
-	m_iExportSync  = 0;
 	m_bExportDone  = false;
 
 	// We'll have to save some session parameters...
@@ -952,7 +1000,6 @@ bool qtractorAudioEngine::fileExport ( const QString& sExportPath,
 	m_pExportFile  = NULL;
 	m_iExportStart = 0;
 	m_iExportEnd   = 0;
-	m_iExportSync  = 0;
 	m_bExportDone  = true;
 
 	// Done whether successfully.
@@ -1375,13 +1422,13 @@ int qtractorAudioEngine::updateConnects (void)
 	// Metronome bus outputs...
 	if (m_bMetroBus && m_pMetroBus) {
 		iUpdate += m_pMetroBus->updateConnects(
-				qtractorBus::Output, m_pMetroBus->outputs(), true);
+			qtractorBus::Output, m_pMetroBus->outputs(), true);
 	}
 
 	// Player bus outputs...
 	if (m_bPlayerBus && m_pPlayerBus) {
 		iUpdate += m_pPlayerBus->updateConnects(
-				qtractorBus::Output, m_pPlayerBus->outputs(), true);
+			qtractorBus::Output, m_pPlayerBus->outputs(), true);
 	}
 
 	// Done.
@@ -1403,7 +1450,7 @@ qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
 
 	if (busMode & qtractorBus::Input) {
 		m_pIAudioMonitor = new qtractorAudioMonitor(iChannels);
-		m_pIPluginList   = new qtractorPluginList(iChannels, 0, 0);
+		m_pIPluginList   = createPluginList();
 	} else {
 		m_pIAudioMonitor = NULL;
 		m_pIPluginList   = NULL;
@@ -1411,7 +1458,7 @@ qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
 
 	if (busMode & qtractorBus::Output) {
 		m_pOAudioMonitor = new qtractorAudioMonitor(iChannels);
-		m_pOPluginList   = new qtractorPluginList(iChannels, 0, 0);
+		m_pOPluginList   = createPluginList();
 	} else {
 		m_pOAudioMonitor = NULL;
 		m_pOPluginList   = NULL;
@@ -1424,7 +1471,9 @@ qtractorAudioBus::qtractorAudioBus ( qtractorAudioEngine *pAudioEngine,
 
 	m_ppIBuffer = NULL;
 	m_ppOBuffer = NULL;
+
 	m_ppXBuffer = NULL;
+	m_ppYBuffer = NULL;
 
 	m_bEnabled  = false;
 
@@ -1469,11 +1518,8 @@ void qtractorAudioBus::setChannels ( unsigned short iChannels )
 	if (m_pOAudioMonitor)
 		m_pOAudioMonitor->setChannels(iChannels);
 
-	if (m_pIPluginList)
-		m_pIPluginList->setBuffer(iChannels, 0, 0);
-
-	if (m_pOPluginList)
-		m_pIPluginList->setBuffer(iChannels, 0, 0);
+	updatePluginList(m_pIPluginList);
+	updatePluginList(m_pOPluginList);
 }
 
 unsigned short qtractorAudioBus::channels (void) const
@@ -1513,7 +1559,7 @@ bool qtractorAudioBus::open (void)
 		m_ppIPorts  = new jack_port_t * [m_iChannels];
 		m_ppIBuffer = new float * [m_iChannels];
 		const QString sIPortName(busName() + "/in_%1");
-		for (i = 0; i < m_iChannels; i++) {
+		for (i = 0; i < m_iChannels; ++i) {
 			m_ppIPorts[i] = jack_port_register(
 				pAudioEngine->jackClient(),
 				sIPortName.arg(i + 1).toUtf8().constData(),
@@ -1528,7 +1574,7 @@ bool qtractorAudioBus::open (void)
 		m_ppOPorts  = new jack_port_t * [m_iChannels];
 		m_ppOBuffer = new float * [m_iChannels];
 		const QString sOPortName(busName() + "/out_%1");
-		for (i = 0; i < m_iChannels; i++) {
+		for (i = 0; i < m_iChannels; ++i) {
 			m_ppOPorts[i] = jack_port_register(
 				pAudioEngine->jackClient(),
 				sOPortName.arg(i + 1).toUtf8().constData(),
@@ -1541,17 +1587,19 @@ bool qtractorAudioBus::open (void)
 	// Allocate internal working bus buffers...
 	unsigned int iBufferSize = pAudioEngine->bufferSize();
 	m_ppXBuffer = new float * [m_iChannels];
-	for (i = 0; i < m_iChannels; i++)
+	m_ppYBuffer = new float * [m_iChannels];
+	for (i = 0; i < m_iChannels; ++i) {
 		m_ppXBuffer[i] = new float [iBufferSize];
+		m_ppYBuffer[i] = NULL;
+	}
 
 	// Plugin lists need some buffer (re)allocation too...
-	unsigned int iSampleRate = pAudioEngine->sampleRate(); 
 	if (m_pIPluginList) {
-		m_pIPluginList->setBuffer(m_iChannels, iBufferSize, iSampleRate);
+		updatePluginList(m_pIPluginList);
 		m_pIPluginList->setName(QObject::tr("%1 In").arg(busName()));
 	}
 	if (m_pOPluginList) {
-		m_pOPluginList->setBuffer(m_iChannels, iBufferSize, iSampleRate);
+		updatePluginList(m_pOPluginList);
 		m_pOPluginList->setName(QObject::tr("%1 Out").arg(busName()));
 	}
 
@@ -1580,7 +1628,7 @@ void qtractorAudioBus::close (void)
 		if (m_ppIPorts) {
 			// Unregister, if we're not shutdown...
 			if (pAudioEngine->jackClient()) {
-				for (i = 0; i < m_iChannels; i++) {
+				for (i = 0; i < m_iChannels; ++i) {
 					if (m_ppIPorts[i]) {
 						jack_port_unregister(
 							pAudioEngine->jackClient(), m_ppIPorts[i]);
@@ -1602,7 +1650,7 @@ void qtractorAudioBus::close (void)
 		if (m_ppOPorts) {
 			// Unregister, if we're not shutdown...
 			if (pAudioEngine->jackClient()) {
-				for (i = 0; i < m_iChannels; i++) {
+				for (i = 0; i < m_iChannels; ++i) {
 					if (m_ppOPorts[i]) {
 						jack_port_unregister(
 							pAudioEngine->jackClient(), m_ppOPorts[i]);
@@ -1621,11 +1669,16 @@ void qtractorAudioBus::close (void)
 
 	// Free internal buffers.
 	if (m_ppXBuffer) {
-		for (i = 0; i < m_iChannels; i++)
+		for (i = 0; i < m_iChannels; ++i)
 			delete [] m_ppXBuffer[i];
 		delete [] m_ppXBuffer;
 		m_ppXBuffer = NULL;
 	}
+
+	if (m_ppYBuffer) {
+		delete [] m_ppYBuffer;
+		m_ppYBuffer = NULL;
+	}
 }
 
 
@@ -1691,16 +1744,15 @@ void qtractorAudioBus::updateBusMode (void)
 		if (m_pIAudioMonitor == NULL)
 			m_pIAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pIPluginList == NULL)
-			m_pIPluginList = new qtractorPluginList(m_iChannels,
-				pAudioEngine->bufferSize(), pAudioEngine->sampleRate());
+			m_pIPluginList = createPluginList();
 	} else {
 		if (m_pIAudioMonitor) {
 			delete m_pIAudioMonitor;
 			m_pIAudioMonitor = NULL;
 		}
-		if (m_pOPluginList) {
-			delete m_pOPluginList;
-			m_pOPluginList = NULL;
+		if (m_pIPluginList) {
+			delete m_pIPluginList;
+			m_pIPluginList = NULL;
 		}
 	}
 
@@ -1709,8 +1761,7 @@ void qtractorAudioBus::updateBusMode (void)
 		if (m_pOAudioMonitor == NULL)
 			m_pOAudioMonitor = new qtractorAudioMonitor(m_iChannels);
 		if (m_pOPluginList == NULL)
-			m_pOPluginList = new qtractorPluginList(m_iChannels,
-				pAudioEngine->bufferSize(), pAudioEngine->sampleRate());
+			m_pOPluginList = createPluginList();
 	} else {
 		if (m_pOAudioMonitor) {
 			delete m_pOAudioMonitor;
@@ -1737,10 +1788,10 @@ void qtractorAudioBus::process_prepare ( unsigned int nframes )
 			m_ppIBuffer[i] = static_cast<float *>
 				(jack_port_get_buffer(m_ppIPorts[i], nframes));
 		}
-		if (m_pIAudioMonitor)
-			m_pIAudioMonitor->process(m_ppIBuffer, nframes);
 		if (m_pIPluginList && m_pIPluginList->activated())
 			m_pIPluginList->process(m_ppIBuffer, nframes);
+		if (m_pIAudioMonitor)
+			m_pIAudioMonitor->process(m_ppIBuffer, nframes);
 	}
 
 	if (busMode() & qtractorBus::Output) {
@@ -1763,15 +1814,16 @@ void qtractorAudioBus::process_commit ( unsigned int nframes )
 	if (!m_bEnabled)
 		return;
 
-	if (m_pOAudioMonitor)
-		m_pOAudioMonitor->process(m_ppOBuffer, nframes);
 	if (m_pOPluginList && m_pOPluginList->activated())
 		m_pOPluginList->process(m_ppOBuffer, nframes);
+	if (m_pOAudioMonitor)
+		m_pOAudioMonitor->process(m_ppOBuffer, nframes);
 }
 
 
 // Bus-buffering methods.
-void qtractorAudioBus::buffer_prepare ( unsigned int nframes )
+void qtractorAudioBus::buffer_prepare ( unsigned int nframes,
+	qtractorAudioBus *pInputBus )
 {
 	if (!m_bEnabled)
 		return;
@@ -1784,10 +1836,43 @@ void qtractorAudioBus::buffer_prepare ( unsigned int nframes )
 	unsigned int offset = pAudioEngine->bufferOffset();
 	unsigned int nbytes = nframes * sizeof(float);
 
-	for (unsigned short i = 0; i < m_iChannels; ++i)
-		::memset(m_ppXBuffer[i] + offset, 0, nbytes);
-}
+	if (pInputBus == NULL) {
+		for (unsigned short i = 0; i < m_iChannels; ++i) {
+			m_ppYBuffer[i] = m_ppXBuffer[i] + offset;
+			::memset(m_ppYBuffer[i], 0, nbytes);
+		}
+		return;
+	}
 
+	unsigned short iBuffers = pInputBus->channels();
+	float **ppBuffer = pInputBus->in();
+
+	if (m_iChannels == iBuffers) {
+		// Exact buffer copy...
+		for (unsigned short i = 0; i < iBuffers; ++i) {
+			m_ppYBuffer[i] = m_ppXBuffer[i] + offset;
+			::memcpy(m_ppYBuffer[i], ppBuffer[i] + offset, nbytes);
+		}
+	} else {
+		// Buffer merge/multiplex...
+		unsigned short i;
+		for (i = 0; i < m_iChannels; ++i) {
+			m_ppYBuffer[i] = m_ppXBuffer[i] + offset;
+			::memset(m_ppYBuffer[i], 0, nbytes);
+		}
+		if (m_iChannels > iBuffers) {
+			unsigned short j = 0;
+			for (i = 0; i < m_iChannels; ++i) {
+				::memcpy(m_ppYBuffer[i], ppBuffer[j] + offset, nbytes);
+				if (++j >= iBuffers)
+					j = 0;
+			}
+		} else { // (m_iChannels < iBuffers)
+			(*m_pfnBufferAdd)(m_ppXBuffer, ppBuffer,
+				nframes, m_iChannels, iBuffers, offset);
+		}
+	}
+}
 
 void qtractorAudioBus::buffer_commit ( unsigned int nframes )
 {
@@ -1799,56 +1884,14 @@ void qtractorAudioBus::buffer_commit ( unsigned int nframes )
 	if (pAudioEngine == NULL)
 		return;
 
-	(*m_pfnBufferAdd)(m_ppOBuffer, m_ppXBuffer, nframes,
-		m_iChannels, m_iChannels, pAudioEngine->bufferOffset());
-}
-
-
-// Input record/passthru bus-buffering method.
-void qtractorAudioBus::buffer_prepare_in (
-	qtractorAudioBus *pInputBus, unsigned int nframes )
-{
-	if (!m_bEnabled)
-		return;
-
-	qtractorAudioEngine *pAudioEngine
-		= static_cast<qtractorAudioEngine *> (engine());
-	if (pAudioEngine == NULL)
-		return;
-
-	unsigned int offset = pAudioEngine->bufferOffset();
-	unsigned int nbytes = nframes * sizeof(float);
-
-	unsigned short iBuffers = pInputBus->channels();
-	float **ppBuffer = pInputBus->in();
-
-	if (m_iChannels == iBuffers) {
-		// Exact buffer copy...
-		for (unsigned short i = 0; i < iBuffers; ++i)
-			::memcpy(m_ppXBuffer[i] + offset, ppBuffer[i], nbytes);
-	} else {
-		// Buffer merge/multiplex...
-		unsigned short i;
-		for (i = 0; i < m_iChannels; ++i)
-			::memset(m_ppXBuffer[i] + offset, 0, nbytes);
-		if (m_iChannels > iBuffers) {
-			unsigned short j = 0;
-			for (i = 0; i < m_iChannels; ++i) {
-				::memcpy(m_ppXBuffer[i] + offset, ppBuffer[j], nbytes);
-				if (++j >= iBuffers)
-					j = 0;
-			}
-		} else { // (m_iChannels < iBuffers)
-			(*m_pfnBufferAdd)(m_ppXBuffer, ppBuffer, nframes,
-				m_iChannels, iBuffers, offset);
-		}
-	}
+	(*m_pfnBufferAdd)(m_ppOBuffer, m_ppXBuffer,
+		nframes, m_iChannels, m_iChannels, pAudioEngine->bufferOffset());
 }
 
 
 float **qtractorAudioBus::buffer (void) const
 {
-	return m_ppXBuffer;
+	return m_ppYBuffer;
 }
 
 
@@ -1900,6 +1943,39 @@ qtractorPluginList *qtractorAudioBus::pluginList_out (void) const
 }
 
 
+// Create plugin-list properly.
+qtractorPluginList *qtractorAudioBus::createPluginList (void) const
+{
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (engine());
+	if (pAudioEngine == NULL)
+		return NULL;
+
+	// Create plugin-list alright...
+	return new qtractorPluginList(m_iChannels,
+		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+		qtractorPluginList::AudioBus);
+}
+
+
+// Update plugin-list buffers properly.
+void qtractorAudioBus::updatePluginList ( qtractorPluginList *pPluginList )
+{
+	// Sanity checks...
+	if (pPluginList == NULL)
+		return;
+
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (engine());
+	if (pAudioEngine == NULL)
+		return;
+	
+	// Set plugin-list buffer alright...
+	pPluginList->setBuffer(m_iChannels,
+		pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+		qtractorPluginList::AudioBus);
+}
+
 
 // Retrieve all current JACK connections for a given bus mode interface;
 // return the effective number of connection attempts...
@@ -1946,7 +2022,7 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 						connects.removeAt(iItem);
 						delete pItem;
 					}
-				} 
+				}
 				else if (!bConnect)
 					connects.append(new ConnectItem(item));
 				iClientPort++;
@@ -1999,7 +2075,7 @@ int qtractorAudioBus::updateConnects ( qtractorBus::BusMode busMode,
 			}
 		}
 	}
-	
+
 	// Done.
 	return iUpdate;
 }
