@@ -61,13 +61,13 @@ public:
 		QDomElement *pElement);
 
 	// Event notifier widget settings.
-	void setNotifyWidget       (QWidget *pNotifyWidget);
+	void setNotifyObject       (QObject *pNotifyObject);
 	void setNotifyShutdownType (QEvent::Type eNotifyShutdownType);
 	void setNotifyXrunType     (QEvent::Type eNotifyXrunType);
 	void setNotifyPortType     (QEvent::Type eNotifyPortType);
 	void setNotifyBufferType   (QEvent::Type eNotifyBufferType);
 
-	QWidget     *notifyWidget() const;
+	QObject     *notifyObject() const;
 	QEvent::Type notifyShutdownType() const;
 	QEvent::Type notifyXrunType() const;
 	QEvent::Type notifyPortType() const;
@@ -159,7 +159,7 @@ private:
 	jack_client_t *m_pJackClient;
 
 	// The event notifier widget.
-	QWidget      *m_pNotifyWidget;
+	QObject      *m_pNotifyObject;
 	QEvent::Type  m_eNotifyShutdownType;
 	QEvent::Type  m_eNotifyXrunType;
 	QEvent::Type  m_eNotifyPortType;
@@ -178,7 +178,6 @@ private:
 	qtractorAudioFile   *m_pExportFile;
 	unsigned long        m_iExportStart;
 	unsigned long        m_iExportEnd;
-	unsigned int         m_iExportSync;
 	bool                 m_bExportDone;
 
 	// Audio metronome stuff.
@@ -192,7 +191,7 @@ private:
 	unsigned long        m_iMetroBeatStart;
 	unsigned int         m_iMetroBeat;
 
-	// Audition/pre-listening player stuff. 
+	// Audition/pre-listening player stuff.
 	qtractorAtomic       m_playerLock;
 	bool                 m_bPlayerOpen;
 	bool                 m_bPlayerBus;
@@ -239,13 +238,10 @@ public:
 	void process_commit(unsigned int nframes);
 
 	// Bus-buffering methods.
-	void buffer_prepare(unsigned int nframes);
+	void buffer_prepare(unsigned int nframes,
+		qtractorAudioBus *pInputBus = NULL);
 	void buffer_commit(unsigned int nframes);
 
-	// Input record/passthru bus-buffering methods.
-	void buffer_prepare_in(
-		qtractorAudioBus *pInputBus, unsigned int nframes);
-
 	float **buffer() const;
 
 	// Frame buffer accessors.
@@ -280,6 +276,12 @@ protected:
 	// Bus mode change event.
 	void updateBusMode();
 
+	// Create plugin-list properly.
+	qtractorPluginList *createPluginList (void) const;
+
+	// Set plugin-list buffers properly.
+	void updatePluginList(qtractorPluginList *pPluginList);
+
 private:
 
 	// Instance variables.
@@ -300,6 +302,7 @@ private:
 	float       **m_ppIBuffer;
 	float       **m_ppOBuffer;
 	float       **m_ppXBuffer;
+	float       **m_ppYBuffer;
 
 	// Special under-work flag...
 	// (r/w access should be atomic)
