@@ -1,7 +1,7 @@
 // qtractorBusForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,12 +27,17 @@
 #include "qtractorAudioEngine.h"
 #include "qtractorMidiEngine.h"
 
-#include "qtractorMainForm.h"
+#include "qtractorPlugin.h"
 
 #include <QHeaderView>
 #include <QMessageBox>
 #include <QMenu>
 
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------
 // class qtractorBusListItem -- Custom bus listview item.
@@ -70,7 +75,6 @@ private:
 };
 
 
-
 //----------------------------------------------------------------------------
 // qtractorBusForm -- UI wrapper form.
 
@@ -97,9 +101,10 @@ qtractorBusForm::qtractorBusForm (
 
 	m_ui.BusListView->setContextMenuPolicy(Qt::CustomContextMenu);
 
-	m_ui.BusTitleTextLabel->setPalette(QPalette(Qt::darkGray));
+	const QColor& rgbDark = palette().dark().color().darker(150);
+	m_ui.BusTitleTextLabel->setPalette(QPalette(rgbDark));
 	m_ui.BusTitleTextLabel->setAutoFillBackground(true);
-	
+
 	// (Re)initial contents.
 	refreshBuses();
 
@@ -128,6 +133,39 @@ qtractorBusForm::qtractorBusForm (
 	QObject::connect(m_ui.AudioAutoConnectCheckBox,
 		SIGNAL(clicked()),
 		SLOT(changed()));
+
+	QObject::connect(m_ui.InputPluginListView,
+		SIGNAL(currentRowChanged(int)),
+		SLOT(stabilizeForm()));
+	QObject::connect(m_ui.AddInputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(addInputPlugin()));
+	QObject::connect(m_ui.RemoveInputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(removeInputPlugin()));
+	QObject::connect(m_ui.MoveUpInputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(moveUpInputPlugin()));
+	QObject::connect(m_ui.MoveDownInputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(moveDownInputPlugin()));
+
+	QObject::connect(m_ui.OutputPluginListView,
+		SIGNAL(currentRowChanged(int)),
+		SLOT(stabilizeForm()));
+	QObject::connect(m_ui.AddOutputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(addOutputPlugin()));
+	QObject::connect(m_ui.RemoveOutputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(removeOutputPlugin()));
+	QObject::connect(m_ui.MoveUpOutputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(moveUpOutputPlugin()));
+	QObject::connect(m_ui.MoveDownOutputPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(moveDownOutputPlugin()));
+
 	QObject::connect(m_ui.RefreshPushButton,
 		SIGNAL(clicked()),
 		SLOT(refreshBuses()));
@@ -143,12 +181,8 @@ qtractorBusForm::qtractorBusForm (
 	QObject::connect(m_ui.ClosePushButton,
 		SIGNAL(clicked()),
 		SLOT(reject()));
-}
 
-
-// Destructor.
-qtractorBusForm::~qtractorBusForm (void)
-{
+	stabilizeForm();
 }
 
 
@@ -209,16 +243,22 @@ void qtractorBusForm::showBus ( qtractorBus *pBus )
 {
 	m_iDirtySetup++;
 
+	// Reset plugin lists...
+	m_ui.InputPluginListView->setPluginList(NULL);
+	m_ui.OutputPluginListView->setPluginList(NULL);
+
 	// Settle current bus reference...
 	m_pBus = pBus;
 
 	// Show bus properties into view pane...
 	if (pBus) {
+		QString sBusTitle = pBus->busName();
+		if (!sBusTitle.isEmpty())
+			sBusTitle += " - ";
 		switch (pBus->busType()) {
-
 		case qtractorTrack::Audio:
 		{
-			m_ui.BusTitleTextLabel->setText(tr("Audio Bus"));
+			sBusTitle += tr("Audio");
 			qtractorAudioBus *pAudioBus
 				= static_cast<qtractorAudioBus *> (pBus);
 			if (pAudioBus) {
@@ -226,22 +266,39 @@ void qtractorBusForm::showBus ( qtractorBus *pBus )
 					pAudioBus->channels());
 				m_ui.AudioAutoConnectCheckBox->setChecked(
 					pAudioBus->isAutoConnect());
+				// Set plugin lists...
+				if (pAudioBus->busMode() & qtractorBus::Input)
+					m_ui.InputPluginListView->setPluginList(
+						pAudioBus->pluginList_in());
+				if (pAudioBus->busMode() & qtractorBus::Output)
+					m_ui.OutputPluginListView->setPluginList(
+						pAudioBus->pluginList_out());
 			}
 			break;
 		}
-
 		case qtractorTrack::Midi:
 		{
-			m_ui.BusTitleTextLabel->setText(tr("MIDI Bus"));
+			sBusTitle += tr("MIDI");
+			qtractorMidiBus *pMidiBus
+				= static_cast<qtractorMidiBus *> (pBus);
+			if (pMidiBus) {
+				// Set plugin lists...
+				if (pMidiBus->busMode() & qtractorBus::Input)
+					m_ui.InputPluginListView->setPluginList(
+						pMidiBus->pluginList_in());
+				if (pMidiBus->busMode() & qtractorBus::Output)
+					m_ui.OutputPluginListView->setPluginList(
+						pMidiBus->pluginList_out());
+			}
 			break;
 		}
-
 		case qtractorTrack::None:
 		default:
-			m_ui.BusTitleTextLabel->setText(tr("Bus"));
 			break;
 		}
-
+		if (!sBusTitle.isEmpty())
+			sBusTitle += ' ';
+		m_ui.BusTitleTextLabel->setText(sBusTitle + tr("Bus"));
 		m_ui.BusNameLineEdit->setText(pBus->busName());
 		m_ui.BusModeComboBox->setCurrentIndex(int(pBus->busMode()) - 1);
 		m_ui.PassthruCheckBox->setChecked(pBus->isPassthru());
@@ -250,7 +307,7 @@ void qtractorBusForm::showBus ( qtractorBus *pBus )
 	// Reset dirty flag...
 	m_iDirtyCount = 0;	
 	m_iDirtySetup--;
-	
+
 	// Done.
 	stabilizeForm();
 }
@@ -266,10 +323,7 @@ void qtractorBusForm::refreshBuses (void)
 	m_pMidiRoot  = NULL;
 	m_ui.BusListView->clear();
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -306,9 +360,6 @@ void qtractorBusForm::refreshBuses (void)
 		m_ui.BusListView->setItemExpanded(m_pMidiRoot, true);
 #endif
 	}
-
-	// Reselect current bus, if any.
-	setBus(m_pBus);
 }
 
 
@@ -329,15 +380,19 @@ void qtractorBusForm::selectBus (void)
 		return;
 
 	// Check if we need an update?...
-	if (m_pBus && pBusItem->bus() != m_pBus && m_iDirtyCount > 0) {
+	qtractorBus *pBus = pBusItem->bus();
+	if (m_pBus && m_pBus != pBus && m_iDirtyCount > 0) {
 		switch (QMessageBox::warning(this,
 			tr("Warning") + " - " QTRACTOR_TITLE,
 			tr("Some settings have been changed.\n\n"
 			"Do you want to apply the changes?"),
 			tr("Apply"), tr("Discard"), tr("Cancel"))) {
 		case 0:     // Apply
-			updateBus();
-			break;
+			if (updateBusEx(m_pBus)) {
+				m_iDirtyTotal++;
+				refreshBuses();
+			}
+			// Fall thru...
 		case 1:     // Discard
 			break;;
 		default:    // Cancel.
@@ -346,22 +401,19 @@ void qtractorBusForm::selectBus (void)
 	}
 
 	// Get new one into view...
-	showBus(pBusItem->bus());
+	showBus(pBus);
 }
 
 
 // Check whether the current view is elligible as a new bus.
-bool qtractorBusForm::canCreateBus (void)
+bool qtractorBusForm::canCreateBus (void) const
 {
 	if (m_iDirtyCount == 0)
 		return false;
 	if (m_pBus == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -391,17 +443,14 @@ bool qtractorBusForm::canCreateBus (void)
 
 
 // Check whether the current view is elligible for update.
-bool qtractorBusForm::canUpdateBus (void)
+bool qtractorBusForm::canUpdateBus (void) const
 {
 	if (m_iDirtyCount == 0)
 		return false;
 	if (m_pBus == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -415,15 +464,12 @@ bool qtractorBusForm::canUpdateBus (void)
 
 
 // Check whether the current view is elligible for deletion.
-bool qtractorBusForm::canDeleteBus (void)
+bool qtractorBusForm::canDeleteBus (void) const
 {
 	if (m_pBus == NULL)
 		return false;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -432,14 +478,73 @@ bool qtractorBusForm::canDeleteBus (void)
 }
 
 
+// Update bus method.
+bool qtractorBusForm::updateBusEx ( qtractorBus *pBus ) const
+{
+	if (pBus == NULL)
+		return false;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	const QString sBusName = m_ui.BusNameLineEdit->text().simplified();
+	if (sBusName.isEmpty())
+		return false;
+
+	qtractorBus::BusMode busMode = qtractorBus::None;
+	switch (m_ui.BusModeComboBox->currentIndex()) {
+	case 0:
+		busMode = qtractorBus::Input;
+		break;
+	case 1:
+		busMode = qtractorBus::Output;
+		break;
+	case 2:
+		busMode = qtractorBus::Duplex;
+		break;
+	}
+
+	// Make it as an unduable command...
+	qtractorUpdateBusCommand *pUpdateBusCommand
+		= new qtractorUpdateBusCommand(pBus);
+
+	// Set all updated properties...
+	qtractorTrack::TrackType busType = pBus->busType();
+	pUpdateBusCommand->setBusType(busType);
+	pUpdateBusCommand->setBusName(sBusName);
+	pUpdateBusCommand->setBusMode(busMode);
+	pUpdateBusCommand->setPassthru(
+		(busMode & qtractorBus::Duplex) == qtractorBus::Duplex
+		&& m_ui.PassthruCheckBox->isChecked());
+
+	// Specialties for bus types...
+	switch (busType) {
+	case qtractorTrack::Audio:
+		pUpdateBusCommand->setChannels(
+			m_ui.AudioChannelsSpinBox->value());
+		pUpdateBusCommand->setAutoConnect(
+			m_ui.AudioAutoConnectCheckBox->isChecked());
+		break;
+	case qtractorTrack::Midi:
+	case qtractorTrack::None:
+	default:
+		break;
+	}
+
+	// Execute and refresh form...
+	return pSession->execute(pUpdateBusCommand);
+}
+
+
 // Create a new bus from current view.
 void qtractorBusForm::createBus (void)
 {
 	if (m_pBus == NULL)
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 	
 	const QString sBusName = m_ui.BusNameLineEdit->text().simplified();
@@ -487,72 +592,27 @@ void qtractorBusForm::createBus (void)
 	}
 
 	// Execute and refresh form...
-	if (pMainForm->commands()->exec(pCreateBusCommand)) {
+	if (pSession->execute(pCreateBusCommand)) {
 		m_iDirtyTotal++;
 		refreshBuses();
 	}
+
+	// Reselect current bus...
+	setBus(m_pBus);
 }
 
 
 // Update current bus in view.
 void qtractorBusForm::updateBus (void)
 {
-	if (m_pBus == NULL)
-		return;
-
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-	
-	const QString sBusName = m_ui.BusNameLineEdit->text().simplified();
-	if (sBusName.isEmpty())
-		return;
-
-	qtractorBus::BusMode busMode = qtractorBus::None;
-	switch (m_ui.BusModeComboBox->currentIndex()) {
-	case 0:
-		busMode = qtractorBus::Input;
-		break;
-	case 1:
-		busMode = qtractorBus::Output;
-		break;
-	case 2:
-		busMode = qtractorBus::Duplex;
-		break;
-	}
-
-	// Make it as an unduable command...
-	qtractorUpdateBusCommand *pUpdateBusCommand
-		= new qtractorUpdateBusCommand(m_pBus);
-
-	// Set all updated properties...
-	qtractorTrack::TrackType busType = m_pBus->busType();
-	pUpdateBusCommand->setBusType(busType);
-	pUpdateBusCommand->setBusName(sBusName);
-	pUpdateBusCommand->setBusMode(busMode);
-	pUpdateBusCommand->setPassthru(
-		(busMode & qtractorBus::Duplex) == qtractorBus::Duplex
-		&& m_ui.PassthruCheckBox->isChecked());
-
-	// Specialties for bus types...
-	switch (busType) {
-	case qtractorTrack::Audio:
-		pUpdateBusCommand->setChannels(
-			m_ui.AudioChannelsSpinBox->value());
-		pUpdateBusCommand->setAutoConnect(
-			m_ui.AudioAutoConnectCheckBox->isChecked());
-		break;
-	case qtractorTrack::Midi:
-	case qtractorTrack::None:
-	default:
-		break;
-	}
-
-	// Execute and refresh form...
-	if (pMainForm->commands()->exec(pUpdateBusCommand)) {
+	// That's it...
+	if (updateBusEx(m_pBus)) {
 		m_iDirtyTotal++;
 		refreshBuses();
 	}
+
+	// Reselect current bus...
+	setBus(m_pBus);
 }
 
 
@@ -562,12 +622,12 @@ void qtractorBusForm::deleteBus (void)
 	if (m_pBus == NULL)
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Prompt user if he/she's sure about this...
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions && pOptions->bConfirmRemove) {
 		// Get some textual type...
 		QString sBusType;
@@ -597,11 +657,17 @@ void qtractorBusForm::deleteBus (void)
 	qtractorDeleteBusCommand *pDeleteBusCommand
 		= new qtractorDeleteBusCommand(m_pBus);
 
+	// Invalidade current bus...
+	m_pBus = NULL;
+
 	// Execute and refresh form...
-	if (pMainForm->commands()->exec(pDeleteBusCommand)) {
+	if (pSession->execute(pDeleteBusCommand)) {
 		m_iDirtyTotal++;
 		refreshBuses();
 	}
+
+	// Done.
+	stabilizeForm();
 }
 
 
@@ -632,6 +698,7 @@ void qtractorBusForm::reject (void)
 			break;
 		default:    // Cancel.
 			bReject = false;
+			break;
 		}
 	}
 
@@ -658,6 +725,52 @@ void qtractorBusForm::stabilizeForm (void)
 	m_ui.CreatePushButton->setEnabled(canCreateBus());
 	m_ui.UpdatePushButton->setEnabled(canUpdateBus());
 	m_ui.DeletePushButton->setEnabled(canDeleteBus());
+
+	// Stabilize current plugin lists state.
+	bool bEnabled;
+	int iItem, iItemCount;
+	qtractorPlugin *pPlugin = NULL;
+	qtractorPluginListItem *pItem = NULL;
+
+	// Input plugin list...
+	bEnabled = (m_ui.InputPluginListView->pluginList() != NULL);
+	m_ui.BusTabWidget->setTabEnabled(1, bEnabled);
+	if (bEnabled) {
+		iItemCount = m_ui.InputPluginListView->count();
+		iItem = -1;
+		pPlugin = NULL;
+		pItem = static_cast<qtractorPluginListItem *> (
+			m_ui.InputPluginListView->currentItem());
+		if (pItem) {
+			iItem = m_ui.InputPluginListView->row(pItem);
+			pPlugin = pItem->plugin();
+		}
+	//	m_ui.AddInputPluginToolButton->setEnabled(true);
+		m_ui.RemoveInputPluginToolButton->setEnabled(pPlugin != NULL);
+		m_ui.MoveUpInputPluginToolButton->setEnabled(pItem && iItem > 0);
+		m_ui.MoveDownInputPluginToolButton->setEnabled(
+			pItem && iItem < iItemCount - 1);
+	}
+
+	// Output plugin list...
+	bEnabled = (m_ui.OutputPluginListView->pluginList() != NULL);
+	m_ui.BusTabWidget->setTabEnabled(2, bEnabled);
+	if (bEnabled) {
+		iItemCount = m_ui.OutputPluginListView->count();
+		iItem = -1;
+		pPlugin = NULL;
+		pItem = static_cast<qtractorPluginListItem *> (
+			m_ui.OutputPluginListView->currentItem());
+		if (pItem) {
+			iItem = m_ui.OutputPluginListView->row(pItem);
+			pPlugin = pItem->plugin();
+		}
+	//	m_ui.AddOutputPluginToolButton->setEnabled(true);
+		m_ui.RemoveOutputPluginToolButton->setEnabled(pPlugin != NULL);
+		m_ui.MoveUpOutputPluginToolButton->setEnabled(pItem && iItem > 0);
+		m_ui.MoveDownOutputPluginToolButton->setEnabled(
+			pItem && iItem < iItemCount - 1);
+	}
 }
 
 
@@ -695,5 +808,56 @@ void qtractorBusForm::contextMenu ( const QPoint& /*pos*/ )
 }
 
 
-// end of qtractorBusForm.cpp
+// Input plugin list slots.
+void qtractorBusForm::addInputPlugin (void)
+{
+	m_ui.InputPluginListView->addPlugin();
+	stabilizeForm();
+}
 
+void qtractorBusForm::removeInputPlugin (void)
+{
+	m_ui.InputPluginListView->removePlugin();
+	stabilizeForm();
+}
+
+void qtractorBusForm::moveUpInputPlugin (void)
+{
+	m_ui.InputPluginListView->moveUpPlugin();
+	stabilizeForm();
+}
+
+void qtractorBusForm::moveDownInputPlugin (void)
+{
+	m_ui.InputPluginListView->moveDownPlugin();
+	stabilizeForm();
+}
+
+
+// Output plugin list slots.
+void qtractorBusForm::addOutputPlugin (void)
+{
+	m_ui.OutputPluginListView->addPlugin();
+	stabilizeForm();
+}
+
+void qtractorBusForm::removeOutputPlugin (void)
+{
+	m_ui.OutputPluginListView->removePlugin();
+	stabilizeForm();
+}
+
+void qtractorBusForm::moveUpOutputPlugin (void)
+{
+	m_ui.OutputPluginListView->moveUpPlugin();
+	stabilizeForm();
+}
+
+void qtractorBusForm::moveDownOutputPlugin (void)
+{
+	m_ui.OutputPluginListView->moveDownPlugin();
+	stabilizeForm();
+}
+
+
+// end of qtractorBusForm.cpp
