@@ -1,7 +1,7 @@
 // qtractorBusForm.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -39,31 +39,44 @@ public:
 
 	// Constructor.
 	qtractorBusForm(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
-	// Destructor.
-	~qtractorBusForm();
 
-    void setBus(qtractorBus *pBus);
-    qtractorBus *bus();
-    bool isDirty();
+	void setBus(qtractorBus *pBus);
+	qtractorBus *bus();
+
+	bool isDirty();
 
 public slots:
 
-    void reject();
-    void refreshBuses();
-    void selectBus();
-    void createBus();
-    void updateBus();
-    void deleteBus();
-    void changed();
-    void stabilizeForm();
-    void contextMenu(const QPoint&);
+	void reject();
+	void refreshBuses();
+	void selectBus();
+	void createBus();
+	void updateBus();
+	void deleteBus();
+	void changed();
+	void stabilizeForm();
+
+	void contextMenu(const QPoint&);
+
+	void addInputPlugin();
+	void removeInputPlugin();
+	void moveUpInputPlugin();
+	void moveDownInputPlugin();
+
+	void addOutputPlugin();
+	void removeOutputPlugin();
+	void moveUpOutputPlugin();
+	void moveDownOutputPlugin();
 
 protected:
 
-    void showBus(qtractorBus *pBus);
-    bool canCreateBus();
-    bool canUpdateBus();
-    bool canDeleteBus();
+	void showBus(qtractorBus *pBus);
+
+	bool canCreateBus() const;
+	bool canUpdateBus() const;
+	bool canDeleteBus() const;
+
+	bool updateBusEx(qtractorBus *pBus) const;
 
 private:
 
@@ -79,7 +92,6 @@ private:
 	int m_iDirtyTotal;
 };
 
-
 #endif	// __qtractorBusForm_h
 
 
