@@ -29,6 +29,16 @@
 #include <QFileInfo>
 #include <QPainter>
 #include <QPolygon>
+#include <QDir>
+
+#ifdef CONFIG_GRADIENT
+#include <QLinearGradient>
+#endif
+
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
 
 
 //-------------------------------------------------------------------------
@@ -53,12 +63,12 @@ qtractorClip::~qtractorClip (void)
 void qtractorClip::clear (void)
 {
 	m_sClipName.clear();
-	
+
 	m_iClipStart      = 0;
 	m_iClipLength     = 0;
 	m_iClipOffset     = 0;
 
-    m_iClipStartTime  = 0;
+	m_iClipStartTime  = 0;
 	m_iClipOffsetTime = 0;
 	m_iClipLengthTime = 0;
 
@@ -68,6 +78,12 @@ void qtractorClip::clear (void)
 	m_iLoopStart      = 0;
 	m_iLoopEnd        = 0;
 
+	m_fGain           = 1.0f;
+
+	// Gain fractionalizer(tm)...
+	m_fractGain.num   = 1;
+	m_fractGain.den   = 8;
+
 	m_iFadeInLength   = 0;
 	m_iFadeOutLength  = 0;
 
@@ -100,7 +116,12 @@ qtractorTrack *qtractorClip::track (void) const
 // Clip filename properties accessors.
 void qtractorClip::setFilename ( const QString& sFilename )
 {
-	m_sFilename = sFilename;
+	QDir dir;
+
+	if (m_pTrack && m_pTrack->session())
+		dir.setPath(m_pTrack->session()->sessionDir());
+
+	m_sFilename = QDir::cleanPath(dir.absoluteFilePath(sFilename));
 }
 
 const QString& qtractorClip::filename (void) const
@@ -108,6 +129,16 @@ const QString& qtractorClip::filename (void) const
 	return m_sFilename;
 }
 
+QString qtractorClip::relativeFilename (void) const
+{
+	QDir dir;
+
+	if (m_pTrack && m_pTrack->session())
+		dir.setPath(m_pTrack->session()->sessionDir());
+
+	return dir.relativeFilePath(m_sFilename);
+}
+
 
 // Clip label accessors.
 const QString& qtractorClip::clipName (void) const
@@ -237,6 +268,26 @@ unsigned long qtractorClip::clipLoopEnd (void) const
 }
 
 
+// Clip-loop points accessors.
+void qtractorClip::setClipGain ( float fGain )
+{
+	m_fGain = fGain;
+
+	// Gain fractionalizer(tm)...
+	m_fractGain.den = 8;
+	while(fGain != int(fGain) && m_fractGain.den < 20) {
+		m_fractGain.den += 2;
+		fGain *= 4.0f;
+	}
+	m_fractGain.num = int(fGain);
+}
+
+float qtractorClip::clipGain (void) const
+{
+	return m_fGain;
+}
+
+
 // Clip fade-in accessors
 void qtractorClip::setFadeInType ( qtractorClip::FadeType fadeType )
 {
@@ -347,7 +398,7 @@ float qtractorClip::gain ( unsigned long iOffset ) const
 	if (iOffset >= m_iClipLength)
 		return 0.0f;
 
-	float fGain = 1.0f;
+	float fGain = m_fGain;
 
 	if (m_iFadeInLength > 0 && iOffset < m_iFadeInLength) {
 		float f  = float(iOffset);
@@ -414,8 +465,15 @@ void qtractorClip::drawClip ( QPainter *pPainter, const QRect& clipRect,
 	unsigned long iClipOffset )
 {
 	// Fill clip background...
-	pPainter->setPen(m_pTrack->background().dark());
+	pPainter->setPen(m_pTrack->background().darker());
+#ifdef CONFIG_GRADIENT
+	QLinearGradient grad(0, clipRect.top(), 0, clipRect.bottom());
+	grad.setColorAt(0.4, m_pTrack->background());
+	grad.setColorAt(1.0, m_pTrack->background().darker(130));
+	pPainter->setBrush(grad);
+#else
 	pPainter->setBrush(m_pTrack->background());
+#endif
 	pPainter->drawRect(clipRect);
 
 	qtractorSession *pSession = m_pTrack->session();
@@ -426,47 +484,55 @@ void qtractorClip::drawClip ( QPainter *pPainter, const QRect& clipRect,
 	QRect rect(clipRect);
 	if (iClipOffset > 0)
 		rect.setLeft(rect.left() - pSession->pixelFromFrame(iClipOffset) + 1);
-	rect.setRight(rect.left() + pSession->pixelFromFrame(m_iClipLength));
 
 	// Draw clip name label...
 	pPainter->drawText(rect,
-		Qt::AlignLeft | Qt::AlignBottom | Qt::TextSingleLine, clipName());
+		Qt::AlignLeft | Qt::AlignBottom | Qt::TextSingleLine, m_sClipName);
+
+	// Draw clip contents (virtual)
+	draw(pPainter, clipRect, iClipOffset);
+
+	// Avoid drawing fade in/out handles
+	// on still empty clips (eg. while recording)
+	if (m_iClipLength < 1)
+		return;
 
 	// Fade in/out handle color...
-	const QColor& rgbFade = track()->foreground().light(160);
+	QColor rgbFade(m_pTrack->foreground());
+	rgbFade.setAlpha(120);
 	pPainter->setPen(rgbFade);
 	pPainter->setBrush(rgbFade);
 
 	// Fade-in slope...
-	int y = rect.top()  + 1;
-	int x = rect.left() + 1;
+	int y = rect.top();
+	int x = rect.left();
 	int w = pSession->pixelFromFrame(m_iFadeInLength);
 	QRect rectFadeIn(x + w, y, 8, 8);
 	if (w > 0 && x + w > clipRect.left()) {
 		QPolygon polyg(3);
 		polyg.setPoint(0, x, y);
-		polyg.setPoint(1, x, rect.bottom() - 1);
+		polyg.setPoint(1, x, rect.bottom());
 		polyg.setPoint(2, x + w, y);
 		pPainter->drawPolygon(polyg);
 	}
 
 	// Fade-out slope...
-	x = rect.right() - 1;
+	x = rect.left() + pSession->pixelFromFrame(m_iClipLength);
 	w = pSession->pixelFromFrame(m_iFadeOutLength);
 	QRect rectFadeOut(x - w - 8, y, 8, 8);
 	if (w > 0 && x - w < clipRect.right()) {
 		QPolygon polyg(3);
 		polyg.setPoint(0, x, y);
-		polyg.setPoint(1, x, rect.bottom() - 1);
+		polyg.setPoint(1, x, rect.bottom());
 		polyg.setPoint(2, x - w, y);
 		pPainter->drawPolygon(polyg);
 	}
 
 	// Fade in/out handles...
 	if (rectFadeIn.intersects(clipRect))
-		pPainter->fillRect(rectFadeIn, rgbFade.dark(120));
+		pPainter->fillRect(rectFadeIn, rgbFade.darker(120));
 	if (rectFadeOut.intersects(clipRect))
-		pPainter->fillRect(rectFadeOut, rgbFade.dark(120));
+		pPainter->fillRect(rectFadeOut, rgbFade.darker(120));
 }
 
 
@@ -498,17 +564,16 @@ QString qtractorClip::toolTip (void) const
 	QString sToolTip = QObject::tr("Name:\t%1").arg(m_sClipName);
 
 	qtractorSession *pSession = NULL;
-	qtractorTrack *pTrack = track();
-	if (pTrack)
-		pSession = pTrack->session();
+	if (m_pTrack)
+		pSession = m_pTrack->session();
 	if (pSession) {
 		sToolTip += '\n';
 		qtractorTimeScale *pTimeScale = pSession->timeScale();
 		sToolTip += QObject::tr("Start:\t%1\nEnd:\t%2\nOffset:\t%3\nLength:\t%4")
 			.arg(pTimeScale->textFromFrame(m_iClipStart))
 			.arg(pTimeScale->textFromFrame(m_iClipStart + m_iClipLength))
-			.arg(pTimeScale->textFromFrame(m_iClipOffset))
-			.arg(pTimeScale->textFromFrame(m_iClipLength));
+			.arg(pTimeScale->textFromFrame(m_iClipOffset, true))
+			.arg(pTimeScale->textFromFrame(m_iClipLength, true));
 	}
 
 	sToolTip += '\n';
@@ -563,6 +628,8 @@ bool qtractorClip::loadElement ( qtractorSessionDocument *pDocument,
 					qtractorClip::setClipOffset(eProp.text().toULong());
 				else if (eProp.tagName() == "length")
 					qtractorClip::setClipLength(eProp.text().toULong());
+				else if (eProp.tagName() == "gain")
+					qtractorClip::setClipGain(eProp.text().toFloat());
 				else if (eProp.tagName() == "fade-in") {
 					qtractorClip::setFadeInType(
 						pDocument->loadFadeType(eProp.attribute("type")));
@@ -602,6 +669,8 @@ bool qtractorClip::saveElement ( qtractorSessionDocument *pDocument,
 		QString::number(qtractorClip::clipOffset()), &eProps);
 	pDocument->saveTextElement("length",
 		QString::number(qtractorClip::clipLength()), &eProps);
+	pDocument->saveTextElement("gain",
+		QString::number(qtractorClip::clipGain()), &eProps);
 
     QDomElement eFadeIn = pDocument->document()->createElement("fade-in");
 	eFadeIn.setAttribute("type",
