@@ -1,7 +1,7 @@
 // qtractorClip.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2008 rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -51,6 +51,7 @@ public:
 	// Filename properties accessors.
 	void setFilename(const QString&  sFilename);
 	const QString& filename() const;
+	QString relativeFilename() const;
 
 	// Clip label accessors.
 	void setClipName(const QString& sClipName);
@@ -81,6 +82,10 @@ public:
 	unsigned long clipLoopStart() const;
 	unsigned long clipLoopEnd() const;
 
+	// Clip gain/volume accessors.
+	void setClipGain(float fGain);
+	float clipGain() const;
+
 	// Fade types.
 	enum FadeType { Linear, Quadratic, Cubic };
 
@@ -101,6 +106,10 @@ public:
 	// Clip time reference settler method.
 	void updateClipTime();
 
+	// Clip paint method.
+	void drawClip(QPainter *pPainter, const QRect& clipRect,
+		unsigned long iClipOffset);
+
 	// Clip (re)open method.
 	virtual void open() = 0;
 
@@ -120,8 +129,8 @@ public:
 	virtual void process(unsigned long iFrameStart, unsigned long iFrameEnd) = 0;
 
 	// Clip paint method.
-	virtual void drawClip(QPainter *pPainter, const QRect& clipRect,
-		unsigned long iClipOffset);
+	virtual void draw(QPainter *pPainter, const QRect& clipRect,
+		unsigned long iClipOffset) = 0;
 
 	// Clip editor method.
 	virtual bool startEditor(QWidget *pParent);
@@ -156,6 +165,9 @@ protected:
 	virtual bool saveClipElement(qtractorSessionDocument *pDocument,
 		QDomElement *pElement) = 0;
 
+	// Gain fractionalizer(tm)...
+	struct { int num, den; } m_fractGain;
+
 private:
 
 	qtractorTrack *m_pTrack;        // Track reference.
@@ -177,6 +189,9 @@ private:
 	unsigned long m_iLoopStart;     // Clip loop start frame-offset.
 	unsigned long m_iLoopEnd;       // Clip loop end frame-offset.
 
+	// Clip gain/volume.
+	float m_fGain;
+
 	// Fade-in/out stuff.
 	unsigned long m_iFadeInLength;  // Fade-in length (in frames).
 	unsigned long m_iFadeOutLength; // Fade-out length (in frames).
