@@ -28,13 +28,15 @@
 #include "qtractorAudioClip.h"
 #include "qtractorMidiClip.h"
 
-#include "qtractorMainForm.h"
 #include "qtractorSession.h"
 #include "qtractorOptions.h"
 
+#include "qtractorMainForm.h"
+
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QLineEdit>
+#include <QUrl>
 
 // Needed for fabs(), logf() and powf()
 #include <math.h>
@@ -74,6 +76,9 @@ qtractorClipForm::qtractorClipForm (
 	QObject::connect(m_ui.FilenameToolButton,
 		SIGNAL(clicked()),
 		SLOT(browseFilename()));
+	QObject::connect(m_ui.ClipGainSpinBox,
+		SIGNAL(valueChanged(double)),
+		SLOT(changed()));
 	QObject::connect(m_ui.TrackChannelSpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(trackChannelChanged(int)));
@@ -135,11 +140,7 @@ qtractorClipForm::~qtractorClipForm (void)
 void qtractorClipForm::setClip ( qtractorClip *pClip, bool bClipNew )
 {
 	// Initialize conveniency options...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -164,6 +165,11 @@ void qtractorClipForm::setClip ( qtractorClip *pClip, bool bClipNew )
 	m_ui.ClipLengthSpinBox->setTimeScale(m_pTimeScale);
 	m_ui.FadeInLengthSpinBox->setTimeScale(m_pTimeScale);
 	m_ui.FadeOutLengthSpinBox->setTimeScale(m_pTimeScale);
+	// These have special delta formats...
+	m_ui.ClipOffsetSpinBox->setDeltaValue(true);
+	m_ui.ClipLengthSpinBox->setDeltaValue(true);
+	m_ui.FadeInLengthSpinBox->setDeltaValue(true);
+	m_ui.FadeOutLengthSpinBox->setDeltaValue(true);
 	// Initialize dialog widgets...
 	m_ui.ClipNameLineEdit->setText(m_pClip->clipName());
 	// Parameters...
@@ -197,6 +203,13 @@ void qtractorClipForm::setClip ( qtractorClip *pClip, bool bClipNew )
 	switch (trackType()) {
 	case qtractorTrack::Audio: {
 		m_ui.FilenameComboBox->setObjectName("Audio" + sSuffix);
+		m_ui.ClipGainTextLabel->setText(tr("&Gain:"));
+		m_ui.ClipGainSpinBox->setSuffix(tr(" dB"));
+		m_ui.ClipGainSpinBox->setRange(-60.0f, +12.0f);
+		float fClipGain = m_pClip->clipGain();
+		if (fClipGain < 0.001f || fClipGain > 4.0f)
+			fClipGain = 1.0f;
+		m_ui.ClipGainSpinBox->setValue(20.0f * ::log10f(fClipGain));
 		qtractorAudioClip *pAudioClip
 			= static_cast<qtractorAudioClip *> (m_pClip);
 		if (pAudioClip) {
@@ -216,6 +229,13 @@ void qtractorClipForm::setClip ( qtractorClip *pClip, bool bClipNew )
 	}
 	case qtractorTrack::Midi: {
 		m_ui.FilenameComboBox->setObjectName("Midi" + sSuffix);
+		m_ui.ClipGainTextLabel->setText(tr("&Volume:"));
+		m_ui.ClipGainSpinBox->setSuffix(tr(" %"));
+		m_ui.ClipGainSpinBox->setRange(0.0f, 1200.0f);
+		float fClipGain = m_pClip->clipGain();
+		if (fClipGain < 1.0f)
+			fClipGain = 1.0f;
+		m_ui.ClipGainSpinBox->setValue(100.0f * fClipGain);
 		qtractorMidiClip *pMidiClip
 			= static_cast<qtractorMidiClip *> (m_pClip);
 		if (pMidiClip)
@@ -233,7 +253,7 @@ void qtractorClipForm::setClip ( qtractorClip *pClip, bool bClipNew )
 		break;
 	}
 
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions)
 		pOptions->loadComboBoxHistory(m_ui.FilenameComboBox);
 
@@ -270,8 +290,8 @@ void qtractorClipForm::accept (void)
 	if (!m_pClip->queryEditor())
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Save settings...
@@ -282,6 +302,17 @@ void qtractorClipForm::accept (void)
 		const QString& sFilename  = m_ui.FilenameComboBox->currentText();
 		unsigned short iTrackChannel = m_ui.TrackChannelSpinBox->value();
 		const QString& sClipName  = m_ui.ClipNameLineEdit->text();
+		float fClipGain = 1.0f;
+		switch (clipType) {
+		case qtractorTrack::Audio:
+			fClipGain = ::powf(10.f, 0.05f * m_ui.ClipGainSpinBox->value());
+			break;
+		case qtractorTrack::Midi:
+			fClipGain = 0.01f * m_ui.ClipGainSpinBox->value();
+			break;
+		default:
+			break;
+		}
 		unsigned long iClipStart  = m_ui.ClipStartSpinBox->value();
 		unsigned long iClipOffset = m_ui.ClipOffsetSpinBox->value();
 		unsigned long iClipLength = m_ui.ClipLengthSpinBox->value();
@@ -326,6 +357,8 @@ void qtractorClipForm::accept (void)
 			default:
 				break;
 			}
+			// Gain/volume...
+			m_pClip->setClipGain(fClipGain);
 			// Parameters...
 			m_pClip->setClipStart(iClipStart);
 			m_pClip->setClipOffset(iClipOffset);
@@ -375,6 +408,9 @@ void qtractorClipForm::accept (void)
 			// Filename and/or track-channel changes...
 			if (iFileChange > 0)
 				pClipCommand->fileClip(m_pClip, sFilename, iTrackChannel);
+			// Gain/volume...
+			if (::fabs(fClipGain - m_pClip->clipGain()) > 0.001f)
+				pClipCommand->gainClip(m_pClip, fClipGain);
 			// Parameters and/or time-stretching changes...
 			if (iClipStart  != m_pClip->clipStart()  ||
 				iClipOffset != m_pClip->clipOffset() ||
@@ -398,9 +434,10 @@ void qtractorClipForm::accept (void)
 		}
 		// Do it (by making it undoable)...
 		if (pClipCommand)
-			pMainForm->commands()->exec(pClipCommand);
+			pSession->execute(pClipCommand);
 		// Account for a new file in game...
-		if (iFileChange > 0) {
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (iFileChange > 0 && pMainForm) {
 			switch (clipType) {
 			case qtractorTrack::Audio:
 				pMainForm->addAudioFile(sFilename);
@@ -413,7 +450,7 @@ void qtractorClipForm::accept (void)
 				break;
 			}
 			// Save history conveniency options...
-			qtractorOptions *pOptions = pMainForm->options();
+			qtractorOptions *pOptions = qtractorOptions::getInstance();
 			if (pOptions)
 				pOptions->saveComboBoxHistory(m_ui.FilenameComboBox);
 		}
@@ -559,26 +596,62 @@ qtractorTrack::TrackType qtractorClipForm::trackType (void) const
 void qtractorClipForm::browseFilename (void)
 {
 	QString sType;
+	QString sExt;
 	QString sFilter;
 
-	switch (trackType()) {
+	qtractorTrack::TrackType clipType = trackType();
+	switch (clipType) {
 	case qtractorTrack::Audio:
 		sType   = tr("Audio");
+		sExt    = qtractorAudioFileFactory::defaultExt();
 		sFilter = qtractorAudioFileFactory::filters();
 		break;
 	case qtractorTrack::Midi:
 		sType   = tr("MIDI");
-		sFilter = tr("MIDI Files (*.mid)");
+		sExt    = "mid";
+		sFilter = tr("MIDI files (*.%1 *.smf *.midi)").arg(sExt);
 		break;
 	case qtractorTrack::None:
 	default:
 		return;
 	}
 
-	// Browse for audio file...
-	QString sFilename = QFileDialog::getOpenFileName(
-		this, tr("%1 Clip File").arg(sType),
-		m_ui.FilenameComboBox->currentText(), sFilter);
+	// Browse for file...
+	QString sFilename;
+
+	const QString& sTitle = tr("%1 Clip File").arg(sType) + " - " QTRACTOR_TITLE;
+#if QT_VERSION < 0x040400
+	sFilename = QFileDialog::getOpenFileName(this,
+		sTitle, m_ui.FilenameComboBox->currentText(), sFilter);
+#else
+	QFileDialog fileDialog(this,
+		sTitle, m_ui.FilenameComboBox->currentText(), sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<QUrl> urls(fileDialog.sidebarUrls());
+		urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+		switch (clipType) {
+		case qtractorTrack::Audio:
+			urls.append(QUrl::fromLocalFile(pOptions->sAudioDir));
+			break;
+		case qtractorTrack::Midi:
+			urls.append(QUrl::fromLocalFile(pOptions->sMidiDir));
+			break;
+		case qtractorTrack::None:
+		default:
+			break;
+		}
+		fileDialog.setSidebarUrls(urls);
+	}
+	// Show dialog...
+	if (fileDialog.exec())
+		sFilename = fileDialog.selectedFiles().first();
+#endif
 
 	if (!sFilename.isEmpty()) {
 		m_ui.FilenameComboBox->setEditText(sFilename);
@@ -595,11 +668,7 @@ void qtractorClipForm::fileChanged (
 	if (m_iDirtySetup > 0)
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
