@@ -167,7 +167,11 @@ public:
 	// Connection list stuff.
 	struct ConnectItem
 	{
+		// Default contructor
+		ConnectItem() : index(0), client(-1), port (-1) {}
+		// Item members.
 		unsigned short index;
+		int client, port;
 		QString clientName;
 		QString portName;
 	};
@@ -187,8 +191,9 @@ public:
 				ConnectItem *pItem = iter.next();
 				if (pItem->index      == item.index &&
 					pItem->clientName == item.clientName &&
-					pItem->portName   == item.portName) 
+					pItem->portName   == item.portName) {
 					return pItem;
+				}
 			}
 			return NULL;
 		}
@@ -197,7 +202,7 @@ public:
 	// Connection lists accessors.
 	ConnectList& inputs()  { return m_inputs;  }
 	ConnectList& outputs() { return m_outputs; }
-	
+
 	// Retrieve/restore client:port connections;
 	// return the effective number of connection attempts.
 	virtual int updateConnects(BusMode busMode,
