@@ -1,7 +1,7 @@
 // qtractorExportForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,12 +28,14 @@
 #include "qtractorAudioFile.h"
 
 #include "qtractorMainForm.h"
+
 #include "qtractorSession.h"
 #include "qtractorOptions.h"
 
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QFileInfo>
+#include <QUrl>
 
 
 //----------------------------------------------------------------------------
@@ -67,6 +69,9 @@ qtractorExportForm::qtractorExportForm (
 	QObject::connect(m_ui.LoopRangeRadioButton,
 		SIGNAL(toggled(bool)),
 		SLOT(rangeChanged()));
+	QObject::connect(m_ui.PunchRangeRadioButton,
+		SIGNAL(toggled(bool)),
+		SLOT(rangeChanged()));
 	QObject::connect(m_ui.EditRangeRadioButton,
 		SIGNAL(toggled(bool)),
 		SLOT(rangeChanged()));
@@ -113,12 +118,8 @@ void qtractorExportForm::setExportType ( qtractorTrack::TrackType exportType )
 	m_exportType = exportType;
 
 	QIcon icon;
-	QString sExportType;
-	qtractorEngine   *pEngine   = NULL;
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
+	qtractorEngine  *pEngine  = NULL;
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession) {
 		// Copy from global time-scale instance...
 		if (m_pTimeScale)
@@ -130,34 +131,36 @@ void qtractorExportForm::setExportType ( qtractorTrack::TrackType exportType )
 		case qtractorTrack::Audio:
 			pEngine = pSession->audioEngine();
 			icon = QIcon(":/icons/trackAudio.png");
-			sExportType = tr("Audio");
+			m_sExportType = tr("Audio");
+			m_sExportExt  = qtractorAudioFileFactory::defaultExt();
 			break;
 		case qtractorTrack::Midi:
 			pEngine = pSession->midiEngine();
 			icon = QIcon(":/icons/trackMidi.png");
-			sExportType = tr("MIDI");
+			m_sExportType = tr("MIDI");
+			m_sExportExt  = "mid";
 			break;
 		case qtractorTrack::None:
 		default:
+			m_sExportType.clear();
+			m_sExportExt.clear();
 			break;
 		}
 	}
 
 	// Grab export file history, one that might me useful...
 	m_ui.ExportPathComboBox->setObjectName(
-		sExportType + m_ui.ExportPathComboBox->objectName());
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions)
-			pOptions->loadComboBoxHistory(m_ui.ExportPathComboBox);
-	}
+		m_sExportType + m_ui.ExportPathComboBox->objectName());
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions)
+		pOptions->loadComboBoxHistory(m_ui.ExportPathComboBox);
 
 	// Fill in the output bus names list...
 	m_ui.ExportBusNameComboBox->clear();
 	if (pEngine) {
 		QDialog::setWindowIcon(icon);
 		QDialog::setWindowTitle(
-			tr("Export %1").arg(sExportType) + " - " QTRACTOR_TITLE);
+			tr("Export %1").arg(m_sExportType) + " - " QTRACTOR_TITLE);
 		for (qtractorBus *pBus = pEngine->buses().first();
 				pBus; pBus = pBus->next()) {
 			if (pBus->busMode() & qtractorBus::Output)
@@ -200,8 +203,13 @@ qtractorTrack::TrackType qtractorExportForm::exportType (void) const
 // Accept settings (OK button slot).
 void qtractorExportForm::accept (void)
 {
+	// Enforce (again) default file extension...
+	QString sExportPath = m_ui.ExportPathComboBox->currentText();
+
+	if (QFileInfo(sExportPath).suffix() != m_sExportExt)
+		sExportPath += '.' + m_sExportExt;
+
 	// Check (again) wether the file already exists...
-	const QString& sExportPath = m_ui.ExportPathComboBox->currentText();
 	if (QFileInfo(sExportPath).exists()) {
 		if (QMessageBox::warning(this,
 			tr("Warning") + " - " QTRACTOR_TITLE,
@@ -215,11 +223,9 @@ void qtractorExportForm::accept (void)
 		}
 	}
 
-	qtractorSession  *pSession  = NULL;
+	qtractorSession  *pSession  = qtractorSession::getInstance();
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
-	if (pSession) {
+	if (pSession && pMainForm) {
 		// It can take a minute...
 		m_ui.ExportPathTextLabel->setEnabled(false);
 		m_ui.ExportPathComboBox->setEnabled(false);
@@ -229,8 +235,7 @@ void qtractorExportForm::accept (void)
 		m_ui.FormatGroupBox->setEnabled(false);
 		m_ui.OkPushButton->setEnabled(false);
 		// Carry on...
-		switch (m_exportType) {
-		case qtractorTrack::Audio: {
+		if (m_exportType == qtractorTrack::Audio) {
 			// Audio file export...
 			qtractorAudioEngine *pAudioEngine = pSession->audioEngine();
 			if (pAudioEngine) {
@@ -243,12 +248,16 @@ void qtractorExportForm::accept (void)
 					tr("Audio file export: \"%1\" started...")
 					.arg(sExportPath));
 				// Do the export as commanded...
-				if (pAudioEngine->fileExport(
+				QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+				bool bResult = pAudioEngine->fileExport(
 					sExportPath,
 					m_ui.ExportStartSpinBox->value(),
 					m_ui.ExportEndSpinBox->value(),
-					pExportBus)) {
+					pExportBus);
+				QApplication::restoreOverrideCursor();
+				if (bResult) {
 					// Log the success...
+					pMainForm->addAudioFile(sExportPath);
 					pMainForm->appendMessages(
 						tr("Audio file export: \"%1\" complete.")
 						.arg(sExportPath));
@@ -259,9 +268,9 @@ void qtractorExportForm::accept (void)
 						.arg(sExportPath));
 				}
 			}
-			break;
 		}
-		case qtractorTrack::Midi: {
+		else
+		if (m_exportType == qtractorTrack::Midi) {
 			// MIDI file export...
 			qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
 			if (pMidiEngine) {
@@ -274,12 +283,16 @@ void qtractorExportForm::accept (void)
 					tr("MIDI file export: \"%1\" started...")
 					.arg(sExportPath));
 				// Do the export as commanded...
-				if (pMidiEngine->fileExport(
+				QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+				bool bResult = pMidiEngine->fileExport(
 					sExportPath,
 					m_ui.ExportStartSpinBox->value(),
 					m_ui.ExportEndSpinBox->value(),
-					pExportBus)) {
+					pExportBus);
+				QApplication::restoreOverrideCursor();
+				if (bResult) {
 					// Log the success...
+					pMainForm->addMidiFile(sExportPath);
 					pMainForm->appendMessages(
 						tr("MIDI file export: \"%1\" complete.")
 						.arg(sExportPath));
@@ -290,13 +303,9 @@ void qtractorExportForm::accept (void)
 						.arg(sExportPath));
 				}
 			}
-			break;
-		}
-		default:
-			break;
 		}
 		// Save other conveniency options...
-		qtractorOptions *pOptions = pMainForm->options();
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions)
 			pOptions->saveComboBoxHistory(m_ui.ExportPathComboBox);
 	}
@@ -309,10 +318,7 @@ void qtractorExportForm::accept (void)
 // Reject settings (Cancel button slot).
 void qtractorExportForm::reject (void)
 {
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession) {
 		// It can take a minute...
 		m_ui.CancelPushButton->setEnabled(false);
@@ -338,11 +344,7 @@ void qtractorExportForm::reject (void)
 // Choose the target filename of export.
 void qtractorExportForm::browseExportPath (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -350,34 +352,49 @@ void qtractorExportForm::browseExportPath (void)
 	if (sExportPath.isEmpty())
 		sExportPath = pSession->sessionDir();
 
-	QString sExportType;
-	QString sExportExt;
-	switch (m_exportType) {
-	case qtractorTrack::Audio:
-		sExportType = tr("Audio");
-		sExportExt  = qtractorAudioFileFactory::defaultExt();
-		break;
-	case qtractorTrack::Midi:
-		sExportType = tr("MIDI");
-		sExportExt  = "mid";
-		break;
-	case qtractorTrack::None:
-	default:
-		return;
-	}
-
 	// Actual browse for the file...
+	const QString& sTitle  = tr("Export %1 File").arg(m_sExportType) + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("%1 files (*.%1)").arg(m_sExportExt);
+#if QT_VERSION < 0x040400
 	sExportPath = QFileDialog::getSaveFileName(this,
-		tr("Export %1 File").arg(sExportType) + " - " QTRACTOR_TITLE,
-		sExportPath, tr("%1 files (*.%2)").arg(sExportType).arg(sExportExt));
+		sTitle, sExportPath, sFilter);
+#else
+	QFileDialog fileDialog(this,
+		sTitle, sExportPath, sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+	fileDialog.setFileMode(QFileDialog::AnyFile);
+	fileDialog.setDefaultSuffix(m_sExportExt);
+	// Stuff sidebar...
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<QUrl> urls(fileDialog.sidebarUrls());
+		urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+		switch (m_exportType) {
+		case qtractorTrack::Audio:
+			urls.append(QUrl::fromLocalFile(pOptions->sAudioDir));
+			break;
+		case qtractorTrack::Midi:
+			urls.append(QUrl::fromLocalFile(pOptions->sMidiDir));
+			break;
+		case qtractorTrack::None:
+		default:
+			break;
+		}
+		fileDialog.setSidebarUrls(urls);
+	}
+	// Show dialog...
+	if (fileDialog.exec())
+		sExportPath = fileDialog.selectedFiles().first();
+#endif
 
 	// Have we cancelled it?
 	if (sExportPath.isEmpty())
 		return;
 
 	// Enforce default file extension...
-	if (QFileInfo(sExportPath).suffix() != sExportExt)
-		sExportPath += '.' + sExportExt;
+	if (QFileInfo(sExportPath).suffix() != m_sExportExt)
+		sExportPath += '.' + m_sExportExt;
 
 	// Finallly set as wanted...
 	m_ui.ExportPathComboBox->setEditText(sExportPath);
@@ -390,11 +407,7 @@ void qtractorExportForm::browseExportPath (void)
 // Display format has changed.
 void qtractorExportForm::rangeChanged (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -408,6 +421,11 @@ void qtractorExportForm::rangeChanged (void)
 		m_ui.ExportEndSpinBox->setValue(pSession->loopEnd(), false);
 	}
 	else
+	if (m_ui.PunchRangeRadioButton->isChecked()) {
+		m_ui.ExportStartSpinBox->setValue(pSession->punchIn(), false);
+		m_ui.ExportEndSpinBox->setValue(pSession->punchOut(), false);
+	}
+	else
 	if (m_ui.EditRangeRadioButton->isChecked()) {
 		m_ui.ExportStartSpinBox->setValue(pSession->editHead(), false);
 		m_ui.ExportEndSpinBox->setValue(pSession->editTail(), false);
@@ -451,16 +469,12 @@ void qtractorExportForm::formatChanged (void)
 // Stabilize current form state.
 void qtractorExportForm::stabilizeForm (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
-	m_ui.LoopRangeRadioButton->setEnabled(
-		pSession->loopStart() < pSession->loopEnd());
+	m_ui.LoopRangeRadioButton->setEnabled(pSession->isLooping());
+	m_ui.PunchRangeRadioButton->setEnabled(pSession->isPunching());
 	m_ui.EditRangeRadioButton->setEnabled(
 		pSession->editHead() < pSession->editTail());
 
