@@ -1,8 +1,7 @@
 // qtractorExportForm.h
 //
-// ui.h extension file, included from the uic-generated form implementation.
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -62,6 +61,10 @@ private:
 
 	// Instance variables...
 	qtractorTrack::TrackType m_exportType;
+
+	QString m_sExportType;
+	QString m_sExportExt;
+
 	qtractorTimeScale *m_pTimeScale;
 };
 
