@@ -25,16 +25,18 @@
 #include "qtractorRubberBand.h"
 #include "qtractorDocument.h"
 
-#include "qtractorMainForm.h"
 #include "qtractorOptions.h"
+#include "qtractorSession.h"
 
 #include <QMessageBox>
 #include <QApplication>
 #include <QHeaderView>
+#include <QClipboard>
 #include <QFileInfo>
 #include <QToolTip>
 #include <QTimer>
 #include <QUrl>
+#include <QDir>
 
 #include <QMouseEvent>
 #include <QDragLeaveEvent>
@@ -487,7 +489,7 @@ void qtractorFileListView::openFile (void)
 	if (files.isEmpty())
 		return;
 
-	// Find a proper group parent  group item...
+	// Find a proper group parent group item...
 	qtractorFileGroupItem *pParentItem = currentGroupItem();
 	// Pick each one of the selected files...
 	int iUpdate = 0;
@@ -523,6 +525,75 @@ void qtractorFileListView::newGroup (void)
 }
 
 
+// Copy/cut current file item(s) to clipboard.
+void qtractorFileListView::copyItem ( bool bCut )
+{
+	// Build URL list...
+	QList<QUrl> urls;
+
+	int iUpdate = 0;
+	QList<QTreeWidgetItem *> items = selectedItems();
+	QListIterator<QTreeWidgetItem *> iter(items);
+	while (iter.hasNext()) {
+		QTreeWidgetItem *pItem = iter.next();
+		if (pItem->type() == qtractorFileListView::FileItem) {
+			qtractorFileListItem *pFileItem
+				= static_cast<qtractorFileListItem *> (pItem);
+			if (pFileItem) {
+				urls.append(QUrl::fromLocalFile(pFileItem->path()));
+				if (bCut) {
+					delete pFileItem;
+					iUpdate++;
+				}
+			}
+		}
+	}
+
+	//	Copy it to system clipboard...
+	if (!urls.isEmpty()) {
+		QMimeData *pMimeData = new QMimeData();
+		pMimeData->setUrls(urls);
+		QApplication::clipboard()->setMimeData(pMimeData);
+	}
+
+	// Make proper notifications...
+	if (iUpdate > 0)
+		emit contentsChanged();
+}
+
+
+// Paste file item(s9 from clipboard.
+void qtractorFileListView::pasteItem (void)
+{
+	const QMimeData *pMimeData = QApplication::clipboard()->mimeData();
+	if (!pMimeData->hasUrls())
+		return;
+
+	// Find a proper group parent group item...
+	qtractorFileGroupItem *pParentItem = currentGroupItem();
+	int iUpdate = 0;
+
+	QListIterator<QUrl> iter(pMimeData->urls());
+	while (iter.hasNext()) {
+		const QString& sPath
+			= QDir::toNativeSeparators(iter.next().toLocalFile());
+		if (!sPath.isEmpty()) {
+			qtractorFileListItem *pFileItem	= addFileItem(sPath, pParentItem);
+			// Make all this new open and visible.
+			if (pFileItem) {
+				iUpdate++;
+				if (pParentItem)
+					pParentItem->setOpen(true);
+			}
+		}
+	}
+
+	// Make proper notifications...
+	if (iUpdate > 0)
+		emit contentsChanged();
+}
+
+
 // Rename current group/file item.
 void qtractorFileListView::renameItem (void)
 {
@@ -535,11 +606,7 @@ void qtractorFileListView::renameItem (void)
 // Remove current group/file item.
 void qtractorFileListView::deleteItem (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions == NULL)
 		return;
 
@@ -870,12 +937,21 @@ void qtractorFileListView::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			pDrag->setHotSpot(QPoint(-4, -12));
 			pDrag->start(Qt::CopyAction | Qt:: MoveAction);
 			// We've dragged and maybe dropped it by now...
-			m_pDragItem = NULL;
+			mouseReleaseEvent(pMouseEvent);
 		}
 	}
 }
 
 
+void qtractorFileListView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
+{
+	QTreeWidget::mouseReleaseEvent(pMouseEvent);
+
+	dragLeaveEvent(NULL);
+	m_pDragItem = NULL;
+}
+
+
 // Drag-n-drop stuff.
 bool qtractorFileListView::canDecodeEvent ( QDropEvent *pDropEvent )
 {
@@ -1040,7 +1116,8 @@ void qtractorFileListView::dropEvent ( QDropEvent *pDropEvent )
 		QListIterator<QUrl> iter(pMimeData->urls());
 		iter.toBack();
 		while (iter.hasPrevious()) {
-			const QString& sPath = iter.previous().toLocalFile();
+			const QString& sPath
+				= QDir::toNativeSeparators(iter.previous().toLocalFile());
 			// Is it one from ourselves (file item) ?...
 			if (m_pDragItem && m_pDragItem->type() == FileItem) {
 				if (dropItem(pDropItem, findItem(sPath, FileItem), bOutdent))
@@ -1160,6 +1237,12 @@ bool qtractorFileListView::loadListElement ( qtractorDocument *pDocument,
 	qtractorFileGroupItem *pParentItem
 		= static_cast<qtractorFileGroupItem *> (pItem);
 
+	// Make it all relative to session directory...
+	QDir dir;
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession)
+		dir.setPath(pSession->sessionDir());
+
 	// Load children...
 	for (QDomNode nChild = pElement->firstChild();
 			!nChild.isNull(); nChild = nChild.nextSibling()) {
@@ -1182,7 +1265,8 @@ bool qtractorFileListView::loadListElement ( qtractorDocument *pDocument,
 		}
 		else
 		if (eChild.tagName() == "file") {
-			qtractorFileListItem *pFileItem = createFileItem(eChild.text());
+			qtractorFileListItem *pFileItem = createFileItem(
+				QDir::cleanPath(dir.absoluteFilePath(eChild.text())));
 			if (pFileItem) {
 				pFileItem->setText(0, eChild.attribute("name"));
 				if (pParentItem)
@@ -1230,8 +1314,13 @@ bool qtractorFileListView::saveListElement ( qtractorDocument *pDocument,
 			= static_cast<qtractorFileListItem *> (pItem);
 		QDomElement eFile = pDocument->document()->createElement("file");
 		eFile.setAttribute("name", pFileItem->text(0));
+		// Make it all relative to session directory...
+		QDir dir;
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession)
+			dir.setPath(pSession->sessionDir());
 		eFile.appendChild(pDocument->document()->createTextNode(
-			pFileItem->path()));
+			dir.relativeFilePath(pFileItem->path())));
 		pElement->appendChild(eFile);
 		break;
 	}
