@@ -28,6 +28,9 @@
 #include <QHBoxLayout>
 #include <QToolButton>
 
+#include <QApplication>
+#include <QClipboard>
+
 #include <QContextMenuEvent>
 
 
@@ -43,7 +46,9 @@ qtractorFiles::qtractorFiles ( QWidget *pParent )
 	QDockWidget::setObjectName("qtractorFiles");
 
 	// Create file type selection tab widget.
+	const QFont& font = QDockWidget::font();
 	m_pTabWidget = new QTabWidget(this);
+	m_pTabWidget->setFont(QFont(font.family(), font.pointSize() - 1));
 	m_pTabWidget->setTabPosition(QTabWidget::South);
 	// Create local tabs.
 	m_pAudioListView = new qtractorAudioListView();
@@ -79,26 +84,38 @@ qtractorFiles::qtractorFiles ( QWidget *pParent )
 		QIcon(":/icons/itemGroup.png"), tr("New &Group"), this);
 	m_pOpenFileAction = new QAction(
 		QIcon(":/icons/itemFile.png"), tr("Add &Files..."), this);
+	m_pCutItemAction = new QAction(
+		QIcon(":/icons/editCut.png"), tr("Cu&t"), NULL);
+	m_pCopyItemAction = new QAction(
+		QIcon(":/icons/editCopy.png"), tr("&Copy"), NULL);
+	m_pPasteItemAction = new QAction(
+		QIcon(":/icons/editPaste.png"), tr("P&aste"), NULL);
 	m_pRenameItemAction = new QAction(
 		QIcon(":/icons/formEdit.png"), tr("R&ename"), this);
 	m_pDeleteItemAction = new QAction(
-		QIcon(":/icons/formRemove.png"), tr("&Delete"), this);
+		QIcon(":/icons/formRemove.png"), tr("&Delete"), NULL);
 	m_pPlayItemAction = new QAction(
-		QIcon(":/icons/transportPlay.png"), tr("&Play"), this);
+		QIcon(":/icons/transportPlay.png"), tr("Play"), this);
 	m_pPlayItemAction->setCheckable(true);
 
 	m_pNewGroupAction->setShortcut(tr("Ctrl+G"));
 	m_pOpenFileAction->setShortcut(tr("Ctrl+F"));
+	m_pCutItemAction->setShortcut(tr("Ctrl+X"));
+	m_pCopyItemAction->setShortcut(tr("Ctrl+C"));
+	m_pPasteItemAction->setShortcut(tr("Ctrl+V"));
 	m_pRenameItemAction->setShortcut(tr("Ctrl+E"));
-	m_pDeleteItemAction->setShortcut(tr("Ctrl+D"));
+	m_pDeleteItemAction->setShortcut(tr("Del"));
 //	m_pPlayItemAction->setShortcut(tr("Ctrl+P"));
 
 	// Some actions surely need those
 	// shortcuts firmly attached...
 	QDockWidget::addAction(m_pNewGroupAction);
 	QDockWidget::addAction(m_pOpenFileAction);
+//	QDockWidget::addAction(m_pCutItemAction);
+//	QDockWidget::addAction(m_pCopyItemAction);
+//	QDockWidget::addAction(m_pPasteItemAction);
 	QDockWidget::addAction(m_pRenameItemAction);
-	QDockWidget::addAction(m_pDeleteItemAction);
+//	QDockWidget::addAction(m_pDeleteItemAction);
 	QDockWidget::addAction(m_pPlayItemAction);
 
 	// Prepare the dockable window stuff.
@@ -125,15 +142,30 @@ qtractorFiles::qtractorFiles ( QWidget *pParent )
 	QObject::connect(m_pAudioListView,
 		SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
 		SLOT(stabilizeSlot()));
+	QObject::connect(m_pAudioListView,
+		SIGNAL(itemClicked(QTreeWidgetItem*,int)),
+		SLOT(stabilizeSlot()));
 	QObject::connect(m_pMidiListView,
 		SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)),
 		SLOT(stabilizeSlot()));
+	QObject::connect(m_pMidiListView,
+		SIGNAL(itemClicked(QTreeWidgetItem*,int)),
+		SLOT(stabilizeSlot()));
 	QObject::connect(m_pNewGroupAction,
 		SIGNAL(triggered(bool)),
 		SLOT(newGroupSlot()));
 	QObject::connect(m_pOpenFileAction,
 		SIGNAL(triggered(bool)),
 		SLOT(openFileSlot()));
+	QObject::connect(m_pCutItemAction,
+		SIGNAL(triggered(bool)),
+		SLOT(cutItemSlot()));
+	QObject::connect(m_pCopyItemAction,
+		SIGNAL(triggered(bool)),
+		SLOT(copyItemSlot()));
+	QObject::connect(m_pPasteItemAction,
+		SIGNAL(triggered(bool)),
+		SLOT(pasteItemSlot()));
 	QObject::connect(m_pRenameItemAction,
 		SIGNAL(triggered(bool)),
 		SLOT(renameItemSlot()));
@@ -146,6 +178,10 @@ qtractorFiles::qtractorFiles ( QWidget *pParent )
 	QObject::connect(m_pPlayButton,
 		SIGNAL(toggled(bool)),
 		SLOT(playSlot(bool)));
+
+	QObject::connect(QApplication::clipboard(),
+		SIGNAL(dataChanged()),
+		SLOT(stabilizeSlot()));
 }
 
 
@@ -154,6 +190,9 @@ qtractorFiles::~qtractorFiles (void)
 {
 	delete m_pNewGroupAction;
 	delete m_pOpenFileAction;
+	delete m_pCutItemAction;
+	delete m_pCopyItemAction;
+	delete m_pPasteItemAction;
 	delete m_pRenameItemAction;
 	delete m_pDeleteItemAction;
 	delete m_pPlayItemAction;
@@ -197,6 +236,29 @@ void qtractorFiles::clear (void)
 }
 
 
+// Check whether one of the widgets has focus (oveerride method)
+bool qtractorFiles::hasFocus (void) const
+{
+	if (m_pAudioListView->hasFocus() || m_pMidiListView->hasFocus())
+		return true;
+
+	return QDockWidget::hasFocus();
+}
+
+
+// Tell whether a file item is currently selected.
+bool qtractorFiles::isFileSelected (void) const
+{
+	qtractorFileListView *pFileListView = currentFileListView();
+	if (pFileListView) {
+		QTreeWidgetItem *pItem = pFileListView->currentItem();
+		return (pItem && pItem->type() == qtractorFileListView::FileItem);
+	}
+
+	return false;
+}
+
+
 // Audio file addition convenience method.
 void qtractorFiles::addAudioFile ( const QString& sFilename )
 {
@@ -277,6 +339,33 @@ void qtractorFiles::newGroupSlot (void)
 }
 
 
+// Cut current file item(s) to clipboard.
+void qtractorFiles::cutItemSlot (void)
+{
+	qtractorFileListView *pFileListView = currentFileListView();
+	if (pFileListView)
+		pFileListView->copyItem(true);
+}
+
+
+// Copy current file item(s) to clipboard.
+void qtractorFiles::copyItemSlot (void)
+{
+	qtractorFileListView *pFileListView = currentFileListView();
+	if (pFileListView)
+		pFileListView->copyItem(false);
+}
+
+
+// Paste file item(s) from clipboard.
+void qtractorFiles::pasteItemSlot (void)
+{
+	qtractorFileListView *pFileListView = currentFileListView();
+	if (pFileListView)
+		pFileListView->pasteItem();
+}
+
+
 // Rename current group/file item.
 void qtractorFiles::renameItemSlot (void)
 {
@@ -301,6 +390,12 @@ void qtractorFiles::stabilizeSlot (void)
 	qtractorFileListView *pFileListView = currentFileListView();
 	if (pFileListView) {
 		QTreeWidgetItem *pItem = pFileListView->currentItem();
+		m_pCutItemAction->setEnabled(
+			pItem && pItem->type() == qtractorFileListView::FileItem);
+		m_pCopyItemAction->setEnabled(
+			pItem && pItem->type() == qtractorFileListView::FileItem);
+		m_pPasteItemAction->setEnabled(
+			(QApplication::clipboard()->mimeData())->hasUrls());
 		m_pRenameItemAction->setEnabled(
 			pItem && pItem->type() == qtractorFileListView::GroupItem);
 		m_pDeleteItemAction->setEnabled(
@@ -311,6 +406,11 @@ void qtractorFiles::stabilizeSlot (void)
 		m_pPlayItemAction->setEnabled(bPlayEnabled);
 		m_pPlayButton->setEnabled(bPlayEnabled);
 	}
+
+	// Stabilize main form as well...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm)
+		pMainForm->stabilizeForm();
 }
 
 
@@ -345,8 +445,12 @@ void qtractorFiles::contextMenuEvent (
 
 	// Construct context menu.
 	menu.addAction(m_pNewGroupAction);
-	menu.addSeparator();
 	menu.addAction(m_pOpenFileAction);
+	menu.addSeparator();
+	menu.addAction(m_pCutItemAction);
+	menu.addAction(m_pCopyItemAction);
+	menu.addAction(m_pPasteItemAction);
+	menu.addSeparator();
 	menu.addAction(m_pRenameItemAction);
 	menu.addAction(m_pDeleteItemAction);
 	menu.addSeparator();
