@@ -23,13 +23,13 @@
 
 #include "qtractorAbout.h"
 #include "qtractorInstrument.h"
+#include "qtractorSession.h"
 #include "qtractorOptions.h"
 
-#include "qtractorMainForm.h"
-
 #include <QHeaderView>
-#include <QFileDialog>
 #include <QMessageBox>
+#include <QFileDialog>
+#include <QUrl>
 
 
 //----------------------------------------------------------------------
@@ -138,24 +138,44 @@ qtractorInstrumentForm::~qtractorInstrumentForm (void)
 // Import new intrument file(s) into listing.
 void qtractorInstrumentForm::importSlot (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions == NULL)
 		return;
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
-	QStringList files = QFileDialog::getOpenFileNames(
-			this,                               // Parent.
-			tr("Import Instrument Files"),      // Caption.
-			pOptions->sInstrumentDir,           // Start here.
-			tr("Instrument Files (*.ins)")      // Filter files.
-	);
+	QStringList files;
+
+	const QString  sExt("ins");
+	const QString& sTitle  = tr("Import Instrument Files") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("Instrument files (*.%1)").arg(sExt);
+#if QT_VERSION < 0x040400
+	// Ask for the filename to open...
+	files = QFileDialog::getOpenFileNames(this,
+		sTitle, pOptions->sInstrumentDir, sFilter);
+#else
+	// Construct open-files dialog...
+	QFileDialog fileDialog(this,
+		sTitle, pOptions->sInstrumentDir, sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFiles);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+	urls.append(QUrl::fromLocalFile(pOptions->sInstrumentDir));
+	fileDialog.setSidebarUrls(urls);
+	// Show dialog...
+	if (fileDialog.exec())
+		files = fileDialog.selectedFiles();
+#endif
 
 	if (files.isEmpty())
 		return;
@@ -200,11 +220,11 @@ void qtractorInstrumentForm::importSlot (void)
 // Remove a file from instrument list.
 void qtractorInstrumentForm::removeSlot (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
@@ -258,11 +278,11 @@ void qtractorInstrumentForm::moveDownSlot (void)
 // Reload the complete instrument definitions, from list.
 void qtractorInstrumentForm::reloadSlot (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
@@ -288,31 +308,51 @@ void qtractorInstrumentForm::reloadSlot (void)
 // Export the whole state into a single instrument file.
 void qtractorInstrumentForm::exportSlot (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions == NULL)
 		return;
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
-	QString sPath = QFileDialog::getSaveFileName(
-			this,                               // Parent.
-			tr("Export Instrument File"),       // Caption.
-			pOptions->sInstrumentDir,           // Start here.
-			tr("Instrument Files (*.ins)")      // Filter files.
-	);
+	QString sPath;
+
+	const QString  sExt("ins");
+	const QString& sTitle  = tr("Export Instrument File") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("Instrument files (*.%1)").arg(sExt);
+#if QT_VERSION < 0x040400
+	// Ask for the filename to open...
+	sPath = QFileDialog::getOpenFileName(this,
+		sTitle, pOptions->sInstrumentDir, sFilter);
+#else
+	// Construct open-files dialog...
+	QFileDialog fileDialog(this,
+		sTitle, pOptions->sInstrumentDir, sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+	fileDialog.setFileMode(QFileDialog::AnyFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+	urls.append(QUrl::fromLocalFile(pOptions->sInstrumentDir));
+	fileDialog.setSidebarUrls(urls);
+	// Show dialog...
+	if (fileDialog.exec())
+		sPath = fileDialog.selectedFiles().first();
+#endif
 
 	if (sPath.isEmpty())
 		return;
 
 	// Enforce .ins extension...
 	if (QFileInfo(sPath).suffix().isEmpty())
-		sPath += ".ins";
+		sPath += '.' + sExt;
 
 	// Check if already exists...
 	if (QFileInfo(sPath).exists()) {
@@ -389,9 +429,9 @@ void qtractorInstrumentForm::stabilizeForm (void)
 
 	m_ui.ReloadPushButton->setEnabled(m_iDirtyCount > 0);
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm && pMainForm->instruments())
-		m_ui.ExportPushButton->setEnabled(pMainForm->instruments()->count() > 0);
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession && pSession->instruments())
+		m_ui.ExportPushButton->setEnabled((pSession->instruments())->count() > 0);
 	else
 		m_ui.ExportPushButton->setEnabled(false);
 }
@@ -400,11 +440,11 @@ void qtractorInstrumentForm::stabilizeForm (void)
 // Refresh all instrument definition views.
 void qtractorInstrumentForm::refreshForm (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
@@ -463,7 +503,7 @@ void qtractorInstrumentForm::refreshForm (void)
 					QString("Based On = %1").arg(patch.basedOn()));
 			}
 			qtractorInstrumentData::ConstIterator it;
-			for (it = patch.begin(); it != patch.end(); ++it) {
+			for (it = patch.constBegin(); it != patch.constEnd(); ++it) {
 				int iProg = it.key();
 				pProgItem = new QTreeWidgetItem(pBankItem, pProgItem);
 				pProgItem->setText(0,
@@ -590,7 +630,7 @@ void qtractorInstrumentForm::listInstrumentData (
 			tr("Based On = %1").arg(data.basedOn()));
 	}
 	qtractorInstrumentData::ConstIterator it;
-	for (it = data.begin(); it != data.end(); ++it) {
+	for (it = data.constBegin(); it != data.constEnd(); ++it) {
 		pItem = new QTreeWidgetItem(pParentItem, pItem);
 		pItem->setText(0,
 			QString("%1 = %2").arg(it.key()).arg(it.value()));
