@@ -98,7 +98,13 @@ public:
 	const LADSPA_Descriptor *ladspa_descriptor() const;
 	LADSPA_Handle ladspa_handle(unsigned short iInstance) const;
 
-private:
+	// Audio port numbers.
+	unsigned long audioIn(unsigned short i)
+		{ return m_piAudioIns[i]; }
+	unsigned long audioOut(unsigned short i)
+		{ return m_piAudioOuts[i]; }
+
+protected:
 
 	// Instance variables.
 	LADSPA_Handle *m_phInstances;
