@@ -38,6 +38,7 @@ class qtractorMessages;
 class qtractorConnections;
 class qtractorMixer;
 class qtractorMmcEvent;
+class qtractorMidiControlEvent;
 class qtractorSpinBox;
 
 class qtractorMidiEditorForm;
@@ -45,6 +46,7 @@ class qtractorMidiEditor;
 
 class QLabel;
 class QComboBox;
+class QProgressBar;
 class QActionGroup;
 class QDoubleSpinBox;
 class QPalette;
@@ -66,16 +68,12 @@ public:
 
 	static qtractorMainForm *getInstance();
 
-	void setOptions(qtractorOptions *pOptions);
-	qtractorOptions *options() const;
+	void setup(qtractorOptions *pOptions);
 
-	qtractorSession *session() const;
 	qtractorTracks *tracks() const;
 	qtractorFiles *files() const;
 	qtractorConnections *connections() const;
 	qtractorMixer *mixer() const;
-	qtractorCommandList *commands() const;
-	qtractorInstrumentList *instruments() const;
 	qtractorThumbView *thumbView() const;
 
 	QString sessionName(const QString& sFilename);
@@ -95,6 +93,9 @@ public:
 	QMenu *trackMenu() const
 		{ return m_ui.trackMenu; }
 
+	QProgressBar *progressBar() const
+		{ return m_pProgressBar; }
+
 	void contextMenuEvent(QContextMenuEvent *pEvent);
 
 	int rolling() const;
@@ -114,6 +115,7 @@ public slots:
 	void editCut();
 	void editCopy();
 	void editPaste();
+	void editPasteRepeat();
 	void editDelete();
 	void editSelectModeClip();
 	void editSelectModeRange();
@@ -125,6 +127,8 @@ public slots:
 	void editClipNew();
 	void editClipEdit();
 	void editClipSplit();
+	void editClipNormalize();
+	void editClipExport();
 
 	void trackAdd();
 	void trackRemove();
@@ -143,6 +147,7 @@ public slots:
 	void trackMoveUp();
 	void trackMoveDown();
 	void trackMoveBottom();
+	void trackAutoMonitor(bool bOn);
 	void trackImportAudio();
 	void trackImportMidi();
 	void trackExportAudio();
@@ -179,6 +184,8 @@ public slots:
 	void transportLoopSet();
 	void transportPlay();
 	void transportRecord();
+	void transportPunch();
+	void transportPunchSet();
 	void transportMetro();
 	void transportFollow();
 	void transportAutoBackward();
@@ -227,6 +234,7 @@ protected:
 	void customEvent(QEvent *pEvent);
 
 	void mmcEvent(qtractorMmcEvent *pMmcEvent);
+	void midiControlEvent(qtractorMidiControlEvent *pCtlEvent);
 
 	bool queryClose();
 
@@ -236,8 +244,8 @@ protected:
 	bool editSession();
 	bool closeSession();
 
-	bool loadSessionFile(const QString& sFilename);
-	bool saveSessionFile(const QString& sFilename);
+	bool loadSessionFile(const QString& sFilename, bool bTemplate = false);
+	bool saveSessionFile(const QString& sFilename, bool bTemplate = false);
 
 	bool startSession();
 	bool checkRestartSession();
@@ -276,8 +284,6 @@ private:
 	// Instance variables...
 	qtractorOptions *m_pOptions;
 	qtractorSession *m_pSession;
-	qtractorCommandList *m_pCommands;
-	qtractorInstrumentList *m_pInstruments;
 	qtractorFiles *m_pFiles;
 	qtractorMessages *m_pMessages;
 	qtractorConnections *m_pConnections;
@@ -290,8 +296,10 @@ private:
 	qtractorSpinBox *m_pTransportTimeSpinBox;
 	QDoubleSpinBox *m_pTempoSpinBox;
 	QComboBox *m_pSnapPerBeatComboBox;
+	QProgressBar *m_pProgressBar;
 	qtractorThumbView *m_pThumbView;
 	unsigned long m_iPlayHead;
+	long m_iDeltaTimer;
 	int m_iPeakTimer;
 	int m_iPlayTimer;
 	int m_iTransportUpdate;
