@@ -28,12 +28,15 @@
 #include "qtractorMidiEditor.h"
 #include "qtractorMidiEditorForm.h"
 
-#include "qtractorMainForm.h"
 #include "qtractorOptions.h"
 
 #include <QFileInfo>
 #include <QPainter>
 
+#if QT_VERSION < 0x040300
+#define lighter light
+#endif
+
 
 //----------------------------------------------------------------------
 // class qtractorMidiClip -- Audio file/buffer clip.
@@ -47,8 +50,9 @@ qtractorMidiClip::qtractorMidiClip ( qtractorTrack *pTrack )
 	m_pSeq  = new qtractorMidiSequence();
 
 	m_iTrackChannel = 0;
-	m_iFormat       = defaultFormat();
-	m_bSessionFlag  = false;
+	m_iFormat = defaultFormat();
+	m_bSessionFlag = false;
+	m_iRevision = 0;
 
 	m_pMidiEditorForm = NULL;
 }
@@ -62,9 +66,11 @@ qtractorMidiClip::qtractorMidiClip ( const qtractorMidiClip& clip )
 
 	setFilename(clip.filename());
 	setTrackChannel(clip.trackChannel());
+	setClipGain(clip.clipGain());
 
-	m_iFormat      = clip.format();
+	m_iFormat = clip.format();
 	m_bSessionFlag = false;
+	m_iRevision = 0;
 
 	m_pMidiEditorForm = NULL;
 }
@@ -73,8 +79,10 @@ qtractorMidiClip::qtractorMidiClip ( const qtractorMidiClip& clip )
 // Destructor.
 qtractorMidiClip::~qtractorMidiClip (void)
 {
+	close(true);
+
 	if (m_pMidiEditorForm) {
-		m_pMidiEditorForm->close();
+		m_pMidiEditorForm->forceClose();
 		delete m_pMidiEditorForm;
 	}
 
@@ -225,6 +233,32 @@ bool qtractorMidiClip::isSessionFlag (void) const
 }
 
 
+// Revisionist methods.
+void qtractorMidiClip::setRevision ( unsigned short iRevision )
+{
+	m_iRevision = iRevision;
+}
+
+unsigned short qtractorMidiClip::revision (void) const
+{
+	return m_iRevision;
+}
+
+
+QString qtractorMidiClip::createFilePathRevision ( bool bForce )
+{
+	QString sFilename = filename();
+
+	if (m_iRevision == 0 || bForce)
+		sFilename = qtractorMidiFile::createFilePathRevision(sFilename);
+
+	if (!bForce)
+		m_iRevision++;
+
+	return sFilename;
+}
+
+
 // Sequence properties accessors.
 qtractorMidiSequence *qtractorMidiClip::sequence (void) const
 {
@@ -306,16 +340,24 @@ void qtractorMidiClip::close ( bool bForce )
 	if (pSession == NULL)
 		return;
 
+	// Take pretended clip-length...
+	unsigned long iClipLength = clipLength();
+	if (iClipLength > 0)
+		m_pSeq->setTimeLength(pSession->tickFromFrame(iClipLength));
+
 	// Actual sequence closure...
 	m_pSeq->close();
 
 	// Commit the final clip length...
-	if (clipLength() < 1)
-		setClipLength(pSession->frameFromTick(m_pSeq->duration()));
+	if (m_pSeq->events().count() < 1)
+		iClipLength = 0;
+	else if (iClipLength < 1)
+		iClipLength = pSession->frameFromTick(m_pSeq->duration());
+	setClipLength(iClipLength);
 
 	// Now's time to write the whole thing...
 	bool bNewFile = (m_pFile && m_pFile->mode() == qtractorMidiFile::Write);
-	if (bNewFile && clipLength() > 0) {
+	if (bNewFile && iClipLength > 0) {
 		// Write SMF header...
 		unsigned short iTracks = 1;
 		if (m_iFormat == 1)
@@ -326,10 +368,6 @@ void qtractorMidiClip::close ( bool bForce )
 			m_pFile->writeTrack(NULL);  // Setup track (SMF format 1).
 		m_pFile->writeTrack(m_pSeq);    // Channel track.
 		m_pFile->close();
-		// Nite to know that actual clip length
-		// shall be the whole recorded segment,
-		// not just until the last stored event...
-		setClipLength(pSession->playHead() - clipStart());
 	}
 
 	// Get rid of owned allocations...
@@ -342,12 +380,12 @@ void qtractorMidiClip::close ( bool bForce )
 	m_pSeq->clear();
 
 	// If proven empty, remove the file.
-	if (bForce && bNewFile && clipLength() < 1)
+	if (bForce && bNewFile && iClipLength < 1)
 		QFile::remove(filename());
 
 	// Get rid of editor form, if any.
 	if (m_pMidiEditorForm) {
-		m_pMidiEditorForm->close();
+		m_pMidiEditorForm->forceClose();
 		delete m_pMidiEditorForm;
 		m_pMidiEditorForm = NULL;
 	}
@@ -359,7 +397,7 @@ void qtractorMidiClip::open (void)
 {
 	// WTF? is there an editor outstanding still there?
 	if (m_pMidiEditorForm) {
-		m_pMidiEditorForm->close();
+		m_pMidiEditorForm->forceClose();
 		delete m_pMidiEditorForm;
 		m_pMidiEditorForm = NULL;
 	}
@@ -381,13 +419,15 @@ void qtractorMidiClip::process ( unsigned long iFrameStart,
 	qtractorTrack *pTrack = track();
 	if (pTrack == NULL)
 		return;
-	if (pTrack->isRecord())
-		return;
 
 	qtractorSession *pSession = pTrack->session();
 	if (pSession == NULL)
 		return;
 
+	qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
+	if (pMidiEngine == NULL)
+		return;
+
 	// Track mute state...
 	bool bMute = (pTrack->isMute()
 		|| (pSession->soloTracks() && !pTrack->isSolo()));
@@ -409,7 +449,7 @@ void qtractorMidiClip::process ( unsigned long iFrameStart,
 			break;
 		if (iTimeEvent >= iTimeStart
 			&& (!bMute || pEvent->type() != qtractorMidiEvent::NOTEON))
-			pSession->midiEngine()->enqueue(pTrack, pEvent, iTimeEvent,
+			pMidiEngine->enqueue(pTrack, pEvent, iTimeEvent,
 				gain(pSession->frameFromTick(pEvent->time())));
 		pEvent = pEvent->next();
 	}
@@ -417,12 +457,9 @@ void qtractorMidiClip::process ( unsigned long iFrameStart,
 
 
 // Audio clip paint method.
-void qtractorMidiClip::drawClip ( QPainter *pPainter, const QRect& clipRect,
+void qtractorMidiClip::draw ( QPainter *pPainter, const QRect& clipRect,
 	unsigned long iClipOffset )
 {
-	// Fill clip background...
-	qtractorClip::drawClip(pPainter, clipRect, iClipOffset);
-
 	qtractorSession *pSession = track()->session();
 	if (pSession == NULL)
 		return;
@@ -436,9 +473,9 @@ void qtractorMidiClip::drawClip ( QPainter *pPainter, const QRect& clipRect,
 	unsigned long iTimeEnd
 		= iTimeStart + pSession->tickFromPixel(clipRect.width());
 
-	const QColor fg = track()->foreground();
+	const QColor& fg = track()->foreground();
 	pPainter->setPen(fg);
-	pPainter->setBrush(fg.light());
+	pPainter->setBrush(fg.lighter());
 
 	int cx = pSession->pixelFromTick(iTimeStart);
 	int h1 = clipRect.height() - 4;
@@ -453,11 +490,13 @@ void qtractorMidiClip::drawClip ( QPainter *pPainter, const QRect& clipRect,
 				+ pSession->pixelFromTick(pEvent->time()) - cx;
 			int y = clipRect.bottom()
 				- (h1 * (pEvent->note() - m_pSeq->noteMin() + 1)) / iNoteSpan;
-			int w = pSession->pixelFromTick(pEvent->duration());
+			int w = (pEvent->duration() > 0
+				? pSession->pixelFromTick(pEvent->duration())
+				: clipRect.right() - x); // Pending note-off? (while recording)
 			if (h > 4) {
 				if (w < 5) w = 5;
 				pPainter->fillRect(x, y, w, h - 1, fg);
-				pPainter->fillRect(x + 1, y + 1, w - 4, h - 4, fg.light());
+				pPainter->fillRect(x + 1, y + 1, w - 4, h - 4, fg.lighter());
 			} else {
 				if (w < 3) w = 3;
 				pPainter->drawRect(x, y, w, h - 1);
@@ -469,7 +508,7 @@ void qtractorMidiClip::drawClip ( QPainter *pPainter, const QRect& clipRect,
 
 
 // Clip editor method.
-bool qtractorMidiClip::startEditor ( QWidget */*pParent*/ )
+bool qtractorMidiClip::startEditor ( QWidget *pParent )
 {
 	qtractorTrack *pTrack = track();
 	if (pTrack == NULL)
@@ -477,10 +516,6 @@ bool qtractorMidiClip::startEditor ( QWidget */*pParent*/ )
 
 	if (m_pMidiEditorForm == NULL) {
 		// Build up the editor form...
-		qtractorOptions  *pOptions  = NULL;
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm)
-			pOptions = pMainForm->options();
 		// What style do we create tool childs?
 		Qt::WindowFlags wflags = Qt::Window
 		#if QT_VERSION >= 0x040200
@@ -489,10 +524,11 @@ bool qtractorMidiClip::startEditor ( QWidget */*pParent*/ )
 			| Qt::WindowTitleHint
 			| Qt::WindowSystemMenuHint
 			| Qt::WindowMinMaxButtonsHint;
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions && pOptions->bKeepToolsOnTop)
 			wflags |= Qt::Tool;
 		// Do it...
-		m_pMidiEditorForm = new qtractorMidiEditorForm(NULL, wflags);
+		m_pMidiEditorForm = new qtractorMidiEditorForm(pParent, wflags);
 		// Set its most standing properties...
 		m_pMidiEditorForm->show();
 		m_pMidiEditorForm->setup(this);
@@ -562,6 +598,10 @@ QString qtractorMidiClip::toolTip (void) const
 			.arg(m_pFile->ticksPerBeat());
 	}
 
+	if (clipGain() > 1.0f)
+		sToolTip += QObject::tr(" (%1% vol)")
+			.arg(100.0f * clipGain(), 0, 'g', 3);
+
 	return sToolTip;
 }
 
@@ -583,6 +623,8 @@ bool qtractorMidiClip::loadClipElement (
 			qtractorMidiClip::setFilename(eChild.text());
 		else if (eChild.tagName() == "track-channel")
 			qtractorMidiClip::setTrackChannel(eChild.text().toUShort());
+		else if (eChild.tagName() == "revision")
+			qtractorMidiClip::setRevision(eChild.text().toUShort());
 	}
 
 	return true;
@@ -602,10 +644,9 @@ bool qtractorMidiClip::saveClipElement (
 		if (pSession) {
 			// Have a new filename revision...
 			const QString& sFilename
-				= qtractorMidiEditor::createFilePathRevision(
-					qtractorMidiClip::filename());
+				= qtractorMidiClip::createFilePathRevision();
 			// Save/replace the clip track...
-			qtractorMidiEditor::saveCopyFile(sFilename,
+			qtractorMidiFile::saveCopyFile(sFilename,
 				qtractorMidiClip::filename(),
 				qtractorMidiClip::trackChannel(),
 				qtractorMidiClip::sequence(),
@@ -620,15 +661,64 @@ bool qtractorMidiClip::saveClipElement (
 	}
 
 	QDomElement eMidiClip = pDocument->document()->createElement("midi-clip");
-	pDocument->saveTextElement("filename", qtractorMidiClip::filename(), &eMidiClip);
+	pDocument->saveTextElement("filename",
+		qtractorMidiClip::relativeFilename(), &eMidiClip);
 	pDocument->saveTextElement("track-channel",
 		QString::number(qtractorMidiClip::trackChannel()), &eMidiClip);
+	pDocument->saveTextElement("revision",
+		QString::number(qtractorMidiClip::revision()), &eMidiClip);
+
 	pElement->appendChild(eMidiClip);
 
 	return true;
 }
 
 
+// MIDI clip export method.
+bool qtractorMidiClip::clipExport ( ClipExport pfnClipExport, void *pvArg,
+	unsigned long iOffset, unsigned long iLength ) const
+{
+	qtractorTrack *pTrack = track();
+	if (pTrack == NULL)
+		return false;
+
+	qtractorSession *pSession = pTrack->session();
+	if (pSession == NULL)
+		return false;
+
+	iOffset += clipOffset();
+	if (iLength < 1)
+		iLength = clipLength();
+
+	qtractorMidiSequence *pSeq = sequence();
+	unsigned short iTicksPerBeat = pSession->ticksPerBeat();
+	unsigned long iTimeOffset = pSeq->timep(pSeq->timeOffset(), iTicksPerBeat);
+	unsigned long iTimeStart = pSession->tickFromFrame(iOffset);
+	iTimeStart = (iTimeStart > iTimeOffset ? iTimeStart - iTimeOffset : 0);
+	unsigned long iTimeEnd = iTimeStart + pSession->tickFromFrame(iLength);
+
+	qtractorMidiSequence seq(pSeq->name(), pSeq->channel(), iTicksPerBeat);
+
+	for (qtractorMidiEvent *pEvent = pSeq->events().first();
+			pEvent; pEvent = pEvent->next()) {
+		unsigned long iTime = seq.timeq(pEvent->time(), pSeq->ticksPerBeat());
+		if (iTime >= iTimeStart &&	iTime < iTimeEnd) {
+			qtractorMidiEvent *pNewEvent = new qtractorMidiEvent(*pEvent);
+			pNewEvent->setTime(iTime - iTimeStart);
+			if (pNewEvent->type() == qtractorMidiEvent::NOTEON) {
+				pNewEvent->setVelocity((unsigned char)
+					(clipGain() * float(pNewEvent->velocity())) & 0x7f);
+			}
+			seq.insertEvent(pNewEvent);
+		}
+	}
+
+	(*pfnClipExport)(&seq, pvArg);
+
+	return true;
+}
+
+
 // Default MIDI file format (for capture/record) accessors.
 unsigned short qtractorMidiClip::g_iDefaultFormat = 0;
 
