@@ -66,6 +66,12 @@ public:
 	void setSessionFlag(bool bSessionFlag);
 	bool isSessionFlag() const;
 
+	// Revisionist methods.
+	void setRevision(unsigned short iRevision);
+	unsigned short revision() const;
+
+	QString createFilePathRevision(bool bForce = false);
+
 	// Sequence properties accessors.
 	qtractorMidiSequence *sequence() const;
 	unsigned short channel() const;
@@ -88,7 +94,7 @@ public:
 	void process(unsigned long iFrameStart, unsigned long iFrameEnd);
 
 	// Clip paint method.
-	void drawClip(QPainter *pPainter, const QRect& rect,
+	void draw(QPainter *pPainter, const QRect& clipRect,
 		unsigned long iClipOffset);
 
 	// Clip editor methods.
@@ -99,6 +105,12 @@ public:
 	// MIDI clip tool-tip.
 	QString toolTip() const;
 
+	// MIDI clip export method.
+	typedef void (*ClipExport)(qtractorMidiSequence *, void *);
+
+	bool clipExport(ClipExport pfnClipExport, void *pvArg,
+		unsigned long iOffset = 0, unsigned long iLength = 0) const;
+
 	// Default MIDI file format accessors
 	// (specific to capture/recording)
 	static void setDefaultFormat(unsigned short iFormat);
@@ -122,6 +134,9 @@ private:
 	unsigned short m_iFormat;
 	bool           m_bSessionFlag;
 
+	// Revisionist count.
+	unsigned short m_iRevision;
+
 	// To optimize and keep track of current playback
 	// position, mostly like an sequence cursor/iterator.
 	qtractorMidiCursor m_playCursor;
