@@ -1,7 +1,7 @@
 // qtractorMidiEditEvent.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,7 +27,6 @@
 #include "qtractorMidiSequence.h"
 
 #include "qtractorSession.h"
-#include "qtractorMainForm.h"
 
 #include <QResizeEvent>
 #include <QMouseEvent>
@@ -41,6 +40,11 @@
 #include <QStyle>
 #endif
 
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------------
 // qtractorMidiEditEventScale -- MIDI event scale widget.
@@ -298,7 +302,7 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 		if (x1 < 0)
 			x1 = 0;
 		if (x1 < w)
-			p.fillRect(x1, 0, w - x1, h, rgbBase.dark(105));
+			p.fillRect(x1, 0, w - x1, h, rgbBase.darker(105));
 	}
 
 	// Draw horizontal lines...
@@ -404,11 +408,8 @@ void qtractorMidiEditEvent::updatePixmap ( int cx, int /*cy*/ )
 		pEvent = pEvent->next();
 	}
 
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
 	// Draw loop boundaries, if applicable...
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession && pSession->isLooping()) {
 		p.setPen(Qt::darkCyan);
 		x = pTimeScale->pixelFromFrame(pSession->loopStart()) - dx;
@@ -479,9 +480,6 @@ void qtractorMidiEditEvent::mousePressEvent ( QMouseEvent *pMouseEvent )
 
 	// Remember what and where we'll be dragging/selecting...
 	m_pEditor->dragMoveStart(this, pos, pMouseEvent->modifiers());
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
@@ -507,9 +505,6 @@ void qtractorMidiEditEvent::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		= qtractorScrollView::viewportToContents(pMouseEvent->pos());
 
 	m_pEditor->dragMoveCommit(this, pos, pMouseEvent->modifiers());
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
