@@ -1,7 +1,7 @@
 // qtractorMidiEditTime.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -24,6 +24,8 @@
 #include "qtractorMidiEditor.h"
 #include "qtractorMidiEditView.h"
 
+#include "qtractorSessionCommand.h"
+
 #include "qtractorSession.h"
 #include "qtractorMainForm.h"
 
@@ -54,7 +56,8 @@ qtractorMidiEditTime::qtractorMidiEditTime (
 
 	qtractorScrollView::setFrameStyle(QFrame::Panel | QFrame::Raised);
 
-	qtractorScrollView::viewport()->setFocusPolicy(Qt::ClickFocus);
+	qtractorScrollView::setFocusPolicy(Qt::NoFocus);
+//	qtractorScrollView::viewport()->setFocusPolicy(Qt::ClickFocus);
 //	qtractorScrollView::viewport()->setFocusProxy(this);
 //	qtractorScrollView::viewport()->setAcceptDrops(true);
 //	qtractorScrollView::setDragAutoScroll(false);
@@ -132,12 +135,12 @@ void qtractorMidiEditTime::updatePixmap ( int cx, int /*cy*/)
 		iBeat++;
 	}
 
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+	
 	// Draw loop boundaries, if applicable...
-	if (pSession && pSession->isLooping()) {
+	if (pSession->isLooping()) {
 		QPolygon polyg(3);
 	//	h -= 4;
 		int d = (h >> 2) + 1;
@@ -160,6 +163,31 @@ void qtractorMidiEditTime::updatePixmap ( int cx, int /*cy*/)
 			p.drawPolygon(polyg);
 		}
 	}
+
+	// Draw punch in/out boundaries, if applicable...
+	if (pSession->isPunching()) {
+		QPolygon polyg(3);
+	//	h -= 4;
+		int d = (h >> 2) + 1;
+		p.setPen(Qt::darkMagenta);
+		p.setBrush(Qt::magenta);
+		x = pTimeScale->pixelFromFrame(pSession->punchIn()) - dx;
+		if (x >= 0 && x < w) {
+			polyg.putPoints(0, 3,
+				x + d, h - d,
+				x, h,
+				x, h - d);
+			p.drawPolygon(polyg);
+		}
+		x = pTimeScale->pixelFromFrame(pSession->punchOut()) - dx;
+		if (x >= 0 && x < w) {
+			polyg.putPoints(0, 3,
+				x, h - d,
+				x, h,
+				x - d, h - d);
+			p.drawPolygon(polyg);
+		}
+	}
 }
 
 
@@ -263,27 +291,46 @@ bool qtractorMidiEditTime::dragHeadStart ( const QPoint& pos )
 		return true;
 	}
 
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
-	// Check loop-points, translating to edit-head/tail headers...
-	if (pSession && pSession->isLooping()) {
-		qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
-		int dx = d + pTimeScale->pixelFromFrame(m_pEditor->offset());
-		// Check loop-start header...
-		rect.moveLeft(pTimeScale->pixelFromFrame(pSession->loopStart()) - dx);
-		if (rect.contains(pos)) {
-			m_dragCursor = DragLoopStart;
-			qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
-			return true;
+	// Check loop and punch points...
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession) {
+		// Loop points...
+		if (pSession->isLooping()) {
+			qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
+			int dx = d + pTimeScale->pixelFromFrame(m_pEditor->offset());
+			// Check loop-start header...
+			rect.moveLeft(pTimeScale->pixelFromFrame(pSession->loopStart()) - dx);
+			if (rect.contains(pos)) {
+				m_dragCursor = DragLoopStart;
+				qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
+				return true;
+			}
+			// Check loop-end header...
+			rect.moveLeft(pTimeScale->pixelFromFrame(pSession->loopEnd()) - dx);
+			if (rect.contains(pos)) {
+				m_dragCursor = DragLoopEnd;
+				qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
+				return true;
+			}
 		}
-		// Check loop-end header...
-		rect.moveLeft(pTimeScale->pixelFromFrame(pSession->loopEnd()) - dx);
-		if (rect.contains(pos)) {
-			m_dragCursor = DragLoopEnd;
-			qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
-			return true;
+		// Punch in/out points...
+		if (pSession->isPunching()) {
+			qtractorTimeScale *pTimeScale = m_pEditor->timeScale();
+			int dx = d + pTimeScale->pixelFromFrame(m_pEditor->offset());
+			// Check punch-in header...
+			rect.moveLeft(pTimeScale->pixelFromFrame(pSession->punchIn()) - dx);
+			if (rect.contains(pos)) {
+				m_dragCursor = DragPunchIn;
+				qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
+				return true;
+			}
+			// Check punch-out header...
+			rect.moveLeft(pTimeScale->pixelFromFrame(pSession->punchOut()) - dx);
+			if (rect.contains(pos)) {
+				m_dragCursor = DragPunchOut;
+				qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
+				return true;
+			}
 		}
 	}
 
@@ -320,10 +367,7 @@ void qtractorMidiEditTime::mousePressEvent ( QMouseEvent *pMouseEvent )
 	// Force null state.
 	m_dragState = DragNone;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -366,9 +410,6 @@ void qtractorMidiEditTime::mousePressEvent ( QMouseEvent *pMouseEvent )
 	}
 
 	qtractorScrollView::mousePressEvent(pMouseEvent);
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
@@ -405,12 +446,14 @@ void qtractorMidiEditTime::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 		pMainForm->updateTransportTime(iFrame);
 		break;
 	case DragEditHead:
+	case DragPunchIn:
 	case DragLoopStart:
 		// Edit-head positioning...
 		m_pEditor->editView()->ensureVisible(pos.x(), y, 16, 0);
 		m_pEditor->setEditHead(iFrame);
 		break;
 	case DragEditTail:
+	case DragPunchOut:
 	case DragLoopEnd:
 		// Edit-head positioning...
 		m_pEditor->editView()->ensureVisible(pos.x(), y, 16, 0);
@@ -443,10 +486,7 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 {
 	qtractorScrollView::mouseReleaseEvent(pMouseEvent);
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -485,16 +525,40 @@ void qtractorMidiEditTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		// New loop-start boundary...
 		if (pSession->editHead() < pSession->loopEnd()) {
 			// Yep, new loop-start point...
-			pSession->setLoop(pSession->editHead(), pSession->loopEnd());
-			m_pEditor->updateContents();
-			m_pEditor->contentsChangeNotify();
+			m_pEditor->commands()->exec(
+				new qtractorSessionLoopCommand(pSession,
+					pSession->editHead(), pSession->loopEnd()));
+		/*	m_pEditor->updateContents();
+			m_pEditor->contentsChangeNotify(); */
 		}
 		break;
 	case DragLoopEnd:
 		// New loop-end boundary...
 		if (pSession->loopStart() < pSession->editTail()) {
 			// Yep, new loop-end point...
-			pSession->setLoop(pSession->loopStart(), pSession->editTail());
+			m_pEditor->commands()->exec(
+				new qtractorSessionLoopCommand(pSession,
+					pSession->loopStart(), pSession->editTail()));
+		/*	m_pEditor->updateContents();
+			m_pEditor->contentsChangeNotify(); */
+		}
+		break;
+	case DragPunchIn:
+		// New punch-in boundary...
+		if (pSession->editHead() < pSession->punchOut()) {
+			// Yep, new punch-in point...
+			pSession->setPunch(pSession->editHead(), pSession->punchOut());
+			// For visual feedback...
+			m_pEditor->updateContents();
+			m_pEditor->contentsChangeNotify();
+		}
+		break;
+	case DragPunchOut:
+		// New punch-out boundary...
+		if (pSession->punchIn() < pSession->editTail()) {
+			// Yep, new punch-out point...
+			pSession->setPunch(pSession->punchIn(), pSession->editTail());
+			// For visual feedback...
 			m_pEditor->updateContents();
 			m_pEditor->contentsChangeNotify();
 		}
@@ -530,10 +594,7 @@ void qtractorMidiEditTime::keyPressEvent ( QKeyEvent *pKeyEvent )
 	if (pKeyEvent->key() == Qt::Key_Escape) {
 		// Restore uncommitted play-head position?...
 		if (m_dragState == DragPlayHead) {
-			qtractorSession  *pSession  = NULL;
-			qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-			if (pMainForm)
-				pSession = pMainForm->session();
+			qtractorSession *pSession = qtractorSession::getInstance();
 			if (pSession)
 				m_pEditor->setPlayHead(pSession->playHead());
 		}
