@@ -29,7 +29,6 @@
 #include "qtractorMidiSequence.h"
 
 #include "qtractorSession.h"
-#include "qtractorMainForm.h"
 
 #include <QResizeEvent>
 #include <QMouseEvent>
@@ -43,6 +42,11 @@
 #include <QStyle>
 #endif
 
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------------
 // qtractorMidiEditView -- MIDI sequence main view widget.
@@ -256,7 +260,7 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 	const QColor& rgbBack  = m_pEditor->background();
 	const QColor& rgbDark  = pal.dark().color();
 	const QColor& rgbLight = pal.mid().color();
-	const QColor& rgbSharp = rgbBase.dark(110);
+	const QColor& rgbSharp = rgbBase.darker(110);
 
 	m_pixmap = QPixmap(w, h);
 	m_pixmap.fill(rgbBase);
@@ -274,7 +278,7 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 		if (x1 < 0)
 			x1 = 0;
 		if (x1 < w)
-			p.fillRect(x1, 0, w - x1, h, rgbBase.dark(105));
+			p.fillRect(x1, 0, w - x1, h, rgbBase.darker(105));
 	}
 
 	// Draw horizontal lines...
@@ -368,11 +372,8 @@ void qtractorMidiEditView::updatePixmap ( int cx, int cy )
 		pEvent = pEvent->next();
 	}
 
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
 	// Draw loop boundaries, if applicable...
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession && pSession->isLooping()) {
 		p.setPen(Qt::darkCyan);
 		x = pTimeScale->pixelFromFrame(pSession->loopStart()) - dx;
@@ -452,9 +453,6 @@ void qtractorMidiEditView::mousePressEvent ( QMouseEvent *pMouseEvent )
 
 	// Remember what and where we'll be dragging/selecting...
 	m_pEditor->dragMoveStart(this, pos, pMouseEvent->modifiers());
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
@@ -483,9 +481,6 @@ void qtractorMidiEditView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		= qtractorScrollView::viewportToContents(pMouseEvent->pos());
 
 	m_pEditor->dragMoveCommit(this, pos, pMouseEvent->modifiers());
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
