@@ -264,118 +264,6 @@ const QString& qtractorMidiEditor::defaultControllerName ( unsigned char control
 qtractorMidiEditor::ClipBoard qtractorMidiEditor::g_clipboard;
 
 
-//----------------------------------------------------------------------------
-// qtractorMidiEditor -- All-in-one SMF file writer/creator method.
-
-bool qtractorMidiEditor::saveCopyFile ( const QString& sNewFilename,
-	const QString& sOldFilename, unsigned short iTrackChannel,
-	qtractorMidiSequence *pSeq, qtractorTimeScale *pTimeScale,
-	unsigned short iFormat )
-{
-	qtractorMidiFile file;
-	float fTempo = (pTimeScale ? pTimeScale->tempo() : 120.0f);
-	unsigned short iTicksPerBeat = (pTimeScale ? pTimeScale->ticksPerBeat() : 96);
-	unsigned short iBeatsPerBar  = (pTimeScale ? pTimeScale->beatsPerBar()  : 4);
-	unsigned short iSeq, iSeqs = 0;
-	qtractorMidiSequence **ppSeqs = NULL;
-	const QString sTrackName = QObject::tr("Track %1");
-	
-	if (pSeq == NULL)
-		return false;
-
-	// Open and load the whole source file...
-	if (file.open(sOldFilename)) {
-		iTicksPerBeat = file.ticksPerBeat();
-		iFormat = file.format();
-		iSeqs = (iFormat == 1 ? file.tracks() : 16);	
-		ppSeqs = new qtractorMidiSequence * [iSeqs];
-		for (iSeq = 0; iSeq < iSeqs; ++iSeq) {	
-			ppSeqs[iSeq] = new qtractorMidiSequence(
-				sTrackName.arg(iSeq + 1), iSeq, iTicksPerBeat);
-		}
-		if (file.readTracks(ppSeqs, iSeqs)) {
-			fTempo = file.tempo();
-			iBeatsPerBar = file.beatsPerBar();
-		}
-		file.close();
-	}
-	
-	// Open and save the whole target file...
-	if (!file.open(sNewFilename, qtractorMidiFile::Write))
-		return false;
-
-	file.setTempo(fTempo);
-	file.setBeatsPerBar(iBeatsPerBar);
-
-	if (ppSeqs == NULL)
-		iSeqs = (iFormat == 0 ? 1 : 2);
-
-	// Write SMF header...
-	file.writeHeader(iFormat, (iFormat == 0 ? 1 : iSeqs), iTicksPerBeat);
-
-	// Write SMF tracks(s)...
-	if (ppSeqs) {
-		// Replace the target track-channel events... 
-		ppSeqs[iTrackChannel]->replaceEvents(pSeq);
-		// Write the whole new tracks...
-		file.writeTracks(ppSeqs, iSeqs);
-	} else {
-		// Most probabley this is a brand new file...
-		if (iFormat == 1)
-			file.writeTrack(NULL);
-		file.writeTrack(pSeq);
-	}
-
-	file.close();
-
-	// Free locally allocated track/sequence array.
-	if (ppSeqs) {
-		for (iSeq = 0; iSeq < iSeq; ++iSeq)
-			delete ppSeqs[iSeq];
-		delete [] ppSeqs;
-	}
-
-	// HACK: This invasive operation is so important that
-	// it surely deserves being in the front page...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		pMainForm->appendMessages(
-			tr("MIDI file save: \"%1\", track-channel: %2.")
-			.arg(sNewFilename).arg(iTrackChannel));
-		pMainForm->addMidiFile(sNewFilename);
-	}
-
-	return true;
-}
-
-
-// Create filename revision (name says it all).
-QString qtractorMidiEditor::createFilePathRevision (
-	const QString& sFilename, int iRevision )
-{
-	QFileInfo fi(sFilename);
-	QDir adir(fi.absoluteDir());
-
-	QRegExp rxRevision("(.+)\\-(\\d+)$");
-	QString sBasename = fi.baseName();
-	if (rxRevision.exactMatch(sBasename)) {
-		sBasename = rxRevision.cap(1);
-		iRevision = rxRevision.cap(2).toInt();
-	}
-	
-	sBasename += "-%1." + fi.completeSuffix();
-	do { fi.setFile(adir, sBasename.arg(++iRevision)); }
-	while (fi.exists());
-
-#ifdef CONFIG_DEBUG
-	qDebug("qtractorMidiEditor::createFilePathRevision(\"%s\")",
-		fi.absoluteFilePath().toUtf8().constData());
-#endif
-
-	return fi.absoluteFilePath();
-}
-
-
 //----------------------------------------------------------------------------
 // qtractorMidiEditor -- The main MIDI sequence editor widget.
 
@@ -685,12 +573,9 @@ void qtractorMidiEditor::setEditHead ( unsigned long iEditHead, bool bSync )
 		setEditTail(iEditHead, bSync);
 
 	if (bSync) {
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm) {
-			qtractorSession *pSession = pMainForm->session();
-			if (pSession)
-				pSession->setEditHead(iEditHead);
-		}
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession)
+			pSession->setEditHead(iEditHead);
 	}
 
 	m_iEditHead = iEditHead;
@@ -718,12 +603,9 @@ void qtractorMidiEditor::setEditTail ( unsigned long iEditTail, bool bSync )
 		setEditHead(iEditTail, bSync);
 
 	if (bSync) {
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm) {
-			qtractorSession *pSession = pMainForm->session();
-			if (pSession)
-				pSession->setEditTail(iEditTail);
-		}
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession)
+			pSession->setEditTail(iEditTail);
 	}
 
 	m_iEditTail = iEditTail;
@@ -1085,9 +967,8 @@ void qtractorMidiEditor::cutClipboard (void)
 	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
 	while (iter.hasNext()) {
 		qtractorMidiEditSelect::Item *pItem = iter.next();
-		qtractorMidiEvent *pEvent = pItem->event;
-		g_clipboard.items.append(new qtractorMidiEvent(*pEvent));
-		pEditCommand->removeEvent(pEvent);
+		g_clipboard.items.append(new qtractorMidiEvent(*(pItem->event)));
+		pEditCommand->removeEvent(pItem->event);
 	}
 
 	// Make it as an undoable command...
@@ -1108,16 +989,39 @@ void qtractorMidiEditor::copyClipboard (void)
 
 	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
 	while (iter.hasNext()) {
-		qtractorMidiEvent *pEvent = iter.next()->event;
-		g_clipboard.items.append(new qtractorMidiEvent(*pEvent));
+		qtractorMidiEditSelect::Item *pItem = iter.next();
+		g_clipboard.items.append(new qtractorMidiEvent(*(pItem->event)));
 	}
 
 	selectionChangeNotify();
 }
 
 
+// Retrieve current paste period.
+// (as from current clipboard width)
+unsigned long qtractorMidiEditor::pastePeriod (void) const
+{
+	unsigned long t0 = 0;
+	unsigned long t1 = 0;
+
+	QListIterator<qtractorMidiEvent *> iter(g_clipboard.items);
+	while (iter.hasNext()) {
+		qtractorMidiEvent *pEvent = iter.next();
+		unsigned long t2 = pEvent->time();
+		if (t0 > t2 || t0 == 0)
+			t0 = t2;
+		t2 += pEvent->duration();
+		if (t1 < t2)
+			t1 = t2;
+	}
+
+	return m_pTimeScale->frameFromTick(t1 - t0);
+}
+
+
 // Paste from clipboard.
-void qtractorMidiEditor::pasteClipboard (void)
+void qtractorMidiEditor::pasteClipboard (
+	unsigned short iPasteCount, unsigned long iPastePeriod )
 {
 	if (m_pMidiClip == NULL)
 		return;
@@ -1135,6 +1039,16 @@ void qtractorMidiEditor::pasteClipboard (void)
 	m_select.clear();
 	resetDragState(pScrollView);
 
+	// Multi-paste period...
+	if (iPastePeriod < 1)
+		iPastePeriod = pastePeriod();
+		
+	int x0 = 0;
+	int dx = m_pTimeScale->pixelFromFrame(iPastePeriod);
+
+	unsigned long t0 = 0;
+	unsigned long dt = m_pTimeScale->tickFromFrame(iPastePeriod);
+	
 	// This is the edit-view spacifics...
 	int h1 = m_pEditList->itemHeight();
 	int ch = m_pEditView->contentsHeight(); // + 1;
@@ -1145,41 +1059,46 @@ void qtractorMidiEditor::pasteClipboard (void)
 		y0 = ((y0 >> 3) << 2);
 
 	QListIterator<qtractorMidiEvent *> iter(g_clipboard.items);
-	while (iter.hasNext()) {
-		qtractorMidiEvent *pEvent = iter.next();
-		// Common event coords...
-		int y;
-		int x  = m_pTimeScale->pixelFromTick(pEvent->time());
-		int w1 = m_pTimeScale->pixelFromTick(pEvent->duration()) + 1;
-		if (w1 < 5)
-			w1 = 5;
-		// View item...
-		QRect rectView;
-		if (pEvent->type() == m_pEditView->eventType()) {
-			y = ch - h1 * (pEvent->note() + 1);
-			rectView.setRect(x, y, w1, h1);
-		}
-		// Event item...
-		QRect rectEvent;
-		if (pEvent->type() == m_pEditEvent->eventType()) {
-			if (pEvent->type() == qtractorMidiEvent::PITCHBEND)
-				y = y0 - (y0 * pEvent->pitchBend()) / 8192;
-			else
-				y = y0 - (y0 * pEvent->value()) / 128;
-			if (!m_bNoteDuration)
+	for (unsigned short i = 0; i < iPasteCount; ++i) {
+		iter.toFront();
+		while (iter.hasNext()) {
+			qtractorMidiEvent *pEvent = iter.next();
+			// Common event coords...
+			int y;
+			int x  = m_pTimeScale->pixelFromTick(pEvent->time()) + x0;
+			int w1 = m_pTimeScale->pixelFromTick(pEvent->duration()) + 1;
+			if (w1 < 5)
 				w1 = 5;
-			if (y < y0)
-				rectEvent.setRect(x, y, w1, y0 - y);
-			else if (y > y0)
-				rectEvent.setRect(x, y0, w1, y - y0);
-			else
-				rectEvent.setRect(x, y0 - 2, w1, 4);
-		}
-		m_select.addItem(pEvent, rectEvent, rectView);
-		if (m_pEventDrag == NULL) {
-			m_pEventDrag = pEvent;
-			m_rectDrag = (bEditView ? rectView : rectEvent);
+			// View item...
+			QRect rectView;
+			if (pEvent->type() == m_pEditView->eventType()) {
+				y = ch - h1 * (pEvent->note() + 1);
+				rectView.setRect(x, y, w1, h1);
+			}
+			// Event item...
+			QRect rectEvent;
+			if (pEvent->type() == m_pEditEvent->eventType()) {
+				if (pEvent->type() == qtractorMidiEvent::PITCHBEND)
+					y = y0 - (y0 * pEvent->pitchBend()) / 8192;
+				else
+					y = y0 - (y0 * pEvent->value()) / 128;
+				if (!m_bNoteDuration)
+					w1 = 5;
+				if (y < y0)
+					rectEvent.setRect(x, y, w1, y0 - y);
+				else if (y > y0)
+					rectEvent.setRect(x, y0, w1, y - y0);
+				else
+					rectEvent.setRect(x, y0 - 2, w1, 4);
+			}
+			m_select.addItem(pEvent, rectEvent, rectView, t0);
+			if (m_pEventDrag == NULL) {
+				m_pEventDrag = pEvent;
+				m_rectDrag = (bEditView ? rectView : rectEvent);
+			}
 		}
+		x0 += dx;
+		t0 += dt;
 	}
 
 	// We'll start a brand new floating state...
@@ -1217,8 +1136,10 @@ void qtractorMidiEditor::deleteSelect (void)
 		= new qtractorMidiEditCommand(m_pMidiClip, tr("delete"));
 
 	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
-	while (iter.hasNext())
-		pEditCommand->removeEvent((iter.next())->event);
+	while (iter.hasNext()) {
+		qtractorMidiEditSelect::Item *pItem = iter.next();
+		pEditCommand->removeEvent(pItem->event);
+	}
 
 	m_pCommands->exec(pEditCommand);
 }
@@ -1417,7 +1338,8 @@ qtractorMidiEvent *qtractorMidiEditor::dragEditEvent (
 	int y0 = (m_pEditEvent->viewport())->height();
 	int h1 = m_pEditList->itemHeight();
 
-	int x1 = m_pTimeScale->pixelSnap(pos.x());
+	int x0 = m_pTimeScale->pixelFromFrame(m_iOffset);
+	int x1 = m_pTimeScale->pixelSnap(pos.x() + x0) - x0;
 	int y1 = 0;
 
 	// This will be the new editing event...
@@ -1641,9 +1563,11 @@ void qtractorMidiEditor::dragMoveUpdate ( qtractorScrollView *pScrollView,
 		if ((pos - m_posDrag).manhattanLength()
 			< QApplication::startDragDistance())
 			break;
+	#if 0
 		// Take care of selection modifier...
 		if ((modifiers & (Qt::ShiftModifier | Qt::ControlModifier)) == 0)
 			flags |= SelectClear;
+	#endif
 		// Are we about to move something around?
 		if (m_pEventDrag) {
 			if (m_resizeMode == ResizeNone) {
@@ -1724,12 +1648,9 @@ void qtractorMidiEditor::dragMoveCommit ( qtractorScrollView *pScrollView,
 			// Playhead positioning...
 			setPlayHead(iFrame);
 			// Immediately commited...
-			qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-			if (pMainForm) {
-				qtractorSession *pSession = pMainForm->session();
-				if (pSession)
-					pSession->setPlayHead(iFrame);
-			}
+			qtractorSession *pSession = qtractorSession::getInstance();
+			if (pSession)
+				pSession->setPlayHead(iFrame);
 		}
 		// Fall thru...
 	case DragSelect:
@@ -1966,7 +1887,7 @@ void qtractorMidiEditor::updateDragMove ( qtractorScrollView *pScrollView,
 
 	int cw = pScrollView->contentsWidth();
 	int dx = delta.x();
-	int x0 = m_rectDrag.x();
+	int x0 = m_rectDrag.x() + m_pTimeScale->pixelFromFrame(m_iOffset);
 	int x1 = rect.x() + dx;
 	if (x1 < 0)
 		dx = -(rect.x());
@@ -2029,20 +1950,22 @@ void qtractorMidiEditor::updateDragResize ( qtractorScrollView *pScrollView,
 	// TODO: Plenty of...
 	switch (m_resizeMode) {
 	case ResizeNoteLeft:
-		x0 = m_rectDrag.left();
-		x1 = x0 + delta.x();
+		dx = delta.x();
+		x0 = m_rectDrag.left() + m_pTimeScale->pixelFromFrame(m_iOffset);
+		x1 = m_rectDrag.left() + dx;
 		if (x1 < 0)
-			x1 = 0;
+			dx = -(m_rectDrag.left());
 		if (x1 > m_rectDrag.right())
-			x1 = m_rectDrag.right();
-		dx = m_pTimeScale->pixelSnap(x1) - x0;
+			dx = m_rectDrag.width();
+		dx = m_pTimeScale->pixelSnap(x0 + dx) - x0;
 		break;
 	case ResizeNoteRight:
-		x0 = m_rectDrag.right();
-		x1 = x0 + delta.x();
+		dx = delta.x();
+		x0 = m_rectDrag.right() + m_pTimeScale->pixelFromFrame(m_iOffset);
+		x1 = m_rectDrag.right() + dx;
 		if (x1 < m_rectDrag.left())
-			x1 = m_rectDrag.left();
-		dx = m_pTimeScale->pixelSnap(x1) - x0;
+			dx = -(m_rectDrag.width());
+		dx = m_pTimeScale->pixelSnap(x0 + dx) - x0;
 		break;
 	case ResizeValueTop:
 	case ResizePitchBendTop:
@@ -2083,11 +2006,23 @@ void qtractorMidiEditor::executeDragMove ( qtractorScrollView *pScrollView,
 
 	updateDragMove(pScrollView, pos + m_posStep);
 
-	long iTimeDelta = 0;
-	if (m_posDelta.x() < 0)
-		iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
-	else
-		iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
+	long iTimeDelta, t2, t1;
+	long t0 = long(m_pTimeScale->tickFromFrame(m_iOffset));
+	if (m_pEventDrag) {
+		if (m_posDelta.x() < 0)
+			iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
+		else
+			iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
+		t1 = long(m_pEventDrag->time());
+		t2 = t1 + iTimeDelta;
+	} else {
+		int x1 = (
+			static_cast<qtractorScrollView *> (m_pEditView) == pScrollView
+			? m_select.rectView().x() : m_select.rectEvent().x());
+		t1 = long(m_pTimeScale->tickFromPixel(x1));
+		t2 = long(m_pTimeScale->tickFromPixel(x1 + m_posDelta.x()));
+	}
+	iTimeDelta = long(m_pTimeScale->tickSnap(t2 + t0) - t0) - t1;
 
 	int h1 = m_pEditList->itemHeight();
 	int iNoteDelta = 0;
@@ -2099,7 +2034,8 @@ void qtractorMidiEditor::executeDragMove ( qtractorScrollView *pScrollView,
 
 	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
 	while (iter.hasNext()) {
-		qtractorMidiEvent *pEvent = (iter.next())->event;
+		qtractorMidiEditSelect::Item *pItem = iter.next();
+		qtractorMidiEvent *pEvent = pItem->event;
 		int iNote = int(pEvent->note()) + iNoteDelta;
 		if (iNote < 0)
 			iNote = 0;
@@ -2126,11 +2062,24 @@ void qtractorMidiEditor::executeDragResize ( qtractorScrollView *pScrollView,
 
 	updateDragResize(pScrollView, pos);
 
-	long iTimeDelta = 0;
-	if (m_posDelta.x() < 0)
-		iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
-	else
-		iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
+	long iTimeDelta, t2, t1;
+	long t0 = long(m_pTimeScale->tickFromFrame(m_iOffset));
+	if (m_pEventDrag) {
+		if (m_posDelta.x() < 0)
+			iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
+		else
+			iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
+		t1 = long(m_pEventDrag->time());
+		if (m_resizeMode == ResizeNoteRight)
+			t1 += long(m_pEventDrag->duration());
+		t2 = t1 + iTimeDelta;
+	} else {
+		int x1 = (m_resizeMode == ResizeNoteRight
+			? m_rectDrag.right() : m_rectDrag.left());
+		t1 = long(m_pTimeScale->tickFromPixel(x1));
+		t2 = long(m_pTimeScale->tickFromPixel(x1 + m_posDelta.x()));
+	}
+	iTimeDelta = long(m_pTimeScale->tickSnap(t2 + t0) - t0) - t1;
 
 	int h = (m_pEditEvent->viewport())->height();
 	int iValueDelta = 0;
@@ -2150,7 +2099,8 @@ void qtractorMidiEditor::executeDragResize ( qtractorScrollView *pScrollView,
 	int iValue;
 	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
 	while (iter.hasNext()) {
-		qtractorMidiEvent *pEvent = (iter.next())->event;
+		qtractorMidiEditSelect::Item *pItem = iter.next();
+		qtractorMidiEvent *pEvent = pItem->event;
 		switch (m_resizeMode) {
 		case ResizeNoteLeft:
 			iTime = long(pEvent->time()) + iTimeDelta;
@@ -2238,11 +2188,23 @@ void qtractorMidiEditor::executeDragPaste ( qtractorScrollView *pScrollView,
 
 	updateDragMove(pScrollView, pos + m_posStep);
 
-	long iTimeDelta = 0;
-	if (m_posDelta.x() < 0)
-		iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
-	else
-		iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
+	long iTimeDelta, t2, t1;
+	long t0 = long(m_pTimeScale->tickFromFrame(m_iOffset));
+	if (m_pEventDrag) {
+		if (m_posDelta.x() < 0)
+			iTimeDelta = -long(m_pTimeScale->tickFromPixel(-m_posDelta.x()));
+		else
+			iTimeDelta = +long(m_pTimeScale->tickFromPixel(+m_posDelta.x()));
+		t1 = long(m_pEventDrag->time());
+		t2 = t1 + iTimeDelta;
+	} else {
+		int x1 = (
+			static_cast<qtractorScrollView *> (m_pEditView) == pScrollView
+			? m_select.rectView().x() : m_select.rectEvent().x());
+		t1 = long(m_pTimeScale->tickFromPixel(x1));
+		t2 = long(m_pTimeScale->tickFromPixel(x1 + m_posDelta.x()));
+	}
+	iTimeDelta = long(m_pTimeScale->tickSnap(t2 + t0) - t0) - t1;
 
 	int h1 = m_pEditList->itemHeight();
 	int iNoteDelta = 0;
@@ -2254,15 +2216,15 @@ void qtractorMidiEditor::executeDragPaste ( qtractorScrollView *pScrollView,
 
 	QListIterator<qtractorMidiEditSelect::Item *> iter(m_select.items());
 	while (iter.hasNext()) {
-		qtractorMidiEvent *pEvent
-			= new qtractorMidiEvent(*(iter.next())->event);
+		qtractorMidiEditSelect::Item *pItem = iter.next();
+		qtractorMidiEvent *pEvent = new qtractorMidiEvent(*(pItem->event));
 		int iNote = int(pEvent->note()) + iNoteDelta;
 		if (iNote < 0)
 			iNote = 0;
 		if (iNote > 127)
 			iNote = 127;
 		pEvent->setNote(iNote);
-		long iTime = long(pEvent->time()) + iTimeDelta;
+		long iTime = long(pEvent->time() + pItem->delta) + iTimeDelta;
 		if (iTime < 0)
 			iTime = 0;
 		pEvent->setTime(iTime);
@@ -2486,7 +2448,8 @@ void qtractorMidiEditor::executeTool ( int iToolIndex )
 	toolsForm.setToolIndex(iToolIndex);
 	if (toolsForm.exec()) {
 		qtractorMidiEditCommand *pEditCommand
-			= toolsForm.editCommand(m_pMidiClip, &m_select);
+			= toolsForm.editCommand(m_pMidiClip, &m_select,
+				m_pTimeScale->tickFromFrame(m_iOffset));
 		if (m_pCommands->exec(pEditCommand))
 			adjustEditCommand(pEditCommand);
 	}		
@@ -2514,10 +2477,12 @@ void qtractorMidiEditor::updateInstrumentNames (void)
 	if (pTrack == NULL)
 		return;
 
-	qtractorMainForm *pMainForm	= qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
-	if (pMainForm->instruments() == NULL)
+
+	qtractorInstrumentList *pInstruments = pSession->instruments();
+	if (pInstruments == NULL)
 		return;
 
 	// Get instrument name from patch descriptor...
@@ -2527,7 +2492,7 @@ void qtractorMidiEditor::updateInstrumentNames (void)
 	if (pMidiBus)
 		sInstrument = pMidiBus->patch(pTrack->midiChannel()).instrumentName;
 	// Do we have any?...
-	if (sInstrument.isEmpty()) {
+	if (sInstrument.isEmpty() || !pInstruments->contains(sInstrument)) {
 		// At least have a GM Drums (Channel 10) help...
 		if (pTrack->midiChannel() == 9) {
 			for (int i = 13; g_aNoteNames[i].name; ++i) {
@@ -2542,17 +2507,17 @@ void qtractorMidiEditor::updateInstrumentNames (void)
 
 	// Finally, got instrument descriptor...
 	qtractorInstrumentData::ConstIterator iter;
-	qtractorInstrument& instr = (*pMainForm->instruments())[sInstrument];
+	const qtractorInstrument& instr = (*pInstruments)[sInstrument];
 
 	// Key note names...
 	const qtractorInstrumentData& notes
 		= instr.notes(pTrack->midiBank(), pTrack->midiProgram());
-	for (iter = notes.begin(); iter != notes.end(); ++iter)
+	for (iter = notes.constBegin(); iter != notes.constEnd(); ++iter)
 		m_noteNames.insert(iter.key(), iter.value());
 
 	// Controller names...
 	const qtractorInstrumentData& controllers = instr.control();
-	for (iter = controllers.begin(); iter != controllers.end(); ++iter)
+	for (iter = controllers.constBegin(); iter != controllers.constEnd(); ++iter)
 		m_controllerNames.insert(iter.key(), iter.value());
 }
 
@@ -2617,7 +2582,8 @@ void qtractorMidiEditor::sendNote ( int iNote, int iVelocity )
 QString qtractorMidiEditor::eventToolTip ( qtractorMidiEvent *pEvent ) const
 {
 	QString sToolTip = tr("Time:\t%1\nType:\t")
-		.arg(m_pTimeScale->textFromTick(pEvent->time()));
+		.arg(m_pTimeScale->textFromTick(pEvent->time()
+			+ m_pTimeScale->tickFromFrame(m_iOffset)));
 
 	switch (pEvent->type()) {
 //	case qtractorMidiEvent::NOTEOFF:
@@ -2628,7 +2594,7 @@ QString qtractorMidiEditor::eventToolTip ( qtractorMidiEvent *pEvent ) const
 			.arg(int(pEvent->note()))
 			.arg(noteName(pEvent->note()))
 			.arg(int(pEvent->velocity()))
-			.arg(m_pTimeScale->textFromTick(pEvent->duration()));
+			.arg(m_pTimeScale->textFromTick(pEvent->duration(), true));
 		break;
 	case qtractorMidiEvent::KEYPRESS:
 		sToolTip += tr("Key Press (%1) %2\nValue:\t%3")
@@ -2695,6 +2661,7 @@ bool qtractorMidiEditor::keyPress ( qtractorScrollView *pScrollView,
 		}
 		// Fall thru...
 	case Qt::Key_Escape:
+		m_dragState = DragStep; // HACK: Force selection clearance!
 		resetDragState(pScrollView);
 		break;
 	case Qt::Key_Home:
