@@ -166,7 +166,12 @@ public:
 	// Clipboard commands.
 	void cutClipboard();
 	void copyClipboard();
-	void pasteClipboard();
+	void pasteClipboard(
+		unsigned short iPasteCount = 1, unsigned long iPastePeriod = 0);
+
+	// Retrieve current paste period.
+	// (as from current clipboard width)
+	unsigned long pastePeriod() const;
 
 	// Execute event removal.
 	void deleteSelect();
@@ -264,16 +269,6 @@ public:
 	// Default controller name accessor.
 	static const QString& defaultControllerName(unsigned char controller);
 
-	// All-in-one SMF file writer/creator method.
-	static bool saveCopyFile(const QString& sNewFilename,
-		const QString& sOldFilename, unsigned short iTrackChannel,
-		qtractorMidiSequence *pSeq, qtractorTimeScale *pTimeScale = NULL,
-		unsigned short iFormat = 0);
-
-	// Create filename revision.
-	static QString createFilePathRevision(
-		const QString& sFilename, int iRevision = 0);
-
 public slots:
 
 	// Redirect selection/change notification.
