@@ -37,6 +37,7 @@
 
 #include "qtractorMainForm.h"
 #include "qtractorShortcutForm.h"
+#include "qtractorPasteRepeatForm.h"
 #include "qtractorClipForm.h"
 
 #include "qtractorTimeScale.h"
@@ -47,15 +48,15 @@
 #include <QCloseEvent>
 #include <QComboBox>
 #include <QLabel>
-
+#include <QUrl>
 
 //-------------------------------------------------------------------------
 // qtractorMidiEditorForm -- Main window form implementation.
 
 // Constructor.
 qtractorMidiEditorForm::qtractorMidiEditorForm (
-	QWidget *pParent, Qt::WindowFlags wflags )
-	: QMainWindow(pParent, wflags)
+	QWidget */*pParent*/, Qt::WindowFlags wflags )
+	: QMainWindow(NULL/*pParent*/, wflags)
 {
 	// Setup UI struct...
 	m_ui.setupUi(this);
@@ -170,6 +171,8 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	addAction(m_ui.transportLoopAction);
 	addAction(m_ui.transportLoopSetAction);
 	addAction(m_ui.transportPlayAction);
+	addAction(m_ui.transportPunchAction);
+	addAction(m_ui.transportPunchSetAction);
 
 	// Ah, make it stand right.
 	setFocus();
@@ -216,6 +219,9 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	QObject::connect(m_ui.editCopyAction,
 		SIGNAL(triggered(bool)),
 		SLOT(editCopy()));
+	QObject::connect(m_ui.editPasteRepeatAction,
+		SIGNAL(triggered(bool)),
+		SLOT(editPasteRepeat()));
 	QObject::connect(m_ui.editPasteAction,
 		SIGNAL(triggered(bool)),
 		SLOT(editPaste()));
@@ -331,10 +337,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		SLOT(contentsChanged(qtractorMidiEditor *)));
 
 	// Try to restore old editor state...
-	qtractorOptions  *pOptions  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions) {
 		// Initial decorations toggle state.
 		m_ui.viewMenubarAction->setChecked(pOptions->bMidiMenubar);
@@ -382,6 +385,7 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 	}
 
 	// Make last-but-not-least conections....
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	if (pMainForm) {
 		QObject::connect(m_ui.editSelectRangeAction,
 			SIGNAL(triggered(bool)),
@@ -410,15 +414,21 @@ qtractorMidiEditorForm::qtractorMidiEditorForm (
 		QObject::connect(m_ui.transportRecordAction,
 			SIGNAL(triggered(bool)),
 			pMainForm, SLOT(transportRecord()));
+		QObject::connect(m_ui.transportPunchAction,
+			SIGNAL(triggered(bool)),
+			pMainForm, SLOT(transportPunch()));
+		QObject::connect(m_ui.transportPunchSetAction,
+			SIGNAL(triggered(bool)),
+			pMainForm, SLOT(transportPunchSet()));
 	}
-
+#if 0
 	// Default snap-per-beat setting...
 	qtractorTimeScale *pTimeScale = m_pMidiEditor->timeScale();
 	if (pTimeScale) {
 		m_pSnapPerBeatComboBox->setCurrentIndex(
 			pTimeScale->indexFromSnap(pTimeScale->snapPerBeat()));
 	}
-
+#endif
 	eventTypeChanged(0);
 }
 
@@ -466,10 +476,7 @@ bool qtractorMidiEditorForm::queryClose (void)
 
 	// Try to save current editor view state...
 	if (bQueryClose && isVisible()) {
-		qtractorOptions  *pOptions  = NULL;
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm)
-			pOptions = pMainForm->options();
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions) {
 			// Save decorations state.
 			pOptions->bMidiMenubar = m_ui.MenuBar->isVisible();
@@ -496,6 +503,20 @@ bool qtractorMidiEditorForm::queryClose (void)
 }
 
 
+// Unconditional-close.
+void qtractorMidiEditorForm::forceClose (void)
+{
+	// Give it a chance to save....
+	queryClose();
+
+	// Make it clean...
+	m_iDirtyCount = 0;
+
+	// Close it down!
+	close();
+}
+
+
 // On-show event handler.
 void qtractorMidiEditorForm::showEvent ( QShowEvent *pShowEvent )
 {
@@ -607,18 +628,22 @@ qtractorMidiSequence *qtractorMidiEditorForm::sequence (void) const
 // Special executive setup method.
 void qtractorMidiEditorForm::setup ( qtractorMidiClip *pMidiClip )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
 		return;
 
 	// Get those time-scales in sync...
 	qtractorTimeScale *pTimeScale = m_pMidiEditor->timeScale();
 	pTimeScale->copy(*pSession->timeScale());
 
+	// Default snap-per-beat setting...
+	m_pSnapPerBeatComboBox->setCurrentIndex(
+		pTimeScale->indexFromSnap(pTimeScale->snapPerBeat()));
+
 	// Note that there's two modes for this method:
 	// wether pMidiClip is given non-null wich means
 	// form initialization and first setup or else... 
@@ -672,8 +697,12 @@ void qtractorMidiEditorForm::setup ( qtractorMidiClip *pMidiClip )
 // Save current clip.
 bool qtractorMidiEditorForm::saveClipFile ( bool bPrompt )
 {
+	qtractorMidiClip *pMidiClip = m_pMidiEditor->midiClip();
+	if (pMidiClip == NULL)
+		return false;
+
 	// Suggest a filename, if there's none...
-	QString sFilename = qtractorMidiEditor::createFilePathRevision(filename());
+	QString sFilename = pMidiClip->createFilePathRevision(bPrompt);
 
 	if (sFilename.isEmpty())
 		bPrompt = true;
@@ -681,34 +710,59 @@ bool qtractorMidiEditorForm::saveClipFile ( bool bPrompt )
 	// Ask for the file to save...
 	if (bPrompt) {
 		// If none is given, assume default directory.
-		// Prompt the guy...
-		sFilename = QFileDialog::getSaveFileName(
-			this,                                        // Parent.
-			tr("Save MIDI Clip") + " - " QTRACTOR_TITLE, // Caption.
-			sFilename,                                   // Start here.
-			tr("MIDI files") + " (*.mid)"                // Filter files.
-		);
+		const QString sExt("mid");
+		const QString& sTitle  = tr("Save MIDI Clip") + " - " QTRACTOR_TITLE;
+		const QString& sFilter = tr("MIDI files (*.%1 *.smf *.midi)").arg(sExt);
+	#if QT_VERSION < 0x040400
+		// Ask for the filenames to open...
+		sFilename = QFileDialog::getOpenFileName(this,
+			sTitle, sFilename, sFilter);
+	#else
+		// Construct open-files dialog...
+		QFileDialog fileDialog(this,
+			sTitle, sFilename, sFilter);
+		// Set proper open-file modes...
+		fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+		fileDialog.setFileMode(QFileDialog::ExistingFile);
+		fileDialog.setDefaultSuffix(sExt);
+		// Stuff sidebar...
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
+		if (pOptions) {
+			QList<QUrl> urls(fileDialog.sidebarUrls());
+			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+			urls.append(QUrl::fromLocalFile(pOptions->sMidiDir));
+			fileDialog.setSidebarUrls(urls);
+		}
+		// Show dialog...
+		if (fileDialog.exec())
+			sFilename = fileDialog.selectedFiles().first();
+	#endif
 		// Have we cancelled it?
 		if (sFilename.isEmpty())
 			return false;
 		// Enforce .mid extension...
-		const QString sExt("mid");
-		if (QFileInfo(sFilename).suffix() != sExt)
+		if (QFileInfo(sFilename).suffix().isEmpty())
 			sFilename += '.' + sExt;
 	}
 
 	// Save it right away...
-	bool bResult = qtractorMidiEditor::saveCopyFile(sFilename,
+	bool bResult = qtractorMidiFile::saveCopyFile(sFilename,
 		filename(), trackChannel(), sequence(), timeScale(), format());
 
 	// Have we done it right?
 	if (bResult) {
 		// Aha, but we're not dirty no more.
 		m_iDirtyCount = 0;
-		qtractorMidiClip *pMidiClip = m_pMidiEditor->midiClip();
-		if (pMidiClip) {
-			pMidiClip->setFilename(sFilename);
-			pMidiClip->setDirty(false);
+		pMidiClip->setFilename(sFilename);
+		pMidiClip->setDirty(false);
+		// HACK: This operation is so important that
+		// it surely deserves being in the front page...
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (pMainForm) {
+			pMainForm->appendMessages(
+				tr("MIDI file save: \"%1\", track-channel: %2.")
+				.arg(filename()).arg(trackChannel()));
+			pMainForm->addMidiFile(filename());
 		}
 	}
 
@@ -803,7 +857,7 @@ void qtractorMidiEditorForm::fileTrackProperties (void)
 
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	if (pMainForm && pMainForm->tracks())
-		(pMainForm->tracks())->editTrack(pTrack, this);
+		(pMainForm->tracks())->editTrack(pTrack);
 }
 
 
@@ -868,6 +922,20 @@ void qtractorMidiEditorForm::editPaste (void)
 }
 
 
+// Paste/repeat from clipboard.
+void qtractorMidiEditorForm::editPasteRepeat (void)
+{
+	qtractorPasteRepeatForm pasteForm(this);
+	pasteForm.setRepeatPeriod(m_pMidiEditor->pastePeriod());
+	if (pasteForm.exec()) {
+		m_pMidiEditor->pasteClipboard(
+			pasteForm.repeatCount(),
+			pasteForm.repeatPeriod()
+		);
+	}
+}
+
+
 // Delete current selection.
 void qtractorMidiEditorForm::editDelete (void)
 {
@@ -1089,11 +1157,7 @@ void qtractorMidiEditorForm::viewRefresh (void)
 // Show (and edit) keyboard shortcuts.
 void qtractorMidiEditorForm::helpShortcuts (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions == NULL)
 		return;
 
@@ -1137,15 +1201,7 @@ void qtractorMidiEditorForm::sendNote ( int iNote, int iVelocity )
 	if (pMidiBus == NULL)
 		return;
 
-	pMidiBus->sendNote(pTrack->midiChannel(), iNote, iVelocity);
-
-	// Track output monitoring...
-	if (iVelocity > 0) {
-		qtractorMidiMonitor *pMidiMonitor
-			= static_cast<qtractorMidiMonitor *> (pTrack->monitor());
-		if (pMidiMonitor)
-			pMidiMonitor->enqueue(qtractorMidiEvent::NOTEON, iVelocity);
-	}
+	pMidiBus->sendNote(pTrack, iNote, iVelocity);
 }
 
 
@@ -1170,13 +1226,22 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 	m_pMidiEditor->updateRedoAction(m_ui.editRedoAction);
 
 	bool bSelected = m_pMidiEditor->isSelected();
+	bool bClipboard = m_pMidiEditor->isClipboard();
 	m_ui.editCutAction->setEnabled(bSelected);
 	m_ui.editCopyAction->setEnabled(bSelected);
-	m_ui.editPasteAction->setEnabled(m_pMidiEditor->isClipboard());
+	m_ui.editPasteAction->setEnabled(bClipboard);
+	m_ui.editPasteRepeatAction->setEnabled(bClipboard);
 	m_ui.editDeleteAction->setEnabled(bSelected);
 	m_ui.editSelectNoneAction->setEnabled(bSelected);
+#if 0
 	m_ui.toolsMenu->setEnabled(bSelected);
-
+#else
+	m_ui.toolsQuantizeAction->setEnabled(bSelected);
+	m_ui.toolsTransposeAction->setEnabled(bSelected);
+	m_ui.toolsNormalizeAction->setEnabled(bSelected);
+	m_ui.toolsRandomizeAction->setEnabled(bSelected);
+	m_ui.toolsResizeAction->setEnabled(bSelected);
+#endif
 	// Just having a non-null sequence will indicate
 	// that we're editing a legal MIDI clip...
 	qtractorMidiSequence *pSeq = sequence();
@@ -1216,37 +1281,43 @@ void qtractorMidiEditorForm::stabilizeForm (void)
 		m_pStatusModLabel->clear();
 
 	m_pDurationLabel->setText(
-		(m_pMidiEditor->timeScale())->textFromTick(pSeq->duration()));
+		(m_pMidiEditor->timeScale())->textFromTick(pSeq->duration(), true));
 
-	qtractorSession  *pSession  = NULL;
+	qtractorSession  *pSession  = qtractorSession::getInstance();
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
-	if (pSession) {
+	if (pSession && pMainForm) {
 		unsigned long iPlayHead = pSession->playHead();
+		bool bSelectable = (pSession->editHead() < pSession->editTail());
 		bool bPlaying    = pSession->isPlaying();
 		bool bRecording  = pSession->isRecording();
+		bool bPunching   = pSession->isPunching();
 		bool bLooping    = pSession->isLooping();
-		bool bEnabled    = (!bPlaying || !bRecording);
-		bool bSelectable = (pSession->editHead() < pSession->editTail());
-		bool bBumped = (bEnabled && (pSession->playHead() > 0 || bPlaying));
+		bool bRolling    = (bPlaying && bRecording);
+		bool bBumped     = (!bRolling && (iPlayHead > 0 || bPlaying));
 		int iRolling = pMainForm->rolling();
 		m_ui.transportBackwardAction->setEnabled(bBumped);
 		m_ui.transportRewindAction->setEnabled(bBumped);
-		m_ui.transportFastForwardAction->setEnabled(bEnabled);
-		m_ui.transportForwardAction->setEnabled(bEnabled
-			&& (iPlayHead < pSession->sessionLength()
+		m_ui.transportFastForwardAction->setEnabled(!bRolling);
+		m_ui.transportForwardAction->setEnabled(
+			!bRolling && (iPlayHead < pSession->sessionLength()
 				|| iPlayHead < pSession->editHead()
 				|| iPlayHead < pSession->editTail()));
-		m_ui.transportLoopAction->setEnabled(bEnabled
-			&& (bLooping || bSelectable));
-		m_ui.transportLoopSetAction->setEnabled(bEnabled && bSelectable);
-		m_ui.transportRecordAction->setEnabled(pSession->recordTracks() > 0);
+		m_ui.transportLoopAction->setEnabled(
+			!bRolling && (bLooping || bSelectable));
+		m_ui.transportLoopSetAction->setEnabled(
+			!bRolling && bSelectable);
+		m_ui.transportRecordAction->setEnabled(
+			(!bLooping || !bPunching) && pSession->recordTracks() > 0);
+		m_ui.transportPunchAction->setEnabled(
+			!bLooping && (bPunching || bSelectable));
+		m_ui.transportPunchSetAction->setEnabled(
+			!bLooping && bSelectable);
 		m_ui.transportRewindAction->setChecked(iRolling < 0);
 		m_ui.transportFastForwardAction->setChecked(iRolling > 0);
 		m_ui.transportLoopAction->setChecked(bLooping);
 		m_ui.transportPlayAction->setChecked(bPlaying);
 		m_ui.transportRecordAction->setChecked(bRecording);
+		m_ui.transportPunchAction->setChecked(bPunching);
 	}
 }
 
