@@ -77,6 +77,9 @@ public:
 	// Pre-close event handler.
 	bool queryClose();
 
+	// Unconditional-close.
+	void forceClose();
+
 public slots:
 
 	void fileSave();
@@ -96,6 +99,7 @@ public slots:
 	void editCut();
 	void editCopy();
 	void editPaste();
+	void editPasteRepeat();
 	void editDelete();
 
 	void editSelectNone();
