@@ -35,6 +35,7 @@ class qtractorMidiEvent;
 class qtractorMidiInputThread;
 class qtractorMidiOutputThread;
 class qtractorMidiMonitor;
+class qtractorPluginList;
 
 class QSocketNotifier;
 
@@ -62,9 +63,6 @@ public:
 	// Special slave sync method.
 	void sync();
 
-	// Drift corrected anchor.
-	unsigned long timeStart() const;
-
 	// Read ahead frames configuration.
 	void setReadAhead(unsigned int iReadAhead);
 	unsigned int readAhead() const;
@@ -75,6 +73,9 @@ public:
 	// Reset all MIDI monitoring...
 	void resetAllMonitors();
 
+	// Reset all MIDI controllers...
+	void resetAllControllers();
+
 	// MIDI event capture method.
 	void capture(snd_seq_event_t *pEv);
 
@@ -88,6 +89,10 @@ public:
 	// Special rewind method, on queue loop.
 	void restartLoop();
 
+	// The delta-time accessors.
+	void setTimeStart(long iTimeStart);
+	long timeStart() const;
+
 	// Special track-immediate methods.
 	void trackMute(qtractorTrack *pTrack, bool bMute);
 
@@ -121,11 +126,13 @@ public:
 	void processMetro(unsigned long iFrameStart, unsigned long iFrameEnd);
 
 	// Event notifier widget settings.
-	void setNotifyWidget  (QWidget *pNotifyWidget);
+	void setNotifyObject  (QObject *pNotifyObject);
 	void setNotifyMmcType (QEvent::Type eNotifyMmcType);
+	void setNotifyCtlType (QEvent::Type eNotifyCtlType);
 
-	QWidget     *notifyWidget() const;
+	QObject     *notifyObject() const;
 	QEvent::Type notifyMmcType() const;
+	QEvent::Type notifyCtlType() const;
 
 	// Control bus accessors.
 	void setControlBus(bool bControlBus);
@@ -203,10 +210,12 @@ private:
 	// The delta-time when playback started .
 	long m_iTimeStart;
 	long m_iTimeDelta;
+	long m_iTimeDrift;
 
 	// The event notifier widget.
-	QWidget      *m_pNotifyWidget;
+	QObject      *m_pNotifyObject;
 	QEvent::Type  m_eNotifyMmcType;
+	QEvent::Type  m_eNotifyCtlType;
 
 	// The assigned control buses.
 	bool             m_bControlBus;
@@ -273,15 +282,17 @@ public:
 		{ return m_patches[iChannel & 0x0f]; }
 
 	// Direct MIDI bank/program selection helper.
-	void setPatch(unsigned short iChannel, const QString& sInstrumentName,
-		int iBankSelMethod, int iBank, int iProg);
+	void setPatch(unsigned short iChannel,
+		const QString& sInstrumentName,
+		int iBankSelMethod, int iBank, int iProg,
+		qtractorTrack *pTrack);
 
-	// Direct MIDI controller helper.
-	void setController(unsigned short iChannel,
+	// Direct MIDI controller helpers.
+	void setController(qtractorTrack *pTrack,
 		int iController, int iValue = 0) const;
 
 	// Direct MIDI note helper.
-	void sendNote(unsigned short iChannel,
+	void sendNote(qtractorTrack *pTrack,
 		int iNote, int iVelocity = 0) const;
 
 	// Direct SysEx helper.
@@ -295,6 +306,10 @@ public:
 	qtractorMidiMonitor *midiMonitor_in()  const;
 	qtractorMidiMonitor *midiMonitor_out() const;
 
+	// Plugin-chain accessors.
+	qtractorPluginList *pluginList_in()  const;
+	qtractorPluginList *pluginList_out() const;
+
 	// Retrieve/restore client:port connections.
 	// return the effective number of connection attempts.
 	int updateConnects(BusMode busMode,
@@ -306,9 +321,9 @@ public:
 	void setMasterPanning(float fPanning);
 
 	// MIDI channel volume.
-	void setVolume(unsigned short iChannel, float fVolume);
+	void setVolume(qtractorTrack *pTrack, float fVolume);
 	// MIDI channel stereo panning.
-	void setPanning(unsigned short iChannel, float fPanning);
+	void setPanning(qtractorTrack *pTrack, float fPanning);
 
 	// Document element methods.
 	bool loadElement(qtractorSessionDocument *pDocument,
@@ -318,9 +333,19 @@ public:
 
 protected:
 
+	// Direct MIDI controller common helper.
+	void setControllerEx(unsigned short iChannel, int iController,
+		int iValue = 0, qtractorTrack *pTrack = NULL) const;
+
 	// Bus mode change event.
 	void updateBusMode();
 
+	// Create plugin-list properly.
+	qtractorPluginList *createPluginList (void) const;
+
+	// Set plugin-list buffers properly.
+	void updatePluginList(qtractorPluginList *pPluginList);
+
 	// Document instrument map methods.
 	bool loadMidiMap(qtractorSessionDocument *pDocument,
 		QDomElement *pElement);
@@ -336,11 +361,43 @@ private:
 	qtractorMidiMonitor *m_pIMidiMonitor;
 	qtractorMidiMonitor *m_pOMidiMonitor;
 
+	// Plugin-chain instances.
+	qtractorPluginList *m_pIPluginList;
+	qtractorPluginList *m_pOPluginList;
+
 	// Channel patch mapper.
 	QHash<unsigned short, Patch> m_patches;
 };
 
 
+//----------------------------------------------------------------------
+// qtractorMidiControlEvent - MIDI Control custom event.
+//
+
+class qtractorMidiControlEvent : public QEvent
+{
+public:
+
+	// Contructor.
+	qtractorMidiControlEvent(QEvent::Type eType, unsigned short iChannel,
+		unsigned char controller, unsigned char value)
+		: QEvent(eType), m_channel(iChannel),
+			m_controller(controller), m_value(value) {}
+
+	// Accessors.
+	unsigned short channel()    const { return m_channel; }
+	unsigned short controller() const { return m_controller; }
+	unsigned short value()      const { return m_value; }
+
+private:
+
+	// Instance variables.
+	unsigned short m_channel;
+	unsigned short m_controller;
+	unsigned short m_value;
+};
+
+
 #endif  // __qtractorMidiEngine_h
 
 
