@@ -19,13 +19,18 @@
 
 *****************************************************************************/
 
+#include "qtractorAbout.h"
 #include "qtractorMidiFile.h"
+#include "qtractorTimeScale.h"
+#include "qtractorSession.h"
 
 #include <QFileInfo>
+#include <QDir>
+#include <QRegExp>
 
 // Symbolic header markers.
-#define SMF_MTHD 0x4d546864
-#define SMF_MTRK 0x4d54726b
+#define SMF_MTHD "MThd"
+#define SMF_MTRK "MTrk"
 
 
 //----------------------------------------------------------------------
@@ -82,7 +87,9 @@ bool qtractorMidiFile::open ( const QString& sFilename, int iMode )
 
 	// First word must identify the file as a SMF;
 	// must be literal "MThd"
-	if (readInt(4) != SMF_MTHD) {
+	char header[5];
+	readData((unsigned char *) &header[0], 4); header[4] = (char) 0;
+	if (::strcmp(header, SMF_MTHD)) {
 		close();
 		return false;
 	}
@@ -112,7 +119,8 @@ bool qtractorMidiFile::open ( const QString& sFilename, int iMode )
 	m_pTrackInfo = new TrackInfo [m_iTracks];
 	for (int iTrack = 0; iTrack < m_iTracks; iTrack++) {
 		// Must be a track header "MTrk"...
-		if (readInt(4) != SMF_MTRK) {
+		readData((unsigned char *) &header[0], 4); header[4] = (char) 0;
+		if (::strcmp(header, SMF_MTRK)) {
 			close();
 			return false;
 		}
@@ -191,7 +199,7 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 		unsigned long iTrackTime  = 0;
 		unsigned long iTrackEnd   = m_iOffset + m_pTrackInfo[iTrack].length;
 		unsigned int  iLastStatus = 0;
-		
+
 		// While this track lasts...
 		while (m_iOffset < iTrackEnd) {
 
@@ -245,7 +253,6 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				data2 = readInt(1);
 				// Check if its channel filtered...
 				if (bChannelEvent) {
-					iTime -= pSeq->timeOffset();
 					if (data2 == 0 && type == qtractorMidiEvent::NOTEON)
 						type = qtractorMidiEvent::NOTEOFF;
 					pEvent = new qtractorMidiEvent(iTime, type, data1, data2);
@@ -258,7 +265,6 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				data2 = readInt(1);
 				// Check if its channel filtered...
 				if (bChannelEvent) {
-					iTime -= pSeq->timeOffset();
 					// We don't sequence bank select events here,
 					// just set the primordial bank patch...
 					switch (data1) {
@@ -287,7 +293,6 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				data2 = readInt(1);
 				// Check if its channel filtered...
 				if (bChannelEvent) {
-					iTime -= pSeq->timeOffset();
 					// Create the new event...
 					pEvent = new qtractorMidiEvent(iTime, type, data1, data2);
 					pSeq->addEvent(pEvent);
@@ -299,7 +304,6 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				data2 = readInt(1);
 				// Check if its channel filtered...
 				if (bChannelEvent) {
-					iTime -= pSeq->timeOffset();
 					// We don't sequence prog change events here,
 					// just set the primordial program patch...
 					if (pSeq->program() < 0)
@@ -312,7 +316,6 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				data2 = readInt(1);
 				// Check if its channel filtered...
 				if (bChannelEvent) {
-					iTime -= pSeq->timeOffset();
 					// Create the new event...
 					pEvent = new qtractorMidiEvent(iTime, type, data1, data2);
 					pSeq->addEvent(pEvent);
@@ -329,7 +332,6 @@ bool qtractorMidiFile::readTracks ( qtractorMidiSequence **ppSeqs,
 				}
 				// Check if its channel filtered...
 				if (bChannelEvent) {
-					iTime -= pSeq->timeOffset();
 					pEvent = new qtractorMidiEvent(iTime, type);
 					pEvent->setSysex(data, 1 + len);
 					pSeq->addEvent(pEvent);
@@ -421,7 +423,7 @@ bool qtractorMidiFile::writeHeader ( unsigned short iFormat,
 
 	// First word must identify the file as a SMF;
 	// must be literal "MThd"
-	writeInt(SMF_MTHD, 4);
+	writeData((unsigned char *) SMF_MTHD, 4);
 
 	// Second word should be the total header chunk length...
 	writeInt(6, 4);
@@ -474,7 +476,7 @@ bool qtractorMidiFile::writeTracks ( qtractorMidiSequence **ppSeqs,
 			pSeq = ppSeqs[iSeq];
 
 		// Must be a track header "MTrk"...
-		writeInt(SMF_MTRK, 4);
+		writeData((unsigned char *) SMF_MTRK, 4);
 
 		// Write a dummy track length (we'll overwrite it later)...
 		unsigned long iMTrkOffset = m_iOffset;
@@ -865,4 +867,104 @@ int qtractorMidiFile::writeData ( unsigned char *pData, unsigned short n )
 }
 
 
+// Create filename revision (name says it all).
+QString qtractorMidiFile::createFilePathRevision (
+	const QString& sFilename, int iRevision )
+{
+	QFileInfo fi(sFilename);
+	QDir adir(fi.absoluteDir());
+
+	QRegExp rxRevision("(.+)\\-(\\d+)$");
+	QString sBasename = fi.baseName();
+	if (rxRevision.exactMatch(sBasename)) {
+		sBasename = rxRevision.cap(1);
+		iRevision = rxRevision.cap(2).toInt();
+	}
+
+	sBasename += "-%1." + fi.completeSuffix();
+	do { fi.setFile(adir, sBasename.arg(++iRevision)); }
+	while (fi.exists());
+
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorMidiFile::createFilePathRevision(\"%s\")",
+		fi.absoluteFilePath().toUtf8().constData());
+#endif
+
+	return fi.absoluteFilePath();
+}
+
+
+// All-in-one SMF file writer/creator method.
+bool qtractorMidiFile::saveCopyFile ( const QString& sNewFilename,
+	const QString& sOldFilename, unsigned short iTrackChannel,
+	qtractorMidiSequence *pSeq, qtractorTimeScale *pTimeScale,
+	unsigned short iFormat )
+{
+	qtractorMidiFile file;
+	float fTempo = (pTimeScale ? pTimeScale->tempo() : 120.0f);
+	unsigned short iTicksPerBeat = (pTimeScale ? pTimeScale->ticksPerBeat() : 96);
+	unsigned short iBeatsPerBar  = (pTimeScale ? pTimeScale->beatsPerBar()  : 4);
+	unsigned short iSeq, iSeqs = 0;
+	qtractorMidiSequence **ppSeqs = NULL;
+	const QString sTrackName = QObject::tr("Track %1");
+
+	if (pSeq == NULL)
+		return false;
+
+	// Open and load the whole source file...
+	if (file.open(sOldFilename)) {
+		iTicksPerBeat = file.ticksPerBeat();
+		iFormat = file.format();
+		iSeqs = (iFormat == 1 ? file.tracks() : 16);
+		ppSeqs = new qtractorMidiSequence * [iSeqs];
+		for (iSeq = 0; iSeq < iSeqs; ++iSeq) {
+			ppSeqs[iSeq] = new qtractorMidiSequence(
+				sTrackName.arg(iSeq + 1), iSeq, iTicksPerBeat);
+		}
+		if (file.readTracks(ppSeqs, iSeqs)) {
+			fTempo = file.tempo();
+			iBeatsPerBar = file.beatsPerBar();
+		}
+		file.close();
+	}
+
+	// Open and save the whole target file...
+	if (!file.open(sNewFilename, qtractorMidiFile::Write))
+		return false;
+
+	file.setTempo(fTempo);
+	file.setBeatsPerBar(iBeatsPerBar);
+
+	if (ppSeqs == NULL)
+		iSeqs = (iFormat == 0 ? 1 : 2);
+
+	// Write SMF header...
+	file.writeHeader(iFormat, (iFormat == 0 ? 1 : iSeqs), iTicksPerBeat);
+
+	// Write SMF tracks(s)...
+	if (ppSeqs) {
+		// Replace the target track-channel events... 
+		ppSeqs[iTrackChannel]->replaceEvents(pSeq);
+		// Write the whole new tracks...
+		file.writeTracks(ppSeqs, iSeqs);
+	} else {
+		// Most probabley this is a brand new file...
+		if (iFormat == 1)
+			file.writeTrack(NULL);
+		file.writeTrack(pSeq);
+	}
+
+	file.close();
+
+	// Free locally allocated track/sequence array.
+	if (ppSeqs) {
+		for (iSeq = 0; iSeq < iSeq; ++iSeq)
+			delete ppSeqs[iSeq];
+		delete [] ppSeqs;
+	}
+
+	return true;
+}
+
+
 // end of qtractorMidiFile.h
