@@ -24,6 +24,7 @@
 
 #include "qtractorMidiSequence.h"
 
+class qtractorTimeScale;
 
 //----------------------------------------------------------------------
 // class qtractorMidiFile -- A SMF (Standard MIDI File) class.
@@ -78,6 +79,16 @@ public:
 	bool writeTracks(qtractorMidiSequence **ppSeqs, unsigned short iSeqs);
 	bool writeTrack (qtractorMidiSequence *pSeq);
 
+	// All-in-one SMF file writer/creator method.
+	static bool saveCopyFile(const QString& sNewFilename,
+		const QString& sOldFilename, unsigned short iTrackChannel,
+		qtractorMidiSequence *pSeq, qtractorTimeScale *pTimeScale = NULL,
+		unsigned short iFormat = 0);
+
+	// Create filename revision.
+	static QString createFilePathRevision(
+		const QString& sFilename, int iRevision = 0);
+
 protected:
 
 	// Read methods.
