@@ -26,7 +26,6 @@
 
 #include "qtractorMidiEditCommand.h"
 
-#include "qtractorMainForm.h"
 #include "qtractorOptions.h"
 #include "qtractorSession.h"
 
@@ -63,10 +62,7 @@ qtractorMidiToolsForm::qtractorMidiToolsForm (
 	// Reinitialize random seed.
 	::srand(::time(NULL));
 
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession) {
 		// Copy from global time-scale instance...
 		if (m_pTimeScale)
@@ -151,13 +147,13 @@ qtractorMidiToolsForm::qtractorMidiToolsForm (
 	QObject::connect(m_ui.NormalizeCheckBox,
 		SIGNAL(toggled(bool)),
 		SLOT(stabilizeForm()));
-	QObject::connect(m_ui.NormalizePercentRadioButton,
+	QObject::connect(m_ui.NormalizePercentCheckBox,
 		SIGNAL(toggled(bool)),
 		SLOT(changed()));
 	QObject::connect(m_ui.NormalizePercentSpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(changed()));
-	QObject::connect(m_ui.NormalizeValueRadioButton,
+	QObject::connect(m_ui.NormalizeValueCheckBox,
 		SIGNAL(toggled(bool)),
 		SLOT(changed()));
 	QObject::connect(m_ui.NormalizeValueSpinBox,
@@ -270,10 +266,7 @@ int qtractorMidiToolsForm::toolIndex (void) const
 void qtractorMidiToolsForm::loadPreset ( const QString& sPreset )
 {
 	// An existing preset is about to be loaded...
-	qtractorOptions  *pOptions  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions) {
 		QList<QVariant> vlist;
 		QSettings& settings = pOptions->settings();
@@ -303,9 +296,9 @@ void qtractorMidiToolsForm::loadPreset ( const QString& sPreset )
 		vlist = settings.value("/Normalize").toList();
 		if (vlist.count() > 4) {
 		//	m_ui.NormalizeCheckBox->setChecked(vlist[0].toBool());
-			m_ui.NormalizePercentRadioButton->setChecked(vlist[1].toBool());
+			m_ui.NormalizePercentCheckBox->setChecked(vlist[1].toBool());
 			m_ui.NormalizePercentSpinBox->setValue(vlist[2].toInt());
-			m_ui.NormalizeValueRadioButton->setChecked(vlist[3].toBool());
+			m_ui.NormalizeValueCheckBox->setChecked(vlist[3].toBool());
 			m_ui.NormalizeValueSpinBox->setValue(vlist[4].toInt());
 		}
 		// Randomize tool...
@@ -341,10 +334,7 @@ void qtractorMidiToolsForm::loadPreset ( const QString& sPreset )
 void qtractorMidiToolsForm::savePreset ( const QString& sPreset )
 {
 	// The current state preset is about to be saved...
-	qtractorOptions  *pOptions  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions) {
 		QList<QVariant> vlist;
 		QSettings& settings = pOptions->settings();
@@ -371,9 +361,9 @@ void qtractorMidiToolsForm::savePreset ( const QString& sPreset )
 		// Normalize tool...
 		vlist.clear();
 		vlist.append(m_ui.NormalizeCheckBox->isChecked());
-		vlist.append(m_ui.NormalizePercentRadioButton->isChecked());
+		vlist.append(m_ui.NormalizePercentCheckBox->isChecked());
 		vlist.append(m_ui.NormalizePercentSpinBox->value());
-		vlist.append(m_ui.NormalizeValueRadioButton->isChecked());
+		vlist.append(m_ui.NormalizeValueCheckBox->isChecked());
 		vlist.append(m_ui.NormalizeValueSpinBox->value());
 		settings.setValue("/Normalize", vlist);
 		// Randomize tool...
@@ -412,10 +402,7 @@ void qtractorMidiToolsForm::refreshPresets (void)
 	const QString sOldPreset = m_ui.PresetNameComboBox->currentText();
 	m_ui.PresetNameComboBox->clear();
 	
-	qtractorOptions  *pOptions  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions) {
 		QSettings& settings = pOptions->settings();
 		settings.beginGroup("/MidiTools");
@@ -477,10 +464,7 @@ void qtractorMidiToolsForm::presetDelete (void)
 		return;
 
 	// A preset entry is about to be deleted...
-	qtractorOptions  *pOptions  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions) {
 		// Prompt user if he/she's sure about this...
 		if (pOptions->bConfirmRemove) {
@@ -515,7 +499,8 @@ unsigned long qtractorMidiToolsForm::quantize (
 
 // Create edit command based on given selection.
 qtractorMidiEditCommand *qtractorMidiToolsForm::editCommand (
-	qtractorMidiClip *pMidiClip, qtractorMidiEditSelect *pSelect )
+	qtractorMidiClip *pMidiClip, qtractorMidiEditSelect *pSelect,
+	unsigned long iTimeOffset )
 {
 	// Create command, it will be handed over...
 	qtractorMidiEditCommand *pEditCommand
@@ -536,6 +521,23 @@ qtractorMidiEditCommand *qtractorMidiToolsForm::editCommand (
 	pEditCommand->setName(tools.join(", "));
 
 	QListIterator<qtractorMidiEditSelect::Item *> iter(pSelect->items());
+
+	// First scan pass for the normalize tool:
+	// find maximum value from the selection...
+	int iMaxValue = 0;
+	if (m_ui.NormalizeCheckBox->isChecked()) {
+		// Make it through one time...
+		while (iter.hasNext()) {
+			qtractorMidiEvent *pEvent = iter.next()->event;
+			bool bPitchBend = (pEvent->type() == qtractorMidiEvent::PITCHBEND);
+			int iValue = (bPitchBend ? pEvent->pitchBend() : pEvent->value());
+			if (iMaxValue < iValue)
+				iMaxValue = iValue;
+		}
+		// Get it back to front...
+		iter.toFront();
+	}
+
 	while (iter.hasNext()) {
 		qtractorMidiEvent *pEvent = iter.next()->event;
 		long iTime = pEvent->time();
@@ -546,8 +548,9 @@ qtractorMidiEditCommand *qtractorMidiToolsForm::editCommand (
 		if (m_ui.QuantizeCheckBox->isChecked()) {
 			tools.append(tr("quantize"));
 			if (m_ui.QuantizeTimeCheckBox->isChecked()) {
-				iTime = quantize(iTime,
-					m_ui.QuantizeTimeComboBox->currentIndex(), 1);
+				iTime = quantize(iTime + iTimeOffset,
+					m_ui.QuantizeTimeComboBox->currentIndex(), 1)
+						- iTimeOffset;
 			}
 			if (m_ui.QuantizeDurationCheckBox->isChecked()
 				&& pEvent->type() == qtractorMidiEvent::NOTEON) {
@@ -580,15 +583,14 @@ qtractorMidiEditCommand *qtractorMidiToolsForm::editCommand (
 		// Normalize tool...
 		if (m_ui.NormalizeCheckBox->isChecked()) {
 			tools.append(tr("normalize"));
-			int p = 0, q = 0;
-			if (m_ui.NormalizePercentRadioButton->isChecked()) {
-				p = m_ui.NormalizePercentSpinBox->value();
-				q = 100;
-			}
-			else
-			if (m_ui.NormalizeValueRadioButton->isChecked()) {
+			int p, q = iMaxValue;
+			if (m_ui.NormalizeValueCheckBox->isChecked())
 				p = m_ui.NormalizeValueSpinBox->value();
-				q = (bPitchBend ? 8192 : 128);
+			else
+				p = (bPitchBend ? 8192 : 128);
+			if (m_ui.NormalizePercentCheckBox->isChecked()) {
+				p *= m_ui.NormalizePercentSpinBox->value();
+				q *= 100;
 			}
 			if (q > 0) {
 				iValue = (iValue * p) / q;
@@ -791,14 +793,14 @@ void qtractorMidiToolsForm::stabilizeForm (void)
 
 	bEnabled = m_ui.NormalizeCheckBox->isChecked();
 
-	m_ui.NormalizePercentRadioButton->setEnabled(bEnabled);
-	bEnabled2 = bEnabled && m_ui.NormalizePercentRadioButton->isChecked();
+	m_ui.NormalizePercentCheckBox->setEnabled(bEnabled);
+	bEnabled2 = bEnabled && m_ui.NormalizePercentCheckBox->isChecked();
 	if (bEnabled2)
 		iEnabled++;
 	m_ui.NormalizePercentSpinBox->setEnabled(bEnabled2);
 
-	m_ui.NormalizeValueRadioButton->setEnabled(bEnabled);
-	bEnabled2 = bEnabled && m_ui.NormalizeValueRadioButton->isChecked();
+	m_ui.NormalizeValueCheckBox->setEnabled(bEnabled);
+	bEnabled2 = bEnabled && m_ui.NormalizeValueCheckBox->isChecked();
 	if (bEnabled2)
 		iEnabled++;
 	m_ui.NormalizeValueSpinBox->setEnabled(bEnabled2);
