@@ -54,7 +54,53 @@
 #include <QResizeEvent>
 #include <QMouseEvent>
 
+#include <QPainter>
 
+#ifdef CONFIG_GRADIENT
+#include <QLinearGradient>
+#endif
+
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
+
+//----------------------------------------------------------------------------
+// qtractorMixerStrip::IconLabel -- Custom mixer strip title widget.
+
+class qtractorMixerStrip::IconLabel : public QLabel
+{
+public:
+
+	// Constructor.
+	IconLabel(QWidget *pParent = NULL) : QLabel(pParent) {}
+
+	// Icon accessors.
+	void setIcon(const QIcon& icon)
+		{ m_icon = icon; }
+	const QIcon& icon() const
+		{ return m_icon; }
+
+protected:
+
+	// Custom paint event.
+	void paintEvent(QPaintEvent *)
+	{
+		QPainter painter(this);
+		QRect rect(QLabel::rect());
+		painter.drawPixmap(rect.x(), rect.y(), m_icon.pixmap(rect.height()));
+		rect.setX(rect.x() + rect.height() + 1);
+		painter.drawText(rect, QLabel::alignment(), QLabel::text());
+	}
+
+private:
+
+	// Instance variables.
+	QIcon m_icon;
+};
+
+	
 //----------------------------------------------------------------------------
 // qtractorMixerStrip -- Mixer strip widget.
 
@@ -94,8 +140,8 @@ qtractorMixerStrip::~qtractorMixerStrip (void)
 	if (m_pRecordButton)
 		delete m_pRecordButton;
 
-	if (m_pThruButton)
-		delete m_pThruButton;
+	if (m_pMonitorButton)
+		delete m_pMonitorButton;
 	if (m_pBusButton)
 		delete m_pBusButton;
 
@@ -114,23 +160,26 @@ void qtractorMixerStrip::initMixerStrip (void)
 	m_iMark = 0;
 	m_iUpdate = 0;
 
-	QFont font6(QFrame::font().family(), 6);
+	const QFont& font = QFrame::font();
+	QFont font6(font.family(), font.pointSize() - 2);
 	QFontMetrics fm(font6);
 
 	m_pLayout = new QVBoxLayout(this);
 	m_pLayout->setMargin(4);
 	m_pLayout->setSpacing(4);
 
-	m_pLabel = new QLabel(/*this*/);
+	m_pLabel = new IconLabel(/*this*/);
 	m_pLabel->setFont(font6);
-	m_pLabel->setFixedHeight(fm.lineSpacing());
+	m_pLabel->setFixedHeight(fm.lineSpacing() + 2);
 	m_pLabel->setBackgroundRole(QPalette::Button);
 	m_pLabel->setForegroundRole(QPalette::ButtonText);
 	m_pLabel->setAutoFillBackground(true);
 	m_pLayout->addWidget(m_pLabel);
 
 	m_pPluginListView = new qtractorPluginListView(/*this*/);
-	m_pPluginListView->setFixedHeight(4 * fm.lineSpacing());
+	m_pPluginListView->setFont(font6);
+	m_pPluginListView->setFixedHeight(fm.lineSpacing() << 2);
+	m_pPluginListView->setTinyScrollBar(true);
 	m_pLayout->addWidget(m_pPluginListView);
 
 	QIcon icons;
@@ -140,15 +189,15 @@ void qtractorMixerStrip::initMixerStrip (void)
 		QIcon::Normal, QIcon::On);
 	const QSizePolicy buttonPolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
 
-	m_pThruButton = new QToolButton(/*this*/);
-	m_pThruButton->setFixedHeight(16);
-	m_pThruButton->setSizePolicy(buttonPolicy);
-	m_pThruButton->setFont(font6);
-	m_pThruButton->setIcon(icons);
-	m_pThruButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
-//	m_pThruButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
-	m_pThruButton->setText(tr("monitor"));
-	m_pThruButton->setCheckable(true);
+	m_pMonitorButton = new QToolButton(/*this*/);
+	m_pMonitorButton->setFixedHeight(16);
+	m_pMonitorButton->setSizePolicy(buttonPolicy);
+	m_pMonitorButton->setFont(font6);
+	m_pMonitorButton->setIcon(icons);
+	m_pMonitorButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+//	m_pMonitorButton->setToolButtonStyle(Qt::ToolButtonTextOnly);
+	m_pMonitorButton->setText(tr("monitor"));
+	m_pMonitorButton->setCheckable(true);
 
 	m_pButtonLayout = new QHBoxLayout(/*this*/);
 	m_pButtonLayout->setSpacing(2);
@@ -179,7 +228,7 @@ void qtractorMixerStrip::initMixerStrip (void)
 		QObject::connect(m_pSoloButton,
 			SIGNAL(trackButtonToggled(qtractorTrackButton *, bool)),
 			pMixer, SLOT(trackButtonToggledSlot(qtractorTrackButton *, bool)));
-		m_pThruButton->setToolTip(tr("Monitor (rec)"));
+		m_pMonitorButton->setToolTip(tr("Monitor (rec)"));
 		m_pBusButton = NULL;
 	} else {
 		meterType = m_pBus->busType();
@@ -195,18 +244,18 @@ void qtractorMixerStrip::initMixerStrip (void)
 		QObject::connect(m_pBusButton,
 			SIGNAL(clicked()),
 			SLOT(busButtonSlot()));
-		m_pThruButton->setToolTip(tr("Monitor (thru)"));
+		m_pMonitorButton->setToolTip(tr("Monitor (thru)"));
 		m_pRecordButton = NULL;
 		m_pMuteButton   = NULL;
 		m_pSoloButton   = NULL;
 	}
 
-	updateThruButton();
-	QObject::connect(m_pThruButton,
+	updateMonitorButton();
+	QObject::connect(m_pMonitorButton,
 		SIGNAL(toggled(bool)),
-		SLOT(thruButtonSlot(bool)));
+		SLOT(monitorButtonSlot(bool)));
 
-	m_pLayout->addWidget(m_pThruButton);
+	m_pLayout->addWidget(m_pMonitorButton);
 	m_pLayout->addLayout(m_pButtonLayout);
 
 	// Now, there's whether we are Audio or MIDI related...
@@ -252,14 +301,22 @@ void qtractorMixerStrip::initMixerStrip (void)
 		if (m_pTrack) {
 			pMidiMonitor
 				= static_cast<qtractorMidiMonitor *> (m_pTrack->monitor());
-		} else if (m_pBus) {
+			m_pPluginListView->setPluginList(m_pTrack->pluginList());
+			m_pPluginListView->setEnabled(true);
+		} else {
 			pMidiBus = static_cast<qtractorMidiBus *> (m_pBus);
 			if (pMidiBus) {
-				if (m_busMode & qtractorBus::Input)
+				if (m_busMode & qtractorBus::Input) {
+					m_pPluginListView->setPluginList(
+						pMidiBus->pluginList_in());
 					pMidiMonitor = pMidiBus->midiMonitor_in();
-				else
+				} else {
+					m_pPluginListView->setPluginList(
+						pMidiBus->pluginList_out());
 					pMidiMonitor = pMidiBus->midiMonitor_out();
+				}
 			}
+			m_pPluginListView->setEnabled(true);
 		}
 		// Have we a MIDI monitor/meter?...
 		if (pMidiMonitor) {
@@ -287,7 +344,6 @@ void qtractorMixerStrip::initMixerStrip (void)
 				}
 			}
 		}
-		m_pPluginListView->setEnabled(false);
 		break;
 	}
 	case qtractorTrack::None:
@@ -343,10 +399,10 @@ void qtractorMixerStrip::updateName (void)
 		meterType = m_pTrack->trackType();
 		sName = m_pTrack->trackName();
 		QPalette pal(m_pLabel->palette());
-		pal.setColor(QPalette::Button, m_pTrack->foreground().light());
-		pal.setColor(QPalette::ButtonText, m_pTrack->background().light());
+		pal.setColor(QPalette::Button, m_pTrack->foreground().lighter());
+		pal.setColor(QPalette::ButtonText, m_pTrack->background().lighter());
 		m_pLabel->setPalette(pal);
-		m_pLabel->setAlignment(Qt::AlignLeft);
+		m_pLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
 	} else if (m_pBus) {
 		meterType = m_pBus->busType();
 		if (m_busMode & qtractorBus::Input) {
@@ -354,16 +410,17 @@ void qtractorMixerStrip::updateName (void)
 		} else {
 			sName = tr("%1 Out").arg(m_pBus->busName());
 		}
-		m_pLabel->setAlignment(Qt::AlignHCenter);
+		m_pLabel->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
 	}
-	m_pLabel->setText(sName);
-
+	
 	QString sSuffix;
 	switch (meterType) {
 	case qtractorTrack::Audio:
+		m_pLabel->setIcon(QIcon(":/icons/trackAudio.png"));
 		sSuffix = tr("(Audio)");
 		break;
 	case qtractorTrack::Midi:
+		m_pLabel->setIcon(QIcon(":/icons/trackMidi.png"));
 		sSuffix = tr("(MIDI)");
 		break;
 	case qtractorTrack::None:
@@ -372,17 +429,18 @@ void qtractorMixerStrip::updateName (void)
 		break;
 	}
 
+	m_pLabel->setText(sName);
 	QFrame::setToolTip(sName + ' ' + sSuffix);
 }
 
 
 // Pass-trough button updater.
-void qtractorMixerStrip::updateThruButton (void)
+void qtractorMixerStrip::updateMonitorButton (void)
 {
 	if (m_iUpdate > 0)
 		return;
 
-	if (m_pThruButton == NULL)
+	if (m_pMonitorButton == NULL)
 		return;
 
 	m_iUpdate++;
@@ -390,7 +448,7 @@ void qtractorMixerStrip::updateThruButton (void)
 	bool bOn = false;
 	if (m_pBus) {
 		bOn = m_pBus->isPassthru();
-		m_pThruButton->setEnabled(
+		m_pMonitorButton->setEnabled(
 			(m_pBus->busMode() & qtractorBus::Duplex) == qtractorBus::Duplex);
 	} 
 	else
@@ -399,11 +457,11 @@ void qtractorMixerStrip::updateThruButton (void)
 
 #if 0
 	const QColor& rgbOff = palette().button().color();
-	QPalette pal(m_pThruButton->palette());
-	pal.setColor(QPalette::Button, bOn ? Qt::gree : rgbOff);
-	m_pThruButton->setPalette(pal);
+	QPalette pal(m_pMonitorButton->palette());
+	pal.setColor(QPalette::Button, bOn ? Qt::green : rgbOff);
+	m_pMonitorButton->setPalette(pal);
 #endif
-	m_pThruButton->setChecked(bOn);
+	m_pMonitorButton->setChecked(bOn);
 
 	m_iUpdate--;
 }
@@ -478,7 +536,16 @@ void qtractorMixerStrip::setBus ( qtractorBus *pBus )
 		break;
 	}
 	case qtractorTrack::Midi: {
-		m_pPluginListView->setEnabled(false);
+		qtractorMidiBus *pMidiBus
+			= static_cast<qtractorMidiBus *> (m_pBus);
+		if (pMidiBus) {
+			if (m_busMode & qtractorBus::Input) {
+				m_pPluginListView->setPluginList(pMidiBus->pluginList_in());
+			} else {
+				m_pPluginListView->setPluginList(pMidiBus->pluginList_out());
+			}
+		}
+		m_pPluginListView->setEnabled(true);
 		break;
 	}
 	case qtractorTrack::None:
@@ -486,7 +553,7 @@ void qtractorMixerStrip::setBus ( qtractorBus *pBus )
 		break;
 	}
 
-	updateThruButton();
+	updateMonitorButton();
 	updateName();
 }
 
@@ -506,8 +573,7 @@ void qtractorMixerStrip::setTrack ( qtractorTrack *pTrack )
 	m_pTrack = pTrack;
 
 	m_pPluginListView->setPluginList(m_pTrack->pluginList());
-	m_pPluginListView->setEnabled(
-		m_pTrack->trackType() == qtractorTrack::Audio);
+	m_pPluginListView->setEnabled(true);
 
 	m_pRecordButton->setTrack(m_pTrack);
 	m_pMuteButton->setTrack(m_pTrack);
@@ -515,7 +581,7 @@ void qtractorMixerStrip::setTrack ( qtractorTrack *pTrack )
 
 	setMonitor(m_pTrack->monitor());
 
-	updateThruButton();
+	updateMonitorButton();
 	updateMidiLabel();
 	updateName();
 }
@@ -532,11 +598,51 @@ void qtractorMixerStrip::setSelected ( bool bSelected )
 	m_bSelected = bSelected;
 
 	QPalette pal;
+#ifdef CONFIG_GRADIENT
+	const QSize& hint = QFrame::sizeHint();
+	QLinearGradient grad(0, 0, hint.width() >> 1, hint.height());
 	if (m_bSelected) {
-		pal.setColor(QPalette::Window, pal.midlight().color().dark(150));
-		pal.setColor(QPalette::WindowText, pal.midlight().color().light(150));
+		const QColor& rgbBase = pal.midlight().color();
+		pal.setColor(QPalette::WindowText, rgbBase.lighter());
+		pal.setColor(QPalette::Window, rgbBase.darker(150));
+		grad.setColorAt(0.4, rgbBase.darker(150));
+		grad.setColorAt(0.6, rgbBase.darker(130));
+		grad.setColorAt(1.0, rgbBase.darker());
+	} else {
+		const QColor& rgbBase = pal.button().color();
+	//	pal.setColor(QPalette::WindowText, rgbBase.darker());
+		pal.setColor(QPalette::Window, rgbBase);
+		grad.setColorAt(0.4, rgbBase);
+		grad.setColorAt(0.6, rgbBase.lighter(105));
+		grad.setColorAt(1.0, rgbBase.darker(130));
 	}
+	m_pPluginListView->setPalette(pal);
+	m_pMonitorButton->setPalette(pal);
+	if (m_pBusButton)
+		m_pBusButton->setPalette(pal);
+	if (m_pRecordButton)
+		m_pRecordButton->setPalette(pal);
+	if (m_pMuteButton)
+		m_pMuteButton->setPalette(pal);
+	if (m_pSoloButton)
+		m_pSoloButton->setPalette(pal);
+	m_pMeter->setPalette(pal);
+	pal.setBrush(QPalette::Window, grad);
+#else
+	if (m_bSelected) {
+		const QColor& rgbBase = pal.midlight().color();
+		pal.setColor(QPalette::WindowText, rgbBase.lighter());
+		pal.setColor(QPalette::Window, rgbBase.darker(150));
+	} else {
+		const QColor& rgbBase = pal.button().color();
+	//	pal.setColor(QPalette::WindowText, rgbBase.darker());
+		pal.setColor(QPalette::Window, rgbBase);
+	}
+#endif
 	QFrame::setPalette(pal);
+#ifdef CONFIG_GRADIENT
+	updateTrackButtons();
+#endif
 }
 
 bool qtractorMixerStrip::isSelected (void) const
@@ -621,12 +727,12 @@ void qtractorMixerStrip::busPassthru ( bool bPassthru )
 	if (m_pBus == NULL)
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Here we go...
-	pMainForm->commands()->exec(
+	pSession->execute(
 		new qtractorBusPassthruCommand(m_pBus, bPassthru));
 }
 
@@ -637,12 +743,12 @@ void qtractorMixerStrip::trackMonitor ( bool bMonitor )
 	if (m_pTrack == NULL)
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Here we go...
-	pMainForm->commands()->exec(
+	pSession->execute(
 		new qtractorTrackMonitorCommand(m_pTrack, bMonitor));
 }
 
@@ -655,7 +761,7 @@ void qtractorMixerStrip::busButtonSlot (void)
 
 
 // Common passthru/monitor button slot
-void qtractorMixerStrip::thruButtonSlot ( bool bOn )
+void qtractorMixerStrip::monitorButtonSlot ( bool bOn )
 {
 	if (m_iUpdate > 0)
 		return;
@@ -678,16 +784,16 @@ void qtractorMixerStrip::panChangedSlot ( float fPanning )
 	qDebug("qtractorMixerStrip[%p]::panChangedSlot(%.3g)", this, fPanning);
 #endif
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Put it in the form of an undoable command...
 	if (m_pTrack) {
-		pMainForm->commands()->exec(
+		pSession->execute(
 			new qtractorTrackPanningCommand(m_pTrack, fPanning));
 	} else if (m_pBus) {
-		pMainForm->commands()->exec(
+		pSession->execute(
 			new qtractorBusPanningCommand(m_pBus, m_busMode, fPanning));
 	}
 }
@@ -703,16 +809,16 @@ void qtractorMixerStrip::gainChangedSlot ( float fGain )
 	qDebug("qtractorMixerStrip[%p]::gainChangedSlot(%.3g)\n", this, fGain);
 #endif
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	// Put it in the form of an undoable command...
 	if (m_pTrack) {
-		pMainForm->commands()->exec(
+		pSession->execute(
 			new qtractorTrackGainCommand(m_pTrack, fGain));
 	} else if (m_pBus) {
-		pMainForm->commands()->exec(
+		pSession->execute(
 			new qtractorBusGainCommand(m_pBus, m_busMode, fGain));
 	}
 }
@@ -924,6 +1030,24 @@ void qtractorMixerRack::cleanStrips ( int iMark )
 }
 
 
+// Update all strips track buttons.
+void qtractorMixerRack::updateTrackButtons (void)
+{
+	QListIterator<qtractorMixerStrip *> iter(m_strips);
+	while (iter.hasNext())
+		iter.next()->updateTrackButtons();
+}
+
+
+// Update all strips monitor buttons.
+void qtractorMixerRack::updateMonitorButtons (void)
+{
+	QListIterator<qtractorMixerStrip *> iter(m_strips);
+	while (iter.hasNext())
+		iter.next()->updateMonitorButton();
+}
+
+
 // Resize event handler.
 void qtractorMixerRack::resizeEvent ( QResizeEvent *pResizeEvent )
 {
@@ -1032,7 +1156,7 @@ qtractorMixer::qtractorMixer ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	m_pSplitter = new QSplitter(Qt::Horizontal, this);
 	m_pSplitter->setObjectName("MixerSplitter");
-	m_pSplitter->setChildrenCollapsible(false);
+//	m_pSplitter->setChildrenCollapsible(false);
 //	m_pSplitter->setOpaqueResize(false);
 	m_pSplitter->setHandleWidth(2);
 
@@ -1080,11 +1204,11 @@ qtractorMixer::~qtractorMixer (void)
 // Notify the main application widget that we're emerging.
 void qtractorMixer::showEvent ( QShowEvent *pShowEvent )
 {
-    qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-    if (pMainForm)
-        pMainForm->stabilizeForm();
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm)
+		pMainForm->stabilizeForm();
 
-    QWidget::showEvent(pShowEvent);
+	QWidget::showEvent(pShowEvent);
 }
 
 // Notify the main application widget that we're closing.
@@ -1112,14 +1236,6 @@ void qtractorMixer::closeEvent ( QCloseEvent * /*pCloseEvent*/ )
 }
 
 
-// Session accessor.
-qtractorSession *qtractorMixer::session (void) const
-{
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	return (pMainForm ? pMainForm->session() : NULL);
-}
-
-
 // The splitter layout widget accessor.
 QSplitter *qtractorMixer::splitter (void) const
 {
@@ -1131,16 +1247,13 @@ QSplitter *qtractorMixer::splitter (void) const
 // (with some fair default...)
 void qtractorMixer::loadSplitterSizes (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			QList<int> sizes;
-			sizes.append(140);
-			sizes.append(160);
-			sizes.append(140);
-			pOptions->loadSplitterSizes(m_pSplitter, sizes);
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<int> sizes;
+		sizes.append(140);
+		sizes.append(160);
+		sizes.append(140);
+		pOptions->loadSplitterSizes(m_pSplitter, sizes);
 	}
 }
 
@@ -1148,12 +1261,9 @@ void qtractorMixer::loadSplitterSizes (void)
 // Save splitter sizes...
 void qtractorMixer::saveSplitterSizes (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions)
-			pOptions->saveSplitterSizes(m_pSplitter);
-	}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions)
+		pOptions->saveSplitterSizes(m_pSplitter);
 }
 
 
@@ -1209,7 +1319,7 @@ void qtractorMixer::updateTrackStrip ( qtractorTrack *pTrack, bool bReset )
 // Update buses'racks.
 void qtractorMixer::updateBuses (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -1242,7 +1352,7 @@ void qtractorMixer::updateBuses (void)
 // Update tracks'rack.
 void qtractorMixer::updateTracks (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -1280,10 +1390,33 @@ void qtractorMixer::trackButtonToggledSlot (
 	qtractorTrackButton *pTrackButton, bool bOn )
 {
 	// Put it in the form of an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pMainForm->commands()->exec(
-			new qtractorTrackButtonCommand(pTrackButton, bOn));
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorTrackButtonCommand(pTrackButton, bOn));
+}
+
+
+// Keyboard event handler.
+void qtractorMixer::keyPressEvent ( QKeyEvent *pKeyEvent )
+{
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorMixer::keyPressEvent(%d)", pKeyEvent->key());
+#endif
+	int iKey = pKeyEvent->key();
+	switch (iKey) {
+	case Qt::Key_Escape:
+		close();
+		break;
+	default:
+		QWidget::keyPressEvent(pKeyEvent);
+		break;
+	}
+
+	// Make sure we've get focus back...
+	QWidget::setFocus();
 }
 
 
