@@ -87,7 +87,8 @@ public:
 	void setSelected(bool bSelected);
 	bool isSelected() const;
 
-	// Update track buttons state.
+	// Update monitor/track buttons state.
+	void updateMonitorButton();
 	void updateTrackButtons();
 
 	// Strip refreshment.
@@ -110,7 +111,7 @@ protected slots:
 	void busButtonSlot();
 
 	// Common passthru/monitor button slot
-	void thruButtonSlot(bool bOn);
+	void monitorButtonSlot(bool bOn);
 
 	// Meter slider change slots.
 	void panChangedSlot(float);
@@ -121,7 +122,6 @@ protected:
 	// Common mixer-strip initializer.
 	void initMixerStrip();
 
-	void updateThruButton();
 	void updateMidiLabel();
 	void updateName();
 
@@ -142,8 +142,10 @@ private:
 	qtractorTrack *m_pTrack;
 
 	// Local widgets.
+	class IconLabel;
+
 	QVBoxLayout            *m_pLayout;
-	QLabel                 *m_pLabel;
+	IconLabel              *m_pLabel;
 	qtractorPluginListView *m_pPluginListView;
 	QHBoxLayout            *m_pButtonLayout;
 	qtractorTrackButton    *m_pRecordButton;
@@ -151,7 +153,7 @@ private:
 	qtractorTrackButton    *m_pSoloButton;
 	qtractorMeter          *m_pMeter;
 	QToolButton            *m_pBusButton;
-	QToolButton            *m_pThruButton;
+	QToolButton            *m_pMonitorButton;
 	QLabel                 *m_pMidiLabel;
 
 	// Selection stuff.
@@ -214,6 +216,12 @@ public:
 	void markStrips(int iMark);
 	void cleanStrips(int iMark);
 
+	// Update all strips track buttons.
+	void updateTrackButtons();
+
+	// Update all strips monitor buttons.
+	void updateMonitorButtons();
+
 public slots:
 
 	// Bus context menu slots.
@@ -270,9 +278,6 @@ public:
 	// Default destructor.
 	~qtractorMixer();
 
-	// Session accessors.
-	qtractorSession *session() const;
-
 	// The splitter layout widget accessor.
 	QSplitter *splitter() const;
 
@@ -310,6 +315,9 @@ protected:
 	// Just about to notify main-window that we're closing.
 	void closeEvent(QCloseEvent *);
 
+	// Keyboard event handler.
+	void keyPressEvent(QKeyEvent *);
+
 	// Special splitter-size persistence methods.
 	void loadSplitterSizes();
 	void saveSplitterSizes();
