@@ -41,13 +41,26 @@
 
 
 //-------------------------------------------------------------------------
-// qtractorOptions - Prototype settings structure.
+// qtractorOptions - Prototype settings structure (pseudo-singleton).
 //
 
+// Singleton instance pointer.
+qtractorOptions *qtractorOptions::g_pOptions = NULL;
+
+// Singleton instance accessor (static).
+qtractorOptions *qtractorOptions::getInstance (void)
+{
+	return g_pOptions;
+}
+
+
 // Constructor.
 qtractorOptions::qtractorOptions (void)
 	: m_settings(QTRACTOR_DOMAIN, QTRACTOR_TITLE)
 {
+	// Pseudo-singleton reference setup.
+	g_pOptions = this;
+
 	// And go into general options group.
 	m_settings.beginGroup("/Options");
 
@@ -63,6 +76,13 @@ qtractorOptions::qtractorOptions (void)
 	bKeepToolsOnTop = m_settings.value("/KeepToolsOnTop", true).toBool();
 	iDisplayFormat  = m_settings.value("/DisplayFormat", 1).toInt();
 	iMaxRecentFiles = m_settings.value("/MaxRecentFiles", 5).toInt();
+	iBaseFontSize   = m_settings.value("/BaseFontSize", 0).toInt();
+	m_settings.endGroup();
+
+	// Load logging options...
+	m_settings.beginGroup("/Logging");
+	bMessagesLog     = m_settings.value("/MessagesLog", false).toBool();
+	sMessagesLogPath = m_settings.value("/MessagesLogPath", "qtractor.log").toString();
 	m_settings.endGroup();
 
 	// And go into view options group.
@@ -128,14 +148,23 @@ qtractorOptions::qtractorOptions (void)
 
 	m_settings.endGroup(); // Options group.
 
-	// Last but not least, get the default directories.
+	// Last but not least, get the defaults.
 	m_settings.beginGroup("/Default");
+	bSessionTemplate = m_settings.value("/SessionTemplate", false).toBool();
+	sSessionTemplatePath = m_settings.value("/SessionTemplatePath").toString();
 	sSessionDir    = m_settings.value("/SessionDir").toString();
 	sAudioDir      = m_settings.value("/AudioDir").toString();
 	sMidiDir       = m_settings.value("/MidiDir").toString();
+	sPresetDir     = m_settings.value("/PresetDir").toString();
 	sInstrumentDir = m_settings.value("/InstrumentDir").toString();
+	bAutoMonitor   = m_settings.value("/AutoMonitor", true).toBool();
+	iSnapPerBeat   = m_settings.value("/SnapPerBeat", 4).toInt();
+	fTempo   = float(m_settings.value("/Tempo", 120.0).toDouble());
+	iPasteRepeatCount = m_settings.value("/PasteRepeatCount", 2).toInt();
+	bPasteRepeatPeriod = m_settings.value("/PasteRepeatPeriod", false).toInt();
 	sPluginSearch  = m_settings.value("/PluginSearch").toString();
 	iPluginType    = m_settings.value("/PluginType", 1).toInt();
+	bPluginActivate = m_settings.value("/PluginActivate", false).toBool();
 	m_settings.endGroup();
 
 	// Plug-in paths.
@@ -143,6 +172,8 @@ qtractorOptions::qtractorOptions (void)
 	ladspaPaths = m_settings.value("/LadspaPaths").toStringList();
 	dssiPaths   = m_settings.value("/DssiPaths").toStringList();
 	vstPaths    = m_settings.value("/VstPaths").toStringList();
+	bAudioOutputBus = m_settings.value("/AudioOutputBus", false).toBool();
+	bDummyVstScan = m_settings.value("/DummyVstScan", true).toBool();
 	m_settings.endGroup();
 
 	// Instrument file list.
@@ -228,6 +259,13 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.setValue("/KeepToolsOnTop", bKeepToolsOnTop);
 	m_settings.setValue("/DisplayFormat", iDisplayFormat);
 	m_settings.setValue("/MaxRecentFiles", iMaxRecentFiles);
+	m_settings.setValue("/BaseFontSize", iBaseFontSize);
+	m_settings.endGroup();
+
+	// Save logging options...
+	m_settings.beginGroup("/Logging");
+	m_settings.setValue("/MessagesLog", bMessagesLog);
+	m_settings.setValue("/MessagesLogPath", sMessagesLogPath);
 	m_settings.endGroup();
 
 	// View options group.
@@ -295,12 +333,21 @@ qtractorOptions::~qtractorOptions (void)
 
 	// Default directories.
 	m_settings.beginGroup("/Default");
+	m_settings.setValue("/SessionTemplate", bSessionTemplate);
+	m_settings.setValue("/SessionTemplatePath", sSessionTemplatePath);
 	m_settings.setValue("/SessionDir", sSessionDir);
 	m_settings.setValue("/AudioDir", sAudioDir);
 	m_settings.setValue("/MidiDir", sMidiDir);
+	m_settings.setValue("/PresetDir", sPresetDir);
 	m_settings.setValue("/InstrumentDir", sInstrumentDir);
+	m_settings.setValue("/AutoMonitor", bAutoMonitor);
+	m_settings.setValue("/SnapPerBeat", iSnapPerBeat);
+	m_settings.setValue("/Tempo", double(fTempo));
+	m_settings.setValue("/PasteRepeatCount", iPasteRepeatCount);
+	m_settings.setValue("/PasteRepeatPeriod", bPasteRepeatPeriod);
 	m_settings.setValue("/PluginSearch", sPluginSearch);
 	m_settings.setValue("/PluginType", iPluginType);
+	m_settings.setValue("/PluginActivate", bPluginActivate);
 	m_settings.endGroup();
 
 	// Plug-in paths.
@@ -308,6 +355,8 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.setValue("/LadspaPaths", ladspaPaths);
 	m_settings.setValue("/DssiPaths", dssiPaths);
 	m_settings.setValue("/VstPaths", vstPaths);
+	m_settings.setValue("/AudioOutputBus", bAudioOutputBus);
+	m_settings.setValue("/DummyVstScan", bDummyVstScan);
 	m_settings.endGroup();
 
 	// Instrument file list.
@@ -361,6 +410,9 @@ qtractorOptions::~qtractorOptions (void)
 	m_settings.endGroup();
 
 	m_settings.endGroup();
+
+	// Pseudo-singleton reference shut-down.
+	g_pOptions = NULL;
 }
 
 
