@@ -32,7 +32,7 @@ class QObject;
 
 
 //-------------------------------------------------------------------------
-// qtractorOptions - Prototype settings class.
+// qtractorOptions - Prototype settings class (singleton).
 //
 
 class qtractorOptions
@@ -65,6 +65,11 @@ public:
 	bool    bPeakAutoRemove;
 	bool    bKeepToolsOnTop;
 	int     iDisplayFormat;
+	int     iBaseFontSize;
+
+	// Loggin options...
+	bool    bMessagesLog;
+	QString sMessagesLogPath;
 
 	// View options...
 	bool    bMenubar;
@@ -121,17 +126,36 @@ public:
 	QString sSessionDir;
 	QString sAudioDir;
 	QString sMidiDir;
+	QString sPresetDir;
 	QString sInstrumentDir;
 
+	// Session options.
+	bool    bSessionTemplate;
+	QString sSessionTemplatePath;
+	bool	bAutoMonitor;
+	int     iSnapPerBeat;
+	float   fTempo;
+
+	// Paste-repeat convenient defaults.
+	int     iPasteRepeatCount;
+	bool    bPasteRepeatPeriod;
+
 	// Plugin search string.
 	QString sPluginSearch;
 	int     iPluginType;
+	bool	bPluginActivate;
 
 	// Plug-in paths.
 	QStringList ladspaPaths;
 	QStringList dssiPaths;
 	QStringList vstPaths;
 
+	// Plug-in instrument options.
+	bool bAudioOutputBus;
+
+	// VST dummy plugin scan option.
+	bool bDummyVstScan;
+
 	// The instrument file list.
 	QStringList instrumentFiles;
 
@@ -177,10 +201,16 @@ public:
 	void loadActionShortcuts(QObject *pObject);
 	void saveActionShortcuts(QObject *pObject);
 
+	// Singleton instance accessor.
+	static qtractorOptions *getInstance();
+
 private:
 
 	// Settings member variables.
 	QSettings m_settings;
+
+	// The singleton instance.
+	static qtractorOptions *g_pOptions;
 };
 
 
