@@ -38,6 +38,7 @@
 #include <QMessageBox>
 #include <QValidator>
 #include <QHeaderView>
+#include <QUrl>
 
 
 //----------------------------------------------------------------------------
@@ -99,6 +100,8 @@ qtractorOptionsForm::qtractorOptionsForm (
 #ifdef CONFIG_VST
 	m_ui.PluginTypeComboBox->addItem(
 		qtractorPluginType::textFromHint(qtractorPluginType::Vst));
+#else
+	m_ui.PluginExperimentalGroupBox->hide();
 #endif
 
 	// Initialize dirty control state.
@@ -210,6 +213,18 @@ qtractorOptionsForm::qtractorOptionsForm (
 	QObject::connect(m_ui.MaxRecentFilesSpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(changed()));
+	QObject::connect(m_ui.BaseFontSizeComboBox,
+		SIGNAL(editTextChanged(const QString&)),
+		SLOT(changed()));
+	QObject::connect(m_ui.SessionTemplateCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.SessionTemplatePathComboBox,
+		SIGNAL(editTextChanged(const QString&)),
+		SLOT(changed()));
+	QObject::connect(m_ui.SessionTemplatePathToolButton,
+		SIGNAL(clicked()),
+		SLOT(chooseSessionTemplatePath()));
 	QObject::connect(m_ui.AudioMeterLevelComboBox,
 		SIGNAL(activated(int)),
 		SLOT(changeAudioMeterLevel(int)));
@@ -255,6 +270,12 @@ qtractorOptionsForm::qtractorOptionsForm (
 	QObject::connect(m_ui.PluginPathDownToolButton,
 		SIGNAL(clicked()),
 		SLOT(moveDownPluginPath()));
+	QObject::connect(m_ui.AudioOutputBusCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.DummyVstScanCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(changed()));
 	QObject::connect(m_ui.MessagesFontPushButton,
 		SIGNAL(clicked()),
 		SLOT(chooseMessagesFont()));
@@ -264,6 +285,15 @@ qtractorOptionsForm::qtractorOptionsForm (
 	QObject::connect(m_ui.MessagesLimitLinesSpinBox,
 		SIGNAL(valueChanged(int)),
 		SLOT(changed()));
+	QObject::connect(m_ui.MessagesLogCheckBox,
+		SIGNAL(stateChanged(int)),
+		SLOT(changed()));
+	QObject::connect(m_ui.MessagesLogPathComboBox,
+		SIGNAL(editTextChanged(const QString&)),
+		SLOT(changed()));
+	QObject::connect(m_ui.MessagesLogPathToolButton,
+		SIGNAL(clicked()),
+		SLOT(chooseMessagesLogPath()));
 	QObject::connect(m_ui.OkPushButton,
 		SIGNAL(clicked()),
 		SLOT(accept()));
@@ -289,6 +319,8 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 	m_pOptions->loadComboBoxHistory(m_ui.MetroBarFilenameComboBox);
 	m_pOptions->loadComboBoxHistory(m_ui.MetroBeatFilenameComboBox);
 	m_pOptions->loadComboBoxHistory(m_ui.PluginPathComboBox);
+	m_pOptions->loadComboBoxHistory(m_ui.MessagesLogPathComboBox);
+	m_pOptions->loadComboBoxHistory(m_ui.SessionTemplatePathComboBox);
 
 	// Audio options.
 	int iIndex  = 0;
@@ -367,7 +399,7 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 		|| !font.fromString(m_pOptions->sMessagesFont))
 		font = QFont("Monospace", 8);
 	QPalette pal(m_ui.MessagesFontTextLabel->palette());
-	pal.setColor(m_ui.MessagesFontTextLabel->backgroundRole(), Qt::white);
+	pal.setColor(QPalette::Background, pal.base().color());
 	m_ui.MessagesFontTextLabel->setPalette(pal);
 	m_ui.MessagesFontTextLabel->setFont(font);
 	m_ui.MessagesFontTextLabel->setText(
@@ -377,6 +409,10 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 	m_ui.MessagesLimitCheckBox->setChecked(m_pOptions->bMessagesLimit);
 	m_ui.MessagesLimitLinesSpinBox->setValue(m_pOptions->iMessagesLimitLines);
 
+	// Logging options...
+	m_ui.MessagesLogCheckBox->setChecked(m_pOptions->bMessagesLog);
+	m_ui.MessagesLogPathComboBox->setEditText(m_pOptions->sMessagesLogPath);
+
 	// Other options finally.
 	m_ui.ConfirmRemoveCheckBox->setChecked(m_pOptions->bConfirmRemove);
 	m_ui.StdoutCaptureCheckBox->setChecked(m_pOptions->bStdoutCapture);
@@ -386,12 +422,24 @@ void qtractorOptionsForm::setOptions ( qtractorOptions *pOptions )
 	m_ui.TrackViewDropSpanCheckBox->setChecked(m_pOptions->bTrackViewDropSpan);
 	m_ui.MaxRecentFilesSpinBox->setValue(m_pOptions->iMaxRecentFiles);
 	m_ui.DisplayFormatComboBox->setCurrentIndex(m_pOptions->iDisplayFormat);
+	if (m_pOptions->iBaseFontSize > 0)
+		m_ui.BaseFontSizeComboBox->setEditText(QString::number(m_pOptions->iBaseFontSize));
+	else
+		m_ui.BaseFontSizeComboBox->setCurrentIndex(0);
+
+	// Session options...
+	m_ui.SessionTemplateCheckBox->setChecked(m_pOptions->bSessionTemplate);
+	m_ui.SessionTemplatePathComboBox->setEditText(m_pOptions->sSessionTemplatePath);
 
 	// Plugin path initialization...
 	m_ladspaPaths = m_pOptions->ladspaPaths;
 	m_dssiPaths   = m_pOptions->dssiPaths;
 	m_vstPaths    = m_pOptions->vstPaths;
 
+	// Plugin instruments options.
+	m_ui.AudioOutputBusCheckBox->setChecked(m_pOptions->bAudioOutputBus);
+	m_ui.DummyVstScanCheckBox->setChecked(m_pOptions->bDummyVstScan);
+
 	int iPluginType = m_pOptions->iPluginType - 1;
 	if (iPluginType < 0)
 		iPluginType = 0;
@@ -460,15 +508,25 @@ void qtractorOptionsForm::accept (void)
 		m_pOptions->bTrackViewDropSpan   = m_ui.TrackViewDropSpanCheckBox->isChecked();
 		m_pOptions->iMaxRecentFiles      = m_ui.MaxRecentFilesSpinBox->value();
 		m_pOptions->iDisplayFormat       = m_ui.DisplayFormatComboBox->currentIndex();
+		m_pOptions->iBaseFontSize        = m_ui.BaseFontSizeComboBox->currentText().toInt();
 		// Plugin paths...
 		m_pOptions->iPluginType          = m_ui.PluginTypeComboBox->currentIndex() + 1;
 		m_pOptions->ladspaPaths          = m_ladspaPaths;
 		m_pOptions->dssiPaths            = m_dssiPaths;
 		m_pOptions->vstPaths             = m_vstPaths;
+		// Plugin instruments options.
+		m_pOptions->bAudioOutputBus      = m_ui.AudioOutputBusCheckBox->isChecked();
+		m_pOptions->bDummyVstScan        = m_ui.DummyVstScanCheckBox->isChecked();
 		// Messages options...
 		m_pOptions->sMessagesFont        = m_ui.MessagesFontTextLabel->font().toString();
 		m_pOptions->bMessagesLimit       = m_ui.MessagesLimitCheckBox->isChecked();
 		m_pOptions->iMessagesLimitLines  = m_ui.MessagesLimitLinesSpinBox->value();
+		// Logging options...
+		m_pOptions->bMessagesLog         = m_ui.MessagesLogCheckBox->isChecked();
+		m_pOptions->sMessagesLogPath     = m_ui.MessagesLogPathComboBox->currentText();
+		// Session options...
+		m_pOptions->bSessionTemplate     = m_ui.SessionTemplateCheckBox->isChecked();
+		m_pOptions->sSessionTemplatePath = m_ui.SessionTemplatePathComboBox->currentText();
 		// Custom colors.
 		int iColor;
 		m_pOptions->audioMeterColors.clear();
@@ -484,6 +542,8 @@ void qtractorOptionsForm::accept (void)
 	m_pOptions->saveComboBoxHistory(m_ui.MetroBarFilenameComboBox);
 	m_pOptions->saveComboBoxHistory(m_ui.MetroBeatFilenameComboBox);
 	m_pOptions->saveComboBoxHistory(m_ui.PluginPathComboBox);
+	m_pOptions->saveComboBoxHistory(m_ui.MessagesLogPathComboBox);
+	m_pOptions->saveComboBoxHistory(m_ui.SessionTemplatePathComboBox);
 
 	// Just go with dialog acceptance
 	QDialog::accept();
@@ -529,7 +589,7 @@ void qtractorOptionsForm::changed (void)
 void qtractorOptionsForm::chooseMetroBarFilename (void)
 {
 	QString sFilename = getOpenAudioFileName(
-		tr("Metronome Bar Audio File"),
+		tr("Metronome Bar Audio File") + " - " QTRACTOR_TITLE,
 		m_ui.MetroBarFilenameComboBox->currentText());
 
 	if (sFilename.isEmpty())
@@ -543,7 +603,7 @@ void qtractorOptionsForm::chooseMetroBarFilename (void)
 void qtractorOptionsForm::chooseMetroBeatFilename (void)
 {
 	QString sFilename = getOpenAudioFileName(
-		tr("Metronome Beat Audio File"),
+		tr("Metronome Beat Audio File") + " - " QTRACTOR_TITLE,
 		m_ui.MetroBeatFilenameComboBox->currentText());
 
 	if (sFilename.isEmpty())
@@ -714,11 +774,28 @@ void qtractorOptionsForm::changePluginPath ( const QString& /*sPluginPath*/ )
 // Browse for plugin path.
 void qtractorOptionsForm::choosePluginPath (void)
 {
-	QString sPluginPath = QFileDialog::getExistingDirectory(
-		this,                                  // Parent.
-		tr("Plug-in Directory:"),              // Caption.
-		m_ui.PluginPathComboBox->currentText() // Start here.
-	);
+	QString sPluginPath;
+
+	const QString& sTitle = tr("Plug-in Directory") + " - " QTRACTOR_TITLE;
+#if QT_VERSION < 0x040400
+	// Ask for the directory...
+    sPluginPath = QFileDialog::getExistingDirectory(this,
+		sTitle, m_ui.PluginPathComboBox->currentText());
+#else
+	// Construct open-directory dialog...
+	QFileDialog fileDialog(this,
+		sTitle, m_ui.PluginPathComboBox->currentText());
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::DirectoryOnly);
+	// Stuff sidebar...
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(m_pOptions->sSessionDir));
+	fileDialog.setSidebarUrls(urls);
+	// Show dialog...
+	if (fileDialog.exec())
+		sPluginPath = fileDialog.selectedFiles().first();
+#endif
 
 	if (!sPluginPath.isEmpty()) {
 		m_ui.PluginPathComboBox->setEditText(sPluginPath);
@@ -932,6 +1009,77 @@ void qtractorOptionsForm::chooseMessagesFont (void)
 }
 
 
+// Messages log path browse slot.
+void qtractorOptionsForm::chooseMessagesLogPath (void)
+{
+	QString sFilename;
+
+	const QString  sExt("log");
+	const QString& sTitle  = tr("Messages Log") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("Log files (*.%1)").arg(sExt); 
+#if QT_VERSION < 0x040400
+	// Ask for the filename to open...
+	sFilename = QFileDialog::getOpenFileName(this,
+		sTitle, m_ui.MessagesLogPathComboBox->currentText(), sFilter);
+#else
+	// Construct open-file dialog...
+	QFileDialog fileDialog(this,
+		sTitle, m_ui.MessagesLogPathComboBox->currentText(), sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+	fileDialog.setFileMode(QFileDialog::AnyFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Show dialog...
+	if (fileDialog.exec())
+		sFilename = fileDialog.selectedFiles().first();
+#endif
+
+	if (!sFilename.isEmpty()) {
+		m_ui.MessagesLogPathComboBox->setEditText(sFilename);
+		m_ui.MessagesLogPathComboBox->setFocus();
+		changed();
+	}
+}
+
+
+// Session template path browse slot.
+void qtractorOptionsForm::chooseSessionTemplatePath (void)
+{
+	QString sFilename;
+
+	const QString  sExt("qtt");
+	const QString& sTitle  = tr("Session Template") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("Session template files (*.qtr *.qts *.%1)").arg(sExt); 
+#if QT_VERSION < 0x040400
+	// Ask for the filename to open...
+	sFilename = QFileDialog::getOpenFileName(this,
+		sTitle, m_ui.SessionTemplatePathComboBox->currentText(), sFilter);
+#else
+	// Construct open-files dialog...
+	QFileDialog fileDialog(this,
+		sTitle, m_ui.SessionTemplatePathComboBox->currentText(), sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(m_pOptions->sSessionDir));
+	fileDialog.setSidebarUrls(urls);
+	// Show dialog...
+	if (fileDialog.exec())
+		sFilename = fileDialog.selectedFiles().first();
+#endif
+
+	if (sFilename.isEmpty())
+		return;
+
+	m_ui.SessionTemplatePathComboBox->setEditText(sFilename);
+	m_ui.SessionTemplatePathComboBox->setFocus();
+	changed();
+}
+
+
 // Stabilize current form state.
 void qtractorOptionsForm::stabilizeForm (void)
 {
@@ -989,6 +1137,22 @@ void qtractorOptionsForm::stabilizeForm (void)
 	m_ui.MetroBeatDurationSpinBox->setEnabled(bMidiMetronome);
 	m_ui.MidiMetroBusCheckBox->setEnabled(bMidiMetronome);
 
+	bool bMessagesLog = m_ui.MessagesLogCheckBox->isChecked();
+	m_ui.MessagesLogPathComboBox->setEnabled(bMessagesLog);
+	m_ui.MessagesLogPathToolButton->setEnabled(bMessagesLog);
+	if (bMessagesLog && bValid) {
+		const QString& sPath = m_ui.MessagesLogPathComboBox->currentText();
+		bValid = !sPath.isEmpty();
+	}
+
+	bool bSessionTemplate = m_ui.SessionTemplateCheckBox->isChecked();
+	m_ui.SessionTemplatePathComboBox->setEnabled(bSessionTemplate);
+	m_ui.SessionTemplatePathToolButton->setEnabled(bSessionTemplate);
+	if (bSessionTemplate && bValid) {
+		const QString& sPath = m_ui.SessionTemplatePathComboBox->currentText();
+		bValid = !sPath.isEmpty() && QFileInfo(sPath).exists();
+	}
+
 	const QString& sPluginPath = m_ui.PluginPathComboBox->currentText();
 	m_ui.PluginPathAddToolButton->setEnabled(
 		!sPluginPath.isEmpty() && QDir(sPluginPath).exists()
@@ -1003,9 +1167,31 @@ void qtractorOptionsForm::stabilizeForm (void)
 QString qtractorOptionsForm::getOpenAudioFileName (
 	const QString& sTitle, const QString& sFilename )
 {
+	QString sAudioFile;
+
+#if QT_VERSION < 0x040400
 	// Ask for the filename to open...
-	return QFileDialog::getOpenFileName(
-		this, sTitle, sFilename, qtractorAudioFileFactory::filters());
+	sAudioFile = QFileDialog::getOpenFileName(this,
+		sTitle, sFilename, qtractorAudioFileFactory::filters());
+#else
+	// Construct open-file dialog...
+	QFileDialog fileDialog(this,
+		sTitle, sFilename, qtractorAudioFileFactory::filters());
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFile);
+	fileDialog.setDefaultSuffix(qtractorAudioFileFactory::defaultExt());
+	// Stuff sidebar...
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(m_pOptions->sSessionDir));
+	urls.append(QUrl::fromLocalFile(m_pOptions->sAudioDir));
+	fileDialog.setSidebarUrls(urls);
+	// Show dialog...
+	if (fileDialog.exec())
+		sAudioFile = fileDialog.selectedFiles().first();
+#endif
+
+	return sAudioFile;
 }
 
 
