@@ -70,6 +70,8 @@ protected slots:
 	void moveUpPluginPath();
 	void moveDownPluginPath();
 	void chooseMessagesFont();
+	void chooseMessagesLogPath();
+	void chooseSessionTemplatePath();
 	void stabilizeForm();
 
 protected:
@@ -106,4 +108,3 @@ private:
 
 
 // end of qtractorOptionsForm.h
-
