@@ -24,7 +24,9 @@
 #include "qtractorPluginListView.h"
 #include "qtractorPluginForm.h"
 
-#include "qtractorMainForm.h"
+#include "qtractorAudioEngine.h"
+#include "qtractorMidiBuffer.h"
+
 #include "qtractorOptions.h"
 
 #include "qtractorSessionDocument.h"
@@ -39,6 +41,9 @@
 #include "qtractorVstPlugin.h"
 #endif
 
+#include <QDomDocument>
+#include <QTextStream>
+#include <QFileInfo>
 #include <QDir>
 
 typedef void (*qtractorPluginFile_Function)(void);
@@ -49,17 +54,26 @@ typedef void (*qtractorPluginFile_Function)(void);
 #define PATH_SEP ":"
 #endif
 
+#if defined(__x86_64__)
+#define PATH_LIB "/lib64"
+#else
+#define PATH_LIB "/lib"
+#endif
+
+#define PATH_PRE1 "/usr/local" PATH_LIB
+#define PATH_PRE2 "/usr" PATH_LIB
+
 // Default plugin paths...
 #ifdef CONFIG_LADSPA
-#define LADSPA_PATH "/usr/local/lib/ladspa" PATH_SEP "/usr/lib/ladspa"
+#define LADSPA_PATH PATH_PRE1 "/ladspa" PATH_SEP PATH_PRE2 "/ladspa"
 #endif
 
 #ifdef CONFIG_DSSI
-#define DSSI_PATH "/usr/local/lib/dssi" PATH_SEP "/usr/lib/dssi"
+#define DSSI_PATH PATH_PRE1 "/dssi" PATH_SEP PATH_PRE2 "/dssi"
 #endif
 
 #ifdef CONFIG_VST
-#define VST_PATH "/usr/local/lib/vst" PATH_SEP "/usr/lib/vst"
+#define VST_PATH PATH_PRE1 "/vst" PATH_SEP PATH_PRE2 "/vst"
 #endif
 
 
@@ -152,7 +166,7 @@ bool qtractorPluginFile::open (void)
 {
 	close();
 
-	// ATTN: Not really needed has it will be
+	// ATTN: Not really needed, as it would be
 	// loaded automagically on resolve()...
 	if (!QLibrary::load())
 		return false;
@@ -240,8 +254,13 @@ bool qtractorPluginFile::getTypes ( qtractorPluginTypeList& types,
 	// Try VST plugin types...
 	if (typeHint == qtractorPluginType::Any ||
 		typeHint == qtractorPluginType::Vst) {
-		if ((pType
-			= qtractorVstPluginType::createType(this)) != NULL) {
+		// Need to look at the options...
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
+		if (pOptions && pOptions->bDummyVstScan)
+			pType = qtractorDummyPluginType::createType(this);
+		else
+			pType = qtractorVstPluginType::createType(this);
+		if (pType) {
 			if (pType->open()) {
 				types.append(pType);
 				pType->close();
@@ -259,7 +278,7 @@ bool qtractorPluginFile::getTypes ( qtractorPluginTypeList& types,
 
 
 // Plugin factory method (static).
-qtractorPlugin *qtractorPluginFile::createPlugin(
+qtractorPlugin *qtractorPluginFile::createPlugin (
 	qtractorPluginList *pList,
 	const QString& sFilename, unsigned long iIndex,
 	qtractorPluginType::Hint typeHint )
@@ -335,7 +354,7 @@ unsigned short qtractorPluginType::instances (
 {
 	unsigned short iInstances = 0;
 	if (iChannels > 0 && m_iAudioOuts > 0
-		&& (bMidi || m_iMidiIns < 1 || m_iAudioIns > 0)) {
+		&& ((bMidi && m_iMidiIns > 0) || !bMidi /* || m_iAudioIns > 0 */)) {
 		if (iChannels >= m_iAudioIns)
 			iInstances = (m_iAudioOuts >= iChannels ? 1 : iChannels);
 		else
@@ -390,6 +409,54 @@ QString qtractorPluginType::textFromHint (
 }
 
 
+//----------------------------------------------------------------------------
+// qtractorDummyPluginType -- Dummy plugin type instance.
+//
+
+// Constructor.
+qtractorDummyPluginType::qtractorDummyPluginType (
+	qtractorPluginFile *pFile, unsigned long iIndex, Hint typeHint)
+	: qtractorPluginType(pFile, iIndex, typeHint)
+{
+}
+
+// Must be overriden methods.
+bool qtractorDummyPluginType::open (void)
+{
+	m_sName  = QFileInfo(file()->filename()).baseName();
+	m_sLabel = m_sName.simplified().replace(QRegExp("[\\s|\\.|\\-]+"), "_");
+
+	m_iUniqueID = 0;
+	for (int i = 0; i < m_sLabel.length(); ++i)
+		m_iUniqueID += int(m_sLabel[i].toAscii());
+
+	// Fake the rest...
+	m_iAudioIns  = 2;
+	m_iAudioOuts = 2;
+	m_iMidiIns   = 1;
+
+	return true;
+}
+
+
+void qtractorDummyPluginType::close (void)
+{
+}
+
+
+// Factory method (static)
+qtractorDummyPluginType *qtractorDummyPluginType::createType (
+	qtractorPluginFile *pFile, unsigned long iIndex, Hint typeHint )
+{
+	// Sanity check...
+	if (pFile == NULL)
+		return NULL;
+
+	// Yep, most probably its a dummy plugin effect...
+	return new qtractorDummyPluginType(pFile, iIndex, typeHint);
+}
+
+
 //----------------------------------------------------------------------------
 // qtractorPlugin -- Plugin instance.
 //
@@ -398,7 +465,6 @@ QString qtractorPluginType::textFromHint (
 qtractorPlugin::qtractorPlugin (
 	qtractorPluginList *pList, qtractorPluginType *pType )
 	: m_pList(pList), m_pType(pType), m_iInstances(0),
-		m_iAudioInsCap(0), m_iAudioOutsCap(0),
 		m_bActivated(false), m_pForm(NULL)
 {
 #if 0
@@ -455,16 +521,9 @@ void qtractorPlugin::setInstances ( unsigned short iInstances )
 	m_iInstances = iInstances;
 
 	// Some sanity required here...
-	if (m_iInstances > 0) {
-		m_iAudioInsCap  = m_pType->audioIns();
-		m_iAudioOutsCap = m_pType->audioOuts();
-		unsigned short iAudioOutsCap = (channels() / m_iInstances);
-		if (m_iAudioOutsCap > iAudioOutsCap)
-			m_iAudioOutsCap = iAudioOutsCap;
-	} else {
-		m_iAudioInsCap  = 0;
-		m_iAudioOutsCap = 0;
+	if (m_iInstances < 1) {
 		// We're sorry but dialogs must also go now...
+		closeEditor();
 		if (m_pForm) {
 			m_pForm->close();
 			delete m_pForm;
@@ -497,23 +556,23 @@ void qtractorPlugin::setActivated ( bool bActivated )
 
 
 // Plugin state serialization methods.
-void qtractorPlugin::setValues ( const QStringList& vlist )
+void qtractorPlugin::setValueList ( const QStringList& vlist )
 {
 	// Split it up...
+	m_values.clear();
 	QStringListIterator val(vlist);
-	QListIterator<qtractorPluginParam *> iter(m_params);
-	while (iter.hasNext() && val.hasNext())
-		iter.next()->setValue(val.next().toFloat());
-
+	QListIterator<qtractorPluginParam *> param(m_params);
+	while (val.hasNext() && param.hasNext())
+		m_values[param.next()->index()] = val.next().toFloat();
 }
 
-QStringList qtractorPlugin::values (void) const
+QStringList qtractorPlugin::valueList (void) const
 {
 	// Join it up...
 	QStringList vlist;
-	QListIterator<qtractorPluginParam *> iter(m_params);
-	while (iter.hasNext())
-		vlist.append(QString::number(iter.next()->value()));
+	QListIterator<qtractorPluginParam *> param(m_params);
+	while (param.hasNext())
+		vlist.append(QString::number(param.next()->value()));
 
 	return vlist;
 }
@@ -551,7 +610,7 @@ void qtractorPlugin::clearItems (void)
 
 
 // Special plugin form accessors.
-bool qtractorPlugin::isVisible (void) const
+bool qtractorPlugin::isFormVisible (void) const
 {
 	return (m_pForm ? m_pForm->isVisible() : false);
 }
@@ -561,10 +620,6 @@ qtractorPluginForm *qtractorPlugin::form (void)
 	// Take the change and create the form if it doesn't current exist.
 	if (m_pForm == NULL) {
 		// Build up the plugin form...
-		qtractorOptions  *pOptions  = NULL;
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm)
-			pOptions = pMainForm->options();
 		// What style do we create tool childs?
 		QWidget *pParent = NULL;
 		Qt::WindowFlags wflags = Qt::Window
@@ -574,8 +629,9 @@ qtractorPluginForm *qtractorPlugin::form (void)
 			| Qt::WindowTitleHint
 			| Qt::WindowSystemMenuHint
 			| Qt::WindowMinMaxButtonsHint;
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
 		if (pOptions && pOptions->bKeepToolsOnTop) {
-			pParent = pMainForm;
+		//	pParent = qtractorMainForm::getInstance();
 			wflags |= Qt::Tool;
 		}
 		// Do it...
@@ -606,20 +662,110 @@ const QString& qtractorPlugin::preset (void)
 }
 
 
-// Plugin preset group - common identification prefix.
+// Plugin preset group - common identification group/prefix.
 QString qtractorPlugin::presetGroup (void) const
 {
-	QString sGroup;
+	return "/Plugin/" + presetPrefix();
+}
 
-	// Normalize plugin identification prefix...
-	if (m_pType) {
-		sGroup  = "/Plugin/";
-		sGroup += m_pType->label();
-		sGroup += '_';
-		sGroup += QString::number(m_pType->uniqueID());
+// Normalize plugin identification prefix...
+QString qtractorPlugin::presetPrefix (void) const
+{
+	return m_pType->label() + '_' + QString::number(m_pType->uniqueID());
+}
+
+
+// Load plugin preset from xml file.
+bool qtractorPlugin::loadPreset ( const QString& sFilename )
+{
+	// Open file...
+	QFile file(sFilename);
+	if (!file.open(QIODevice::ReadOnly))
+		return false;
+	// Parse it a-la-DOM :-)
+	QDomDocument doc("qtractorPlugin");
+	if (!doc.setContent(&file)) {
+		file.close();
+		return false;
+	}
+	file.close();
+
+	// Get root element.
+	QDomElement ePreset = doc.documentElement();
+	if (ePreset.tagName() != "preset")
+		return false;
+	// Check if it's on the correct plugin preset...
+	if (ePreset.attribute("type") != presetPrefix())
+		return false;
+
+	// Reset any old configs.
+	clearConfigs();
+	clearValues();
+
+	// Now parse for children...
+	for (QDomNode nChild = ePreset.firstChild();
+			!nChild.isNull(); nChild = nChild.nextSibling()) {
+
+		// Convert node to element, if any.
+		QDomElement eChild = nChild.toElement();
+		if (eChild.isNull())
+			continue;
+
+		// Check for preset item...
+		if (eChild.tagName() == "configs" || eChild.tagName() == "configure") {
+			// Parse for config entries...
+			qtractorPlugin::loadConfigs(&eChild, m_configs);
+		}
+		else
+		if (eChild.tagName() == "params") {
+			// Parse for param entries...
+			qtractorPlugin::loadValues(&eChild, m_values);
+		}
 	}
 
-	return sGroup;
+	// Make it real.
+	realizeConfigs();
+	realizeValues();
+
+	releaseConfigs();
+
+	return true;
+}
+
+
+// Save plugin preset to xml file.
+bool qtractorPlugin::savePreset ( const QString& sFilename )
+{
+	QFileInfo fi(sFilename);
+
+	QDomDocument doc("qtractorPlugin");
+
+	QDomElement ePreset = doc.createElement("preset");
+	ePreset.setAttribute("type", presetPrefix());
+	ePreset.setAttribute("name", fi.baseName());
+	ePreset.setAttribute("version", QTRACTOR_VERSION);
+
+	// Save plugin configs...
+	QDomElement eConfigs = doc.createElement("configs");
+	saveConfigs(&doc, &eConfigs);
+	ePreset.appendChild(eConfigs);
+
+	// Save plugin params...
+	QDomElement eParams = doc.createElement("params");
+	saveValues(&doc, &eParams);
+	ePreset.appendChild(eParams);
+
+	doc.appendChild(ePreset);
+
+	// Finally, we're ready to save to external file.
+	QFile file(sFilename);
+	if (!file.open(QIODevice::WriteOnly | QIODevice::Truncate))
+		return false;
+	QTextStream ts(&file);
+	ts << doc.toString() << endl;
+	file.close();
+
+	return true;
 }
 
 
@@ -637,28 +783,137 @@ qtractorPluginParam *qtractorPlugin::findParam ( unsigned long iIndex ) const
 }
 
 
+// Plugin configure realization.
+void qtractorPlugin::realizeConfigs (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPlugin[%p]::realizeConfigs()", this);
+#endif
+
+	// Set configuration (CLOBs)...
+	Configs::ConstIterator config = m_configs.constBegin();
+	for (; config != m_configs.constEnd(); ++config)
+		configure(config.key(), config.value());
+
+	// Set proper bank/program selection...
+	qtractorMidiManager *pMidiManager = m_pList->midiManager();
+	if (pMidiManager)
+		selectProgram(pMidiManager->currentBank(), pMidiManager->currentProg());
+}
+
+
+// Plugin parameter realization.
+void qtractorPlugin::realizeValues (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPlugin[%p]::realizeValues()", this);
+#endif
+
+	// (Re)set parameter values (initial)...
+	Values::ConstIterator param = m_values.constBegin();
+	for (; param != m_values.constEnd(); ++param) {
+		qtractorPluginParam *pParam = findParam(param.key());
+		if (pParam)
+			pParam->setValue(param.value());
+	}
+}
+
+
+// Load plugin configuration stuff (CLOB).
+void qtractorPlugin::loadConfigs ( QDomElement *pElement, Configs& configs )
+{
+	for (QDomNode nConfig = pElement->firstChild();
+			!nConfig.isNull();
+				nConfig = nConfig.nextSibling()) {
+		// Convert config node to element...
+		QDomElement eConfig = nConfig.toElement();
+		if (eConfig.isNull())
+			continue;
+		if (eConfig.tagName() == "config")
+			configs[eConfig.attribute("key")] = eConfig.text();
+	}
+}
+
+
+// Load plugin parameter values.
+void qtractorPlugin::loadValues ( QDomElement *pElement, Values& values )
+{
+	for (QDomNode nParam = pElement->firstChild();
+			!nParam.isNull(); nParam = nParam.nextSibling()) {
+		// Convert node to element, if any.
+		QDomElement eParam = nParam.toElement();
+		if (eParam.isNull())
+			continue;
+		// Check for config item...
+		if (eParam.tagName() == "param")
+			values[eParam.attribute("index").toULong()] = eParam.text().toFloat();
+	}
+}
+
+
+// Save plugin configuration stuff (CLOB)...
+void qtractorPlugin::saveConfigs (
+	QDomDocument *pDocument, QDomElement *pElement )
+{
+	freezeConfigs();
+
+	// Save plugin configs...
+	Configs::ConstIterator iter = m_configs.constBegin();
+	for (; iter != m_configs.constEnd(); ++iter) {
+		QDomElement eConfig = pDocument->createElement("config");
+		eConfig.setAttribute("key", iter.key());
+		eConfig.appendChild(
+			pDocument->createTextNode(iter.value()));
+		pElement->appendChild(eConfig);
+	}
+
+	releaseConfigs();
+}
+
+
+// Save plugin parameter values.
+void qtractorPlugin::saveValues (
+	QDomDocument *pDocument, QDomElement *pElement )
+{
+	QListIterator<qtractorPluginParam *> param(m_params);
+	while (param.hasNext()) {
+		qtractorPluginParam *pParam = param.next();
+		QDomElement eParam = pDocument->createElement("param");
+		eParam.setAttribute("name", pParam->name());
+		eParam.setAttribute("index", QString::number(pParam->index()));
+		eParam.appendChild(
+			pDocument->createTextNode(QString::number(pParam->value())));
+		pElement->appendChild(eParam);
+	}
+}
+
+	
 //----------------------------------------------------------------------------
 // qtractorPluginList -- Plugin chain list instance.
 //
 
 // Constructor.
 qtractorPluginList::qtractorPluginList ( unsigned short iChannels,
-	unsigned int iBufferSize, unsigned int iSampleRate, bool bMidi )
+	unsigned int iBufferSize, unsigned int iSampleRate, unsigned int iFlags )
 	: m_iChannels(0), m_iBufferSize(0), m_iSampleRate(0),
-		m_bMidi(false),	m_iActivated(0)
+		m_iFlags(0), m_iActivated(0), m_pMidiManager(NULL)
 		
 {
 	m_pppBuffers[0] = NULL;
 	m_pppBuffers[1] = NULL;
 
-	setBuffer(iChannels, iBufferSize, iSampleRate, bMidi);
+	setBuffer(iChannels, iBufferSize, iSampleRate, iFlags);
 }
 
 // Destructor.
 qtractorPluginList::~qtractorPluginList (void)
 {
 	// Reset allocated channel buffers.
-	setBuffer(0, 0, 0, false);
+	setBuffer(0, 0, 0, 0);
+
+	// Now that's one manager less.
+	if (m_pMidiManager)
+		qtractorMidiManager::deleteMidiManager(m_pMidiManager);
 
 	// Clear out all dependables...
 	m_views.clear();
@@ -672,15 +927,18 @@ void qtractorPluginList::setName ( const QString& sName )
 
 	for (qtractorPlugin *pPlugin = first();
 			pPlugin; pPlugin = pPlugin->next()) {
-		if (pPlugin->isVisible())
-			pPlugin->form()->updateCaption();
+		if (pPlugin->isFormVisible())
+			(pPlugin->form())->updateCaption();
 	}
+
+	if (m_pMidiManager)
+		m_pMidiManager->resetAudioOutputBus();
 }
 
 
 // Main-parameters accessor.
 void qtractorPluginList::setBuffer ( unsigned short iChannels,
-	unsigned int iBufferSize, unsigned int iSampleRate, bool bMidi )
+	unsigned int iBufferSize, unsigned int iSampleRate, unsigned int iFlags )
 {
 	unsigned short i;
 
@@ -692,6 +950,9 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 		m_pppBuffers[1] = NULL;
 	}
 
+	// Set proper flags at once.
+	m_iFlags = iFlags;
+
 	// Some sanity is in order, at least for now...
 	if (iChannels == 0 || iBufferSize == 0 || iSampleRate == 0)
 		return;
@@ -700,7 +961,6 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 	m_iChannels   = iChannels;
 	m_iBufferSize = iBufferSize;
 	m_iSampleRate = iSampleRate;
-	m_bMidi       = bMidi;
 
 	// Allocate new interim buffer...
 	if (m_iChannels > 0 && m_iBufferSize > 0) {
@@ -715,12 +975,20 @@ void qtractorPluginList::setBuffer ( unsigned short iChannels,
 	for (qtractorPlugin *pPlugin = first();
 			pPlugin; pPlugin = pPlugin->next())
 		pPlugin->setChannels(m_iChannels);
+
+	// FIXME: This should be better managed...
+	if (m_pMidiManager)
+		m_pMidiManager->updateInstruments();
 }
 
 
 // Reset and (re)activate all plugin chain.
 void qtractorPluginList::resetBuffer (void)
 {
+	// Save and reset activation count...
+	int iActivated = m_iActivated;
+	m_iActivated = 0;
+
 	// Temporarily deactivate all activated plugins...
 	for (qtractorPlugin *pPlugin = first();
 			pPlugin; pPlugin = pPlugin->next()) {
@@ -740,6 +1008,9 @@ void qtractorPluginList::resetBuffer (void)
 		if (pPlugin->isActivated())
 			pPlugin->activate();
 	}
+
+	// Restore activation count.
+	m_iActivated = iActivated;
 }
 
 
@@ -769,19 +1040,20 @@ void qtractorPluginList::addPlugin ( qtractorPlugin *pPlugin )
 
 
 // Insert-guarded plugin method.
-void qtractorPluginList::insertPlugin (	qtractorPlugin *pPlugin,
+void qtractorPluginList::insertPlugin ( qtractorPlugin *pPlugin,
 	qtractorPlugin *pNextPlugin )
 {
 	// We'll get prepared before plugging it in...
 	pPlugin->setChannels(m_iChannels);
 
+	addPluginRef(pPlugin);
 	if (pNextPlugin)
 		insertBefore(pPlugin, pNextPlugin);
 	else
 		append(pPlugin);
 
-	//if (pPlugin->isActivated())
-	//	updateActivated(true);
+	if (pPlugin->isActivated())
+		updateActivated(true);
 
 	// Now update each observer list-view...
 	QListIterator<qtractorPluginListView *> iter(m_views);
@@ -811,6 +1083,9 @@ void qtractorPluginList::movePlugin (
 
 	// Remove and insert back again...
 	pPluginList->unlink(pPlugin);
+	pPluginList->removePluginRef(pPlugin);
+
+	addPluginRef(pPlugin);
 	if (pNextPlugin) {
 		insertBefore(pPlugin, pNextPlugin);
 	} else {
@@ -852,6 +1127,7 @@ void qtractorPluginList::removePlugin ( qtractorPlugin *pPlugin )
 {
 	// Just unlink the plugin from the list...
 	unlink(pPlugin);
+	removePluginRef(pPlugin);
 
 	if (pPlugin->isActivated())
 		updateActivated(false);
@@ -869,18 +1145,59 @@ qtractorPlugin *qtractorPluginList::copyPlugin ( qtractorPlugin *pPlugin )
 		return NULL;
 
 	// Clone the plugin instance...
+	pPlugin->freezeConfigs();
+
+	// MIDI bank program whether necessary...
+	int iBank = 0;
+	int iProg = 0;
+	if (m_pMidiManager && m_pMidiManager->currentBank() >= 0)
+		iBank = m_pMidiManager->currentBank();
+	if (m_pMidiManager && m_pMidiManager->currentProg() >= 0)
+		iProg = m_pMidiManager->currentProg();
+
 	qtractorPlugin *pNewPlugin = qtractorPluginFile::createPlugin(this,
 		(pType->file())->filename(), pType->index(), pType->typeHint());
 	if (pNewPlugin) {
 		pNewPlugin->setPreset(pPlugin->preset());
+		pNewPlugin->setConfigs(pPlugin->configs());
 		pNewPlugin->setValues(pPlugin->values());
 		pNewPlugin->setActivated(pPlugin->isActivated());
 	}
 
+	pPlugin->releaseConfigs();
+
 	return pNewPlugin;
 }
 
 
+// Plugin management helpers.
+void qtractorPluginList::addPluginRef ( qtractorPlugin *pPlugin )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginList[%p]::addPluginRef(%p)", this, pPlugin);
+#endif
+	if ((m_iFlags & Midi) && m_pMidiManager == NULL && count() == 0)
+		m_pMidiManager = qtractorMidiManager::createMidiManager(this);
+
+	if (m_pMidiManager)
+		m_pMidiManager->addPluginRef(pPlugin);
+}
+
+void qtractorPluginList::removePluginRef ( qtractorPlugin *pPlugin )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginList[%p]::removePluginRef(%p)", this, pPlugin);
+#endif
+	if (m_pMidiManager) {
+		m_pMidiManager->removePluginRef(pPlugin);
+		if (count() == 0) {
+			qtractorMidiManager::deleteMidiManager(m_pMidiManager);
+			m_pMidiManager = NULL;
+		}
+	}
+}
+
+
 // List of views management.
 void qtractorPluginList::addView ( qtractorPluginListView *pView )
 {
@@ -939,20 +1256,32 @@ void qtractorPluginList::process ( float **ppBuffer, unsigned int nframes )
 bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 	QDomElement *pElement )
 {
+	int iBank = 0;
+	int iProg = 0;
+
 	// Load plugin-list children...
 	for (QDomNode nPlugin = pElement->firstChild();
 			!nPlugin.isNull();
 				nPlugin = nPlugin.nextSibling()) {
+
 		// Convert clip node to element...
 		QDomElement ePlugin = nPlugin.toElement();
 		if (ePlugin.isNull())
 			continue;
+		if (ePlugin.tagName() == "bank")
+			iBank = ePlugin.text().toInt();
+		else
+		if (ePlugin.tagName() == "program")
+			iProg = ePlugin.text().toInt();
+		else
 		if (ePlugin.tagName() == "plugin") {
 			QString sFilename;
 			unsigned long iIndex = 0;
 			QString sPreset;
 			QStringList vlist;
 			bool bActivated = false;
+			qtractorPlugin::Configs configs;
+			qtractorPlugin::Values values;
 			qtractorPluginType::Hint typeHint
 				= qtractorPluginType::hintFromText(
 					ePlugin.attribute("type"));
@@ -977,6 +1306,16 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 				else
 				if (eParam.tagName() == "activated")
 					bActivated = pDocument->boolFromText(eParam.text());
+				else
+				if (eParam.tagName() == "configs" || eParam.tagName() == "configure") {
+					// Load plugin configuration stuff (CLOB)...
+					qtractorPlugin::loadConfigs(&eParam, configs);
+				}
+				else
+				if (eParam.tagName() == "params") {
+					// Load plugin parameter values...
+					qtractorPlugin::loadValues(&eParam, values);
+				}
 			}
 			if (sFilename.isEmpty())
 				continue;
@@ -985,11 +1324,41 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 					sFilename, iIndex, typeHint);
 			if (pPlugin) {
 				pPlugin->setPreset(sPreset);
-				pPlugin->setValues(vlist);
+				pPlugin->setConfigs(configs);
+				if (!vlist.isEmpty())
+					pPlugin->setValueList(vlist);
+				if (!values.isEmpty())
+					pPlugin->setValues(values);
 				pPlugin->setActivated(bActivated);
+				addPluginRef(pPlugin);
 				append(pPlugin);
 			}
 		}
+		else
+		// Load audio output bus flag...
+		if (ePlugin.tagName() == "audio-output-bus") {
+			if (m_pMidiManager) {
+				m_pMidiManager->setAudioOutputBus(
+					pDocument->boolFromText(ePlugin.text()));
+			}
+		}
+		else
+		// Load audio output connections...
+		if (ePlugin.tagName() == "audio-outputs") {
+			if (m_pMidiManager && m_pMidiManager->isAudioOutputBus()) {
+				qtractorAudioBus *pAudioBus = m_pMidiManager->audioOutputBus();
+				if (pAudioBus) {
+					pAudioBus->loadConnects(
+						pAudioBus->outputs(), pDocument, &ePlugin);
+				}
+			}
+		}
+	}
+
+	// Set current bank/program properly...
+	if (m_pMidiManager) {
+		m_pMidiManager->setCurrentBank(iBank);
+		m_pMidiManager->setCurrentProg(iProg);
 	}
 
 	return true;
@@ -999,9 +1368,21 @@ bool qtractorPluginList::loadElement ( qtractorSessionDocument *pDocument,
 bool qtractorPluginList::saveElement ( qtractorSessionDocument *pDocument,
 	QDomElement *pElement )
 {
+	// Save current MIDI bank/program setting...
+	if (m_pMidiManager && m_pMidiManager->currentBank() >= 0)
+		pDocument->saveTextElement("bank",
+			QString::number(m_pMidiManager->currentBank()), pElement);
+	if (m_pMidiManager && m_pMidiManager->currentProg() >= 0)
+		pDocument->saveTextElement("program",
+			QString::number(m_pMidiManager->currentProg()), pElement);
+
 	// Save plugins...
 	for (qtractorPlugin *pPlugin = first();
 			pPlugin; pPlugin = pPlugin->next()) {
+
+		// Do freeze plugin state...
+		pPlugin->freezeConfigs();
+
 		// Create the new plugin element...
 		QDomElement ePlugin = pDocument->document()->createElement("plugin");
 		ePlugin.setAttribute("type", qtractorPluginType::textFromHint(
@@ -1012,12 +1393,40 @@ bool qtractorPluginList::saveElement ( qtractorSessionDocument *pDocument,
 			QString::number((pPlugin->type())->index()), &ePlugin);
 		pDocument->saveTextElement("preset",
 			pPlugin->preset(), &ePlugin);
-		pDocument->saveTextElement("values",
-			pPlugin->values().join(","), &ePlugin);
+	//	pDocument->saveTextElement("values",
+	//		pPlugin->valueList().join(","), &ePlugin);
 		pDocument->saveTextElement("activated",
 			pDocument->textFromBool(pPlugin->isActivated()), &ePlugin);
+		// Plugin configuration stuff (CLOB)...
+		QDomElement eConfigs = pDocument->document()->createElement("configs");
+		pPlugin->saveConfigs(pDocument->document(), &eConfigs);
+		ePlugin.appendChild(eConfigs);
+		// Plugin parameter values...
+		QDomElement eParams = pDocument->document()->createElement("params");
+		pPlugin->saveValues(pDocument->document(), &eParams);
+		ePlugin.appendChild(eParams);
 		// Add this plugin...
 		pElement->appendChild(ePlugin);
+
+		// May release plugin state...
+		pPlugin->releaseConfigs();
+	}
+
+	// Save audio output-bus connects...
+	if (m_pMidiManager) {
+		pDocument->saveTextElement("audio-output-bus",
+			pDocument->textFromBool(m_pMidiManager->isAudioOutputBus()), pElement);
+		if (m_pMidiManager->isAudioOutputBus()) {
+			qtractorAudioBus *pAudioBus = m_pMidiManager->audioOutputBus();
+			if (pAudioBus) {
+				QDomElement eOutputs
+					= pDocument->document()->createElement("audio-outputs");
+				qtractorBus::ConnectList outputs;
+				pAudioBus->updateConnects(qtractorBus::Output, outputs);
+				pAudioBus->saveConnects(outputs, pDocument, &eOutputs);
+				pElement->appendChild(eOutputs);
+			}
+		}
 	}
 
 	return true;
@@ -1047,16 +1456,19 @@ void qtractorPluginParam::setDefaultValue ( float fDefaultValue )
 // Current port value.
 void qtractorPluginParam::setValue ( float fValue )
 {
-	if (fValue == m_fValue)
-		return;
-
 	if (isBoundedAbove() && fValue > m_fMaxValue)
 		fValue = m_fMaxValue;
 	else
 	if (isBoundedBelow() && fValue < m_fMinValue)
 		fValue = m_fMinValue;
 
+	if (fValue == m_fValue)
+		return;
+
 	m_fValue = fValue;
+
+	// Update specifics.
+	m_pPlugin->updateParam(this, m_fValue);
 }
 
 
