@@ -28,6 +28,8 @@
 #include <QLibrary>
 #include <QSize>
 
+#include <QHash>
+
 
 // Forward declarations.
 class qtractorPluginFile;
@@ -39,7 +41,11 @@ class qtractorPlugin;
 class qtractorPluginListView;
 class qtractorPluginListItem;
 
+class qtractorMidiManager;
+
 class qtractorSessionDocument;
+
+class QDomDocument;
 class QDomElement;
 
 
@@ -58,7 +64,7 @@ public:
 	qtractorPluginType(qtractorPluginFile *pFile, unsigned long iIndex,
 		Hint typeHint) : m_iUniqueID(0), m_iControlIns(0), m_iControlOuts(0),
 			m_iAudioIns(0), m_iAudioOuts(0), m_iMidiIns(0), m_iMidiOuts(0),
-			m_bRealtime(false), m_bEditor(false),
+			m_bRealtime(false), m_bConfigure(false), m_bEditor(false),
 			m_pFile(pFile), m_iIndex(iIndex), m_typeHint(typeHint) {}
 
 	// Destructor (virtual)
@@ -88,6 +94,7 @@ public:
 
 	// Attribute accessors.
 	bool           isRealtime()  const { return m_bRealtime;    }
+	bool           isConfigure() const { return m_bConfigure;   }
 	bool           isEditor()    const { return m_bEditor;      }
 	bool           isMidi()      const { return m_iMidiIns > 0; }
 
@@ -119,6 +126,7 @@ protected:
 
 	// Cached flags.
 	bool m_bRealtime;
+	bool m_bConfigure;
 	bool m_bEditor;
 
 private:
@@ -130,6 +138,28 @@ private:
 };
 
 
+//----------------------------------------------------------------------------
+// qtractorDummyPluginType -- Dummy plugin type instance.
+//
+
+class qtractorDummyPluginType : public qtractorPluginType
+{
+public:
+
+	// Constructor.
+	qtractorDummyPluginType(
+		qtractorPluginFile *pFile, unsigned long iIndex, Hint typeHint);
+
+	// Must be overriden methods.
+	bool open();
+	void close();
+
+	// Factory method (static)
+	static qtractorDummyPluginType *createType(
+		qtractorPluginFile *pFile, unsigned long iIndex = 0, Hint typeHint = Vst);
+};
+
+
 //----------------------------------------------------------------------------
 // qtractorPluginTypeList -- Plugin type list instance.
 //
@@ -274,18 +304,26 @@ public:
 	
 	// Activation methods.
 	void setActivated(bool bActivated);
-	bool isActivated() const { return m_bActivated; }
+	bool isActivated() const
+		{ return m_bActivated; }
 
 	// Parameter list accessor.
 	void addParam(qtractorPluginParam *pParam)
 		{ m_params.append(pParam); }
 
 	// An accessible list of parameters.
-	const QList<qtractorPluginParam *>& params() const { return m_params; }
+	const QList<qtractorPluginParam *>& params() const
+		{ return m_params; }
+
+	// Instance capped number of audio ports.
+	unsigned short audioIns() const
+		{ return m_pType->audioIns(); }
+	unsigned short audioOuts() const
+		{ return m_pType->audioOuts(); }
 
 	// Plugin state serialization methods.
-	void setValues(const QStringList& vlist);
-	QStringList values() const;
+	void setValueList(const QStringList& vlist);
+	QStringList valueList() const;
 
 	// Reset-to-default method.
 	void reset();
@@ -301,6 +339,39 @@ public:
 	virtual void process(
 		float **ppIBuffer, float **ppOBuffer, unsigned int nframes) = 0;
 
+	// Parameter update method.
+	virtual void updateParam(
+		qtractorPluginParam */*pParam*/, float /*fValue*/) {}
+
+	// Specific MIDI instrument selector.
+	virtual void selectProgram(int /*iBank*/, int /*iProg*/) {}
+
+	// Program (patch) descriptor.
+	struct Program
+	{
+		int     bank;
+		int     prog;
+		QString name;
+	};
+
+	// Provisional program/patch accessor.
+	virtual bool getProgram(int /*iIndex*/, Program& /*program*/) const
+		{ return false; }
+
+	// MIDI continuous controller handler.
+	virtual void setController(int /*iController*/, int /*iValue*/) {}
+
+	// Plugin configuration handlers.
+	virtual void configure(
+		const QString& /*sKey*/, const QString& /*sValue*/) {}
+
+	// Tall whether configuration snapshot is needed.
+	virtual bool isConfigure() { return false; }
+
+	// Plugin configuration/state snapshot.
+	virtual void freezeConfigs() {}
+	virtual void releaseConfigs() {}
+
 	// GUI Editor stuff.
 	virtual void openEditor(QWidget */*pParent*/) {}
 	virtual void closeEditor() {};
@@ -317,7 +388,8 @@ public:
 		{ return m_sEditorTitle; }
 
 	// An accessible list of observers.
-	const QList<qtractorPluginListItem *>& items() const { return m_items; }
+	const QList<qtractorPluginListItem *>& items() const
+		{ return m_items; }
 
 	// List of observers management.
 	void addItem(qtractorPluginListItem *pItem);
@@ -325,7 +397,7 @@ public:
 	void clearItems();
 
 	// Special plugin form accessors.
-	bool isVisible() const;
+	bool isFormVisible() const;
 	qtractorPluginForm *form();
 
 	// Plugin default preset name accessor (informational)
@@ -334,21 +406,64 @@ public:
 
 	// Plugin preset group - common identification prefix.
 	QString presetGroup() const;
+	QString presetPrefix() const;
+
+	// Plugin configuration from/to xml file.
+	bool loadPreset(const QString& sFilename);
+	bool savePreset(const QString& sFilename);
 
 	// Plugin parameter lookup.
 	qtractorPluginParam *findParam(unsigned long iIndex) const;
 
+	// Plugin configuration (CLOB) stuff.
+	typedef QHash<QString, QString> Configs;
+
+	void setConfigs(const Configs& configs)
+		{ m_configs = configs; }
+	const Configs& configs() const
+		{ return m_configs; }
+
+	void setConfig(const QString& sKey, const QString& sValue)
+		{ m_configs[sKey] = sValue; }
+	QString config(const QString& sKey) const
+		{ return m_configs[sKey]; }
+
+	// Plugin parameter values stuff.
+	typedef QHash<unsigned long, float> Values;
+
+	void setValues(const Values& values)
+		{ m_values = values; }
+	const Values& values() const
+		{ return m_values; }
+
+	void setValue(unsigned long iIndex, float fValue)
+		{ m_values[iIndex] = fValue; }
+	float value(unsigned long iIndex) const
+		{ return m_values[iIndex]; }
+
+	// Plugin configure realization.
+	void realizeConfigs();
+	void realizeValues();
+
+	// Plugin configure clearance.
+	void clearConfigs()
+		{ m_configs.clear(); }
+	void clearValues()
+		{ m_values.clear(); }
+
+	// Load plugin configuration/parameter values stuff.
+	static void loadConfigs(QDomElement *pElement, Configs& configs);
+	static void loadValues(QDomElement *pElement, Values& values);
+
+	// Save plugin configuration/parameter values stuff.
+	void saveConfigs(QDomDocument *pDocument, QDomElement *pElement);
+	void saveValues(QDomDocument *pDocument, QDomElement *pElement);
+
 protected:
 
 	// Instance number settler.
 	void setInstances(unsigned short iInstances);
 
-	// Instance capped number of audio ports.
-	unsigned short audioInsCap() const
-		{ return m_iAudioInsCap; }
-	unsigned short audioOutsCap() const
-		{ return m_iAudioOutsCap; }
-	
 private:
 
 	// Instance variables.
@@ -357,9 +472,6 @@ private:
 
 	unsigned short m_iInstances;
 
-	unsigned short m_iAudioInsCap;
-	unsigned short m_iAudioOutsCap;
-
 	bool m_bActivated;
 
 	// List of input control ports (parameters).
@@ -374,6 +486,12 @@ private:
 
 	// GUI editor stuff.
 	QString m_sEditorTitle;
+
+	// Plugin configuration (CLOB) stuff.
+	Configs m_configs;
+
+	// Plugin parameter values (part of configuration).
+	Values m_values;
 };
 
 
@@ -385,12 +503,24 @@ class qtractorPluginList : public qtractorList<qtractorPlugin>
 {
 public:
 
+	// Plugin-list type flags.
+	enum Flags {
+		// Basic flags.
+		Audio = 0, Midi = 1,
+		Track = 0, Bus  = 2,
+		// Composite helper flags.
+		AudioTrack = Audio | Track,
+		AudioBus   = Audio | Bus,
+		MidiTrack  = Midi  | Track,
+		MidiBus    = Midi  | Bus
+	};
+
 	// Constructor.
 	qtractorPluginList(
 		unsigned short iChannels,
 		unsigned int iBufferSize,
 		unsigned int iSampleRate,
-		bool bMidi = false);
+		unsigned int iFlags);
 
 	// Destructor.
 	~qtractorPluginList();
@@ -405,7 +535,7 @@ public:
 		unsigned short iChannels,
 		unsigned int iBufferSize,
 		unsigned int iSampleRate,
-		bool bMidi = false);
+		unsigned int iFlags);
 
 	// Reset and (re)activate all plugin chain.
 	void resetBuffer();
@@ -414,13 +544,19 @@ public:
 	unsigned short channels() const { return m_iChannels;   }
 	unsigned int sampleRate() const { return m_iSampleRate; }
 	unsigned int bufferSize() const { return m_iBufferSize; }
-	bool         isMidi()     const { return m_bMidi; }
+	unsigned int flags()      const { return m_iFlags; }
+
+	// Specific MIDI manager accessor.
+	qtractorMidiManager *midiManager() const
+		{ return m_pMidiManager; }
 
 	// Special activation methods.
-	unsigned int activated() const  { return m_iActivated;  }
 	bool isActivatedAll() const;
 	void updateActivated(bool bActivated);
 
+	unsigned int activated() const
+		{ return m_iActivated;  }
+
 	// Guarded plugin methods.
 	void addPlugin(qtractorPlugin *pPlugin);
 	void insertPlugin(qtractorPlugin *pPlugin, qtractorPlugin *pNextPlugin);
@@ -431,7 +567,8 @@ public:
 	qtractorPlugin *copyPlugin(qtractorPlugin *pPlugin);
 
 	// An accessible list of views.
-	const QList<qtractorPluginListView *>& views() const { return m_views; }
+	const QList<qtractorPluginListView *>& views() const
+		{ return m_views; }
 
 	// list of views management.
 	void addView(qtractorPluginListView *pView);
@@ -446,13 +583,19 @@ public:
 	bool saveElement(qtractorSessionDocument *pDocument,
 		QDomElement *pElement);
 
+protected:
+
+	// Plugin management helpers.
+	void addPluginRef(qtractorPlugin *pPlugin);
+	void removePluginRef(qtractorPlugin *pPlugin);
+
 private:
 
 	// Instance variables.
 	unsigned short m_iChannels;
 	unsigned int   m_iBufferSize;
 	unsigned int   m_iSampleRate;
-	bool           m_bMidi;
+	unsigned int   m_iFlags;
 
 	// Activation state.
 	unsigned int   m_iActivated;
@@ -463,6 +606,9 @@ private:
 	// An accessible list of observers.
 	QList<qtractorPluginListView *> m_views;
 
+	// Specific MIDI manager.
+	qtractorMidiManager *m_pMidiManager;
+
 	// Internal running buffer chain references.
 	float **m_pppBuffers[2];
 };
@@ -526,13 +672,12 @@ public:
 		{ return m_fDefaultValue; }
 	
 	// Current parameter value.
-	virtual void setValue(float fValue);
-	virtual float value() const
+	void setValue(float fValue);
+	float value() const
 		{ return m_fValue; }
 
 	// Reset-to-default method.
-	virtual void reset()
-		{ setValue(m_fDefaultValue); }
+	void reset() { setValue(m_fDefaultValue); }
 
 	// Direct parameter value.
 	float *data() { return &m_fValue; }
