@@ -31,6 +31,7 @@
 
 // Forward declarations...
 class qtractorPluginPortWidget;
+class qtractorMidiManager;
 
 
 //----------------------------------------------------------------------
@@ -259,6 +260,31 @@ private:
 };
 
 
+//----------------------------------------------------------------------
+// class qtractorAudioOutputBusCommand - declaration.
+//
+
+class qtractorAudioOutputBusCommand : public qtractorCommand
+{
+public:
+
+	// Constructor.
+	qtractorAudioOutputBusCommand(qtractorMidiManager *pMidiManager,
+		bool bAudioOutputBus);
+
+	// Plugin audio ouput bus command methods.
+	bool redo();
+	bool undo();
+
+private:
+
+	// Instance variables.
+	qtractorMidiManager *m_pMidiManager;
+
+	bool m_bAudioOutputBus;
+};
+
+
 #endif	// __qtractorPluginCommand_h
 
 // end of qtractorPluginCommand.h
