@@ -29,7 +29,7 @@
 #include "qtractorSlider.h"
 
 #include "qtractorOptions.h"
-#include "qtractorMainForm.h"
+#include "qtractorSession.h"
 
 #include <QMessageBox>
 #include <QGridLayout>
@@ -39,6 +39,10 @@
 #include <QCheckBox>
 #include <QDoubleSpinBox>
 
+#include <QFileInfo>
+#include <QFileDialog>
+#include <QUrl>
+
 #include <math.h>
 
 // This shall hold the default preset name.
@@ -56,24 +60,32 @@ qtractorPluginForm::qtractorPluginForm (
 	// Setup UI struct...
 	m_ui.setupUi(this);
 
-	QWidget::setFont(QFont(font().family(), 7));
+	const QFont& font = QWidget::font();
+	QWidget::setFont(QFont(font.family(), font.pointSize() - 1));
 
-	m_pPlugin       = NULL;
-	m_pGridLayout   = NULL;
-	m_iDirtyCount   = 0;
-	m_iUpdate       = 0;
+	m_pPlugin     = NULL;
+	m_pGridLayout = NULL;
+	m_iDirtyCount = 0;
+	m_iUpdate     = 0;
 
     m_ui.PresetComboBox->setValidator(
 		new QRegExpValidator(QRegExp("[\\w-]+"), m_ui.PresetComboBox));
 	m_ui.PresetComboBox->setInsertPolicy(QComboBox::NoInsert);
 
 	// Have some effective feedback when toggling on/off...
-	QIcon icons;
-	icons.addPixmap(
+	QIcon iconParams;
+	iconParams.addPixmap(
+		QPixmap(":/icons/formParamsOff.png"), QIcon::Active, QIcon::Off);
+	iconParams.addPixmap(
+		QPixmap(":/icons/formParamsOn.png"), QIcon::Active, QIcon::On);
+	m_ui.ParamsToolButton->setIcon(iconParams);
+
+	QIcon iconActivate;
+	iconActivate.addPixmap(
 		QPixmap(":/icons/itemLedOff.png"), QIcon::Active, QIcon::Off);
-	icons.addPixmap(
+	iconActivate.addPixmap(
 		QPixmap(":/icons/itemLedOn.png"), QIcon::Active, QIcon::On);
-	m_ui.ActivateToolButton->setIcon(icons);
+	m_ui.ActivateToolButton->setIcon(iconActivate);
 
 	if (g_sDefPreset.isEmpty())
 		g_sDefPreset = tr("(default)");
@@ -85,18 +97,28 @@ qtractorPluginForm::qtractorPluginForm (
 	QObject::connect(m_ui.PresetComboBox,
 		SIGNAL(activated(const QString &)),
 		SLOT(loadPresetSlot(const QString&)));
+	QObject::connect(m_ui.OpenPresetToolButton,
+		SIGNAL(clicked()),
+		SLOT(openPresetSlot()));
 	QObject::connect(m_ui.SavePresetToolButton,
 		SIGNAL(clicked()),
 		SLOT(savePresetSlot()));
 	QObject::connect(m_ui.DeletePresetToolButton,
 		SIGNAL(clicked()),
 		SLOT(deletePresetSlot()));
+	QObject::connect(m_ui.ParamsToolButton,
+		SIGNAL(toggled(bool)),
+		SLOT(paramsSlot(bool)));
 	QObject::connect(m_ui.EditToolButton,
 		SIGNAL(toggled(bool)),
 		SLOT(editSlot(bool)));
 	QObject::connect(m_ui.ActivateToolButton,
 		SIGNAL(toggled(bool)),
 		SLOT(activateSlot(bool)));
+
+	QObject::connect(this,
+		SIGNAL(valueChanged(qtractorPluginParam *, float)),
+		SLOT(valueChangeSlot(qtractorPluginParam *, float)));
 }
 
 
@@ -135,14 +157,14 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 	const QList<qtractorPluginParam *>& params = m_pPlugin->params();
 	int iRows = params.count();
 	bool bEditor = (m_pPlugin->type())->isEditor();
-	// FIXME: Can't stand more than hundred widgets?
-	// or do we have a dedicated editor GUI?
-	if (!bEditor && iRows < 101) {
+	// FIXME: Couldn't stand more than a hundred widgets?
+	// or do we have one dedicated editor GUI?
+	if (!bEditor || iRows < 101) {
 		int iCols = 1;
-		while (iRows > 12 && iCols < 3) {
-			iRows >>= 1;
-			iCols++;
-		}
+		while (iRows > 12 && iCols < 4)
+			iRows = (params.count() / ++iCols);
+		if (params.count() % iCols)	// Adjust to balance.
+			iRows++;
 		int iRow = 0;
 		int iCol = 0;
 		QListIterator<qtractorPluginParam *> iter(params);
@@ -155,20 +177,28 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 			QObject::connect(pParamWidget,
 				SIGNAL(valueChanged(qtractorPluginParam *, float)),
 				SLOT(valueChangeSlot(qtractorPluginParam *, float)));
-			if (++iRow > iRows) {
+			if (++iRow >= iRows) {
 				iRow = 0;
 				iCol++;
 			}
 		}
 	}
-	layout()->addItem(m_pGridLayout);
+	m_ui.ParamsGridWidget->setLayout(m_pGridLayout);
 
 	// Show editor button if available?
 	m_ui.EditToolButton->setVisible(bEditor);
+	//if (bEditor)
+	//	toggleEditor(m_pPlugin->isEditorVisible());
 
 	// Set plugin name as title...
 	updateCaption();
-	adjustSize();
+
+	// This should trigger paramsSlot(!bEditor)
+	// and adjust the size of the params dialog...
+	bool bParams = (iRows > 0 && iRows < 101);
+	m_ui.ParamsToolButton->setVisible(bParams);
+	m_ui.ParamsToolButton->setChecked(!bEditor);
+	paramsSlot(bParams);
 
 	updateActivated();
 	refresh();
@@ -176,8 +206,8 @@ void qtractorPluginForm::setPlugin ( qtractorPlugin *pPlugin )
 	show();
 
 	// Do we have a dedicated editor GUI?
-	// if (bEditor)
-	//	m_pPlugin->openEditor(this);
+	if (!bParams && bEditor)
+		m_pPlugin->openEditor(this);
 }
 
 
@@ -277,9 +307,13 @@ void qtractorPluginForm::updateParamValue (
 	if (m_pPlugin == NULL)
 		return;
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginForm[%p]::updateParamValue(%lu)", this, iIndex);
+#endif
+
 	qtractorPluginParam *pParam = m_pPlugin->findParam(iIndex);
 	if (pParam)
-		valueChangeSlot(pParam, fValue);
+		emit valueChanged(pParam, fValue);
 }
 
 
@@ -289,10 +323,9 @@ void qtractorPluginForm::updateParamWidget ( unsigned long iIndex )
 	if (m_pPlugin == NULL)
 		return;
 
-	if (m_iUpdate > 0)
-		return;
-
-	m_iUpdate++;
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginForm[%p]::updateParamWidget(%lu)", this, iIndex);
+#endif
 
 	QListIterator<qtractorPluginParamWidget *> iter(m_paramWidgets);
 	while (iter.hasNext()) {
@@ -302,10 +335,6 @@ void qtractorPluginForm::updateParamWidget ( unsigned long iIndex )
 			break;
 		}
 	}
-
-	m_pPlugin->idleEditor();
-
-	m_iUpdate--;
 }
 
 
@@ -330,26 +359,35 @@ void qtractorPluginForm::loadPresetSlot ( const QString& sPreset )
 	if (m_iUpdate > 0 || sPreset.isEmpty())
 		return;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	// We'll need this, sure.
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions == NULL)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return;
 
 	if (sPreset == g_sDefPreset) {
 		// Reset to default...
-		pMainForm->commands()->exec(
+		pSession->execute(
 			new qtractorResetPluginCommand(m_pPlugin));
 	} else {
 		// An existing preset is about to be loaded...
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			QSettings& settings = pOptions->settings();
-			// Get the preset entry, if any...
+		QSettings& settings = pOptions->settings();
+		// Should it be load from know file?...
+		if ((m_pPlugin->type())->isConfigure()) {
+			settings.beginGroup(m_pPlugin->presetGroup());
+			m_pPlugin->loadPreset(settings.value(sPreset).toString());
+			settings.endGroup();
+			refresh();
+		} else {
+			//...or make it as usual (parameter list only)...
 			settings.beginGroup(m_pPlugin->presetGroup());
 			QStringList vlist = settings.value(sPreset).toStringList();
 			settings.endGroup();
-			// Is it there?
 			if (!vlist.isEmpty()) {
-				pMainForm->commands()->exec(
+				pSession->execute(
 					new qtractorPresetPluginCommand(m_pPlugin, vlist));
 			}
 		}
@@ -359,6 +397,73 @@ void qtractorPluginForm::loadPresetSlot ( const QString& sPreset )
 }
 
 
+void qtractorPluginForm::openPresetSlot (void)
+{
+	if (m_pPlugin == NULL)
+		return;
+	if (!(m_pPlugin->type())->isConfigure())
+		return;
+
+	if (m_iUpdate > 0)
+		return;
+
+	// We'll need this, sure.
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions == NULL)
+		return;
+
+	// We'll assume that there's an external file...
+	QString sFilename;
+
+	// Prompt if file does not currently exist...
+	const QString  sExt("qtx");
+	const QString& sTitle  = tr("Open Preset") + " - " QTRACTOR_TITLE;
+	const QString& sFilter = tr("Preset files (*.%1)").arg(sExt); 
+#if QT_VERSION < 0x040400
+	// Ask for the filename to save...
+	sFilename = QFileDialog::getSaveFileName(this,
+		sTitle, pOptions->sPresetDir, sFilter);
+#else
+	// Construct save-file dialog...
+	QFileDialog fileDialog(this,
+		sTitle, pOptions->sPresetDir, sFilter);
+	// Set proper open-file modes...
+	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
+	fileDialog.setFileMode(QFileDialog::ExistingFile);
+	fileDialog.setDefaultSuffix(sExt);
+	// Stuff sidebar...
+	QList<QUrl> urls(fileDialog.sidebarUrls());
+	urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+	urls.append(QUrl::fromLocalFile(pOptions->sPresetDir));
+	fileDialog.setSidebarUrls(urls);
+	// Show dialog...
+	if (fileDialog.exec())
+		sFilename = fileDialog.selectedFiles().first();
+#endif
+	// Have we a filename to load a preset from?
+	if (!sFilename.isEmpty()) {
+		if (m_pPlugin->loadPreset(sFilename)) {
+			// Got it loaded alright...
+			QFileInfo fi(sFilename);
+			setPreset(fi.baseName()
+				.replace((m_pPlugin->type())->label() + '-', QString()));
+			pOptions->sPresetDir = fi.absolutePath();
+		} else {
+			// Failure (maybe wrong plugin)...
+			QMessageBox::critical(this,
+				tr("Error") + " - " QTRACTOR_TITLE,
+				tr("Preset could not be loaded\n"
+				"from \"%1\".\n\n"
+				"Sorry.").arg(sFilename),
+				tr("Cancel"));
+		}
+	}
+	refresh();
+
+	stabilize();
+}
+
+
 void qtractorPluginForm::savePresetSlot (void)
 {
 	if (m_pPlugin == NULL)
@@ -368,23 +473,62 @@ void qtractorPluginForm::savePresetSlot (void)
 	if (sPreset.isEmpty() || sPreset == g_sDefPreset)
 		return;
 
+	// We'll need this, sure.
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions == NULL)
+		return;
+
 	// The current state preset is about to be saved...
-	QStringList vlist = m_pPlugin->values();
-	// Is there any?
-	if (!vlist.isEmpty()) {
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-		if (pMainForm) {
-			qtractorOptions *pOptions = pMainForm->options();
-			if (pOptions) {
-				QSettings& settings = pOptions->settings();
-				// Set the preset entry...
-				settings.beginGroup(m_pPlugin->presetGroup());
-				settings.setValue(sPreset, vlist);
-				settings.endGroup();
-				refresh();
+	// this is where we'll make it...
+	QSettings& settings = pOptions->settings();
+	settings.beginGroup(m_pPlugin->presetGroup());
+	// Which mode of preset?
+	if ((m_pPlugin->type())->isConfigure()) {
+		// Sure, we'll have something complex enough
+		// to make it save into an external file...
+		const QString sExt("qtx");
+		QFileInfo fi(QDir(pOptions->sPresetDir),
+			(m_pPlugin->type())->label() + '-' + sPreset + '.' + sExt);
+		QString sFilename = fi.absoluteFilePath();
+		// Prompt if file does not currently exist...
+		if (!fi.exists()) {
+			const QString& sTitle  = tr("Save Preset") + " - " QTRACTOR_TITLE;
+			const QString& sFilter = tr("Preset files (*.%1)").arg(sExt); 
+		#if QT_VERSION < 0x040400
+			// Ask for the filename to save...
+			sFilename = QFileDialog::getSaveFileName(this,
+				sTitle, sFilename, sFilter);
+		#else
+			// Construct save-file dialog...
+			QFileDialog fileDialog(this,
+				sTitle, sFilename, sFilter);
+			// Set proper open-file modes...
+			fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+			fileDialog.setFileMode(QFileDialog::AnyFile);
+			fileDialog.setDefaultSuffix(sExt);
+			// Stuff sidebar...
+			QList<QUrl> urls(fileDialog.sidebarUrls());
+			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+			urls.append(QUrl::fromLocalFile(pOptions->sPresetDir));
+			fileDialog.setSidebarUrls(urls);
+			// Show dialog...
+			if (fileDialog.exec())
+				sFilename = fileDialog.selectedFiles().first();
+		#endif
+		}
+		// We've a filename to save the preset
+		if (!sFilename.isEmpty()) {
+			if (QFileInfo(sFilename).suffix() != sExt)
+				sFilename += '.' + sExt;
+			if (m_pPlugin->savePreset(sFilename)) {
+				settings.setValue(sPreset, sFilename);
+				pOptions->sPresetDir = QFileInfo(sFilename).absolutePath();
 			}
 		}
-	}
+	}	// Just leave it to simple parameter value list...
+	else settings.setValue(sPreset, m_pPlugin->valueList());
+	settings.endGroup();
+	refresh();
 
 	stabilize();
 }
@@ -399,36 +543,66 @@ void qtractorPluginForm::deletePresetSlot (void)
 	if (sPreset.isEmpty() || sPreset == g_sDefPreset)
 		return;
 
-	// A preset entry is about to be deleted...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			// Prompt user if he/she's sure about this...
-			if (pOptions->bConfirmRemove) {
-				if (QMessageBox::warning(this,
-					tr("Warning") + " - " QTRACTOR_TITLE,
-					tr("About to delete preset:\n\n"
-					"\"%1\" (%2)\n\n"
-					"Are you sure?")
-					.arg(sPreset)
-					.arg((m_pPlugin->type())->name()),
-					tr("OK"), tr("Cancel")) > 0)
-					return;
-			}
-			// Go ahead...
-			QSettings& settings = pOptions->settings();
-			settings.beginGroup(m_pPlugin->presetGroup());
-			settings.remove(sPreset);
-			settings.endGroup();
-			refresh();
-		}
+	// We'll need this, sure.
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions == NULL)
+		return;
+
+	// A preset entry is about to be deleted;
+	// prompt user if he/she's sure about this...
+	if (pOptions->bConfirmRemove) {
+		if (QMessageBox::warning(this,
+			tr("Warning") + " - " QTRACTOR_TITLE,
+			tr("About to delete preset:\n\n"
+			"\"%1\" (%2)\n\n"
+			"Are you sure?")
+			.arg(sPreset)
+			.arg((m_pPlugin->type())->name()),
+			tr("OK"), tr("Cancel")) > 0)
+			return;
 	}
+	// Go ahead...
+	QSettings& settings = pOptions->settings();
+	settings.beginGroup(m_pPlugin->presetGroup());
+	if ((m_pPlugin->type())->isConfigure()) {
+		const QString& sFilename = settings.value(sPreset).toString();
+		if (QFileInfo(sFilename).exists())
+			QFile(sFilename).remove();
+	}
+	settings.remove(sPreset);
+	settings.endGroup();
+	refresh();
 
 	stabilize();
 }
 
 
+// Params slot.
+void qtractorPluginForm::paramsSlot ( bool bOn )
+{
+	if (m_pPlugin == NULL)
+		return;
+
+	if (m_iUpdate > 0)
+		return;
+
+	m_iUpdate++;
+
+	m_ui.ParamsGridWidget->setVisible(bOn);
+	if (bOn)
+		m_ui.ParamsGridWidget->show();
+	else
+		m_ui.ParamsGridWidget->hide();
+
+	// Shake it a little bit first, but
+	// make it as tight as possible...
+	resize(width() + 1, height() + 1);
+	adjustSize();
+
+	m_iUpdate--;
+}
+
+
 // Editor slot.
 void qtractorPluginForm::editSlot ( bool bOn )
 {
@@ -461,9 +635,9 @@ void qtractorPluginForm::activateSlot ( bool bOn )
 	m_iUpdate++;
 
 	// Make it a undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pMainForm->commands()->exec(
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession)
+		pSession->execute(
 			new qtractorActivatePluginCommand(m_pPlugin, bOn));
 
 	m_iUpdate--;
@@ -480,12 +654,15 @@ void qtractorPluginForm::valueChangeSlot (
 	if (m_iUpdate > 0)
 		return;
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginForm[%p]::valueChangeSlot(%p, %g)", this, pParam, fValue);
+#endif
 	m_iUpdate++;
 
 	// Make it a undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pMainForm->commands()->exec(
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession)
+		pSession->execute(
 			new qtractorPluginParamCommand(pParam, fValue));
 
 	m_iUpdate--;
@@ -501,19 +678,22 @@ void qtractorPluginForm::refresh (void)
 	if (m_pPlugin == NULL)
 		return;
 
+	if (m_iUpdate > 0)
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginForm[%p]::refresh()", this);
+#endif
 	m_iUpdate++;
 
 	const QString sOldPreset = m_ui.PresetComboBox->currentText();
 	m_ui.PresetComboBox->clear();
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			pOptions->settings().beginGroup(m_pPlugin->presetGroup());
-			m_ui.PresetComboBox->insertItems(0,
-				pOptions->settings().childKeys());
-			pOptions->settings().endGroup();
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		pOptions->settings().beginGroup(m_pPlugin->presetGroup());
+		m_ui.PresetComboBox->insertItems(0,
+			pOptions->settings().childKeys());
+		pOptions->settings().endGroup();
 	}
 	m_ui.PresetComboBox->addItem(g_sDefPreset);
 	m_ui.PresetComboBox->setEditText(sOldPreset);
@@ -535,9 +715,15 @@ void qtractorPluginForm::stabilize (void)
 	bool bExists  = false;
 	bool bEnabled = (m_pPlugin != NULL);
 	m_ui.ActivateToolButton->setEnabled(bEnabled);
-	if (bEnabled)
-		bEnabled = ((m_pPlugin->type())->controlIns() > 0);
+	if (bEnabled) {
+		bEnabled = (
+			(m_pPlugin->type())->controlIns() > 0 ||
+			(m_pPlugin->type())->isConfigure());
+	}
+
 	m_ui.PresetComboBox->setEnabled(bEnabled);
+	m_ui.OpenPresetToolButton->setVisible(
+		bEnabled && (m_pPlugin->type())->isConfigure());
 
 	if (bEnabled) {
 		const QString& sPreset = m_ui.PresetComboBox->currentText();
@@ -611,7 +797,8 @@ qtractorPluginParamWidget::qtractorPluginParamWidget (
 		m_pLabel = new QLabel(/*this*/);
 		if (m_pParam->isDisplay()) {
 			m_pLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
-			m_pLabel->setFixedWidth(72);
+		//	m_pLabel->setFixedWidth(72);
+			m_pLabel->setMinimumWidth(64);
 			m_pLabel->setText(m_pParam->name());
 			m_pGridLayout->addWidget(m_pLabel, 0, 0);
 		} else {
@@ -633,7 +820,8 @@ qtractorPluginParamWidget::qtractorPluginParamWidget (
 			m_pDisplay = new QLabel(/*this*/);
 			m_pDisplay->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 			m_pDisplay->setText(m_pParam->display());
-			m_pDisplay->setFixedWidth(72);
+		//	m_pDisplay->setFixedWidth(72);
+			m_pDisplay->setMinimumWidth(64);
 			m_pGridLayout->addWidget(m_pDisplay, 0, 2);
 		} else {
 			m_pGridLayout->addWidget(m_pSlider, 1, 0, 1, 2);
@@ -671,6 +859,9 @@ void qtractorPluginParamWidget::refresh (void)
 	if (m_iUpdate > 0)
 		return;
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginParamWidget[%p]::refresh()", this);
+#endif
 	m_iUpdate++;
 
 	float fValue = m_pParam->value();
@@ -693,18 +884,38 @@ void qtractorPluginParamWidget::refresh (void)
 // Slider conversion methods.
 int qtractorPluginParamWidget::paramToSlider ( float fValue ) const
 {
-	int   iValue = 0;
-	float fScale = m_pParam->maxValue() - m_pParam->minValue();
-	if (fScale > 1E-6f)
-		iValue = int((10000.0f * (fValue - m_pParam->minValue())) / fScale);
+	int iValue = 0;
+
+	if (m_pParam->isLogarithmic() && m_pParam->minValue() > 1E-6f) {
+		if (fValue > 1E-6f) {
+			float fLogMaxValue = ::logf(m_pParam->maxValue());
+			float fLogMinValue = ::logf(m_pParam->minValue());
+			iValue = ::lroundf(10000.0f
+				* (::logf(fValue) - fLogMinValue)
+				/ (fLogMaxValue - fLogMinValue));
+		}
+	} else {
+		float fScale = m_pParam->maxValue() - m_pParam->minValue();
+		if (fScale > 1E-6f)
+			iValue = int((10000.0f * (fValue - m_pParam->minValue())) / fScale);
+	}
 
 	return iValue;
 }
 
 float qtractorPluginParamWidget::sliderToParam ( int iValue ) const
 {
-	float fDelta = m_pParam->maxValue() - m_pParam->minValue();
-	return m_pParam->minValue() + (float(iValue) * fDelta) / 10000.0f;
+	float fValue = 0.0f;
+
+	if (m_pParam->isLogarithmic() && m_pParam->minValue() > 1E-6f) {
+		float fRatio = m_pParam->maxValue() / m_pParam->minValue();
+		fValue = m_pParam->minValue() * ::powf(fRatio, (float(iValue) / 10000.0f));
+	} else {
+		float fDelta = m_pParam->maxValue() - m_pParam->minValue();
+		fValue = m_pParam->minValue() + (float(iValue) * fDelta) / 10000.0f;
+	}
+
+	return fValue;
 }
 
 
@@ -738,6 +949,9 @@ void qtractorPluginParamWidget::spinBoxValueChanged ( const QString& sText )
 	if (m_iUpdate > 0)
 		return;
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginParamWidget[%p]::spinBoxValueChanged()", this);
+#endif
 	m_iUpdate++;
 
 	float fValue = 0.0f;
@@ -763,6 +977,9 @@ void qtractorPluginParamWidget::sliderValueChanged ( int iValue )
 	if (m_iUpdate > 0)
 		return;
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorPluginParamWidget[%p]::sliderValueChanged()", this);
+#endif
 	m_iUpdate++;
 
 	float fValue = sliderToParam(iValue);
