@@ -73,12 +73,19 @@ public:
 
 	void toggleEditor(bool bOn);
 
+signals:
+
+	// Change notification.
+	void valueChanged(qtractorPluginParam *, float);
+
 protected slots:
 
 	void changePresetSlot(const QString& sPreset);
 	void loadPresetSlot(const QString& sPreset);
+	void openPresetSlot();
 	void savePresetSlot();
 	void deletePresetSlot();
+	void paramsSlot(bool bOn);
 	void editSlot(bool bOn);
 	void activateSlot(bool bOn);
 	void valueChangeSlot(qtractorPluginParam *pParam, float fValue);
