@@ -23,7 +23,6 @@
 #include "qtractorPluginSelectForm.h"
 
 #include "qtractorOptions.h"
-#include "qtractorMainForm.h"
 
 #include <QHeaderView>
 #include <QLineEdit>
@@ -46,8 +45,6 @@ qtractorPluginSelectForm::qtractorPluginSelectForm (
 	// Setup UI struct...
 	m_ui.setupUi(this);
 
-	QDialog::setFont(QFont(font().family(), 7));
-
 	m_iChannels = 0;
 	m_bMidi = false;
 
@@ -97,15 +94,13 @@ qtractorPluginSelectForm::qtractorPluginSelectForm (
 	m_ui.PluginTypeProgressBar->hide();
 
 	// Initialize conveniency options...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			pOptions->loadComboBoxHistory(m_ui.PluginSearchComboBox);
-			m_ui.PluginSearchComboBox->setEditText(
-				pOptions->sPluginSearch);
-			m_ui.PluginTypeComboBox->setCurrentIndex(pOptions->iPluginType);
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		pOptions->loadComboBoxHistory(m_ui.PluginSearchComboBox);
+		m_ui.PluginSearchComboBox->setEditText(
+			pOptions->sPluginSearch);
+		m_ui.PluginTypeComboBox->setCurrentIndex(pOptions->iPluginType);
+		m_ui.PluginActivateCheckBox->setChecked(pOptions->bPluginActivate);
 	}
 
 	// Let the search begin...
@@ -134,6 +129,9 @@ qtractorPluginSelectForm::qtractorPluginSelectForm (
 	QObject::connect(m_ui.PluginListView,
 		SIGNAL(itemDoubleClicked(QTreeWidgetItem *, int)),
 		SLOT(accept()));
+	QObject::connect(m_ui.PluginActivateCheckBox,
+		SIGNAL(clicked()),
+		SLOT(stabilize()));
 	QObject::connect(m_ui.OkPushButton,
 		SIGNAL(clicked()),
 		SLOT(accept()));
@@ -147,14 +145,12 @@ qtractorPluginSelectForm::qtractorPluginSelectForm (
 qtractorPluginSelectForm::~qtractorPluginSelectForm (void)
 {
 	// Save other conveniency options, if convenient thought...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			pOptions->iPluginType = m_ui.PluginTypeComboBox->currentIndex();
-			pOptions->sPluginSearch = m_ui.PluginSearchComboBox->currentText();
-			pOptions->saveComboBoxHistory(m_ui.PluginSearchComboBox);
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		pOptions->iPluginType = m_ui.PluginTypeComboBox->currentIndex();
+		pOptions->sPluginSearch = m_ui.PluginSearchComboBox->currentText();
+		pOptions->saveComboBoxHistory(m_ui.PluginSearchComboBox);
+		pOptions->bPluginActivate= m_ui.PluginActivateCheckBox->isChecked();
 	}
 }
 
@@ -204,15 +200,16 @@ qtractorPluginType::Hint qtractorPluginSelectForm::pluginTypeHint ( int iPlugin
 	return qtractorPluginType::hintFromText(sText);
 }
 
+bool qtractorPluginSelectForm::isPluginActivated (void) const
+{
+	return m_ui.PluginActivateCheckBox->isChecked();
+}
+
 
 // Plugin type hint change slot.
 void qtractorPluginSelectForm::typeHintChanged ( int iTypeHint )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions == NULL)
 		return;
 
@@ -262,6 +259,9 @@ void qtractorPluginSelectForm::refresh (void)
 		// Tell the world we'll take some time...
 		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 		int iFile = 0;
+		qtractorPluginType::Hint typeHint
+			= qtractorPluginType::hintFromText(
+				m_ui.PluginTypeComboBox->currentText());
 		m_ui.PluginTypeProgressBar->setMaximum(g_pluginPath.files().count());
 		m_ui.PluginTypeProgressBar->show();
 		QListIterator<qtractorPluginFile *> file_iter(g_pluginPath.files());
@@ -270,7 +270,7 @@ void qtractorPluginSelectForm::refresh (void)
 			QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
 			qtractorPluginFile *pFile = file_iter.next();
 			if (pFile->open()) {
-				pFile->getTypes(g_pluginTypes);
+				pFile->getTypes(g_pluginTypes, typeHint);
 				pFile->close();
 			}
 		}
@@ -299,7 +299,8 @@ void qtractorPluginSelectForm::refresh (void)
 			int iMidiOuts    = pType->midiOuts();
 			int iControlIns  = pType->controlIns();
 			int iControlOuts = pType->controlOuts();
-			unsigned short iInstances = pType->instances(m_iChannels, m_bMidi);
+			// FIXME: We don't care whether it's a MIDI chain at this time...
+			unsigned short iInstances = pType->instances(m_iChannels, false);
 			cols.clear();
 			cols << sName;
 			cols << QString("%1:%2").arg(iAudioIns).arg(iAudioOuts);
