@@ -31,15 +31,21 @@
 
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiClip.h"
+#include "qtractorMidiBuffer.h"
 
 #include "qtractorPlugin.h"
 
+#include "qtractorInstrument.h"
+#include "qtractorCommand.h"
+
 #include <QApplication>
 #include <QDateTime>
 #include <QFileInfo>
 #include <QRegExp>
 #include <QDir>
 
+#include <stdlib.h>
+
 
 //-------------------------------------------------------------------------
 // qtractorSession::Properties -- Session properties structure.
@@ -70,12 +76,46 @@ qtractorSession::Properties& qtractorSession::Properties::copy (
 //-------------------------------------------------------------------------
 // qtractorSession -- Session container.
 
+// Singleton instance pointer.
+qtractorSession *qtractorSession::g_pSession = NULL;
+
+
+// Singleton instance accessor (static).
+qtractorSession *qtractorSession::getInstance (void)
+{
+	// Create the singleton instance, if not already...
+	if (g_pSession == NULL) {
+		g_pSession = new qtractorSession();
+		::atexit(Destroy);
+	}
+
+	return g_pSession;
+}
+
+
+// Singleton instance destroyer.
+void qtractorSession::Destroy (void)
+{
+	// OK. We're done with ourselves.
+	if (g_pSession) {
+		delete g_pSession;
+		g_pSession = NULL;
+	}
+}
+
+
 // Constructor.
 qtractorSession::qtractorSession (void)
 {
 	m_tracks.setAutoDelete(true);
 	m_cursors.setAutoDelete(false);
 
+	m_midiManagers.setAutoDelete(false);
+
+	// Singleton ownings.
+	m_pCommands    = new qtractorCommandList();
+	m_pInstruments = new qtractorInstrumentList();
+
 	// The dubious permanency of the crucial device engines.
 	m_pMidiEngine       = new qtractorMidiEngine(this);
 	m_pAudioEngine      = new qtractorAudioEngine(this);
@@ -86,6 +126,7 @@ qtractorSession::qtractorSession (void)
 	clear();
 }
 
+
 // Default destructor.
 qtractorSession::~qtractorSession (void)
 {
@@ -95,6 +136,9 @@ qtractorSession::~qtractorSession (void)
 	delete m_pAudioPeakFactory;
 	delete m_pAudioEngine;
 	delete m_pMidiEngine;
+
+	delete m_pInstruments;
+	delete m_pCommands;
 }
 
 
@@ -152,22 +196,30 @@ void qtractorSession::clear (void)
 	ATOMIC_SET(&m_locks, 0);
 	ATOMIC_SET(&m_mutex, 0);
 
+	m_pCurrentTrack = NULL;
+
 	m_tracks.clear();
 	m_cursors.clear();
 
 	m_props.clear();
 
 	m_midiTags.clear();
+	m_midiManagers.clear();
 
 	m_pAudioEngine->clear();
 	m_pMidiEngine->clear();
 
+	m_pCommands->clear();
+
 	m_iSessionLength = 0;
 
 	m_iRecordTracks  = 0;
 	m_iMuteTracks    = 0;
 	m_iSoloTracks    = 0;
 
+	m_iAudioRecord   = 0;
+	m_iMidiRecord    = 0;
+
 	m_iMidiTag       = 0;
 
 	m_iEditHead      = 0;
@@ -180,6 +232,11 @@ void qtractorSession::clear (void)
 	m_iLoopStartTime = 0;
 	m_iLoopEndTime   = 0;
 
+	m_iPunchIn       = 0;
+	m_iPunchOut      = 0;
+	m_iPunchInTime   = 0;
+	m_iPunchOutTime  = 0;
+
 	m_bRecording     = false;
 
 	updateTimeScale();
@@ -194,6 +251,27 @@ void qtractorSession::clear (void)
 }
 
 
+// The global undoable command execuive.
+bool qtractorSession::execute ( qtractorCommand *pCommand )
+{
+	return m_pCommands->exec(pCommand);
+}
+
+
+// The global undoable command list reference.
+qtractorCommandList *qtractorSession::commands (void) const
+{
+	return m_pCommands;
+}
+
+
+// Session instruments repository.
+qtractorInstrumentList *qtractorSession::instruments (void) const
+{
+	return m_pInstruments;
+}
+
+
 // Session directory path accessors.
 void qtractorSession::setSessionDir ( const QString& sSessionDir )
 {
@@ -327,21 +405,21 @@ bool qtractorSession::beatIsBar ( unsigned int iBeat ) const
 }
 
 
-// Pixels per beat (width).	
+// Pixels per beat (width).
 void qtractorSession::setPixelsPerBeat ( unsigned short iPixelsPerBeat )
 {
 	m_props.timeScale.setPixelsPerBeat(iPixelsPerBeat);
 }
 
 unsigned short qtractorSession::pixelsPerBeat (void) const
-{ 
+{
 	return m_props.timeScale.pixelsPerBeat();
 }
 
 
 // Horizontal zoom factor.
 void qtractorSession::setHorizontalZoom ( unsigned short iHorizontalZoom )
-{ 
+{
 	m_props.timeScale.setHorizontalZoom(iHorizontalZoom);
 }
 
@@ -353,12 +431,12 @@ unsigned short qtractorSession::horizontalZoom (void) const
 
 // Vertical zoom factor.
 void qtractorSession::setVerticalZoom ( unsigned short iVerticalZoom )
-{ 
+{
 	m_props.timeScale.setVerticalZoom(iVerticalZoom);
 }
 
 unsigned short qtractorSession::verticalZoom (void) const
-{ 
+{
 	return m_props.timeScale.verticalZoom();
 }
 
@@ -490,7 +568,7 @@ unsigned long qtractorSession::locateFromFrame ( unsigned long iFrame ) const
 
 
 // Update scale divisor factors.
-void qtractorSession::updateTimeScale (void) 
+void qtractorSession::updateTimeScale (void)
 {
 	// Recompute scale divisor factors...
 	m_props.timeScale.updateScale();
@@ -516,6 +594,12 @@ void qtractorSession::updateTimeScale (void)
 		}
 	}
 
+	// Update punch points...
+	if (m_iPunchIn < m_iPunchOut) {
+		m_iPunchIn  = frameFromTick(m_iPunchInTime);
+		m_iPunchOut = frameFromTick(m_iPunchOutTime);
+	}
+
 	// Do not forget those edit points too...
 	m_iEditHead = frameFromTick(m_iEditHeadTime);
 	m_iEditTail = frameFromTick(m_iEditTailTime);
@@ -549,6 +633,12 @@ void qtractorSession::updateTimeResolution (void)
 		m_iLoopEndTime   = tickFromFrame(m_iLoopEnd);
 	}
 
+	// Update punch points...
+	if (m_iPunchIn < m_iPunchOut) {
+		m_iPunchInTime  = tickFromFrame(m_iPunchIn);
+		m_iPunchOutTime = tickFromFrame(m_iPunchOut);
+	}
+
 	// Do not forget those edit points too...
 	m_iEditHeadTime = tickFromFrame(m_iEditHead);
 	m_iEditTailTime = tickFromFrame(m_iEditTail);
@@ -773,6 +863,37 @@ unsigned int qtractorSession::soloTracks (void) const
 }
 
 
+// Temporary current track accessors.
+void qtractorSession::setCurrentTrack ( qtractorTrack *pTrack )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorSession::setCurrentTrack(%p)", pTrack);
+#endif
+	m_pCurrentTrack = pTrack;
+}
+
+qtractorTrack *qtractorSession::currentTrack (void) const
+{
+	return m_pCurrentTrack;
+}
+
+
+// Temporary current track predicates.
+bool qtractorSession::isTrackMonitor ( qtractorTrack *pTrack ) const
+{
+	return pTrack->isMonitor()
+		|| pTrack == m_pCurrentTrack;
+}
+
+bool qtractorSession::isTrackMidiChannel (
+	qtractorTrack *pTrack, unsigned short iChannel ) const
+{
+	return pTrack->isMidiOmni()
+		|| pTrack->midiChannel() == iChannel
+		|| pTrack == m_pCurrentTrack;
+}
+
+
 // Session cursor factory methods.
 qtractorSessionCursor *qtractorSession::createSessionCursor (
 	unsigned long iFrame, qtractorTrack::TrackType syncType )
@@ -809,8 +930,7 @@ void qtractorSession::resetAllPlugins (void)
 	// All tracks...
 	for (qtractorTrack *pTrack = m_tracks.first();
 			pTrack; pTrack = pTrack->next()) {
-		if (pTrack->pluginList())
-			pTrack->pluginList()->resetBuffer();
+		(pTrack->pluginList())->resetBuffer();
 	}
 	
 	// All audio buses...
@@ -845,8 +965,8 @@ qtractorAudioEngine *qtractorSession::audioEngine (void) const
 // Wait for application stabilization.
 void qtractorSession::stabilize ( int msecs )
 {
-#ifdef CONFIG_DEBUG
-	fprintf(stderr, "qtractorSession::stabilize(%d)\n", msecs);
+#ifdef CONFIG_DEBUG_0
+	qDebug("qtractorSession::stabilize(%d)", msecs);
 #endif
 	// Wait a litle bit before continue...
 	QTime t;
@@ -868,18 +988,47 @@ void qtractorSession::setPlaying ( bool bPlaying )
 {
 	// For all armed tracks...
 	if (bPlaying && isRecording()) {
-		unsigned long iClipStart = m_pAudioEngine->sessionCursor()->frame();
+		// Take a snapshot on where recording
+		// clips are about to start...
+		unsigned long iPlayHead  = playHead();
+		unsigned long iClipStart = iPlayHead;
+		if (isPunching()) {
+			unsigned long iPunchIn = punchIn();
+			if (iClipStart < iPunchIn)
+				iClipStart = iPunchIn;
+		}
+		// Of course, mark those clips alright...
 		for (qtractorTrack *pTrack = m_tracks.first();
 				pTrack; pTrack = pTrack->next()) {
 			qtractorClip *pClipRecord = pTrack->clipRecord();
-			if (pClipRecord)
+			if (pClipRecord) {
 				pClipRecord->setClipStart(iClipStart);
+				// MIDI adjust to playing queue start...
+				if (pTrack->trackType() == qtractorTrack::Midi
+					&& iClipStart > iPlayHead) {
+					qtractorMidiClip *pMidiClip
+						= static_cast<qtractorMidiClip *> (pClipRecord);
+					if (pMidiClip) {
+						pMidiClip->sequence()->setTimeOffset(
+							tickFromFrame(iClipStart - iPlayHead));
+					}
+				}
+			}
 		}
 	}
 
 	// Do it.
 	m_pAudioEngine->setPlaying(bPlaying);
 	m_pMidiEngine->setPlaying(bPlaying);
+
+	// Have all MIDI instrument plugins be shut...
+	if (!bPlaying) {
+		qtractorMidiManager *pMidiManager = m_midiManagers.first();
+		while (pMidiManager) {
+			pMidiManager->reset();
+			pMidiManager = pMidiManager->next();
+		}
+	}
 }
 
 bool qtractorSession::isPlaying() const
@@ -1017,6 +1166,63 @@ bool qtractorSession::isLooping (void) const
 }
 
 
+// Session punch points accessors.
+void qtractorSession::setPunch (
+	unsigned long iPunchIn, unsigned long iPunchOut )
+{
+	// Local prepare...
+	if (iPunchIn >= iPunchOut) {
+		iPunchIn  = 0;
+		iPunchOut = 0;
+	}
+
+	// Local commit...
+	m_iPunchIn  = iPunchIn;
+	m_iPunchOut = iPunchOut;
+
+	// Time-normalized references too...
+	m_iPunchInTime  = tickFromFrame(iPunchIn);
+	m_iPunchOutTime = tickFromFrame(iPunchOut);
+}
+
+unsigned long qtractorSession::punchIn (void) const
+{
+	return m_iPunchIn;
+}
+
+unsigned long qtractorSession::punchOut (void) const
+{
+	return m_iPunchOut;
+}
+
+bool qtractorSession::isPunching (void) const
+{
+	return (m_iPunchIn < m_iPunchOut) && !isLooping();
+}
+
+
+unsigned long qtractorSession::punchInTime (void) const
+{
+	return m_iPunchInTime;
+}
+
+unsigned long qtractorSession::punchOutTime (void) const
+{
+	return m_iPunchOutTime;
+}
+
+
+unsigned long qtractorSession::frameTime (void) const
+{
+	return	m_pAudioEngine->sessionCursor()->frameTime();
+}
+
+unsigned long qtractorSession::framePos (void) const
+{
+	return	m_pAudioEngine->sessionCursor()->framePos();
+}
+
+
 // Sanitize a given name.
 QString qtractorSession::sanitize ( const QString& s )
 {
@@ -1026,10 +1232,12 @@ QString qtractorSession::sanitize ( const QString& s )
 
 // Create a brand new filename (absolute file path).
 QString qtractorSession::createFilePath ( const QString& sTrackName,
-	int iClipNo, const QString& sExt ) 
+	int iClipNo, const QString& sExt )
 {
-	QString sFilename = qtractorSession::sanitize(m_props.sessionName)
-		+ '-' + qtractorSession::sanitize(sTrackName) + "-%1." + sExt;
+	QString sFilename = qtractorSession::sanitize(m_props.sessionName);
+	if (!sFilename.isEmpty())
+		sFilename += '-';
+	sFilename += qtractorSession::sanitize(sTrackName) + "-%1." + sExt;
 
 	if (iClipNo < 1)
 		iClipNo++;
@@ -1039,7 +1247,7 @@ QString qtractorSession::createFilePath ( const QString& sTrackName,
 		fi.setFile(m_props.sessionDir, sFilename.arg(++iClipNo));
 
 #ifdef CONFIG_DEBUG
-	fprintf(stderr, "qtractorSession::createFilePath(\"%s\")\n",
+	qDebug("qtractorSession::createFilePath(\"%s\")",
 		fi.absoluteFilePath().toUtf8().constData());
 #endif
 
@@ -1070,7 +1278,7 @@ bool qtractorSession::isRecording() const
 void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
 {
 #ifdef CONFIG_DEBUG
-	fprintf(stderr, "qtractorSession::trackRecord(\"%s\", %d)\n",
+	qDebug("qtractorSession::trackRecord(\"%s\", %d)",
 		pTrack->trackName().toUtf8().constData(), (int) bRecord);
 #endif
 
@@ -1080,37 +1288,79 @@ void qtractorSession::trackRecord ( qtractorTrack *pTrack, bool bRecord )
 		// Check whether we set recording off...
 		if (recordTracks() < 1)
 			setRecording(false);
+		// One-down current tracks in record mode.
+		switch (pTrack->trackType()) {
+		case qtractorTrack::Audio:
+			m_iAudioRecord--;
+			break;
+		case qtractorTrack::Midi:
+			m_iMidiRecord--;
+			break;
+		default:
+			break;
+		}
+	#if 0
+		// Re-sync as appropriate...
+		if (isPlaying())
+			trackMute(pTrack, false);
+	#endif
 		// Done.
 		return;
 	}
 
 	// Here's the place to create and set the capture clip...
+	unsigned long iClipStart = playHead();
+
+	if (isPunching()) {
+		unsigned long iPunchIn = punchIn();
+		if (iClipStart < iPunchIn)
+			iClipStart = iPunchIn;
+	}
+
 	switch (pTrack->trackType()) {
 	case qtractorTrack::Audio:
 	{
 		qtractorAudioClip *pAudioClip = new qtractorAudioClip(pTrack);
-		pAudioClip->setClipStart(playHead());
+		pAudioClip->setClipStart(iClipStart);
 		pAudioClip->openAudioFile(
 			createFilePath(pTrack->trackName(), 0,
 				qtractorAudioFileFactory::defaultExt()),
 			qtractorAudioFile::Write);
 		pTrack->setClipRecord(pAudioClip);
+		// One-up audio tracks in record mode.
+		m_iAudioRecord++;
 		break;
 	}
 	case qtractorTrack::Midi:
 	{
 		qtractorMidiClip *pMidiClip = new qtractorMidiClip(pTrack);
-		pMidiClip->setClipStart(playHead());
+		pMidiClip->setClipStart(iClipStart);
 		pMidiClip->openMidiFile(
 			createFilePath(pTrack->trackName(), 0, "mid"),
 			qtractorMidiClip::defaultFormat(),
 			qtractorMidiFile::Write);
 		pTrack->setClipRecord(pMidiClip);
+		// MIDI adjust to playing queue start
+		// iif armed while already playing ...
+		if (isPlaying()) {
+			unsigned long iTime = tickFromFrame(iClipStart);
+			unsigned long iTimeStart = m_pMidiEngine->timeStart();
+			if (iTime > iTimeStart)
+				pMidiClip->sequence()->setTimeOffset(iTime - iTimeStart);
+		}
+		// One-up MIDI tracks in record mode.
+		m_iMidiRecord++;
 		break;
 	}
 	default:
 		break;
 	}
+
+#if 0
+	// Mute track as appropriate...
+	if (isPlaying())
+		trackMute(pTrack, true);
+#endif
 }
 
 // Immediate track mute (engine indirection).
@@ -1119,10 +1369,10 @@ void qtractorSession::trackMute ( qtractorTrack *pTrack, bool bMute )
 	// For the time being, only needed for ALSA sequencer...
 	if (pTrack->trackType() == qtractorTrack::Midi)
 		m_pMidiEngine->trackMute(pTrack, bMute);
-
-	// But not, plugins must also be muted (sort of)...
-	if (bMute && pTrack->pluginList())
-		pTrack->pluginList()->resetBuffer();
+	else if (bMute) // Audio plugins must also be muted (sort of)...
+		(pTrack->pluginList())->resetBuffer();
+	else // Muted audio tracks must re-synchronize...
+		m_pAudioEngine->sessionCursor()->updateTrack(pTrack);
 }
 
 
@@ -1140,14 +1390,26 @@ void qtractorSession::trackSolo ( qtractorTrack *pTrack, bool bSolo )
 	for (qtractorTrack *pTrackMute = m_tracks.first();
 			pTrackMute; pTrackMute = pTrackMute->next()) {
 		// For all other track, but this one.
-		if (pTrack == pTrackMute || pTrackMute->isMute())
+		if (pTrackMute->isMute())
 			continue;
 		// (Un)mute each other track...
-		trackMute(pTrackMute, bSolo);
+		trackMute(pTrackMute, pTrackMute == pTrack ? !bSolo : bSolo);
 	}
 }
 
 
+// Track recording specifics.
+unsigned short qtractorSession::audioRecord (void) const
+{
+	return m_iAudioRecord;
+}
+
+unsigned short qtractorSession::midiRecord (void) const
+{
+	return m_iMidiRecord;
+}
+
+
 // Audio peak factory accessor.
 qtractorAudioPeakFactory *qtractorSession::audioPeakFactory (void) const
 {
@@ -1185,13 +1447,45 @@ void qtractorSession::releaseMidiTag ( qtractorTrack *pTrack )
 
 
 // MIDI session/tracks instrument patching.
-void qtractorSession::setMidiPatch ( qtractorInstrumentList *pInstruments )
+void qtractorSession::setMidiPatch (void)
 {
 	for (qtractorTrack *pTrack = m_tracks.first();
 			pTrack; pTrack = pTrack->next()) {
 		if (pTrack->trackType() == qtractorTrack::Midi)
-			pTrack->setMidiPatch(pInstruments);
+			pTrack->setMidiPatch(m_pInstruments);
 	}
+
+	m_pMidiEngine->resetAllControllers();
+}
+
+
+// MIDI manager list accessors.
+qtractorMidiManager *qtractorSession::createMidiManager (
+	qtractorPluginList *pPluginList )
+{
+	qtractorMidiManager *pMidiManager
+		= new qtractorMidiManager(this, pPluginList);
+	m_midiManagers.append(pMidiManager);
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorSession[%p]::createMidiManager(%p)", this, pMidiManager);
+#endif
+	return pMidiManager;
+}
+
+void qtractorSession::deleteMidiManager (
+	qtractorMidiManager *pMidiManager )
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorSession[%p]::deleteMidiManager(%p)", this, pMidiManager);
+#endif
+	m_midiManagers.remove(pMidiManager);
+	delete pMidiManager;
+}
+
+
+const qtractorList<qtractorMidiManager>& qtractorSession::midiManagers (void) const
+{
+	return m_midiManagers;
 }
 
 
@@ -1230,12 +1524,18 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 	QDomElement *pElement )
 {
 	qtractorSession::clear();
-	qtractorSession::setSessionName(pElement->attribute("name"));
+
+	// Templates have no session name...
+	if (!pDocument->isTemplate())
+		qtractorSession::setSessionName(pElement->attribute("name"));
 
 	// Session state should be postponed...
 	unsigned long iLoopStart = 0;
 	unsigned long iLoopEnd   = 0;
 
+	unsigned long iPunchIn   = 0;
+	unsigned long iPunchOut  = 0;
+
 	// Load session children...
 	for (QDomNode nChild = pElement->firstChild();
 			!nChild.isNull();
@@ -1284,11 +1584,15 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 					iLoopStart = eState.text().toULong();
 				else if (eState.tagName() == "loop-end")
 					iLoopEnd = eState.text().toULong();
+				else if (eState.tagName() == "punch-in")
+					iPunchIn = eState.text().toULong();
+				else if (eState.tagName() == "punch-out")
+					iPunchOut = eState.text().toULong();
 			}
 		}
 		else
 		// Load file lists...
-		if (eChild.tagName() == "files") {
+		if (eChild.tagName() == "files" && !pDocument->isTemplate()) {
 			for (QDomNode nList = eChild.firstChild();
 					!nList.isNull();
 						nList = nList.nextSibling()) {
@@ -1401,6 +1705,8 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 	// Check whether some deferred state needs to be set...
 	if (iLoopStart < iLoopEnd)
 		qtractorSession::setLoop(iLoopStart, iLoopEnd);
+	if (iPunchIn < iPunchOut)
+		qtractorSession::setPunch(iPunchIn, iPunchOut);
 
 	return true;
 }
@@ -1409,8 +1715,11 @@ bool qtractorSession::loadElement ( qtractorSessionDocument *pDocument,
 bool qtractorSession::saveElement ( qtractorSessionDocument *pDocument,
 	QDomElement *pElement )
 {
-	pElement->setAttribute("name", qtractorSession::sessionName());
-	pElement->setAttribute("version", QTRACTOR_VERSION);
+	// Templates should have no session name...
+	if (!pDocument->isTemplate())
+		pElement->setAttribute("name", qtractorSession::sessionName());
+
+	pElement->setAttribute("version", PACKAGE_STRING);
 
 	// Save session properties...
 	QDomElement eProps = pDocument->document()->createElement("properties");
@@ -1434,31 +1743,40 @@ bool qtractorSession::saveElement ( qtractorSessionDocument *pDocument,
 		QString::number(qtractorSession::loopStart()), &eState);
 	pDocument->saveTextElement("loop-end",
 		QString::number(qtractorSession::loopEnd()), &eState);
+	pDocument->saveTextElement("punch-in",
+		QString::number(qtractorSession::punchIn()), &eState);
+	pDocument->saveTextElement("punch-out",
+		QString::number(qtractorSession::punchOut()), &eState);
 	pElement->appendChild(eState);
 
-	// Save file lists...
-	QDomElement eFiles = pDocument->document()->createElement("files");
-	// Audio files...
-	QDomElement eAudioList = pDocument->document()->createElement("audio-list");
-	qtractorAudioListView *pAudioList = NULL;
-	if (pDocument->files())
-		pAudioList = pDocument->files()->audioListView();
-	if (pAudioList == NULL)
-		return false;
-	if (!pAudioList->saveElement(pDocument, &eAudioList))
-		return false;
-	eFiles.appendChild(eAudioList);
-	// MIDI files...
-	QDomElement eMidiList = pDocument->document()->createElement("midi-list");
-	qtractorMidiListView *pMidiList = NULL;
-	if (pDocument->files())
-		pMidiList = pDocument->files()->midiListView();
-	if (pMidiList == NULL)
-		return false;
-	if (!pMidiList->saveElement(pDocument, &eMidiList))
-		return false;
-	eFiles.appendChild(eMidiList);
-	pElement->appendChild(eFiles);
+	// Files are not saved when in template mode...
+	if (!pDocument->isTemplate()) {
+		// Save file lists...
+		QDomElement eFiles = pDocument->document()->createElement("files");
+		// Audio files...
+		QDomElement eAudioList
+			= pDocument->document()->createElement("audio-list");
+		qtractorAudioListView *pAudioList = NULL;
+		if (pDocument->files())
+			pAudioList = pDocument->files()->audioListView();
+		if (pAudioList == NULL)
+			return false;
+		if (!pAudioList->saveElement(pDocument, &eAudioList))
+			return false;
+		eFiles.appendChild(eAudioList);
+		// MIDI files...
+		QDomElement eMidiList
+			= pDocument->document()->createElement("midi-list");
+		qtractorMidiListView *pMidiList = NULL;
+		if (pDocument->files())
+			pMidiList = pDocument->files()->midiListView();
+		if (pMidiList == NULL)
+			return false;
+		if (!pMidiList->saveElement(pDocument, &eMidiList))
+			return false;
+		eFiles.appendChild(eMidiList);
+		pElement->appendChild(eFiles);
+	}
 
 	// Save device lists...
 	QDomElement eDevices = pDocument->document()->createElement("devices");
