@@ -36,19 +36,21 @@ class qtractorAudioEngine;
 class qtractorAudioPeakFactory;
 class qtractorSessionCursor;
 class qtractorSessionDocument;
+class qtractorMidiManager;
+class qtractorInstrumentList;
+class qtractorCommandList;
+class qtractorCommand;
 
 class QDomElement;
 
 
 //-------------------------------------------------------------------------
-// qtractorSession -- Session container.
+// qtractorSession -- Session container (singleton).
 
 class qtractorSession
 {
 public:
 
-	// Constructor.
-	qtractorSession();
 	// Default destructor.
 	~qtractorSession();
 
@@ -172,6 +174,15 @@ public:
 	void setSoloTracks(bool bSolo);
 	unsigned int soloTracks() const;
 
+	// Temporary current track accessors.
+	void setCurrentTrack(qtractorTrack *pTrack);
+	qtractorTrack *currentTrack() const;
+
+	// Temporary current track predicates.
+	bool isTrackMonitor(qtractorTrack *pTrack) const;
+	bool isTrackMidiChannel(qtractorTrack *pTrack,
+		unsigned short iChannel) const;
+
 	// Session cursor factory methods.
 	qtractorSessionCursor *createSessionCursor(unsigned long iFrame = 0,
 		qtractorTrack::TrackType syncType = qtractorTrack::None);
@@ -224,6 +235,19 @@ public:
 	unsigned long loopEnd() const;
 	bool isLooping() const;
 
+	// Session punch points accessors.
+	void setPunch(unsigned long iPunchIn, unsigned long iPunchOut);
+	unsigned long punchIn() const;
+	unsigned long punchOut() const;
+	bool isPunching() const;
+
+	unsigned long punchInTime() const;
+	unsigned long punchOutTime() const;
+
+	// Absolute frame time and offset.
+	unsigned long frameTime() const;
+	unsigned long framePos() const;
+
 	// Sanitize a given name.
 	static QString sanitize(const QString& s); 
 
@@ -235,6 +259,10 @@ public:
 	void setRecording(bool bRecording);
 	bool isRecording() const;
 
+	// Track recording specifics.
+	unsigned short audioRecord() const;
+	unsigned short midiRecord() const;
+
 	// Special track-immediate methods.
 	void trackRecord(qtractorTrack *pTrack, bool bRecord);
 	void trackMute(qtractorTrack *pTrack, bool bMute);
@@ -249,7 +277,14 @@ public:
 	void releaseMidiTag(qtractorTrack *pTrack);
 
 	// MIDI session/tracks instrument patching.
-	void setMidiPatch(qtractorInstrumentList *pInstruments);
+	void setMidiPatch();
+
+	// MIDI managers factory methods.
+	qtractorMidiManager *createMidiManager(qtractorPluginList *pPluginList);
+	void deleteMidiManager(qtractorMidiManager *pMidiManager);
+
+	// MIDI manager list accessors.
+	const qtractorList<qtractorMidiManager>& midiManagers() const;
 
 	// Auto time-stretching global flag (when tempo changes)
 	void setAutoTimeStretch(bool bAutoTimeStretch);
@@ -292,6 +327,26 @@ public:
 	// Alternate properties accessor.
 	Properties& properties();
 
+	// Session command executive (undo/redo)
+	bool execute(qtractorCommand *pCommand);
+
+	// Session command list reference (undo/redo)
+	qtractorCommandList *commands() const;
+
+	// Instrument names mapping.
+	qtractorInstrumentList *instruments() const;
+
+	// Singleton instance accessor.
+	static qtractorSession *getInstance();
+
+	// Singleton destroyer.
+	static void Destroy();
+
+protected:
+
+	// Constructor.
+	qtractorSession();
+
 private:
 
 	Properties     m_props;             // Session properties.
@@ -302,6 +357,8 @@ private:
 	unsigned int   m_iMuteTracks;       // Current number of muted tracks.
 	unsigned int   m_iSoloTracks;       // Current number of solo tracks.
 
+	qtractorTrack *m_pCurrentTrack;     // Temporary current track.
+
 	// The track list.
 	qtractorList<qtractorTrack> m_tracks;
 
@@ -315,6 +372,10 @@ private:
 	// Audio peak factory (singleton) instance.
 	qtractorAudioPeakFactory *m_pAudioPeakFactory;
 
+	// Track recording counts.
+	unsigned short m_iAudioRecord;
+	unsigned short m_iMidiRecord;
+
 	// MIDI track tagging specifics.
 	unsigned short m_iMidiTag;
 	QList<unsigned short> m_midiTags;
@@ -333,15 +394,34 @@ private:
 	unsigned long m_iLoopStartTime;
 	unsigned long m_iLoopEndTime;
 
+	// Session punch points.
+	unsigned long m_iPunchIn;
+	unsigned long m_iPunchOut;
+	// Session time-normalized punch points.
+	unsigned long m_iPunchInTime;
+	unsigned long m_iPunchOutTime;
+
 	// Consolidated record state.
 	bool m_bRecording;
 
 	// Auto time-stretching global flag (when tempo changes)
 	bool m_bAutoTimeStretch;
 
+	// MIDI plugin manager list.
+	qtractorList<qtractorMidiManager> m_midiManagers;
+
 	// RT-safeness hackish lock-mutex.
 	qtractorAtomic m_locks;
 	qtractorAtomic m_mutex;
+
+	// Instrument names mapping.
+	qtractorInstrumentList *m_pInstruments;
+
+	// Session command executive (undo/redo)
+	qtractorCommandList *m_pCommands;
+
+	// The singleton instance.
+	static qtractorSession *g_pSession;
 };
 
 
