@@ -58,6 +58,12 @@ qtractorThumbView::qtractorThumbView( QWidget *pParent )
 
 	QFrame::setFocusPolicy(Qt::ClickFocus);
 
+	// Local contents length (in frames).
+	m_iContentsLength = 0;
+
+	// Local play-head positioning.
+	m_iPlayHeadX  = 0;
+
 	m_dragState   = DragNone;
 	m_pRubberBand = new qtractorRubberBand(QRubberBand::Rectangle, this, 2);
 //	QPalette pal(m_pRubberBand->palette());
@@ -70,130 +76,56 @@ qtractorThumbView::qtractorThumbView( QWidget *pParent )
 }
 
 
-// Update thumb-position.
-void qtractorThumbView::updateThumb ( int dx )
+// (Re)create the complete view pixmap.
+void qtractorThumbView::updateContents (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	int w = QFrame::width();
+	int h = QFrame::height();
+	if (w < 1 || h < 1)
 		return;
 
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
-		return;
-
-	qtractorTracks *pTracks = pMainForm->tracks();
-	if (pTracks == NULL)
-		return;
-
-	int w = QFrame::width()  - 2;
-	int h = QFrame::height() - 2;
-
-	int w2 = 0;
-	int x2 = dx;
-	int f2 = pSession->pixelFromFrame(pSession->sessionLength());
-	if (f2 > 0) {
-		f2 += pSession->pixelFromBeat(2 * pSession->beatsPerBar());
-		w2 += (w * pTracks->trackView()->viewport()->width()) / f2;
-		x2 += (w * pTracks->trackView()->contentsX()) / f2;
-	} else {
-		w2 += w;
-	}
-
-	if (w2 < 8)
-		w2 = 8;
-	else
-	if (w2 > w)
-		w2 = w;
-
-	if (x2 < 1)
-		x2 = 1;
-	else
-	if (x2 > w - w2)
-		x2 = w - w2;
-
-	m_pRubberBand->setGeometry(x2, 1, w2, h);
-}
-
-
-// Update view-position.
-void qtractorThumbView::updateView ( int dx )
-{
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
-		return;
-
-	qtractorTracks *pTracks = pMainForm->tracks();
-	if (pTracks == NULL)
-		return;
-
-	int w = QFrame::width() - 2;
-
-	int f2 = pSession->pixelFromFrame(pSession->sessionLength())
-		+ pSession->pixelFromBeat(2 * pSession->beatsPerBar());
-
-	int cx = pTracks->trackView()->contentsX() + (dx * f2) / w;
-	int cy = pTracks->trackView()->contentsY();
-
-	if (cx < 0)
-		cx = 0;
-
-	pTracks->trackView()->setContentsPos(cx, cy);
-}
-
-
-// Session track-line paint method.
-void qtractorThumbView::paintEvent ( QPaintEvent *pPaintEvent )
-{
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
-		return;
-
-	qtractorTracks *pTracks = pMainForm->tracks();
-	if (pTracks == NULL)
-		return;
-
-	int w = QFrame::width()  - 2;
-	int h = QFrame::height() - 2;
-	if (w < 2 || h < 2) {
-		QFrame::paintEvent(pPaintEvent);
-		return;
-	}
-
-	QPainter painter(this);
-
 	const QPalette& pal = QFrame::palette();
-	painter.fillRect(1, 1, w, h, pal.dark().color());
 
-	int x2, w2;
-	int f2 = pSession->sessionLength();
-	if (f2 > 0)
-		f2 += pSession->frameFromBeat(2 * pSession->beatsPerBar());
+	m_pixmap = QPixmap(w, h);
+	m_pixmap.fill(pal.dark().color());
+
+	QPainter painter(&m_pixmap);
+	painter.initFrom(this);
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorTracks *pTracks = pMainForm->tracks();
+	if (pTracks == NULL)
+		return;
+
+	// Local contents length (in frames).
+	m_iContentsLength = pSession->sessionLength();
+	if (m_iContentsLength > 0)
+		m_iContentsLength += pSession->frameFromBeat(2 * pSession->beatsPerBar());
 	else
-		f2 += pSession->frameFromPixel(pTracks->trackView()->viewport()->width());
-	f2 /= w;
-	f2++;
+		m_iContentsLength += pSession->frameFromPixel(pTracks->trackView()->width());
 
-	int c2 = pTracks->trackView()->contentsHeight();
+	int ch = pTracks->trackView()->contentsHeight();
+	int f2 = 1 + (m_iContentsLength / w);
+	int x2, w2;
 
-	if (c2 > 0) {
+	if (ch > 0) {
 		int y2 = 1;
 		qtractorTrack *pTrack = pSession->tracks().first();
 		while (pTrack && y2 < h) {
-			int h2 = ((h * pTrack->zoomHeight()) / c2);
+			int h2 = ((h * pTrack->zoomHeight()) / ch);
 			if (h2 < 3)
 				h2 = 3;
 			qtractorClip *pClip = pTrack->clips().first();
 			while (pClip) {
-				x2 = (pClip->clipStart()  / f2);
-				w2 = (pClip->clipLength() / f2);
+				x2 = int(pClip->clipStart()  / f2);
+				w2 = int(pClip->clipLength() / f2);
 				painter.fillRect(x2, y2, w2 - 1, h2 - 1, pTrack->background());
 				pClip = pClip->next();
 			}
@@ -205,28 +137,191 @@ void qtractorThumbView::paintEvent ( QPaintEvent *pPaintEvent )
 	// Draw the loop-bound lines, if any...
 	if (pSession->isLooping()) {
 		painter.setPen(Qt::darkCyan);
-		x2 = int(pSession->loopStart()) / f2;
+		x2 = int(pSession->loopStart() / f2);
 		if (x2 < w)
-			painter.drawLine(x2, 1, x2, h);
-		x2 = int(pSession->loopEnd()) / f2;
+			painter.drawLine(x2, 0, x2, h);
+		x2 = int(pSession->loopEnd() / f2);
 		if (x2 < w)
-			painter.drawLine(x2, 1, x2, h);
+			painter.drawLine(x2, 0, x2, h);
 	}
 
-	// Finally, daw current edit-bound lines...
+	// May trigger an update now.
+	update();
+}
+
+
+// Update thumb-position.
+void qtractorThumbView::updateThumb ( int dx )
+{
+	int w = QFrame::width();
+	int h = QFrame::height();
+	if (w < 1 || h < 1)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorTracks *pTracks = pMainForm->tracks();
+	if (pTracks == NULL)
+		return;
+
+	int cw = pSession->pixelFromFrame(m_iContentsLength) + 1;
+	int x2 = dx + (w * pTracks->trackView()->contentsX()) / cw;
+	int w2 = (w * pTracks->trackView()->viewport()->width()) / cw;
+
+	if (w2 < 8)
+		w2 = 8;
+	else
+	if (w2 > w)
+		w2 = w;
+
+	if (x2 < 0)
+		x2 = 0;
+	else
+	if (x2 > w - w2)
+		x2 = w - w2;
+
+	m_pRubberBand->setGeometry(x2, 0, w2, h);
+}
+
+
+// Update playhead-position.
+void qtractorThumbView::updatePlayHead ( unsigned long iPlayHead )
+{
+	int w = QFrame::width();
+	int h = QFrame::height();
+	if (w < 1 || h < 1)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorTracks *pTracks = pMainForm->tracks();
+	if (pTracks == NULL)
+		return;
+
+	int f2 = 1 + (m_iContentsLength / w);
+
+	// Extra: update current playhead position...
+	int x2 = int(iPlayHead / f2);
+	if (m_iPlayHeadX != x2) {
+		// Override old playhead line...
+		update(QRect(m_iPlayHeadX, 0, 1, h));
+		// New position is in...
+		m_iPlayHeadX = x2;
+		// And draw it...
+		update(QRect(m_iPlayHeadX, 0, 1, h));
+	}
+}
+
+
+// Update view-position.
+void qtractorThumbView::updateView ( int dx )
+{
+	int w = QFrame::width();
+	if (w < 1)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorTracks *pTracks = pMainForm->tracks();
+	if (pTracks == NULL)
+		return;
+
+	int cw = pSession->pixelFromFrame(m_iContentsLength) + 1;
+	int cx = pTracks->trackView()->contentsX() + (dx * cw) / w;
+	int cy = pTracks->trackView()->contentsY();
+
+	if (cx < 0)
+		cx = 0;
+
+	pTracks->trackView()->setContentsPos(cx, cy);
+}
+
+
+// Set playhead-position (indirect).
+void qtractorThumbView::setPlayHeadX ( int iPlayHeadX )
+{
+	int w = QFrame::width();
+	if (w < 1)
+		return;
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorTracks *pTracks = pMainForm->tracks();
+	if (pTracks == NULL)
+		return;
+
+	int f2 = 1 + (m_iContentsLength / w);
+	pSession->setPlayHead(f2 * iPlayHeadX);
+}
+
+
+// Session track-line paint method.
+void qtractorThumbView::paintEvent ( QPaintEvent *pPaintEvent )
+{
+	QPainter painter(this);
+	
+	// Render the famous pixmap region...
+	const QRect& rect = pPaintEvent->rect();
+	painter.drawPixmap(rect, m_pixmap, rect);
+
+	int w = QFrame::width();
+	int h = QFrame::height();
+	if (w < 1 || h < 1)
+		return;
+	
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
+		return;
+
+	qtractorTracks *pTracks = pMainForm->tracks();
+	if (pTracks == NULL)
+		return;
+
+	int f2 = 1 + (m_iContentsLength / w);
+	int x2;
+
+	// Draw current edit-bound lines...
 	painter.setPen(Qt::blue);
-	x2 = int(pSession->editHead()) / f2;
-	if (x2 < w)
-		painter.drawLine(x2, 1, x2, h);
-	x2 = int(pSession->editTail()) / f2;
-	if (x2 < w)
-		painter.drawLine(x2, 1, x2, h);
+	x2 = int(pSession->editHead() / f2);
+	if (x2 >= rect.left() && x2 <= rect.right())
+		painter.drawLine(x2, 0, x2, h);
+	x2 = int(pSession->editTail() / f2);
+	if (x2 >= rect.left() && x2 <= rect.right())
+		painter.drawLine(x2, 0, x2, h);
 
-	// done for now, QFrame will need right away...
-	painter.end();
-
-	// Do it the QFrame way...
-	QFrame::paintEvent(pPaintEvent);
+	// Draw current play-head as well...
+	painter.setPen(Qt::red);
+	x2 = m_iPlayHeadX;
+	if (x2 >= rect.left() && x2 <= rect.right())
+		painter.drawLine(x2, 0, x2, h);
 }
 
 
@@ -235,6 +330,7 @@ void qtractorThumbView::resizeEvent ( QResizeEvent *pResizeEvent )
 {
 	QFrame::resizeEvent(pResizeEvent);
 
+	updateContents();
 	updateThumb();
 }
 
@@ -287,10 +383,15 @@ void qtractorThumbView::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 		const QPoint& pos = pMouseEvent->pos();
 		if (m_dragState == DragMove)
 			updateView(pos.x() - m_posDrag.x());
-		else
-		if (m_dragState == DragClick) {
-			const QRect& rect = m_pRubberBand->geometry();
-			updateView(pos.x() - ((rect.left() + rect.right()) >> 1));
+		else {
+			if (m_dragState == DragClick) {
+				const QRect& rect = m_pRubberBand->geometry();
+				updateView(pos.x() - ((rect.left() + rect.right()) >> 1));
+			}
+			// Make it change playhead?...
+			if (pMouseEvent->modifiers()
+				& (Qt::ShiftModifier | Qt::ControlModifier))
+				setPlayHeadX(pos.x());
 		}
 	}
 
