@@ -46,6 +46,12 @@ public:
 	// Constructor.
 	qtractorThumbView(QWidget *pParent = 0);
 
+	// Update playhead-position.
+	void updatePlayHead(unsigned long iPlayHead);
+
+	// (Re)create the complete view pixmap.
+	void updateContents();
+
 public slots:
 
 	// Update thumb-position.
@@ -56,6 +62,9 @@ protected:
 	// Update view-position.
 	void updateView(int dx);
 
+	// Set playhead-position (indirect).
+	void setPlayHeadX(int iPlayHeadX);
+
 	// Session track-line paint method.
 	void paintEvent(QPaintEvent *pPaintEvent);
 
@@ -75,6 +84,15 @@ protected:
 
 private:
 
+	// Local double-buffering pixmap.
+	QPixmap m_pixmap;
+
+	// Local contents length (in frames).
+	unsigned long m_iContentsLength;
+	
+	// Local playhead positioning.
+	int m_iPlayHeadX;
+
 	// The thumb rubber-band widget.
 	qtractorRubberBand *m_pRubberBand;
 
