@@ -24,10 +24,6 @@
 
 #include "qtractorTimeStretch.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 
 // Cross-correlation value calculation over the overlap period.
 //
@@ -42,10 +38,13 @@ static inline bool sse_enabled (void)
 	bool bSSE = false;
 #if defined(__GNUC__)
 #if defined(__i386__) || defined(__x86_64__)
-	unsigned int a, b, c, d;
+	unsigned int eax, ebx, ecx, edx;
 	__asm__ __volatile__ (
-		"cpuid": "=a" (a), "=b" (b), "=c" (c), "=d" (d) : "a" (1));
-	bSSE = (d & (1 << 25));
+		"movl %%ebx, %%esi\n\t" \
+		"cpuid\n\t" \
+		"xchgl %%ebx, %%esi" \
+		: "=a" (eax), "=S" (ebx), "=c" (ecx), "=d" (edx) \
+		: "0" (1));
 #endif
 #endif
 	return bSSE;
@@ -120,8 +119,9 @@ static inline double std_cross_corr (
 // Constructor.
 qtractorTimeStretch::qtractorTimeStretch (
 	unsigned short iChannels, unsigned int iSampleRate )
+	: m_iChannels(0)
 {
-	m_iChannels = iChannels;
+	setChannels(iChannels);
 
 	m_fTempo = 1.0f;
 	m_bQuickSeek = false;
@@ -569,230 +569,4 @@ void qtractorTimeStretch::calcOverlapLength (void)
 }
 
 
-//---------------------------------------------------------------------------
-// qtractorTimeStretch::FifoBuffer - Sample frame buffer (FIFO).
-//
-
-// Constructor.
-qtractorTimeStretch::FifoBuffer::FifoBuffer ( unsigned short iChannels )
-{
-	m_iChannels = 0;
-	m_iSizeInFrames = 0;
-	m_ppBuffer = NULL;
-	m_ppBufferUnaligned = NULL;
-	m_iFramesInBuffer = 0;
-	m_iFramePos = 0;
-	
-	setChannels(iChannels);
-}
-
-
-// Destructor.
-qtractorTimeStretch::FifoBuffer::~FifoBuffer (void)
-{
-	if (m_ppBuffer) {
-		delete [] m_ppBufferUnaligned;
-		delete [] m_ppBuffer;
-	}
-}
-
-
-// Sets number of channels, 1=mono, 2=stereo, ...
-void qtractorTimeStretch::FifoBuffer::setChannels ( unsigned short iChannels )
-{
-	if (m_iChannels == iChannels)
-		return;
-
-	m_iChannels = iChannels;
-	m_iSizeInFrames = 0;
-
-//	ensureCapacity(1024);
-}
-
-
-// Get the assigned number of channels, 1=mono, 2=stereo.
-unsigned short qtractorTimeStretch::FifoBuffer::channels (void) const
-{
-	return m_iChannels;
-}
-
-
-// Read frames from beginning of the sample buffer.
-unsigned int qtractorTimeStretch::FifoBuffer::readFrames (
-	float **ppFrames, unsigned int iFrames, unsigned int iOffset ) const
-{
-	unsigned int iMaxFrames
-		= (iFrames > m_iFramesInBuffer ? m_iFramesInBuffer : iFrames);
-	for (unsigned int i = 0; i < m_iChannels; ++i)
-		::memcpy(ppFrames[i], ptrBegin(i) + iOffset, iMaxFrames * sizeof(float));
-	return iMaxFrames;
-}
-
-
-// Adjusts book-keeping so that given number of frames are removed
-// from beginning of the sample buffer without copying them anywhere. 
-//
-// Used to reduce the number of samples in the buffer when accessing
-// the sample buffer directly with ptrBegin() function.
-unsigned int qtractorTimeStretch::FifoBuffer::receiveFrames (
-	float **ppFrames, unsigned int iFrames, unsigned int iOffset )
-{
-	return receiveFrames(readFrames(ppFrames, iFrames, iOffset));
-}
-
-unsigned int qtractorTimeStretch::FifoBuffer::receiveFrames (
-	unsigned int iFrames )
-{
-	if (iFrames >= m_iFramesInBuffer) {
-		unsigned int iFramesInBuffer = m_iFramesInBuffer;
-		m_iFramesInBuffer = 0;
-		return iFramesInBuffer;
-	}
-
-	m_iFramesInBuffer -= iFrames;
-	m_iFramePos += iFrames;
-
-	return iFrames;
-}
-
-
-// Write samples/frames to the end of sample frame buffer.
-unsigned int qtractorTimeStretch::FifoBuffer::writeFrames (
-	float **ppFrames, unsigned int iFrames, unsigned int iOffset )
-{
-	ensureCapacity(iFrames);
-	for (unsigned short i = 0; i < m_iChannels; ++i)
-		::memcpy(ptrEnd(i), ppFrames[i] + iOffset, iFrames * sizeof(float));
-	return iFrames;
-}
-
-
-// Adjusts the book-keeping to increase number of frames
-// in the buffer without copying any actual frames.
-//
-// This function is used to update the number of frames in the
-// sample buffer when accessing the buffer directly with ptrEnd()
-// function. Please be careful though.
-void qtractorTimeStretch::FifoBuffer::putFrames (
-	float **ppFrames, unsigned int iFrames, unsigned int iOffset )
-{
-	m_iFramesInBuffer += writeFrames(ppFrames, iFrames, iOffset);
-}
-
-void qtractorTimeStretch::FifoBuffer::putFrames ( unsigned int iFrames )
-{
-	ensureCapacity(iFrames);
-	m_iFramesInBuffer += iFrames;
-}
-
-
-// Returns a pointer to the beginning of the output samples. 
-// This function is provided for accessing the output samples directly. 
-// Please be careful for not to corrupt the book-keeping!
-//
-// When using this function to output samples, also remember
-// to remove the/ output samples from the buffer by calling 
-// the receiveFrames(iFrames) function.
-float *qtractorTimeStretch::FifoBuffer::ptrBegin (
-	unsigned short iChannel ) const
-{
-	return m_ppBuffer[iChannel] + m_iFramePos;
-}
-
-
-// Returns a pointer to the end of the used part of the sample buffer
-// (i.e. where the new samples are to be inserted). This function may
-// be used for inserting new samples into the sample buffer directly.
-// Please be careful not corrupt the book-keeping!
-//
-// When using this function as means for inserting new samples, also
-// remember to increase the sample count afterwards, by calling the 
-// putFrames(iFrames) function.
-float *qtractorTimeStretch::FifoBuffer::ptrEnd (
-	unsigned short iChannel ) const
-{
-	return m_ppBuffer[iChannel] + m_iFramePos + m_iFramesInBuffer;
-}
-
-
-// Returns the number of frames currently in the buffer
-unsigned int qtractorTimeStretch::FifoBuffer::frames (void) const
-{
-	return m_iFramesInBuffer;
-}
-
-
-// Returns nonzero if the sample frame buffer is empty
-bool qtractorTimeStretch::FifoBuffer::isEmpty (void) const
-{
-	return (m_iFramesInBuffer == 0);
-}
-
-
-// Clears the sample frame buffer.
-void qtractorTimeStretch::FifoBuffer::clear (void)
-{
-	m_iFramesInBuffer = 0;
-	m_iFramePos = 0;
-}
-
-
-// Returns the current buffer capacity in terms of frames.
-unsigned int qtractorTimeStretch::FifoBuffer::capacity (void) const
-{
-	return m_iSizeInFrames;
-}
-
-
-// Ensures that the buffer has capacity for at least this many frames.
-void qtractorTimeStretch::FifoBuffer::ensureCapacity (
-	const unsigned int iSlackCapacity )
-{
-	unsigned int iRequiredCapacity
-		= m_iFramePos + m_iFramesInBuffer + (iSlackCapacity << 1);
-
-	if (iRequiredCapacity > m_iSizeInFrames) {
-		// Realloc by making it a double :P
-		iRequiredCapacity <<= 1;
-		// Enlarge the buffer in 4KB steps (round up to next 4KB boundary)
-		unsigned int iSizeInBytes
-			= (iRequiredCapacity * sizeof(float) + 4095) & -4096;
-		// assert(iSizeInBytes % 2 == 0);
-		float **ppTemp = new float * [m_iChannels];
-		float **ppTempUnaligned = new float * [m_iChannels];
-		m_iSizeInFrames = (iSizeInBytes / sizeof(float)) + (16 / sizeof(float));
-		for (unsigned short i = 0; i < m_iChannels; ++i) {
-			ppTempUnaligned[i] = new float [m_iSizeInFrames];
-			ppTemp[i] = (float *)
-				(((unsigned long) ppTempUnaligned[i] + 15) & -16);
-			if (m_ppBuffer) {
-				if (m_iFramesInBuffer > 0) {
-					::memcpy(ppTemp[i], ptrBegin(i),
-						m_iFramesInBuffer * sizeof(float));
-				}
-				delete [] m_ppBufferUnaligned[i];
-			}
-		}
-		if (m_ppBuffer) {
-			delete [] m_ppBufferUnaligned;
-			delete [] m_ppBuffer;
-		}
-		m_ppBuffer = ppTemp;
-		m_ppBufferUnaligned = ppTempUnaligned;
-		m_iFramePos = 0;
-		// Done realloc.
-	} else if (m_iFramePos > (m_iSizeInFrames >> 2)) {
-		// Rewind the buffer by moving data...
-		if (m_iFramesInBuffer > 0) {
-			for (unsigned short i = 0; i < m_iChannels; ++i) {
-				::memmove(m_ppBuffer[i], ptrBegin(i),
-					m_iFramesInBuffer * sizeof(float));
-			}
-		}
-		// Done rewind.
-		m_iFramePos = 0;
-	}
-}
-
-
 // end of qtractorTimeStretch.cpp
