@@ -27,9 +27,10 @@
 
 #include "qtractorSessionDocument.h"
 #include "qtractorAudioEngine.h"
-#include "qtractorMidiEngine.h"
 #include "qtractorAudioMonitor.h"
+#include "qtractorMidiEngine.h"
 #include "qtractorMidiMonitor.h"
+#include "qtractorMidiBuffer.h"
 #include "qtractorInstrument.h"
 #include "qtractorPlugin.h"
 #include "qtractorMixer.h"
@@ -110,7 +111,11 @@ qtractorTrack::qtractorTrack ( qtractorSession *pSession, TrackType trackType )
 
 	m_clips.setAutoDelete(true);
 
-	m_pPluginList = new qtractorPluginList(0, 0, 0);
+	unsigned int iFlags = qtractorPluginList::Track;
+	if (trackType == qtractorTrack::Midi)
+		iFlags |= qtractorPluginList::Midi;
+
+	m_pPluginList = new qtractorPluginList(0, 0, 0, iFlags);
 
 	setHeight(48);	// Default track height (qtractorTrackList::ItemHeightBase).
 	clear();
@@ -120,13 +125,12 @@ qtractorTrack::qtractorTrack ( qtractorSession *pSession, TrackType trackType )
 qtractorTrack::~qtractorTrack (void)
 {
 	close();
+	clear();
 
 	if (m_pPluginList)
 		delete m_pPluginList;
 	if (m_pMonitor)
 		delete m_pMonitor;
-
-	clear();
 }
 
 
@@ -160,12 +164,14 @@ bool qtractorTrack::open (void)
 
 	// Depending on track type...
 	qtractorEngine *pEngine = NULL;
+	qtractorAudioEngine *pAudioEngine = m_pSession->audioEngine();
+	qtractorMidiEngine  *pMidiEngine  = m_pSession->midiEngine();
 	switch (m_props.trackType) {
 	case qtractorTrack::Audio:
-		pEngine = m_pSession->audioEngine();
+		pEngine = pAudioEngine;
 		break;
 	case qtractorTrack::Midi:
-		pEngine = m_pSession->midiEngine();
+		pEngine = pMidiEngine;
 		break;
 	default:
 		break;
@@ -220,15 +226,14 @@ bool qtractorTrack::open (void)
 	// (Re)allocate (output) monitor...
 	switch (m_props.trackType) {
 	case qtractorTrack::Audio: {
-		qtractorAudioEngine *pAudioEngine
-			= static_cast<qtractorAudioEngine *> (pEngine);
 		qtractorAudioBus *pAudioBus
 			= static_cast<qtractorAudioBus *> (m_pOutputBus);
 		if (pAudioBus) {
 			m_pMonitor = new qtractorAudioMonitor(pAudioBus->channels(),
 				m_props.gain, m_props.panning);
 			m_pPluginList->setBuffer(pAudioBus->channels(),
-				pAudioEngine->bufferSize(), pAudioEngine->sampleRate());
+				pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+				qtractorPluginList::AudioTrack);
 		}
 		break;
 	}
@@ -238,7 +243,27 @@ bool qtractorTrack::open (void)
 		if (pMidiBus) {
 			m_pMonitor = new qtractorMidiMonitor(m_pSession,
 				m_props.gain, m_props.panning);
-			m_pPluginList->setBuffer(0, 0, 0);
+		}
+		// Get audio bus as for the plugin list...
+		qtractorAudioBus *pAudioBus = NULL;
+		qtractorMidiManager *pMidiManager = m_pPluginList->midiManager();
+		if (pMidiManager)
+			pAudioBus = pMidiManager->audioOutputBus();
+		if (pAudioBus == NULL) {
+			// Output bus gets to be the first available output bus...
+			for (qtractorBus *pBus = (pAudioEngine->buses()).first();
+					pBus; pBus = pBus->next()) {
+				if (pBus->busMode() & qtractorBus::Output) {
+					pAudioBus = static_cast<qtractorAudioBus *> (pBus);
+					break;
+				}
+			}
+		}
+		// Set plugin-list buffer alright...
+		if (pAudioBus) {
+			m_pPluginList->setBuffer(pAudioBus->channels(),
+				pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+				qtractorPluginList::MidiTrack);
 		}
 		break;
 	}
@@ -325,9 +350,15 @@ void qtractorTrack::setTrackType ( qtractorTrack::TrackType trackType )
 	// Set new track type, now...
 	m_props.trackType = trackType;
 
-	// Acquire a new midi-tag...
-	if (m_props.trackType == qtractorTrack::Midi)
+	// Acquire a new midi-tag and setup new plugin-list flags...
+	unsigned int iFlags = qtractorPluginList::Track;
+	if (m_props.trackType == qtractorTrack::Midi) {
 		m_pSession->acquireMidiTag(this);
+		iFlags |= qtractorPluginList::Midi;
+	}
+
+	// (Re)set plugin-list...
+	m_pPluginList->setBuffer(0, 0, 0, iFlags);
 }
 
 
@@ -654,6 +685,7 @@ void qtractorTrack::setClipRecord ( qtractorClip *pClipRecord )
 {
 	if (m_pClipRecord)
 		delete m_pClipRecord;
+
 	m_pClipRecord = pClipRecord;
 }
 
@@ -710,61 +742,79 @@ void qtractorTrack::process ( qtractorClip *pClip,
 {
 	// Audio-buffers needs some preparation...
 	unsigned int nframes = iFrameEnd - iFrameStart;
+	qtractorAudioBus *pInputBus = NULL;
 	qtractorAudioBus *pOutputBus = NULL;
 	qtractorAudioMonitor *pAudioMonitor = NULL;
 	if (m_props.trackType == qtractorTrack::Audio) {
 		pAudioMonitor = static_cast<qtractorAudioMonitor *> (m_pMonitor);
+		// Input/Output stuff...
+		pInputBus  = static_cast<qtractorAudioBus *> (m_pInputBus);
 		pOutputBus = static_cast<qtractorAudioBus *> (m_pOutputBus);
-		if (pOutputBus == NULL)
-			return;
-		pOutputBus->buffer_prepare(nframes);
+		// Prepare this track buffer...
+		if (pOutputBus)
+			pOutputBus->buffer_prepare(nframes, isMonitor() ? pInputBus : NULL);
 	}
 
-	// Now, for every clip...
-	while (pClip && pClip->clipStart() < iFrameEnd
-		&& iFrameStart < pClip->clipStart() + pClip->clipLength()) {
-		pClip->process(iFrameStart, iFrameEnd);
-		pClip = pClip->next();
+	// Playback...
+	if (!isMute() && (!m_pSession->soloTracks() || isSolo())) {
+		// Now, for every clip...
+		while (pClip && pClip->clipStart() < iFrameEnd) {
+			if (iFrameStart < pClip->clipStart() + pClip->clipLength())
+				pClip->process(iFrameStart, iFrameEnd);
+			pClip = pClip->next();
+		}
 	}
 
 	// Audio buffers needs monitoring and commitment...
 	if (pAudioMonitor) {
-		// Check whether it ain't muted, of course...
-		bool bOutput = (pOutputBus
-			&& !isMute() && (!m_pSession->soloTracks() || isSolo()));
 		// Audio-recording?
-		if (isRecord()) {
-			qtractorAudioBus *pInputBus
-				= static_cast<qtractorAudioBus *> (m_pInputBus);
-			if (pInputBus) {
-				// Input pre-monitoring...
-				pAudioMonitor->process(
-					pInputBus->in(), nframes, pInputBus->channels());
-				// Effective audio-recording?
-				qtractorAudioClip *pAudioClip
-					= static_cast<qtractorAudioClip *> (m_pClipRecord);
-				if (pAudioClip) {
+		if (isRecord() && pInputBus) {
+			// Need the audio buffer offset on this...
+			unsigned int offset = m_pSession->audioEngine()->bufferOffset();
+			// Effective audio-recording?
+			qtractorAudioClip *pAudioClip
+				= static_cast<qtractorAudioClip *> (m_pClipRecord);
+			if (pAudioClip) {
+				// Clip recording...
+				if (m_pSession->isPunching()) {
+					// Punch-in/out recording...
+					unsigned long iPunchIn  = m_pSession->punchIn();
+					unsigned long iPunchOut = m_pSession->punchOut();
+					if (iPunchIn < iFrameEnd && iPunchOut > iFrameStart) {
+						unsigned int offs = offset;
+						unsigned int nfrs = nframes;
+						if (iPunchIn >= iFrameStart) {
+							// Punch-in...
+							offs += (iPunchIn - iFrameStart);
+							if (iPunchOut < iFrameEnd)	// Punch-out (unlikely...)
+								nfrs = (iPunchOut - iPunchIn);
+							else
+								nfrs = (iFrameEnd - iPunchIn);
+						}
+						else
+						if (iPunchOut < iFrameEnd)	// Punch-out (likely...)
+							nfrs = (iPunchOut - iFrameStart);
+						pAudioClip->write(
+							pInputBus->in(), nfrs, pInputBus->channels(), offs);
+					}
+				} else {
+					// Regular full-length recording...
 					pAudioClip->write(
-						pInputBus->in(), nframes, pInputBus->channels());
-				}
-				// Post-monitoring override...
-				if (bOutput && isMonitor()) {
-					// Plugin chain post-processing...
-					pOutputBus->buffer_prepare_in(pInputBus, nframes);
-					if (m_pPluginList->activated())
-						m_pPluginList->process(pOutputBus->buffer(), nframes);
-					pOutputBus->buffer_commit(nframes);
-					bOutput = false;
+						pInputBus->in(), nframes, pInputBus->channels(), offset);
 				}
 			}
+			// Record non-passthru metering...
+			pAudioMonitor->process_meter(
+				pInputBus->in(), nframes, pInputBus->channels(), offset);
 		}
-		// Audio-output...
-		if (bOutput) {
-			// Output pre-monitoring...
-			pAudioMonitor->process(pOutputBus->buffer(), nframes);
+		// Output monitor processing...
+		if (pOutputBus) {
 			// Plugin chain post-processing...
-			if (m_pPluginList->activated())
+			if (m_pPluginList->activated() > 0)
 				m_pPluginList->process(pOutputBus->buffer(), nframes);
+			// Monitor passthru...
+			pAudioMonitor->process(pOutputBus->buffer(), nframes);
+			// Actually render it...
 			pOutputBus->buffer_commit(nframes);
 		}
 	}
@@ -868,7 +918,7 @@ void qtractorTrack::setMidiPatch ( qtractorInstrumentList *pInstruments )
 		iBankSelMethod = (*pInstruments)[patch.instrumentName].bankSelMethod();
 
 	pMidiBus->setPatch(midiChannel(), patch.instrumentName,
-		iBankSelMethod, midiBank(), midiProgram());
+		iBankSelMethod, midiBank(), midiProgram(), this);
 }
 
 
@@ -972,7 +1022,7 @@ bool qtractorTrack::loadElement ( qtractorSessionDocument *pDocument,
 		}
 		else
 		// Load clips...
-		if (eChild.tagName() == "clips") {
+		if (eChild.tagName() == "clips" && !pDocument->isTemplate()) {
 			for (QDomNode nClip = eChild.firstChild();
 					!nClip.isNull();
 						nClip = nClip.nextSibling()) {
@@ -1070,18 +1120,21 @@ bool qtractorTrack::saveElement ( qtractorSessionDocument *pDocument,
 		qtractorTrack::foreground().name(), &eView);
 	pElement->appendChild(eView);
 
-	// Save track clips...
-	QDomElement eClips = pDocument->document()->createElement("clips");
-	for (qtractorClip *pClip = qtractorTrack::clips().first();
-			pClip; pClip = pClip->next()) {
-		// Create the new clip element...
-		QDomElement eClip = pDocument->document()->createElement("clip");
-		if (!pClip->saveElement(pDocument, &eClip))
-			return false;
-		// Add this clip...
-		eClips.appendChild(eClip);
+	// Clips are not saved when in template mode...
+	if (!pDocument->isTemplate()) {
+		// Save track clips...
+		QDomElement eClips = pDocument->document()->createElement("clips");
+		for (qtractorClip *pClip = qtractorTrack::clips().first();
+				pClip; pClip = pClip->next()) {
+			// Create the new clip element...
+			QDomElement eClip = pDocument->document()->createElement("clip");
+			if (!pClip->saveElement(pDocument, &eClip))
+				return false;
+			// Add this clip...
+			eClips.appendChild(eClip);
+		}
+		pElement->appendChild(eClips);
 	}
-	pElement->appendChild(eClips);
 
 	// Save track plugins...
 	QDomElement ePlugins = pDocument->document()->createElement("plugins");
