@@ -59,12 +59,13 @@ bool qtractorTrackCommand::addTrack (void)
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorTrackCommand::addTrack(%p)", m_pTrack);
 #endif
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return false;
 
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
 		return false;
 
 	qtractorTracks *pTracks = pMainForm->tracks();
@@ -80,8 +81,6 @@ bool qtractorTrackCommand::addTrack (void)
 	pSession->insertTrack(m_pTrack, pAfterTrack);
 	// And the new track list view item too...
 	iTrack = pTracks->trackList()->insertTrack(iTrack, m_pTrack);
-	if (iTrack >= 0)
-		pTracks->trackList()->setCurrentTrackRow(iTrack);
 	// Special MIDI track cases...
 	if (m_pTrack->trackType() == qtractorTrack::Midi)
 	    pTracks->updateMidiTrack(m_pTrack);
@@ -91,6 +90,9 @@ bool qtractorTrackCommand::addTrack (void)
 	if (pMixer)
 		pMixer->updateTracks();
 
+	// Let the change get visible.
+	pTracks->trackList()->setCurrentTrackRow(iTrack);
+
 	// Avoid disposal of the track reference.
 	setAutoDelete(false);
 
@@ -103,12 +105,13 @@ bool qtractorTrackCommand::removeTrack (void)
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorTrackCommand::removeTrack(%p)", m_pTrack);
 #endif
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return false;
 
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
 		return false;
 
 	qtractorTracks *pTracks = pMainForm->tracks();
@@ -201,12 +204,12 @@ qtractorMoveTrackCommand::qtractorMoveTrackCommand (
 // Track-move command methods.
 bool qtractorMoveTrackCommand::redo (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
 		return false;
 
-	qtractorSession *pSession = pMainForm->session();
-	if (pSession == NULL)
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm == NULL)
 		return false;
 
 	qtractorTracks *pTracks = pMainForm->tracks();
@@ -293,15 +296,12 @@ qtractorImportTrackCommand::qtractorImportTrackCommand (void)
 	m_iSaveCount   = 0;
 	m_pSaveCommand = NULL;
 
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorSession *pSession = pMainForm->session();
-		if (pSession) {
-			m_sessionProps = pSession->properties();
-			m_pSaveCommand
-				= new qtractorPropertyCommand<qtractorSession::Properties>
-					(name(), pSession->properties(), m_sessionProps);
-		}
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession) {
+		m_sessionProps = pSession->properties();
+		m_pSaveCommand
+			= new qtractorPropertyCommand<qtractorSession::Properties>
+				(name(), pSession->properties(), m_sessionProps);
 	}
 }
 
@@ -380,6 +380,10 @@ qtractorEditTrackCommand::qtractorEditTrackCommand (
 // Overridden track-edit command methods.
 bool qtractorEditTrackCommand::redo (void)
 {
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	if (pMainForm == NULL)
 		return false;
@@ -388,6 +392,11 @@ bool qtractorEditTrackCommand::redo (void)
 	if (pTracks == NULL)
 		return false;
 
+	// Howdy, maybe we're already have a name on recording...
+	bool bRecord = m_pTrack->isRecord();
+	if (bRecord)
+		pSession->trackRecord(m_pTrack, false);
+
 	// Make the track property change...
 	bool bResult = qtractorPropertyCommand<qtractorTrack::Properties>::redo();
 	// Reopen to assign a probable new bus...
@@ -402,6 +411,9 @@ bool qtractorEditTrackCommand::redo (void)
 				.arg(m_pTrack->inputBusName())
 				.arg(m_pTrack->outputBusName()));
 	}
+	else // Reassign recording...
+	if (bRecord)
+		pSession->trackRecord(m_pTrack, true);
 
 	// Refresh track item, at least the names...
 	pTracks->trackList()->updateTrack(m_pTrack);
@@ -416,6 +428,11 @@ bool qtractorEditTrackCommand::redo (void)
 	return bResult;
 }
 
+bool qtractorEditTrackCommand::undo (void)
+{
+	return redo();
+}
+
 
 //----------------------------------------------------------------------
 // class qtractorTrackButtonCommand - implementation.
@@ -444,6 +461,22 @@ qtractorTrackButtonCommand::qtractorTrackButtonCommand (
 		break;
 	}
 
+	// Toggle/update all other?
+	Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+	if (m_toolType != qtractorTrack::Record
+		&& (modifiers & (Qt::ShiftModifier | Qt::ControlModifier))) {
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession) {
+			if (modifiers & Qt::ControlModifier)
+				bOn = !bOn;
+			for (qtractorTrack *pTrackEx = pSession->tracks().first();
+					pTrackEx; pTrackEx = pTrackEx->next()) {
+				if (pTrackEx != track())
+					m_tracks.append(new TrackItem(pTrackEx, bOn));
+			}
+		}
+	}
+
 	setRefresh(false);
 }
 
@@ -452,12 +485,18 @@ qtractorTrackButtonCommand::~qtractorTrackButtonCommand (void)
 {
 	if (m_pClipCommand)
 		delete m_pClipCommand;
+
+	qDeleteAll(m_tracks);
 }
 
 
 // Track-button command method.
 bool qtractorTrackButtonCommand::redo (void)
 {
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	if (pMainForm == NULL)
 		return false;
@@ -465,8 +504,8 @@ bool qtractorTrackButtonCommand::redo (void)
 	qtractorTrack *pTrack = track();
 	if (pTrack == NULL)
 		return false;
-	
-	bool bOn = false;	
+
+	bool bOn = false;
 	qtractorMmcEvent::SubCommand scmd = qtractorMmcEvent::TRACK_NONE;
 
 	switch (m_toolType) {
@@ -476,7 +515,7 @@ bool qtractorTrackButtonCommand::redo (void)
 		bOn = pTrack->isRecord();
 		if (bOn && !m_bOn
 			&& m_pClipCommand == NULL && m_iRecordCount == 0) {
-			m_pClipCommand = new qtractorClipCommand(QString::null);
+			m_pClipCommand = new qtractorClipCommand(QString());
 			// Do all the record stuffing here...
 			if (m_pClipCommand->addClipRecord(pTrack)) {
 				// Yes, we've recorded something...
@@ -505,30 +544,58 @@ bool qtractorTrackButtonCommand::redo (void)
 		pTrack->setSolo(m_bOn);
 		break;
 	}
-	
-	// Track-list update...
-	qtractorSession *pSession = pMainForm->session();
-	qtractorTracks  *pTracks  = pMainForm->tracks();
-	if (pSession && pTracks) {
-		// Send MMC MASKED_WRITE command...
-		pSession->midiEngine()->sendMmcMaskedWrite(scmd,
-			pTracks->trackList()->trackRow(pTrack), m_bOn);
-		// Update track list item...
-		pTracks->trackList()->updateTrack(pTrack);
-	}
 
-	// Mixer turn...
-	qtractorMixer *pMixer = pMainForm->mixer();
-	if (pMixer) {
-		qtractorMixerStrip *pStrip
-			= pMixer->trackRack()->findStrip(pTrack->monitor());
-		if (pStrip)
-			pStrip->updateTrackButtons();
-	}
+	// Track-list and mixer update...
+	qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
+	qtractorTrackList  *pTrackList  = pMainForm->tracks()->trackList();
+
+	// Send MMC MASKED_WRITE command...
+	pMidiEngine->sendMmcMaskedWrite(scmd,
+		pTrackList->trackRow(pTrack), m_bOn);
+	// Update track list item...
+	pTrackList->updateTrack(pTrack);
 
 	// Reset for undo.
 	m_bOn = bOn;
 
+	// Toggle/update all other?
+	qtractorMixerRack *pTrackRack = pMainForm->mixer()->trackRack();
+	if (m_tracks.isEmpty()) {
+		// Update one track mixer strip...
+		qtractorMixerStrip *pStrip
+			= pTrackRack->findStrip(pTrack->monitor());
+		if (pStrip)
+			pStrip->updateTrackButtons();
+		// Done with single mode.
+	} else {
+		// Exclusive mode.
+		QListIterator<TrackItem *> iter(m_tracks);
+		while (iter.hasNext()) {
+			TrackItem *pTrackItem = iter.next();
+			pTrack = pTrackItem->track;
+			bOn = false;
+			if (m_toolType == qtractorTrack::Mute) {
+				bOn = pTrack->isMute();
+				pTrack->setMute(pTrackItem->on);
+			}
+			else
+			if (m_toolType == qtractorTrack::Solo) {
+				bOn = pTrack->isSolo();
+				pTrack->setSolo(pTrackItem->on);
+			}
+			// Send MMC MASKED_WRITE command...
+			pMidiEngine->sendMmcMaskedWrite(scmd,
+				pTrackList->trackRow(pTrack), pTrackItem->on);
+			// Update track list item...
+			pTrackList->updateTrack(pTrack);
+			// Swap for undo...
+			pTrackItem->on = bOn;
+		}
+		// Update all track mixer strips...
+		pTrackRack->updateTrackButtons();
+		// Done with exclusive mode.
+	}
+
 	return true;
 }
 
@@ -552,13 +619,39 @@ qtractorTrackMonitorCommand::qtractorTrackMonitorCommand (
 {
 	m_bMonitor = bMonitor;
 
+	// Toggle/update all other?
+	Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+	if (modifiers & (Qt::ShiftModifier | Qt::ControlModifier)) {
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession) {
+			if (modifiers & Qt::ControlModifier)
+				bMonitor = !bMonitor;
+			for (qtractorTrack *pTrackEx = pSession->tracks().first();
+					pTrackEx; pTrackEx = pTrackEx->next()) {
+				if (pTrackEx != pTrack)
+					m_tracks.append(new TrackItem(pTrackEx, bMonitor));
+			}
+		}
+	}
+
 	setRefresh(false);
 }
 
 
+// Destructor.
+qtractorTrackMonitorCommand::~qtractorTrackMonitorCommand (void)
+{
+	qDeleteAll(m_tracks);
+}
+
+
 // Track-monitor command method.
 bool qtractorTrackMonitorCommand::redo (void)
 {
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
 	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	if (pMainForm == NULL)
 		return false;
@@ -574,10 +667,30 @@ bool qtractorTrackMonitorCommand::redo (void)
 	// Set undo value...
 	m_bMonitor = bMonitor;
 
-	// Update (reset) all applicable mixer strips...
-	qtractorMixer *pMixer = pMainForm->mixer();
-	if (pMixer)
-		pMixer->updateTrackStrip(pTrack, true);
+	// Toggle/update all other?
+	qtractorMixerRack *pTrackRack = pMainForm->mixer()->trackRack();
+	if (m_tracks.isEmpty()) {
+		// Update one track mixer strip...
+		qtractorMixerStrip *pStrip
+			= pTrackRack->findStrip(pTrack->monitor());
+		if (pStrip)
+			pStrip->updateMonitorButton();
+		// Done with single mode.
+	} else {
+		// Exclusive mode.
+		QListIterator<TrackItem *> iter(m_tracks);
+		while (iter.hasNext()) {
+			TrackItem *pTrackItem = iter.next();
+			pTrack = pTrackItem->track;
+			bMonitor = pTrack->isMonitor();
+			pTrack->setMonitor(pTrackItem->on);
+			// Swap for undo...
+			pTrackItem->on = bMonitor;
+		}
+		// Update all track mixer strips...
+		pTrackRack->updateMonitorButtons();
+		// Done with exclusive mode.
+	}
 
 	return true;
 }
@@ -599,10 +712,10 @@ qtractorTrackGainCommand::qtractorTrackGainCommand (
 
 	// Try replacing an previously equivalent command...
 	static qtractorTrackGainCommand *s_pPrevGainCommand = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (s_pPrevGainCommand && pMainForm) {
+	if (s_pPrevGainCommand) {
+		qtractorSession *pSession = qtractorSession::getInstance();
 		qtractorCommand *pLastCommand
-			= pMainForm->commands()->lastCommand();
+			= (pSession->commands())->lastCommand();
 		qtractorCommand *pPrevCommand
 			= static_cast<qtractorCommand *> (s_pPrevGainCommand);
 		if (pPrevCommand == pLastCommand
@@ -618,7 +731,7 @@ qtractorTrackGainCommand::qtractorTrackGainCommand (
 				if (iPrevSign == iCurrSign) {
 					m_fPrevGain = fLastGain;
 					m_bPrevGain = true;
-					pMainForm->commands()->removeLastCommand();
+					(pSession->commands())->removeLastCommand();
 				}
 			}
 		}
@@ -630,10 +743,6 @@ qtractorTrackGainCommand::qtractorTrackGainCommand (
 // Track-gain command method.
 bool qtractorTrackGainCommand::redo (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
 	qtractorTrack *pTrack = track();
 	if (pTrack == NULL)
 		return false;
@@ -647,7 +756,7 @@ bool qtractorTrackGainCommand::redo (void)
 		qtractorMidiBus *pMidiBus
 			= static_cast<qtractorMidiBus *> (pTrack->outputBus());
 		if (pMidiBus)
-			pMidiBus->setVolume(pTrack->midiChannel(), m_fGain);
+			pMidiBus->setVolume(pTrack, m_fGain);
 	}
 
 	// Set undo value...
@@ -656,12 +765,15 @@ bool qtractorTrackGainCommand::redo (void)
 	m_fGain     = fGain;
 
 	// Mixer/Meter turn...
-	qtractorMixer *pMixer = pMainForm->mixer();
-	if (pMixer) {
-		qtractorMixerStrip *pStrip
-			= pMixer->trackRack()->findStrip(pTrack->monitor());
-		if (pStrip && pStrip->meter())
-			pStrip->meter()->updateGain();
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		qtractorMixer *pMixer = pMainForm->mixer();
+		if (pMixer) {
+			qtractorMixerStrip *pStrip
+				= pMixer->trackRack()->findStrip(pTrack->monitor());
+			if (pStrip && pStrip->meter())
+				pStrip->meter()->updateGain();
+		}
 	}
 
 	return true;
@@ -685,10 +797,10 @@ qtractorTrackPanningCommand::qtractorTrackPanningCommand (
 
 	// Try replacing an previously equivalent command...
 	static qtractorTrackPanningCommand *s_pPrevPanningCommand = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (s_pPrevPanningCommand && pMainForm) {
+	if (s_pPrevPanningCommand) {
+		qtractorSession *pSession = qtractorSession::getInstance();
 		qtractorCommand *pLastCommand
-			= pMainForm->commands()->lastCommand();
+			= (pSession->commands())->lastCommand();
 		qtractorCommand *pPrevCommand
 			= static_cast<qtractorCommand *> (s_pPrevPanningCommand);
 		if (pPrevCommand == pLastCommand
@@ -704,7 +816,7 @@ qtractorTrackPanningCommand::qtractorTrackPanningCommand (
 				if (iPrevSign == iCurrSign) {
 					m_fPrevPanning = fLastPanning;
 					m_bPrevPanning = true;
-					pMainForm->commands()->removeLastCommand();
+					(pSession->commands())->removeLastCommand();
 				}
 			}
 		}
@@ -716,10 +828,6 @@ qtractorTrackPanningCommand::qtractorTrackPanningCommand (
 // Track-panning command method.
 bool qtractorTrackPanningCommand::redo (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
 	qtractorTrack *pTrack = track();
 	if (pTrack == NULL)
 		return false;
@@ -733,7 +841,7 @@ bool qtractorTrackPanningCommand::redo (void)
 		qtractorMidiBus *pMidiBus
 			= static_cast<qtractorMidiBus *> (pTrack->outputBus());
 		if (pMidiBus)
-			pMidiBus->setPanning(pTrack->midiChannel(), m_fPanning);
+			pMidiBus->setPanning(pTrack, m_fPanning);
 	}
 	
 	// Set undo value...
@@ -742,12 +850,15 @@ bool qtractorTrackPanningCommand::redo (void)
 	m_fPanning     = fPanning;
 
 	// Mixer/Meter turn...
-	qtractorMixer *pMixer = pMainForm->mixer();
-	if (pMixer) {
-		qtractorMixerStrip *pStrip
-			= pMixer->trackRack()->findStrip(pTrack->monitor());
-		if (pStrip && pStrip->meter())
-			pStrip->meter()->updatePanning();
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		qtractorMixer *pMixer = pMainForm->mixer();
+		if (pMixer) {
+			qtractorMixerStrip *pStrip
+				= pMixer->trackRack()->findStrip(pTrack->monitor());
+			if (pStrip && pStrip->meter())
+				pStrip->meter()->updatePanning();
+		}
 	}
 
 	return true;
