@@ -29,7 +29,11 @@
 #include "qtractorAudioEngine.h"
 #include "qtractorMidiEngine.h"
 
-#include "qtractorMainForm.h"
+#include "qtractorMidiBuffer.h"
+#include "qtractorPlugin.h"
+
+#include "qtractorCommand.h"
+
 #include "qtractorBusForm.h"
 
 #include <QColorDialog>
@@ -38,6 +42,11 @@
 #include <QValidator>
 #include <QPainter>
 
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------
 // class qtractorColorItemDelegate -- Custom color view item delegate.
@@ -94,6 +103,9 @@ qtractorTrackForm::qtractorTrackForm (
 	// No settings descriptor initially (the caller will set it).
 	m_pTrack = NULL;
 
+	// Current MIDI output bus to be cached.
+	m_pMidiBus = NULL;
+
 	// Set some dialog validators...
 	m_ui.BankComboBox->setValidator(new QIntValidator(m_ui.BankComboBox));
 	m_ui.ProgComboBox->setValidator(new QIntValidator(m_ui.ProgComboBox));
@@ -115,7 +127,7 @@ qtractorTrackForm::qtractorTrackForm (
 	m_ui.BackgroundColorComboBox->clear();
 	for (int i = 1; i < 28; i++) {
 		const QColor rgbBack = qtractorTrack::trackColor(i);
-		const QColor rgbFore = rgbBack.dark();
+		const QColor rgbFore = rgbBack.darker();
 		m_ui.ForegroundColorComboBox->addItem(rgbFore.name());
 		m_ui.BackgroundColorComboBox->addItem(rgbBack.name());
 	}
@@ -128,13 +140,13 @@ qtractorTrackForm::qtractorTrackForm (
 	m_iOldBank = -1;
 	m_iOldProg = -1;
 
+	// No last acceptable command yet.
+	m_pLastCommand = NULL;
+
 	// Initialize dirty control state.
 	m_iDirtySetup = 0;
 	m_iDirtyCount = 0;
 
-	// Already time for instrument cacheing...
-	updateInstruments();
-
 	// UI signal/slot connections...
 	QObject::connect(m_ui.TrackNameTextEdit,
 		SIGNAL(textChanged()),
@@ -184,6 +196,23 @@ qtractorTrackForm::qtractorTrackForm (
 	QObject::connect(m_ui.BackgroundColorToolButton,
 		SIGNAL(clicked()),
 		SLOT(selectBackgroundColor()));
+
+	QObject::connect(m_ui.PluginListView,
+		SIGNAL(currentRowChanged(int)),
+		SLOT(stabilizeForm()));
+	QObject::connect(m_ui.AddPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(addPlugin()));
+	QObject::connect(m_ui.RemovePluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(removePlugin()));
+	QObject::connect(m_ui.MoveUpPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(moveUpPlugin()));
+	QObject::connect(m_ui.MoveDownPluginToolButton,
+		SIGNAL(clicked()),
+		SLOT(moveDownPlugin()));
+
 	QObject::connect(m_ui.OkPushButton,
 		SIGNAL(clicked()),
 		SLOT(accept()));
@@ -202,25 +231,31 @@ qtractorTrackForm::~qtractorTrackForm (void)
 // Populate (setup) dialog controls from settings descriptors.
 void qtractorTrackForm::setTrack ( qtractorTrack *pTrack )
 {
-	// Set reference descriptor.
-	m_pTrack = pTrack;
-
 	// Avoid dirty this all up.
 	m_iDirtySetup++;
 
+	// Set target track reference descriptor.
+	m_pTrack = pTrack;
+
 	// Track properties cloning...
 	m_props = m_pTrack->properties();
 
+	// Get reference of the last acceptable command...
+	m_pLastCommand = ((m_pTrack->session())->commands())->lastCommand();
+
+	// Set plugin list...
+	m_ui.PluginListView->setPluginList(m_pTrack->pluginList());
+
 	// Initialize dialog widgets...
 	m_ui.TrackNameTextEdit->setPlainText(m_props.trackName);
 	qtractorEngine *pEngine = NULL;
 	switch (m_props.trackType) {
 	case qtractorTrack::Audio:
-		pEngine = pTrack->session()->audioEngine();
+		pEngine = (m_pTrack->session())->audioEngine();
 		m_ui.AudioRadioButton->setChecked(true);
 		break;
 	case qtractorTrack::Midi:
-		pEngine = pTrack->session()->midiEngine();
+		pEngine = (m_pTrack->session())->midiEngine();
 		m_ui.MidiRadioButton->setChecked(true);
 		break;
 	default:
@@ -235,6 +270,12 @@ void qtractorTrackForm::setTrack ( qtractorTrack *pTrack )
 		m_ui.OutputBusNameComboBox->setCurrentIndex(
 			m_ui.OutputBusNameComboBox->findText(m_props.outputBusName));
 
+	// Force MIDI output bus recaching.
+	m_pMidiBus = midiBus();
+
+	// Already time for instrument cacheing...
+	updateInstruments();
+
 	m_ui.OmniCheckBox->setChecked(m_props.midiOmni);
 	m_ui.ChannelSpinBox->setValue(m_props.midiChannel + 1);
 	updateChannel(m_ui.ChannelSpinBox->value(),
@@ -272,7 +313,7 @@ const qtractorTrack::Properties& qtractorTrackForm::properties (void) const
 
 
 // Selected track type determinator.
-qtractorTrack::TrackType qtractorTrackForm::trackType (void)
+qtractorTrack::TrackType qtractorTrackForm::trackType (void) const
 {
 	qtractorTrack::TrackType trackType = qtractorTrack::None;
 
@@ -338,10 +379,15 @@ void qtractorTrackForm::reject (void)
 	}
 
 	if (bReject) {
-		// Try to restore the previously saved patch...
-		if (m_pOldMidiBus) {
-			m_pOldMidiBus->setPatch(m_iOldChannel, m_sOldInstrumentName,
-				m_iOldBankSelMethod, m_iOldBank, m_iOldProg);
+		// Bogus track? Unlikely, but...
+		if (m_pTrack) {
+			// Backout all commands made this far...
+			((m_pTrack->session())->commands())->backout(m_pLastCommand);
+			// Try to restore the previously saved patch...
+			if (m_pOldMidiBus) {
+				m_pOldMidiBus->setPatch(m_iOldChannel, m_sOldInstrumentName,
+					m_iOldBankSelMethod, m_iOldBank, m_iOldProg, m_pTrack);
+			}
 		}
 		QDialog::reject();
 	}
@@ -355,11 +401,30 @@ void qtractorTrackForm::stabilizeForm (void)
 	bValid = bValid && !m_ui.TrackNameTextEdit->toPlainText().isEmpty();
 	bValid = bValid && trackType() != qtractorTrack::None;
 	m_ui.OkPushButton->setEnabled(bValid);
+
+	// Stabilize current plugin list state.
+	int iItem = -1;
+	int iItemCount = m_ui.PluginListView->count();
+
+	qtractorPlugin *pPlugin = NULL;
+	qtractorPluginListItem *pItem
+		= static_cast<qtractorPluginListItem *> (
+			m_ui.PluginListView->currentItem());
+	if (pItem) {
+		iItem = m_ui.PluginListView->row(pItem);
+		pPlugin = pItem->plugin();
+	}
+
+//	m_ui.AddPluginToolButton->setEnabled(true);
+	m_ui.RemovePluginToolButton->setEnabled(pPlugin != NULL);
+
+	m_ui.MoveUpPluginToolButton->setEnabled(pItem && iItem > 0);
+	m_ui.MoveDownPluginToolButton->setEnabled(pItem && iItem < iItemCount - 1);
 }
 
 
 // Retrieve currently assigned MIDI output-bus, if applicable.
-qtractorMidiBus *qtractorTrackForm::midiBus (void)
+qtractorMidiBus *qtractorTrackForm::midiBus (void) const
 {
 	if (m_pTrack == NULL)
 		return NULL;
@@ -380,7 +445,7 @@ qtractorMidiBus *qtractorTrackForm::midiBus (void)
 
 
 // Retrieve currently selected MIDI bank number.
-int qtractorTrackForm::midiBank (void)
+int qtractorTrackForm::midiBank (void) const
 {
 	int iBankIndex = m_ui.BankComboBox->currentIndex();
 	const QString& sBankText = m_ui.BankComboBox->currentText();
@@ -393,7 +458,7 @@ int qtractorTrackForm::midiBank (void)
 
 
 // Retrieve currently selected MIDI program number.
-int qtractorTrackForm::midiProgram (void)
+int qtractorTrackForm::midiProgram (void) const
 {
 	int iProgIndex = m_ui.ProgComboBox->currentIndex();
 	const QString& sProgText = m_ui.ProgComboBox->currentText();
@@ -408,11 +473,14 @@ int qtractorTrackForm::midiProgram (void)
 // Refresh instrument list.
 void qtractorTrackForm::updateInstruments (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
+	if (m_pTrack == NULL)
 		return;
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorSession *pSession = m_pTrack->session();
+	if (pSession == NULL)
+		return;
+
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
@@ -421,16 +489,43 @@ void qtractorTrackForm::updateInstruments (void)
 
 	m_ui.InstrumentComboBox->clear();
 	const QIcon& icon = QIcon(":/icons/itemInstrument.png");
-	for (qtractorInstrumentList::Iterator iter = pInstruments->begin();
-			iter != pInstruments->end(); ++iter) {
+
+	// Take care of MIDI plugin instrument names...
+	updateInstrumentsAdd(icon,
+		(m_pTrack->pluginList())->midiManager());
+
+	// And, maybe some from the MIDI output bus...
+	if (m_pMidiBus && m_pMidiBus->pluginList_out())
+		updateInstrumentsAdd(icon,
+			(m_pMidiBus->pluginList_out())->midiManager());
+
+	// Regular instrument names...
+	qtractorInstrumentList::ConstIterator iter = pInstruments->constBegin();
+	for ( ; iter != pInstruments->constEnd(); ++iter)
 		m_ui.InstrumentComboBox->addItem(icon, iter.value().instrumentName());
-	}
 
 	// Done.
 	m_iDirtySetup--;
 }
 
 
+// Refresh instrument list of given MIDI buffer manager.
+void qtractorTrackForm::updateInstrumentsAdd (
+	const QIcon& icon, qtractorMidiManager *pMidiManager )
+{
+	if (pMidiManager == NULL)
+		return;
+
+	pMidiManager->updateInstruments();
+
+	const qtractorMidiManager::Instruments& list
+		= pMidiManager->instruments();
+	qtractorMidiManager::Instruments::ConstIterator iter = list.constBegin();
+	for ( ; iter != list.constEnd(); ++iter)
+		m_ui.InstrumentComboBox->addItem(icon, iter.key());
+}
+
+
 // Update track type and buses.
 void qtractorTrackForm::updateTrackType ( qtractorTrack::TrackType trackType )
 {
@@ -497,8 +592,7 @@ void qtractorTrackForm::updateChannel ( int iChannel,
 		return;
 
 	// MIDI bus...
-	qtractorMidiBus *pMidiBus = midiBus();
-	if (pMidiBus == NULL)
+	if (m_pMidiBus == NULL)
 		return;
 
 #ifdef CONFIG_DEBUG
@@ -510,7 +604,7 @@ void qtractorTrackForm::updateChannel ( int iChannel,
 	m_iDirtySetup++;
 
 	// MIDI channel patch...
-	const qtractorMidiBus::Patch& patch = pMidiBus->patch(iChannel);
+	const qtractorMidiBus::Patch& patch = m_pMidiBus->patch(iChannel);
 	if (iBankSelMethod < 0)
 		iBankSelMethod = patch.bankSelMethod;
 #if 0
@@ -537,12 +631,13 @@ void qtractorTrackForm::updateChannel ( int iChannel,
 void qtractorTrackForm::updateBanks ( const QString& sInstrumentName,
 	int iBankSelMethod, int iBank, int iProg )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
 	if (m_pTrack == NULL)
 		return;
+
+	qtractorSession *pSession = m_pTrack->session();
+	if (pSession == NULL)
+		return;
+
 //	if (sInstrumentName.isEmpty())
 //		return;
 
@@ -551,7 +646,7 @@ void qtractorTrackForm::updateBanks ( const QString& sInstrumentName,
 		sInstrumentName.toUtf8().constData(), iBankSelMethod, iBank, iProg);
 #endif
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
@@ -566,22 +661,33 @@ void qtractorTrackForm::updateBanks ( const QString& sInstrumentName,
 	m_ui.BankComboBox->addItem(icon, tr("(None)"));
 	m_banks[iBankIndex++] = -1;
 
+	// Care of MIDI plugin instrument banks...
+	bool bMidiManager = updateBanksAdd(icon,
+		(m_pTrack->pluginList())->midiManager(),
+		sInstrumentName, iBank, iBankIndex);
+	if (!bMidiManager && m_pMidiBus && m_pMidiBus->pluginList_out()) {
+		bMidiManager = updateBanksAdd(icon,
+			(m_pMidiBus->pluginList_out())->midiManager(),
+			sInstrumentName, iBank, iBankIndex);
+	}
+
 	// Get instrument set alright...
-	if ((*pInstruments).contains(sInstrumentName)) {
+	if (!bMidiManager && pInstruments->contains(sInstrumentName)) {
 		// Instrument reference...
 		const qtractorInstrument& instr = (*pInstruments)[sInstrumentName];
 		// Bank selection method...
 		if (iBankSelMethod < 0)
 			iBankSelMethod = instr.bankSelMethod();
 		// Refresh patch bank mapping...
-		qtractorInstrumentPatches::ConstIterator it;
-		for (it = instr.patches().begin(); it != instr.patches().end(); ++it) {
+		qtractorInstrumentPatches::ConstIterator it
+			= instr.patches().constBegin();
+		for (; it != instr.patches().constEnd(); ++it) {
 			if (it.key() >= 0) {
 				m_ui.BankComboBox->addItem(icon, it.value().name());
 				m_banks[iBankIndex++] = it.key();
 			}
 		}
-#if 1
+	#if 0
 		// In case bank address is generic...
 		if (m_ui.BankComboBox->count() < 2) {
 			const qtractorInstrumentData& patch = instr.patch(iBank);
@@ -590,20 +696,29 @@ void qtractorTrackForm::updateBanks ( const QString& sInstrumentName,
 				m_banks[iBankIndex] = iBank;
 			}
 		}
-#endif
+	#endif
 		// For proper bank selection...
+		iBankIndex = -1;
 		if (iBank >= 0) {
-			const qtractorInstrumentData& bank = instr.patch(iBank);
-			iBankIndex = m_ui.BankComboBox->findText(bank.name());
+			const qtractorInstrumentData& patch = instr.patch(iBank);
+			if (!patch.name().isEmpty())
+				iBankIndex = m_ui.BankComboBox->findText(patch.name());
 		}
 	}
-	else iBankIndex = -1;
+	else if (!bMidiManager)
+		iBankIndex = -1;
 
 	// Bank selection method...
 	if (iBankSelMethod < 0)
 		iBankSelMethod = 0;
 	m_ui.BankSelMethodComboBox->setCurrentIndex(iBankSelMethod);
 
+	// If there's banks we must choose at least one...
+	if (iBank < 0 && m_banks.count() > 1) {
+		iBankIndex = 1;
+		iBank = m_banks[iBankIndex];
+	}
+
 	// Do the proper bank selection...
 	if (iBank < 0) {
 		m_ui.BankComboBox->setCurrentIndex(0);
@@ -625,12 +740,13 @@ void qtractorTrackForm::updateBanks ( const QString& sInstrumentName,
 void qtractorTrackForm::updatePrograms (  const QString& sInstrumentName,
 	int iBank, int iProg )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
 	if (m_pTrack == NULL)
 		return;
+
+	qtractorSession *pSession = m_pTrack->session();
+	if (pSession == NULL)
+		return;
+
 //	if (sInstrumentName.isEmpty())
 //		return;
 
@@ -639,7 +755,7 @@ void qtractorTrackForm::updatePrograms (  const QString& sInstrumentName,
 		sInstrumentName.toUtf8().constData(), iBank, iProg);
 #endif
 
-	qtractorInstrumentList *pInstruments = pMainForm->instruments();
+	qtractorInstrumentList *pInstruments = pSession->instruments();
 	if (pInstruments == NULL)
 		return;
 
@@ -655,15 +771,25 @@ void qtractorTrackForm::updatePrograms (  const QString& sInstrumentName,
 	m_ui.ProgComboBox->addItem(icon, tr("(None)"));
 	m_progs[iProgIndex++] = -1;
 
+	// Take care of MIDI plugin instrument programs...
+	bool bMidiManager = updateProgramsAdd(icon,
+		(m_pTrack->pluginList())->midiManager(),
+		sInstrumentName, iBank, iProg, iProgIndex);
+	if (!bMidiManager && m_pMidiBus && m_pMidiBus->pluginList_out()) {
+		bMidiManager = updateProgramsAdd(icon,
+			(m_pMidiBus->pluginList_out())->midiManager(),
+			sInstrumentName, iBank, iProg, iProgIndex);
+	}
+
 	// Get instrument set alright...
-	if ((*pInstruments).contains(sInstrumentName)) {
+	if (!bMidiManager && pInstruments->contains(sInstrumentName)) {
 		// Instrument reference...
 		const qtractorInstrument& instr = (*pInstruments)[sInstrumentName];
 		// Bank reference...
 		const qtractorInstrumentData& bank = instr.patch(iBank);
 		// Enumerate the explicit given program list...
-		qtractorInstrumentData::ConstIterator it;
-		for (it = bank.begin(); it != bank.end(); ++it) {
+		qtractorInstrumentData::ConstIterator it = bank.constBegin();
+		for (; it != bank.constEnd(); ++it) {
 			if (it.key() >= 0 && !it.value().isEmpty()) {
 				m_ui.ProgComboBox->addItem(icon, it.value());
 				m_progs[iProgIndex++] = it.key();
@@ -674,14 +800,21 @@ void qtractorTrackForm::updatePrograms (  const QString& sInstrumentName,
 		if (iProg >= 0 && bank.contains(iProg))
 			iProgIndex = m_ui.ProgComboBox->findText(bank[iProg]);
 	}
-	else iProgIndex = -1;
-
+	else if (!bMidiManager)
+		iProgIndex = -1;
+#if 0
+	// If there's programs we must choose at least one...
+	if (iProg < 0 && m_progs.count() > 1) {
+		iProgIndex = 1;
+		iProg = m_progs[iProgIndex];
+	}
+#endif
 	// In case program address is generic...
 	if (m_ui.ProgComboBox->count() < 2) {
 		// Just make a generic program list...
 		for (int i = 0; i < 128; ++i) {
 			m_ui.ProgComboBox->addItem(icon, QString("%1  - -").arg(i + 1));
-			m_progs[i] = i;
+			m_progs[i + 1] = i;
 		}
 		if (iProg >= 0)
 			iProgIndex = iProg;
@@ -701,6 +834,80 @@ void qtractorTrackForm::updatePrograms (  const QString& sInstrumentName,
 }
 
 
+// Update instrument banks.
+bool qtractorTrackForm::updateBanksAdd (
+	const QIcon& icon, qtractorMidiManager *pMidiManager,
+	const QString& sInstrumentName, int iBank, int& iBankIndex )
+{
+	if (pMidiManager == NULL)
+		return false;
+
+	const qtractorMidiManager::Instruments& list
+		= pMidiManager->instruments();
+	if (!list.contains(sInstrumentName))
+		return false;
+
+	// Refresh bank mapping...
+	const qtractorMidiManager::Banks& banks = list[sInstrumentName];
+	qtractorMidiManager::Banks::ConstIterator iter = banks.constBegin();
+	for ( ; iter != banks.constEnd(); ++iter) {
+		m_ui.BankComboBox->addItem(icon, iter.value().name);
+		m_banks[iBankIndex++] = iter.key();
+	}
+	// Reset given bank combobox index.
+	iBankIndex = -1;
+	// For proper bank selection...
+	if (banks.contains(iBank)) {
+		const qtractorMidiManager::Bank& bank = banks[iBank];
+		iBankIndex = m_ui.BankComboBox->findText(bank.name);
+	}
+
+	// Mark that we've have something.
+	return true;
+}
+
+
+// Update instrument programs.
+bool qtractorTrackForm::updateProgramsAdd (
+	const QIcon& icon, qtractorMidiManager *pMidiManager,
+	const QString& sInstrumentName, int iBank, int iProg, int& iProgIndex )
+{
+	if (pMidiManager == NULL)
+		return false;
+
+	const qtractorMidiManager::Instruments& list
+		= pMidiManager->instruments();
+	if (!list.contains(sInstrumentName))
+		return false;
+
+	// Bank reference...
+	const qtractorMidiManager::Banks& banks
+		= list[sInstrumentName];
+	if (banks.contains(iBank)) {
+		const qtractorMidiManager::Progs& progs = banks[iBank].progs;
+		// Refresh program mapping...
+		const QString sProg("%1 - %2");
+		qtractorMidiManager::Progs::ConstIterator iter = progs.constBegin();
+		for ( ; iter != progs.constEnd(); ++iter) {
+			m_ui.ProgComboBox->addItem(icon,
+				sProg.arg(iter.key()).arg(iter.value()));
+			m_progs[iProgIndex++] = iter.key();
+		}
+		// Reset given program combobox index.
+		iProgIndex = -1;
+		// For proper program selection...
+		if (progs.contains(iProg)) {
+			iProgIndex = m_ui.ProgComboBox->findText(
+				sProg.arg(iProg).arg(progs[iProg]));
+		}
+	}
+	else iProgIndex = -1;
+
+	// Mark that we've have something.
+	return true;
+}
+
+
 // Update and set a color item.
 void qtractorTrackForm::updateColorItem ( QComboBox *pComboBox,
 	const QColor& color )
@@ -782,7 +989,7 @@ void qtractorTrackForm::trackTypeChanged (void)
 	if (m_pOldMidiBus) {
 		// Restore previously current/saved patch...
 		m_pOldMidiBus->setPatch(m_iOldChannel, m_sOldInstrumentName,
-			m_iOldBankSelMethod, m_iOldBank, m_iOldProg);
+			m_iOldBankSelMethod, m_iOldBank, m_iOldProg, m_pTrack);
 		// Reset restorable patch reference...
 		m_pOldMidiBus = NULL;
 		m_iOldChannel = -1;
@@ -806,11 +1013,44 @@ void qtractorTrackForm::inputBusNameChanged ( const QString& /* sBusName */ )
 
 
 // Make changes due to output-bus name.
-void qtractorTrackForm::outputBusNameChanged ( const QString& /* sBusName */ )
+void qtractorTrackForm::outputBusNameChanged ( const QString& sBusName )
 {
 	if (m_iDirtySetup > 0)
 		return;
 
+	if (m_pTrack == NULL)
+		return;
+
+	// It all depends on the track type we're into...
+	qtractorTrack::TrackType trackType = qtractorTrackForm::trackType();
+
+	// (Re)initialize plugin-list audio output bus properly...
+	qtractorAudioEngine *pAudioEngine = m_pTrack->session()->audioEngine();
+	if (pAudioEngine) {
+		// Get the audio bus applicable for the plugin list...
+		qtractorAudioBus *pAudioBus = NULL;
+		if (trackType == qtractorTrack::Audio)
+			pAudioBus = static_cast<qtractorAudioBus *> (
+				pAudioEngine->findBus(sBusName));
+		// FIXME: Master audio bus as reference, still...
+		if (pAudioBus == NULL)
+			pAudioBus = static_cast<qtractorAudioBus *> (
+				pAudioEngine->buses().first());
+		// If an audio bus has been found applicable,
+		// must set plugin-list channel buffers...
+		if (pAudioBus) {
+			m_pTrack->pluginList()->setBuffer(pAudioBus->channels(),
+				pAudioEngine->bufferSize(), pAudioEngine->sampleRate(),
+				trackType == qtractorTrack::Audio
+					? qtractorPluginList::AudioTrack
+					: qtractorPluginList::MidiTrack);
+		}
+	}
+
+	// Recache the applicable MIDI output bus ...
+	if (trackType == qtractorTrack::Midi)
+		m_pMidiBus = midiBus();
+
 	channelChanged(m_ui.ChannelSpinBox->value());
 }
 
@@ -923,8 +1163,7 @@ void qtractorTrackForm::progChanged (void)
 		return;
 
 	// Of course, only applicable on MIDI tracks...
-	qtractorMidiBus *pMidiBus = midiBus();
-	if (pMidiBus) {
+	if (m_pMidiBus) {
 		// Patch parameters...
 		unsigned short iChannel = m_ui.ChannelSpinBox->value() - 1;
 		const QString& sInstrumentName = m_ui.InstrumentComboBox->currentText();
@@ -932,16 +1171,16 @@ void qtractorTrackForm::progChanged (void)
 		int iBank = midiBank();
 		int iProg = midiProgram();
 		// Keep old bus/channel patching consistency.
-		if (pMidiBus != m_pOldMidiBus || iChannel != m_iOldChannel) {
+		if (m_pMidiBus != m_pOldMidiBus || iChannel != m_iOldChannel) {
 			// Restore previously saved patch...
 			if (m_pOldMidiBus) {
 				m_pOldMidiBus->setPatch(m_iOldChannel, m_sOldInstrumentName,
-					m_iOldBankSelMethod, m_iOldBank, m_iOldProg);
+					m_iOldBankSelMethod, m_iOldBank, m_iOldProg, m_pTrack);
 			}
 			// Save current channel patch...
-			const qtractorMidiBus::Patch& patch = pMidiBus->patch(iChannel);
+			const qtractorMidiBus::Patch& patch = m_pMidiBus->patch(iChannel);
 			if (!patch.instrumentName.isEmpty()) {
-				m_pOldMidiBus = pMidiBus;
+				m_pOldMidiBus = m_pMidiBus;
 				m_iOldChannel = iChannel;
 				m_sOldInstrumentName = patch.instrumentName;
 				m_iOldBankSelMethod  = patch.bankSelMethod;
@@ -950,10 +1189,10 @@ void qtractorTrackForm::progChanged (void)
 			}
 		}
 		// Patch it directly...
-		pMidiBus->setPatch(iChannel, sInstrumentName,
-			iBankSelMethod, iBank, iProg);
+		m_pMidiBus->setPatch(iChannel, sInstrumentName,
+			iBankSelMethod, iBank, iProg, m_pTrack);
 	}
-	
+
 	// Flag that it changed anyhow!
 	changed();
 }
@@ -983,4 +1222,32 @@ void qtractorTrackForm::selectBackgroundColor (void)
 }
 
 
+// Plugin list slots.
+void qtractorTrackForm::addPlugin (void)
+{
+	m_ui.PluginListView->addPlugin();
+	updateInstruments();
+	changed();
+}
+
+void qtractorTrackForm::removePlugin (void)
+{
+	m_ui.PluginListView->removePlugin();
+	updateInstruments();
+	changed();
+}
+
+void qtractorTrackForm::moveUpPlugin (void)
+{
+	m_ui.PluginListView->moveUpPlugin();
+	changed();
+}
+
+void qtractorTrackForm::moveDownPlugin (void)
+{
+	m_ui.PluginListView->moveDownPlugin();
+	changed();
+}
+
+
 // end of qtractorTrackForm.cpp
