@@ -32,6 +32,9 @@
 #include "qtractorAudioEngine.h"
 #include "qtractorMidiEngine.h"
 
+#include "qtractorMidiBuffer.h"
+#include "qtractorPlugin.h"
+
 #include "qtractorRubberBand.h"
 
 #include "qtractorMainForm.h"
@@ -46,6 +49,15 @@
 #include <QKeyEvent>
 #include <QPainter>
 
+#ifdef CONFIG_GRADIENT
+#include <QLinearGradient>
+#endif
+
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------------
 // qtractorTrackListHeaderModel -- Track-list header model.
@@ -190,9 +202,8 @@ qtractorTrackList::qtractorTrackList ( qtractorTracks *pTracks, QWidget *pParent
 //	qtractorScrollView::setDragAutoScroll(false);
 	qtractorScrollView::setMouseTracking(true);
 
-
-//	const QFont& font = qtractorScrollView::font();
-//	qtractorScrollView::setFont(QFont(font.family(), font.pointSize() - 1));
+	const QFont& font = qtractorScrollView::font();
+	qtractorScrollView::setFont(QFont(font.family(), font.pointSize() - 1));
 
 	QObject::connect(m_pHeader,
 		SIGNAL(sectionResized(int,int,int)),
@@ -232,12 +243,45 @@ qtractorTrackList::Item::Item ( qtractorTrackList *pTrackList,
 	update(pTrackList);
 }
 
+
 // Track-list model item destructor
 qtractorTrackList::Item::~Item (void)
 {
 	if (widget) delete widget;
 }
 
+
+// Track-list model item bank/program names helper.
+bool qtractorTrackList::Item::updateBankProgram (
+	qtractorMidiManager *pMidiManager, const QString& sInstrument,
+	QString& sBank, QString& sProgram ) const
+{
+	if (pMidiManager == NULL)
+		return false;
+
+	const qtractorMidiManager::Instruments& list
+		= pMidiManager->instruments();
+	if (!list.contains(sInstrument))
+		return false;
+
+	const qtractorMidiManager::Banks& banks
+		= list[sInstrument];
+	int iBank = track->midiBank();
+	if (banks.contains(iBank)) {
+		const qtractorMidiManager::Bank& bank
+			= banks[iBank];
+		int iProg = track->midiProgram();
+		if (bank.progs.contains(iProg)) {
+			sProgram = QString("%1 - %2").arg(iProg)
+				.arg(bank.progs[iProg]);
+			sBank = bank.name;
+		}
+	}
+
+	return true;
+}
+
+
 // Track-list model item cache updater.
 void qtractorTrackList::Item::update ( qtractorTrackList *pTrackList )
 {
@@ -295,20 +339,35 @@ void qtractorTrackList::Item::update ( qtractorTrackList *pTrackList )
 			if (track->midiBank() >= 0)
 				sBank = QString::number(track->midiBank());
 			if (pMidiBus) {
-				qtractorMainForm *pMainForm
-					= qtractorMainForm::getInstance();
 				const qtractorMidiBus::Patch& patch
 					= pMidiBus->patch(iChannel);
-				if (!patch.instrumentName.isEmpty()
-					&& pMainForm && pMainForm->instruments()) {
+				if (!patch.instrumentName.isEmpty()) {
 					sInstrument = patch.instrumentName;
-					qtractorInstrument& instr
-						= (*pMainForm->instruments())[sInstrument];
-					const qtractorInstrumentData& bank
-						= instr.patch(track->midiBank());
-					if (bank.contains(track->midiProgram())) {
-						sProgram = bank[track->midiProgram()];
-						sBank = bank.name();
+					bool bMidiManager = updateBankProgram(
+						(track->pluginList())->midiManager(),
+						sInstrument, sBank, sProgram);
+					if (!bMidiManager && pMidiBus->pluginList_out()) {
+						bMidiManager = updateBankProgram(
+							(pMidiBus->pluginList_out())->midiManager(),
+							sInstrument, sBank, sProgram);
+					}
+					if (!bMidiManager) {
+						qtractorInstrumentList *pInstruments = NULL;
+						qtractorSession *pSession
+							= qtractorSession::getInstance();
+						if (pSession)
+							pInstruments = pSession->instruments();
+						if (pInstruments
+							&& pInstruments->contains(sInstrument)) {
+							qtractorInstrument& instr
+								= (*pInstruments)[sInstrument];
+							const qtractorInstrumentData& bank
+								= instr.patch(track->midiBank());
+							if (bank.contains(track->midiProgram())) {
+								sProgram = bank[track->midiProgram()];
+								sBank = bank.name();
+							}
+						}
 					}
 				}
 			}
@@ -403,7 +462,7 @@ qtractorTrack *qtractorTrackList::track ( int iTrack ) const
 // Retrive the given track row rectangular (in viewport coordinates).
 QRect qtractorTrackList::trackRect ( int iTrack ) const
 {
-	QRect rect(0, 0, qtractorScrollView::viewport()->width(), 0);
+	QRect rect(0, 0, qtractorScrollView::viewport()->width(), ItemHeightBase);
 
 	if (iTrack >= 0 && iTrack < m_items.count()) {
 		int y1, y2;
@@ -464,20 +523,15 @@ void qtractorTrackList::setCurrentTrackRow ( int iTrack )
 	if (iCurrentTrack == m_iCurrentTrack)
 		return;
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorTrackList::setCurrentTrackRow(%d)", iCurrentTrack);
+#endif
+
 	m_iCurrentTrack = iCurrentTrack;
 
-	int hh = m_pHeader->sizeHint().height();
-	int  y = qtractorScrollView::contentsY() - hh;
-	const QRect& rect = trackRect(m_iCurrentTrack);
-	if (rect.top() < hh) {
-		qtractorScrollView::ensureVisible(
-			qtractorScrollView::contentsX(), y + rect.top(), 0, hh);
-	} else if (rect.bottom() > qtractorScrollView::viewport()->height()) {
-		qtractorScrollView::ensureVisible(
-			qtractorScrollView::contentsX(), y + rect.bottom(), 0, hh);
-	}
-
-	updateContents();
+	// Make sure the new current track is visible...
+	if (!ensureVisibleRect(trackRect(m_iCurrentTrack)))
+		updateContents();
 
 	emit selectionChanged();
 }
@@ -493,7 +547,6 @@ int qtractorTrackList::trackRowCount (void) const
 }
 
 
-
 // Retrieves current selected track reference.
 qtractorTrack *qtractorTrackList::currentTrack (void) const
 {
@@ -553,8 +606,8 @@ void qtractorTrackList::clear (void)
 // Update all tracks item height.
 void qtractorTrackList::updateContentsHeight (void)
 {
-	// Remember to give some room to drop something...
-	int iContentsHeight = m_pHeader->sizeHint().height() + ItemHeightBase;
+	// Remember to give some room to drop something at the bottom...
+	int iContentsHeight = m_pHeader->sizeHint().height() + (ItemHeightBase << 1);
 	QListIterator<Item *> iter(m_items);
 	while (iter.hasNext()) {
 		qtractorTrack *pTrack = iter.next()->track;
@@ -628,11 +681,11 @@ void qtractorTrackList::drawCell ( QPainter *pPainter, int iRow, int iCol,
 	const Item *pItem = m_items.at(iRow);
 	QColor bg, fg;
 	if (iCol == Number) {
-		bg = (pItem->track)->foreground().light();
-		fg = (pItem->track)->background().light();
+		bg = (pItem->track)->foreground().lighter();
+		fg = (pItem->track)->background().lighter();
 	} else if (m_iCurrentTrack == iRow) {
-		bg = pal.midlight().color().dark(150);
-		fg = pal.midlight().color().light(150);
+		bg = pal.midlight().color().darker(150);
+		fg = pal.midlight().color().lighter();
 	} else {
 		bg = pal.window().color();
 		fg = pal.windowText().color();
@@ -640,7 +693,14 @@ void qtractorTrackList::drawCell ( QPainter *pPainter, int iRow, int iCol,
 
 	// Draw text and decorations if any...
 	QRect rectText(rect.topLeft() + QPoint(4, 4), rect.size() - QSize(8, 8));
+#ifdef CONFIG_GRADIENT
+	QLinearGradient grad(0, rect.top(), 0, rect.bottom());
+	grad.setColorAt(0.4, bg);
+	grad.setColorAt(1.0, bg.darker(120));
+	pPainter->fillRect(rect, grad);
+#else
 	pPainter->fillRect(rect, bg);
+#endif
 	pPainter->setPen(fg);
 	if (iCol == Number) {
 		pPainter->drawText(rectText,
@@ -675,10 +735,10 @@ void qtractorTrackList::drawCell ( QPainter *pPainter, int iRow, int iCol,
 	}
 
 	// Do some simple embossing...
-	pPainter->setPen(bg.light(150));
+	pPainter->setPen(bg.lighter(150));
 	pPainter->drawLine(rect.left(), rect.top(), rect.left(), rect.bottom());
 	pPainter->drawLine(rect.left(), rect.top(), rect.right(), rect.top());
-	pPainter->setPen(bg.dark(150));
+	pPainter->setPen(bg.darker(150));
 	pPainter->drawLine(rect.right(), rect.top(), rect.right(), rect.bottom());
 	pPainter->drawLine(rect.left(), rect.bottom(), rect.right(), rect.bottom());
 }
@@ -709,7 +769,7 @@ void qtractorTrackList::updatePixmap ( int cx, int cy )
 	int iColCount = m_pHeader->count();
 	int hh = m_pHeader->sizeHint().height();
 	// Account for the item dropping headroom...
-	int ch = qtractorScrollView::contentsHeight() - ItemHeightBase;
+	int ch = qtractorScrollView::contentsHeight() - (ItemHeightBase << 1);
 
 	int x, y1, y2, h1;
 	y1 = y2 = 0;
@@ -833,9 +893,6 @@ void qtractorTrackList::mousePressEvent ( QMouseEvent *pMouseEvent )
 	// just 'coz we're using ruberbands, which are
 	// in fact based on real widgets (WM entities)...
 	qtractorScrollView::mousePressEvent(pMouseEvent);
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
@@ -849,8 +906,10 @@ void qtractorTrackList::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 		if (m_iDragTrack >= 0) {
 			int iTrack = trackRowAt(pMouseEvent->pos());
 			if (iTrack >= 0) {
-				m_posDrag = trackRect(iTrack).topLeft();
+				const QRect& rect = trackRect(iTrack);
+				m_posDrag = rect.topLeft();
 				moveRubberBand(m_posDrag);
+				ensureVisibleRect(rect);
 			}
 		}
 		break;
@@ -922,10 +981,7 @@ void qtractorTrackList::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 	qtractorScrollView::mouseReleaseEvent(pMouseEvent);
 
 	// We'll need a reference for issuing commands...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -941,7 +997,7 @@ void qtractorTrackList::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 				if (pTrackDrag && pTrackDrop
 					&& pTrackDrag != pTrackDrop
 					&& pTrackDrag != pTrackDrop->prev()) {
-					pMainForm->commands()->exec(
+					pSession->execute(
 						new qtractorMoveTrackCommand(pTrackDrag, pTrackDrop));
 				}
 			}
@@ -956,7 +1012,7 @@ void qtractorTrackList::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			// Go for it...
 			qtractorTrack *pTrack = track(m_iDragTrack);
 			if (pTrack) {
-				pMainForm->commands()->exec(
+				pSession->execute(
 					new qtractorResizeTrackCommand(pTrack, iZoomHeight));
 			}
 		}
@@ -969,16 +1025,12 @@ void qtractorTrackList::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 
 	// Force null state.
 	resetDragState();
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
 // Draw a dragging separator line.
 void qtractorTrackList::moveRubberBand ( const QPoint& posDrag )
 {
-
 	// Create the rubber-band if there's none...
 	if (m_pRubberBand == NULL) {
 		m_pRubberBand = new qtractorRubberBand(
@@ -999,6 +1051,27 @@ void qtractorTrackList::moveRubberBand ( const QPoint& posDrag )
 }
 
 
+// Make sure the given (track) rectangle is visible.
+bool qtractorTrackList::ensureVisibleRect ( const QRect& rect )
+{
+	int hh = m_pHeader->sizeHint().height();
+	int cx = qtractorScrollView::contentsX();
+	int cy = qtractorScrollView::contentsY();
+
+	if (rect.top() < hh) {
+		qtractorScrollView::ensureVisible(cx, cy + rect.top(), 0, 24);
+		return true;
+	}
+
+	if (rect.bottom() > qtractorScrollView::viewport()->height()) {
+		qtractorScrollView::ensureVisible(cx, cy + rect.bottom() - hh, 0, 24);
+		return true;
+	}
+
+	return false;
+}
+
+
 // Reset drag/select/move state.
 void qtractorTrackList::resetDragState (void)
 {
@@ -1102,9 +1175,6 @@ void qtractorTrackList::keyPressEvent ( QKeyEvent *pKeyEvent )
 		qtractorScrollView::keyPressEvent(pKeyEvent);
 		break;
 	}
-
-	// Make sure we've get focus back...
-	qtractorScrollView::setFocus();
 }
 
 
