@@ -33,6 +33,7 @@ class qtractorTracks;
 class qtractorTrackList;
 class qtractorTrackButton;
 class qtractorRubberBand;
+class qtractorMidiManager;
 
 class QHeaderView;
 
@@ -175,6 +176,9 @@ protected:
 	// Show and move rubber-band item.
 	void moveRubberBand(const QPoint& posDrag);
 
+	// Make sure the given (track) rectangle is visible.
+	bool ensureVisibleRect(const QRect& rect);
+
 	// Keyboard event handler.
 	void keyPressEvent(QKeyEvent *pKeyEvent);
 
@@ -215,6 +219,10 @@ private:
 		Item(qtractorTrackList *pTrackList, qtractorTrack *pTrack);
 		// Destructor.
 		~Item();
+		// Bank/program names helper.
+		bool updateBankProgram (qtractorMidiManager *pMidiManager,
+			const QString& sInstrument,
+			QString& sBank, QString& sProgram ) const;
 		// Item updater.
 		void update(qtractorTrackList *pTrackList);
 		// Item members.
