@@ -1,7 +1,7 @@
 // qtractorTrackTime.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -25,6 +25,8 @@
 #include "qtractorSession.h"
 #include "qtractorTracks.h"
 
+#include "qtractorSessionCommand.h"
+
 #include "qtractorMainForm.h"
 
 #include <QApplication>
@@ -56,7 +58,8 @@ qtractorTrackTime::qtractorTrackTime ( qtractorTracks *pTracks,
 
 	qtractorScrollView::setFrameStyle(QFrame::Panel | QFrame::Raised);
 
-	qtractorScrollView::viewport()->setFocusPolicy(Qt::ClickFocus);
+	qtractorScrollView::setFocusPolicy(Qt::NoFocus);
+//	qtractorScrollView::viewport()->setFocusPolicy(Qt::ClickFocus);
 //	qtractorScrollView::viewport()->setFocusProxy(this);
 //	qtractorScrollView::viewport()->setAcceptDrops(true);
 //	qtractorScrollView::setDragAutoScroll(false);
@@ -91,7 +94,7 @@ void qtractorTrackTime::updatePixmap ( int cx, int /* cy */)
 	m_pixmap = QPixmap(w, h);
 	m_pixmap.fill(pal.window().color());
 
-	qtractorSession *pSession = m_pTracks->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -153,6 +156,31 @@ void qtractorTrackTime::updatePixmap ( int cx, int /* cy */)
 			painter.drawPolygon(polyg);
 		}
 	}
+
+	// Draw punch in/out boundaries, if applicable...
+	if (pSession->isPunching()) {
+		QPolygon polyg(3);
+	//	h -= 4;
+		int d = (h >> 2);
+		painter.setPen(Qt::darkMagenta);
+		painter.setBrush(Qt::magenta);
+		x = pSession->pixelFromFrame(pSession->punchIn()) - cx;
+		if (x >= 0 && x < w) {
+			polyg.putPoints(0, 3,
+				x + d, h - d,
+				x, h,
+				x, h - d);
+			painter.drawPolygon(polyg);
+		}
+		x = pSession->pixelFromFrame(pSession->punchOut()) - cx;
+		if (x >= 0 && x < w) {
+			polyg.putPoints(0, 3,
+				x, h - d,
+				x, h,
+				x - d, h - d);
+			painter.drawPolygon(polyg);
+		}
+	}
 }
 
 
@@ -161,6 +189,7 @@ void qtractorTrackTime::updateContents ( const QRect& rect )
 {
 	updatePixmap(
 		qtractorScrollView::contentsX(), qtractorScrollView::contentsY());
+
 	qtractorScrollView::updateContents(rect);
 }
 
@@ -247,7 +276,7 @@ void qtractorTrackTime::contentsXMovingSlot ( int cx, int /*cy*/ )
 // Check if some position header is to be dragged...
 bool qtractorTrackTime::dragHeadStart ( const QPoint& pos )
 {
-	qtractorSession *pSession = m_pTracks->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -265,7 +294,7 @@ bool qtractorTrackTime::dragHeadStart ( const QPoint& pos )
 		return true;
 	}
 
-	// Check loop-points, translating to edit-head/tail headers...
+	// Check loop-point headers...
 	if (pSession->isLooping()) {
 		// Check loop-start header...
 		rect.moveLeft(pSession->pixelFromFrame(pSession->loopStart()) - d);
@@ -283,6 +312,24 @@ bool qtractorTrackTime::dragHeadStart ( const QPoint& pos )
 		}
 	}
 
+	// Check punch-point headers...
+	if (pSession->isPunching()) {
+		// Check punch-in header...
+		rect.moveLeft(pSession->pixelFromFrame(pSession->punchIn()) - d);
+		if (rect.contains(pos)) {
+			m_dragCursor = DragPunchIn;
+			qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
+			return true;
+		}
+		// Check punch-out header...
+		rect.moveLeft(pSession->pixelFromFrame(pSession->punchOut()) - d);
+		if (rect.contains(pos)) {
+			m_dragCursor = DragPunchOut;
+			qtractorScrollView::setCursor(QCursor(Qt::SizeHorCursor));
+			return true;
+		}
+	}
+
 	// Check edit-head header...
 	rect.moveLeft(m_pTracks->trackView()->editHeadX() - d);
 	if (rect.contains(pos)) {
@@ -316,7 +363,7 @@ void qtractorTrackTime::mousePressEvent ( QMouseEvent *pMouseEvent )
 	// Force null state.
 	m_dragState = DragNone;
 
-	qtractorSession *pSession = m_pTracks->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession) {
 		// Direct snap positioning...
 		const QPoint& pos = viewportToContents(pMouseEvent->pos());
@@ -358,12 +405,9 @@ void qtractorTrackTime::mousePressEvent ( QMouseEvent *pMouseEvent )
 // Handle selection/dragging -- mouse pointer move.
 void qtractorTrackTime::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return;
-
-	qtractorSession *pSession = m_pTracks->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession) {
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 		// Are we already moving/dragging something?
 		const QPoint& pos = viewportToContents(pMouseEvent->pos());
 		unsigned long iFrame = pSession->frameSnap(
@@ -386,15 +430,18 @@ void qtractorTrackTime::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			m_pTracks->trackView()->ensureVisible(pos.x(), y, 16, 0);
 			m_pTracks->trackView()->setPlayHead(iFrame);
 			// Let the change get some immediate visual feedback...
-			pMainForm->updateTransportTime(iFrame);
+			if (pMainForm)
+				pMainForm->updateTransportTime(iFrame);
 			break;
 		case DragLoopStart:
+		case DragPunchIn:
 		case DragEditHead:
 			// Edit-head positioning...
 			m_pTracks->trackView()->ensureVisible(pos.x(), y, 16, 0);
 			m_pTracks->trackView()->setEditHead(iFrame);
 			break;
 		case DragLoopEnd:
+		case DragPunchOut:
 		case DragEditTail:
 			// Edit-tail positioning...
 			m_pTracks->trackView()->ensureVisible(pos.x(), y, 16, 0);
@@ -428,7 +475,7 @@ void qtractorTrackTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 {
 	qtractorScrollView::mouseReleaseEvent(pMouseEvent);
 
-	qtractorSession *pSession = m_pTracks->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession) {
 		// Which mouse state?
 		const bool bModifier = (pMouseEvent->modifiers()
@@ -455,9 +502,18 @@ void qtractorTrackTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			// New loop-start boundary...
 			if (pSession->editHead() < pSession->loopEnd()) {
 				// Yep, new loop-start point...
-				pSession->setLoop(pSession->editHead(), pSession->loopEnd());
-				m_pTracks->trackView()->updateContents();
-				updateContents();
+				pSession->execute(
+					new qtractorSessionLoopCommand(pSession,
+						pSession->editHead(), pSession->loopEnd()));
+			}
+			break;
+		case DragPunchIn:
+			// New punch-in boundary...
+			if (pSession->editHead() < pSession->punchOut()) {
+				// Yep, new punch-in point...
+				pSession->setPunch(
+					pSession->editHead(), pSession->punchOut());
+				// For visual feedback...
 				m_pTracks->contentsChangeNotify();
 			}
 			break;
@@ -469,9 +525,18 @@ void qtractorTrackTime::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 			// New loop-end boundary...
 			if (pSession->loopStart() < pSession->editTail()) {
 				// Yep, new loop-end point...
-				pSession->setLoop(pSession->loopStart(), pSession->editTail());
-				m_pTracks->trackView()->updateContents();
-				updateContents();
+				pSession->execute(
+					new qtractorSessionLoopCommand(pSession,
+						pSession->loopStart(), pSession->editTail()));
+			}
+			break;
+		case DragPunchOut:
+			// New punch-out boundary...
+			if (pSession->punchOut() < pSession->editTail()) {
+				// Yep, new punch-out point...
+				pSession->setPunch(
+					pSession->punchIn(), pSession->editTail());
+				// For visual feedback...
 				m_pTracks->contentsChangeNotify();
 			}
 			break;
@@ -513,13 +578,14 @@ void qtractorTrackTime::keyPressEvent ( QKeyEvent *pKeyEvent )
 #endif
 	switch (pKeyEvent->key()) {
 	case Qt::Key_Escape:
+	{
 		// Restore uncommitted play-head position?...
-		if (m_dragState == DragPlayHead && m_pTracks->session()) {
-			m_pTracks->trackView()->setPlayHead(
-				m_pTracks->session()->playHead());
-		}
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession && m_dragState == DragPlayHead)
+			m_pTracks->trackView()->setPlayHead(pSession->playHead());
 		resetDragState();
 		break;
+	}
 	default:
 		qtractorScrollView::keyPressEvent(pKeyEvent);
 		break;
