@@ -1,7 +1,7 @@
 // qtractorTrackTime.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2007, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2008, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -99,7 +99,8 @@ private:
 	enum DragState {
 		DragNone = 0, DragStart, DragSelect,
 		DragPlayHead, DragEditHead, DragEditTail,
-		DragLoopStart, DragLoopEnd
+		DragLoopStart, DragLoopEnd,
+		DragPunchIn, DragPunchOut
 	} m_dragState, m_dragCursor;
 
 	QRect  m_rectDrag;
