@@ -31,7 +31,9 @@
 #include "qtractorTrackCommand.h"
 #include "qtractorClipCommand.h"
 
+#include "qtractorAudioEngine.h"
 #include "qtractorAudioClip.h"
+
 #include "qtractorMidiEngine.h"
 #include "qtractorMidiClip.h"
 
@@ -41,16 +43,26 @@
 #include "qtractorTrackForm.h"
 #include "qtractorClipForm.h"
 
+#include "qtractorPasteRepeatForm.h"
+
 #include "qtractorMidiEditorForm.h"
 
 #include <QVBoxLayout>
 #include <QToolButton>
+#include <QProgressBar>
 #include <QMessageBox>
+#include <QFileDialog>
 #include <QFileInfo>
 #include <QDate>
+#include <QUrl>
 
 #include <QHeaderView>
 
+#if QT_VERSION < 0x040300
+#define lighter(x)	light(x)
+#define darker(x)	dark(x)
+#endif
+
 
 //----------------------------------------------------------------------------
 // qtractorTracks -- The main session track listview widget.
@@ -85,15 +97,12 @@ qtractorTracks::qtractorTracks ( QWidget *pParent )
 
 	// Get previously saved splitter sizes,
 	// (with some fair default...)
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions) {
-			QList<int> sizes;
-			sizes.append(160);
-			sizes.append(480);
-			pOptions->loadSplitterSizes(this, sizes);
-		}
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions) {
+		QList<int> sizes;
+		sizes.append(160);
+		sizes.append(480);
+		pOptions->loadSplitterSizes(this, sizes);
 	}
 
 	// Early track list stabilization.
@@ -114,28 +123,9 @@ qtractorTracks::qtractorTracks ( QWidget *pParent )
 qtractorTracks::~qtractorTracks (void)
 {
 	// Save splitter sizes...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm) {
-		qtractorOptions *pOptions = pMainForm->options();
-		if (pOptions)
-			pOptions->saveSplitterSizes(this);
-	}
-}
-
-
-// Session helper accessor.
-qtractorSession *qtractorTracks::session (void) const
-{
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	return (pMainForm ? pMainForm->session() : NULL);
-}
-
-
-// Instrument list accessor.
-qtractorInstrumentList *qtractorTracks::instruments (void) const
-{
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	return (pMainForm ? pMainForm->instruments() : NULL);
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
+	if (pOptions)
+		pOptions->saveSplitterSizes(this);
 }
 
 
@@ -159,7 +149,7 @@ qtractorTrackView *qtractorTracks::trackView (void) const
 // Horizontal zoom factor.
 void qtractorTracks::horizontalZoomStep ( int iZoomStep )
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 		
@@ -182,7 +172,7 @@ void qtractorTracks::horizontalZoomStep ( int iZoomStep )
 // Vertical zoom factor.
 void qtractorTracks::verticalZoomStep ( int iZoomStep )
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 		
@@ -218,7 +208,7 @@ void qtractorTracks::zoomOut (void)
 
 void qtractorTracks::zoomReset (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -254,7 +244,7 @@ void qtractorTracks::verticalZoomOutSlot (void)
 
 void qtractorTracks::viewZoomResetSlot (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -269,7 +259,7 @@ void qtractorTracks::viewZoomResetSlot (void)
 // (usually after zoom change)
 void qtractorTracks::centerContents (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 		
@@ -291,7 +281,7 @@ void qtractorTracks::centerContents (void)
 // Update/sync integral contents from session tracks.
 void qtractorTracks::updateContents ( bool bRefresh )
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -319,6 +309,7 @@ void qtractorTracks::updateContents ( bool bRefresh )
 	if (iRefresh > 0) {
 		m_pTrackView->updateContentsWidth();
 		m_pTrackList->updateContentsHeight();
+	//	m_pTrackView->setFocus();
 	}
 }
 
@@ -326,7 +317,7 @@ void qtractorTracks::updateContents ( bool bRefresh )
 // Retrieves current (selected) track reference.
 qtractorTrack *qtractorTracks::currentTrack (void) const
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return NULL;
 
@@ -361,7 +352,7 @@ qtractorClip *qtractorTracks::currentClip (void) const
 // Edit/create a brand new clip.
 bool qtractorTracks::newClip (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -411,10 +402,9 @@ bool qtractorTracks::newClip (void)
 		pClip->setFilename(sFilename);
 		pClip->setClipName(QFileInfo(sFilename).baseName());
 		// Proceed to setup the MDII clip properly...
-		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 		qtractorMidiClip *pMidiClip
 			= static_cast<qtractorMidiClip *> (pClip);
-		if (pMainForm && pMidiClip) {
+		if (pMidiClip) {
 			// Initialize MIDI event container...
 			qtractorMidiSequence *pSeq = pMidiClip->sequence();
 			pSeq->setName(pMidiClip->clipName());
@@ -428,11 +418,13 @@ bool qtractorTracks::newClip (void)
 				// SMF format 1 (2 tracks, 1 channel)
 				pMidiClip->setTrackChannel(1);
 			}
+			// Make it a brand new revision...
+			pMidiClip->setRevision(1);
 			// Insert the clip right away...
 			qtractorClipCommand *pClipCommand
 				= new qtractorClipCommand(tr("new clip"));
 			pClipCommand->addClip(pClip, pTrack);
-			pMainForm->commands()->exec(pClipCommand);
+			pSession->execute(pClipCommand);
 			// Just start the MIDI editor on it...
 			return pClip->startEditor(this);
 		}
@@ -467,11 +459,7 @@ bool qtractorTracks::editClip ( qtractorClip *pClip )
 // Split given(current) clip.
 bool qtractorTracks::splitClip ( qtractorClip *pClip )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -506,7 +494,377 @@ bool qtractorTracks::splitClip ( qtractorClip *pClip )
 	}
 
 	// That's it...
-	return pMainForm->commands()->exec(pClipCommand);
+	return pSession->execute(pClipCommand);
+}
+
+
+// Audio clip normalize callback.
+struct audioClipNormalizeData
+{	// Ctor.
+	audioClipNormalizeData(unsigned short iChannels)
+		: count(0), channels(iChannels), max(0.0f) {};
+	// Members.
+	unsigned int count;
+	unsigned short channels;
+	float max;
+};
+
+static void audioClipNormalize (
+	float **ppFrames, unsigned int iFrames, void *pvArg )
+{
+	audioClipNormalizeData *pData
+		= static_cast<audioClipNormalizeData *> (pvArg);
+
+	for (unsigned short i = 0; i < pData->channels; ++i) {
+		float *pFrames = ppFrames[i];
+		for (unsigned int n = 0; n < iFrames; ++n) {
+			float fSample = *pFrames++;
+			if (fSample < 0.0f) // Take absolute value...
+				fSample = -(fSample);
+			if (pData->max < fSample)
+				pData->max = fSample;
+		}
+	}
+
+	if (++(pData->count) > 100) {
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (pMainForm) {
+			QProgressBar *pProgressBar = pMainForm->progressBar();
+			pProgressBar->setValue(pProgressBar->value() + iFrames);
+		}
+		qtractorSession::stabilize();
+		pData->count = 0;
+	}
+}
+
+
+// MIDI clip normalize callback.
+static void midiClipNormalize (
+	qtractorMidiSequence *pSeq, void *pvArg )
+{
+	unsigned char *pMax = (unsigned char *) (pvArg);
+	for (qtractorMidiEvent *pEvent = pSeq->events().first();
+			pEvent; pEvent = pEvent->next()) {
+		if (pEvent->type() == qtractorMidiEvent::NOTEON &&
+			*pMax < pEvent->velocity()) {
+			*pMax = pEvent->velocity();
+		}
+	}
+}
+
+
+// Normalize given(current) clip.
+bool qtractorTracks::normalizeClip ( qtractorClip *pClip )
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	if (pClip == NULL)
+		pClip = m_pTrackView->currentClip();
+	if (pClip == NULL)
+		return false;
+
+	qtractorTrack *pTrack = pClip->track();
+	if (pTrack == NULL)
+		return false;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+
+	unsigned long iOffset = 0;
+	unsigned long iLength = pClip->clipLength();
+
+	if (pClip->isClipSelected()) {
+		iOffset = pClip->clipSelectStart() - pClip->clipStart();
+		iLength = pClip->clipSelectEnd() - pClip->clipSelectStart();
+	}
+
+	// Default non-normalized setting...
+	float fGain = pClip->clipGain();
+
+	if (pTrack->trackType() == qtractorTrack::Audio) {
+		// Normalize audio clip...
+		qtractorAudioClip *pAudioClip
+			= static_cast<qtractorAudioClip *> (pClip);
+		if (pAudioClip == NULL)
+			return false;
+		qtractorAudioBus *pAudioBus
+			= static_cast<qtractorAudioBus *> (pTrack->outputBus());
+		if (pAudioBus == NULL)
+			return false;
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+		QProgressBar *pProgressBar = NULL;
+		if (pMainForm)
+			pProgressBar = pMainForm->progressBar();
+		if (pProgressBar) {
+			pProgressBar->setRange(0, iLength / 100);
+			pProgressBar->reset();
+			pProgressBar->show();
+		}
+		audioClipNormalizeData data(pAudioBus->channels());
+		pAudioClip->clipExport(audioClipNormalize, &data, iOffset, iLength);
+		if (data.max > 0.1f && data.max < 1.1f)
+			fGain /= data.max;
+		if (pProgressBar)
+			pProgressBar->hide();
+		QApplication::restoreOverrideCursor();
+	}
+	else
+	if (pTrack->trackType() == qtractorTrack::Midi) {
+		// Normalize MIDI clip...
+		qtractorMidiClip *pMidiClip
+			= static_cast<qtractorMidiClip *> (pClip);
+		if (pMidiClip == NULL)
+			return false;
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+		unsigned char max = 0;
+		pMidiClip->clipExport(midiClipNormalize, &max, iOffset, iLength);
+		if (max > 0x0c && max < 0x7f)
+			fGain *= (127.0f / float(max));
+		QApplication::restoreOverrideCursor();
+	}
+
+	// Make it as an undoable command...
+	qtractorClipCommand *pClipCommand
+		= new qtractorClipCommand(tr("clip normalize"));
+	pClipCommand->gainClip(pClip, fGain);
+
+	// That's it...
+	return pSession->execute(pClipCommand);
+}
+
+
+// Audio clip export calback.
+struct audioClipExportData
+{	// Ctor.
+	audioClipExportData(qtractorAudioFile *pFile)
+		: count(0), file(pFile) {};
+	// Members.
+	unsigned int count;
+	qtractorAudioFile *file;
+};
+
+static void audioClipExport (
+	float **ppFrames, unsigned int iFrames, void *pvArg )
+{
+	audioClipExportData *pData
+		= static_cast<audioClipExportData *> (pvArg);
+
+	(pData->file)->write(ppFrames, iFrames);
+
+	if (++(pData->count) > 100) {
+		qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+		if (pMainForm) {
+			QProgressBar *pProgressBar = pMainForm->progressBar();
+			pProgressBar->setValue(pProgressBar->value() + iFrames);
+		}
+		qtractorSession::stabilize();
+		pData->count = 0;
+	}
+}
+
+
+// MIDI clip export calback.
+static void midiClipExport (
+	qtractorMidiSequence *pSeq, void *pvArg )
+{
+	qtractorMidiFile *pMidiFile
+		= static_cast<qtractorMidiFile *> (pvArg);
+	pMidiFile->writeTrack(pSeq);
+}
+
+
+// Export given(current) clip.
+bool qtractorTracks::exportClip ( qtractorClip *pClip )
+{
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return false;
+
+	if (pClip == NULL)
+		pClip = m_pTrackView->currentClip();
+	if (pClip == NULL)
+		return false;
+
+	qtractorTrack *pTrack = pClip->track();
+	if (pTrack == NULL)
+		return false;
+
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+
+	unsigned long iOffset = 0;
+	unsigned long iLength = pClip->clipLength();
+
+	if (pClip->isClipSelected()) {
+		iOffset = pClip->clipSelectStart() - pClip->clipStart();
+		iLength = pClip->clipSelectEnd() - pClip->clipSelectStart();
+	}
+
+	if (pTrack->trackType() == qtractorTrack::Audio) {
+		// Export audio clip...
+		qtractorAudioClip *pAudioClip
+			= static_cast<qtractorAudioClip *> (pClip);
+		if (pAudioClip == NULL)
+			return false;
+		qtractorAudioBus *pAudioBus
+			= static_cast<qtractorAudioBus *> (pTrack->outputBus());
+		if (pAudioBus == NULL)
+			return false;
+		const QString& sExt = qtractorAudioFileFactory::defaultExt();
+		const QString& sTitle  = tr("Export Audio Clip") + " - " QTRACTOR_TITLE;
+		const QString& sFilter = tr("Audio files (*.%1)").arg(sExt); 
+	#if QT_VERSION < 0x040400
+		// Ask for the filename to save...
+		QString sFilename = QFileDialog::getSaveFileName(this, sTitle,
+			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+	#else
+		// Construct save-file dialog...
+		QFileDialog fileDialog(this, sTitle,
+			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+		// Set proper open-file modes...
+		fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+		fileDialog.setFileMode(QFileDialog::AnyFile);
+		fileDialog.setDefaultSuffix(sExt);
+		// Stuff sidebar...
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
+		if (pOptions) {
+			QList<QUrl> urls(fileDialog.sidebarUrls());
+			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+			urls.append(QUrl::fromLocalFile(pOptions->sAudioDir));
+			fileDialog.setSidebarUrls(urls);
+		}
+		// Show dialog...
+		if (!fileDialog.exec())
+			return false;
+		QString sFilename = fileDialog.selectedFiles().first();
+	#endif
+		if (sFilename.isEmpty())
+			return false;
+		if (QFileInfo(sFilename).suffix() != sExt)
+			sFilename += '.' + sExt;
+		qtractorAudioFile *pAudioFile
+			= qtractorAudioFileFactory::createAudioFile(sFilename,
+				pAudioBus->channels(), pSession->sampleRate());
+		if (pAudioFile == NULL)
+			return false;
+		if (pMainForm) {
+			pMainForm->appendMessages(
+				tr("Audio clip export: \"%1\" started...")
+				.arg(sFilename));
+		}
+		bool bResult = false;
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+		QProgressBar *pProgressBar = NULL;
+		if (pMainForm)
+			pProgressBar = pMainForm->progressBar();
+		if (pProgressBar) {
+			pProgressBar->setRange(0, iLength / 100);
+			pProgressBar->reset();
+			pProgressBar->show();
+		}
+		if (pAudioFile->open(sFilename, qtractorAudioFile::Write)) {
+			audioClipExportData data(pAudioFile);
+			pAudioClip->clipExport(
+				audioClipExport, &data, iOffset, iLength);
+			pAudioFile->close();
+			bResult = true;
+		}
+		delete pAudioFile;
+		if (pProgressBar)
+			pProgressBar->hide();
+		QApplication::restoreOverrideCursor();
+		if (pMainForm) {
+			if (bResult) {
+				pMainForm->addAudioFile(sFilename);
+				pMainForm->appendMessages(
+					tr("Audio clip export: \"%1\" complete.")
+					.arg(sFilename));
+			} else {
+				pMainForm->appendMessagesError(
+					tr("Audio clip export:\n\n\"%1\"\n\nfailed.")
+					.arg(sFilename));
+			}
+		}
+	}
+	else
+	if (pTrack->trackType() == qtractorTrack::Midi) {
+		// Export MIDI clip...
+		qtractorMidiClip *pMidiClip
+			= static_cast<qtractorMidiClip *> (pClip);
+		if (pMidiClip == NULL)
+			return false;
+		const QString  sExt("mid");
+		const QString& sTitle  = tr("Export MIDI Clip") + " - " QTRACTOR_TITLE;
+		const QString& sFilter = tr("MIDI files (*.%1 *.smf *.midi)").arg(sExt); 
+	#if QT_VERSION < 0x040400
+		// Ask for the filename to save...
+		QString sFilename = QFileDialog::getSaveFileName(this, sTitle,
+			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+	#else
+		// Construct save-file dialog...
+		QFileDialog fileDialog(this, sTitle,
+			pSession->createFilePath(pTrack->trackName(), 0, sExt), sFilter);
+		// Set proper open-file modes...
+		fileDialog.setAcceptMode(QFileDialog::AcceptSave);
+		fileDialog.setFileMode(QFileDialog::AnyFile);
+		fileDialog.setDefaultSuffix(sExt);
+		// Stuff sidebar...
+		qtractorOptions *pOptions = qtractorOptions::getInstance();
+		if (pOptions) {
+			QList<QUrl> urls(fileDialog.sidebarUrls());
+			urls.append(QUrl::fromLocalFile(pOptions->sSessionDir));
+			urls.append(QUrl::fromLocalFile(pOptions->sMidiDir));
+			fileDialog.setSidebarUrls(urls);
+		}
+		// Show dialog...
+		if (!fileDialog.exec())
+			return false;
+		QString sFilename = fileDialog.selectedFiles().first();
+	#endif
+		if (sFilename.isEmpty())
+			return false;
+		if (QFileInfo(sFilename).suffix() != sExt)
+			sFilename += '.' + sExt;
+		if (pMainForm) {
+			pMainForm->appendMessages(
+				tr("MIDI clip export: \"%1\" started...")
+				.arg(sFilename));
+		}
+		bool bResult = true;
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+		qtractorMidiFile *pMidiFile = new qtractorMidiFile();
+		if (pMidiFile->open(sFilename, qtractorMidiFile::Write)) {
+			pMidiFile->setTempo(pSession->tempo());
+			pMidiFile->setBeatsPerBar(pSession->beatsPerBar());
+			unsigned short iFormat = qtractorMidiClip::defaultFormat();
+			unsigned short iTracks = (iFormat == 0 ? 1 : 2);
+			pMidiFile->writeHeader(iFormat, iTracks, pSession->ticksPerBeat());
+			if (iFormat == 1)
+				pMidiFile->writeTrack(NULL);  // Setup track (SMF format 1).
+			pMidiClip->clipExport(
+				midiClipExport, pMidiFile, iOffset, iLength);
+			pMidiFile->close();
+			bResult = true;
+		}
+		delete pMidiFile;
+		QApplication::restoreOverrideCursor();
+		if (pMainForm) {
+			if (bResult) {
+				pMainForm->addMidiFile(sFilename);
+				pMainForm->appendMessages(
+					tr("MIDI clip export: \"%1\" complete.")
+					.arg(sFilename));
+			} else {
+				pMainForm->appendMessagesError(
+					tr("MIDI clip export:\n\n\"%1\"\n\nfailed.")
+					.arg(sFilename));
+			}
+		}
+	}
+	else return false; // WTF?
+
+	// Done.
+	return true;
 }
 
 
@@ -517,13 +875,6 @@ bool qtractorTracks::isClipSelected (void) const
 }
 
 
-// Whether there's any clip on clipboard.
-bool qtractorTracks::isClipboardEmpty (void) const
-{
-	return m_pTrackView->isClipboardEmpty();
-}
-
-
 // Clipboard methods.
 void qtractorTracks::cutClipboard (void)
 {
@@ -541,6 +892,20 @@ void qtractorTracks::pasteClipboard (void)
 }
 
 
+// Special paste/repeat prompt.
+void qtractorTracks::pasteRepeatClipboard (void)
+{
+	qtractorPasteRepeatForm pasteForm(this);
+	pasteForm.setRepeatPeriod(m_pTrackView->pastePeriod());
+	if (pasteForm.exec()) {
+		m_pTrackView->pasteClipboard(
+			pasteForm.repeatCount(),
+			pasteForm.repeatPeriod());
+	}
+}
+
+
+
 // Delete current selection.
 void qtractorTracks::deleteClipSelect (void)
 {
@@ -551,7 +916,7 @@ void qtractorTracks::deleteClipSelect (void)
 // Select range interval between edit head and tail.
 void qtractorTracks::selectEditRange (void)
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -596,11 +961,7 @@ void qtractorTracks::selectAll ( bool bSelect )
 // Adds a new track into session.
 bool qtractorTracks::addTrack (void)
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -611,7 +972,7 @@ bool qtractorTracks::addTrack (void)
 	pTrack->setTrackName(QString("Track %1").arg(iTrack));
 	pTrack->setMidiChannel(pSession->midiTag() % 16);
 	pTrack->setBackground(color);
-	pTrack->setForeground(color.dark());
+	pTrack->setForeground(color.darker());
 
 	// Open dialog for settings...
 	qtractorTrackForm trackForm(this);
@@ -625,7 +986,7 @@ bool qtractorTracks::addTrack (void)
 	pTrack->properties() = trackForm.properties();
 
 	// Put it in the form of an undoable command...
-	return pMainForm->commands()->exec(
+	return pSession->execute(
 		new qtractorAddTrackCommand(pTrack));
 }
 
@@ -633,11 +994,7 @@ bool qtractorTracks::addTrack (void)
 // Remove given(current) track from session.
 bool qtractorTracks::removeTrack ( qtractorTrack *pTrack )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -647,8 +1004,12 @@ bool qtractorTracks::removeTrack ( qtractorTrack *pTrack )
 	if (pTrack == NULL)
 		return false;
 
+	// Don't remove tracks engaged in recording...
+	if (pTrack->isRecord() && pSession->isRecording() && pSession->isPlaying())
+		return false;
+
 	// Prompt user if he/she's sure about this...
-	qtractorOptions *pOptions = pMainForm->options();
+	qtractorOptions *pOptions = qtractorOptions::getInstance();
 	if (pOptions && pOptions->bConfirmRemove) {
 		if (QMessageBox::warning(this,
 			tr("Warning") + " - " QTRACTOR_TITLE,
@@ -661,19 +1022,15 @@ bool qtractorTracks::removeTrack ( qtractorTrack *pTrack )
 	}
 
 	// Put it in the form of an undoable command...
-	return pMainForm->commands()->exec(
+	return pSession->execute(
 		new qtractorRemoveTrackCommand(pTrack));
 }
 
 
 // Edit given(current) track properties.
-bool qtractorTracks::editTrack ( qtractorTrack *pTrack, QWidget *pParent )
+bool qtractorTracks::editTrack ( qtractorTrack *pTrack )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -683,16 +1040,18 @@ bool qtractorTracks::editTrack ( qtractorTrack *pTrack, QWidget *pParent )
 	if (pTrack == NULL)
 		return false;
 
+	// Don't edit tracks engaged in recording...
+	if (pTrack->isRecord() && pSession->isRecording() && pSession->isPlaying())
+		return false;
+
 	// Open dialog for settings...
-	if (pParent == NULL)
-		pParent = this;
-	qtractorTrackForm trackForm(pParent);
+	qtractorTrackForm trackForm(this);
 	trackForm.setTrack(pTrack);
 	if (!trackForm.exec())
 		return false;
 
 	// Put it in the form of an undoable command...
-	return pMainForm->commands()->exec(
+	return pSession->execute(
 		new qtractorEditTrackCommand(pTrack, trackForm.properties()));
 }
 
@@ -701,11 +1060,7 @@ bool qtractorTracks::editTrack ( qtractorTrack *pTrack, QWidget *pParent )
 bool qtractorTracks::addAudioTracks ( QStringList files,
 	unsigned long iClipStart )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -728,8 +1083,10 @@ bool qtractorTracks::addAudioTracks ( QStringList files,
 	// or will have each clip intto several tracks...
 	bool bDropSpan = m_pTrackView->isDropSpan();
 	qtractorTrack *pTrack = NULL;
+	int iTrackClip = 0;
 
 	// For each one of those files, if any...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	QStringListIterator iter(files);
 	while (iter.hasNext()) {
 		// This is one of the selected filenames....
@@ -738,11 +1095,10 @@ bool qtractorTracks::addAudioTracks ( QStringList files,
 		if (pTrack == NULL || !bDropSpan) {
 			const QColor& color = qtractorTrack::trackColor(++iTrack);
 			pTrack = new qtractorTrack(pSession, qtractorTrack::Audio);
-			if (bDropSpan)
-				pTrack->setTrackName(tr("Track %1").arg(iTrack));
 			pTrack->setBackground(color);
-			pTrack->setForeground(color.dark());
+			pTrack->setForeground(color.darker());
 			pImportTrackCommand->addTrack(pTrack);
+			iTrackClip = 0;
 		}
 		// Add the clip at once...
 		qtractorAudioClip *pAudioClip = new qtractorAudioClip(pTrack);
@@ -751,21 +1107,24 @@ bool qtractorTracks::addAudioTracks ( QStringList files,
 		// Time to add the new track/clip into session;
 		// actuallly, this is wheen the given audio file gets open...
 		pTrack->addClip(pAudioClip);
+		if (iTrackClip == 0)
+			pTrack->setTrackName(pAudioClip->clipName());
+		iTrackClip++;
 		// Add the new track to composite command...
 		if (bDropSpan)
 			iClipStart += pAudioClip->clipLength();
-		else
-			pTrack->setTrackName(pAudioClip->clipName());
-		// Don't forget to add this one to local repository.
-		pMainForm->addAudioFile(sPath);
 		iUpdate++;
-		// Log this successful import operation...
-		sDescription += tr("Audio file import \"%1\" on %2 %3.\n")
-			.arg(QFileInfo(sPath).fileName())
-			.arg(QDate::currentDate().toString("MMM dd yyyy"))
-			.arg(QTime::currentTime().toString("hh:mm:ss"));
-		pMainForm->appendMessages(
-			tr("Audio file import: \"%1\".").arg(sPath));
+		// Don't forget to add this one to local repository.
+		if (pMainForm) {
+			pMainForm->addAudioFile(sPath);
+			// Log this successful import operation...
+			sDescription += tr("Audio file import \"%1\" on %2 %3.\n")
+				.arg(QFileInfo(sPath).fileName())
+				.arg(QDate::currentDate().toString("MMM dd yyyy"))
+				.arg(QTime::currentTime().toString("hh:mm:ss"));
+			pMainForm->appendMessages(
+				tr("Audio file import: \"%1\".").arg(sPath));
+		}
 		// Make things temporarily stable...
 		qtractorSession::stabilize();
 	}
@@ -780,7 +1139,7 @@ bool qtractorTracks::addAudioTracks ( QStringList files,
 	pSession->setDescription(sDescription);
 
 	// Put it in the form of an undoable command...
-	return pMainForm->commands()->exec(pImportTrackCommand);
+	return pSession->execute(pImportTrackCommand);
 }
 
 
@@ -788,11 +1147,7 @@ bool qtractorTracks::addAudioTracks ( QStringList files,
 bool qtractorTracks::addMidiTracks ( QStringList files,
 	unsigned long iClipStart )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -817,6 +1172,7 @@ bool qtractorTracks::addMidiTracks ( QStringList files,
 		sDescription += '\n';
 
 	// For each one of those files, if any...
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
 	QStringListIterator iter(files);
 	while (iter.hasNext()) {
 		// This is one of the selected filenames....
@@ -834,7 +1190,7 @@ bool qtractorTracks::addMidiTracks ( QStringList files,
 				= new qtractorTrack(pSession, qtractorTrack::Midi);
 		//	pTrack->setTrackName(QFileInfo(sPath).baseName());
 			pTrack->setBackground(color);
-			pTrack->setForeground(color.dark());
+			pTrack->setForeground(color.darker());
 			// Add the clip at once...
 			qtractorMidiClip *pMidiClip = new qtractorMidiClip(pTrack);
 			pMidiClip->setFilename(sPath);
@@ -860,9 +1216,10 @@ bool qtractorTracks::addMidiTracks ( QStringList files,
 				pTrack->setTrackName(pMidiClip->clipName());
 				pTrack->setMidiChannel(pMidiClip->channel());
 				pImportTrackCommand->addTrack(pTrack);
-				// Don't forget to add this one to local repository.
-				pMainForm->addMidiFile(sPath);
 				iUpdate++;
+				// Don't forget to add this one to local repository.
+				if (pMainForm)
+					pMainForm->addMidiFile(sPath);
 			} else {
 				// Get rid of these, now...
 				pTrack->unlinkClip(pMidiClip);
@@ -871,7 +1228,7 @@ bool qtractorTracks::addMidiTracks ( QStringList files,
 			}
 		}
 		// Log this successful import operation...
-		if (iUpdate > 0) {
+		if (iUpdate > 0 && pMainForm) {
 			sDescription += tr("MIDI file import \"%1\" on %2 %3.\n")
 				.arg(QFileInfo(sPath).fileName())
 				.arg(QDate::currentDate().toString("MMM dd yyyy"))
@@ -893,7 +1250,7 @@ bool qtractorTracks::addMidiTracks ( QStringList files,
 	pSession->setDescription(sDescription);
 
 	// Put it in the form of an undoable command...
-	return pMainForm->commands()->exec(pImportTrackCommand);
+	return pSession->execute(pImportTrackCommand);
 }
 
 
@@ -901,11 +1258,7 @@ bool qtractorTracks::addMidiTracks ( QStringList files,
 bool qtractorTracks::addMidiTrackChannel ( const QString& sPath,
 	int iTrackChannel, unsigned long iClipStart )
 {
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm == NULL)
-		return false;
-
-	qtractorSession *pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return false;
 
@@ -922,7 +1275,7 @@ bool qtractorTracks::addMidiTrackChannel ( const QString& sPath,
 		= new qtractorTrack(pSession, qtractorTrack::Midi);
 //	pTrack->setTrackName(QFileInfo(sPath).baseName());
 	pTrack->setBackground(color);
-	pTrack->setForeground(color.dark());
+	pTrack->setForeground(color.darker());
 	// Add the clip at once...
 	qtractorMidiClip *pMidiClip = new qtractorMidiClip(pTrack);
 	pMidiClip->setFilename(sPath);
@@ -934,24 +1287,28 @@ bool qtractorTracks::addMidiTrackChannel ( const QString& sPath,
 	pTrack->setMidiChannel(pMidiClip->channel());
 	// Add the new track to composite command...
 	pImportTrackCommand->addTrack(pTrack);
-	// Don't forget to add this one to local repository.
-	pMainForm->addMidiFile(sPath);
 
-	// Make things temporarily stable...
-	pMainForm->appendMessages(
-		tr("MIDI file import: \"%1\""
-			", track-channel: %2.").arg(sPath).arg(iTrackChannel));
+	// Don't forget to add this one to local repository.
+	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
+	if (pMainForm) {
+		pMainForm->addMidiFile(sPath);
+		// Make things temporarily stable...
+		pMainForm->appendMessages(
+			tr("MIDI file import: \"%1\""
+				", track-channel: %2.").arg(sPath).arg(iTrackChannel));
+	}
+
 	qtractorSession::stabilize();
 
 	// Put it in the form of an undoable command...
-	return pMainForm->commands()->exec(pImportTrackCommand);
+	return pSession->execute(pImportTrackCommand);
 }
 
 
 // MIDI track/bus/channel alias active maintenance method.
 void qtractorTracks::updateMidiTrack ( qtractorTrack *pMidiTrack )
 {
-	qtractorSession *pSession = session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 	if (pSession == NULL)
 		return;
 
@@ -988,7 +1345,8 @@ void qtractorTracks::updateMidiTrack ( qtractorTrack *pMidiTrack )
 	pMidiBus->setPatch(iChannel, patch.instrumentName,
 		pMidiTrack->midiBankSelMethod(),
 		pMidiTrack->midiBank(),
-		pMidiTrack->midiProgram());
+		pMidiTrack->midiProgram(),
+		pMidiTrack);
 }
 
 
@@ -1025,10 +1383,12 @@ void qtractorTracks::trackButtonToggledSlot (
 	qtractorTrackButton *pTrackButton, bool bOn )
 {
 	// Put it in the form of an undoable command...
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pMainForm->commands()->exec(
-			new qtractorTrackButtonCommand(pTrackButton, bOn));
+	qtractorSession *pSession = qtractorSession::getInstance();
+	if (pSession == NULL)
+		return;
+
+	pSession->execute(
+		new qtractorTrackButtonCommand(pTrackButton, bOn));
 }
 
 
