@@ -50,12 +50,6 @@ public:
 	// Destructor.
 	~qtractorTracks();
 
-	// Session helper accessor.
-	qtractorSession *session() const;
-
-	// Instrument list accessor.
-	qtractorInstrumentList *instruments() const;
-
 	// Child widgets accessors.
 	qtractorTrackList *trackList() const;
 	qtractorTrackTime *trackTime() const;
@@ -69,8 +63,7 @@ public:
 
 	bool addTrack();
 	bool removeTrack(qtractorTrack *pTrack = NULL);
-	bool editTrack(qtractorTrack *pTrack = NULL,
-		QWidget *pParent = NULL);
+	bool editTrack(qtractorTrack *pTrack = NULL);
 
 	qtractorTrackItemWidget *currentTrackWidget() const;
 
@@ -89,18 +82,20 @@ public:
 	bool newClip();
 	bool editClip(qtractorClip *pClip = NULL);
 	bool splitClip(qtractorClip *pClip = NULL);
+	bool normalizeClip(qtractorClip *pClip = NULL);
+	bool exportClip(qtractorClip *pClip = NULL);
 
 	// Whether there's any clip currently selected.
 	bool isClipSelected() const;
 
-	// Whether there's any clip on clipboard.
-	bool isClipboardEmpty() const;
-
 	// Clipboard methods.
 	void cutClipboard();
 	void copyClipboard();
 	void pasteClipboard();
 
+	// Special paste/repeat prompt.
+	void pasteRepeatClipboard();
+
 	// Delete selection method.
 	void deleteClipSelect();
 
