@@ -27,7 +27,8 @@
 
 #include "qtractorPluginForm.h"
 
-#include "qtractorMainForm.h"
+#include "qtractorMidiBuffer.h"
+
 #include "qtractorSession.h"
 
 #include <QApplication>
@@ -38,10 +39,8 @@
 #if defined(Q_WS_X11)
 #include <QX11Info>
 #include <X11/Xlib.h>
-#include <X11/Xutil.h>
 #include <X11/Xatom.h>
 typedef void (*XEventProc)(XEvent *);
-#include <QMouseEvent>
 #endif
 
 #if !defined(VST_2_3_EXTENSIONS) 
@@ -92,12 +91,12 @@ static XEventProc getXEventProc ( Display *pDisplay, Window w )
 	unsigned long iBytes, iCount;
 	unsigned char *pData;
 	XEventProc eventProc = NULL;
-	Atom atomType, atom = XInternAtom(pDisplay, "_XEventProc", false);
+	Atom aType, aName = XInternAtom(pDisplay, "_XEventProc", false);
 
 	g_bXError = false;
 	XErrorHandler oldErrorHandler = XSetErrorHandler(tempXErrorHandler);
-	XGetWindowProperty(pDisplay, w, atom, 0, 1, false,
-		AnyPropertyType, &atomType,  &iSize, &iCount, &iBytes, &pData);
+	XGetWindowProperty(pDisplay, w, aName, 0, 1, false,
+		AnyPropertyType, &aType,  &iSize, &iCount, &iBytes, &pData);
 	if (g_bXError == false && iCount == 1)
 		eventProc = (XEventProc) (*(int *) pData);
 	XSetErrorHandler(oldErrorHandler);
@@ -115,37 +114,15 @@ static Window getXChildWindow ( Display *pDisplay, Window w )
 	return (iChildren > 0 ? pwChildren[0] : 0);
 }
 
-static unsigned int getXButton ( Qt::MouseButtons buttons )
-{
-	int button = 0;
-	if (buttons & Qt::LeftButton)
-		button = Button1;
-	else
-	if (buttons & Qt::MidButton)
-		button = Button2;
-	else
-	if (buttons & Qt::RightButton)
-		button = Button3;
-	return button;
-}
-
-static unsigned int getXButtonState ( Qt::MouseButtons buttons )
-{
-	int state = 0;
-	if (buttons & Qt::LeftButton)
-		state |= Button1Mask;
-	if (buttons & Qt::MidButton)
-		state |= Button2Mask;
-	if (buttons & Qt::RightButton)
-		state |= Button3Mask;
-	return state;
-}
-
 #endif // Q_WS_X11
 
 
 //----------------------------------------------------------------------------
 // qtractorVstPlugin::EditorWidget -- Plugin editor wrapper widget.
+//
+
+// Dynamic singleton list of VST editors.
+static QList<qtractorVstPlugin::EditorWidget *> g_vstEditors;
 
 class qtractorVstPlugin::EditorWidget : public QWidget
 {
@@ -154,32 +131,103 @@ public:
 	// Constructor.
 	EditorWidget(QWidget *pParent, Qt::WindowFlags wflags = 0)
 		: QWidget(pParent, wflags),
-#if defined(Q_WS_X11)
-			m_pDisplay(QX11Info::display()),
-			m_wEditor(NULL),
-			m_eventProc(NULL),
-#endif
-			m_pPlugin(NULL) {}
+	#if defined(Q_WS_X11)
+		m_pDisplay(QX11Info::display()),
+		m_wVstEditor(NULL),
+		m_pVstEventProc(NULL),
+		m_bButtonPress(false),
+	#endif
+		m_pVstPlugin(NULL) {}
+
+	// Destructor.
+	~EditorWidget() { close(); }
 
 	// Specialized editor methods.
-	void setPlugin(qtractorPlugin *pPlugin)
+	void open(qtractorVstPlugin *pVstPlugin)
 	{
-		m_pPlugin = pPlugin;
-#if defined(Q_WS_X11)
-		m_wEditor = getXChildWindow(m_pDisplay, (Window) winId());
-		if (m_wEditor) {
-			m_eventProc = getXEventProc(m_pDisplay, m_wEditor);
-			if (m_eventProc)
-				XSelectInput(m_pDisplay, m_wEditor, NoEventMask);
+		m_pVstPlugin = pVstPlugin;
+
+		// Start the proper (child) editor...
+		long  value = 0;
+		void *ptr = (void *) winId();
+	#if defined(Q_WS_X11)
+		value = (long) m_pDisplay;
+	#endif
+		m_pVstPlugin->vst_dispatch(0, effEditOpen, 0, value, ptr, 0.0f);
+	
+		// Make it the right size
+		struct ERect {
+			short top;
+			short left;
+			short bottom;
+			short right;
+		} *pRect;
+	
+		if (m_pVstPlugin->vst_dispatch(0, effEditGetRect, 0, 0, &pRect, 0.0f)) {
+			QWidget::setFixedSize(
+				pRect->right - pRect->left,
+				pRect->bottom - pRect->top);
 		}
-#endif
-		QWidget::setWindowTitle(m_pPlugin->editorTitle());
+
+	#if defined(Q_WS_X11)
+		m_wVstEditor = getXChildWindow(m_pDisplay, (Window) winId());
+		if (m_wVstEditor) {
+			m_pVstEventProc = getXEventProc(m_pDisplay, m_wVstEditor);
+			if (m_pVstEventProc)
+				g_vstEditors.append(this);
+		}
+	#endif
+
+		QWidget::setWindowTitle(tr("%1 - Editor")
+			.arg(m_pVstPlugin->editorTitle()));
+
+		// Final stabilization...
+		m_pVstPlugin->setEditorVisible(true);
+		m_pVstPlugin->idleEditor();
 	}
 
-	qtractorPlugin *plugin() const { return m_pPlugin; }
+	// Close the editor widget.
+	void close()
+	{
+		if (m_pVstPlugin) {
+			m_pVstPlugin->vst_dispatch(0, effEditClose, 0, 0, NULL, 0.0f);
+			m_pVstPlugin->setEditorVisible(false);
+		}
+
+		int iIndex = g_vstEditors.indexOf(this);
+		if (iIndex >= 0)
+			g_vstEditors.removeAt(iIndex);
+	}
 
 #if defined(Q_WS_X11)
-	Display *display() const { return m_pDisplay; }
+
+	// Local X11 event filter.
+	bool x11EventFilter(XEvent *pEvent)
+	{
+		if (m_pVstEventProc && pEvent->xany.window == m_wVstEditor) {
+			// Avoid mouse tracking events...
+			switch (pEvent->xany.type) {
+			case ButtonPress:
+				m_bButtonPress = true;
+				break;
+			case ButtonRelease:
+				m_bButtonPress = false;
+				break;
+			case MotionNotify:
+				if (!m_bButtonPress)
+					return false;
+				// Fall thru...
+			default:
+				break;
+			}
+			// Process as intended...
+			(*m_pVstEventProc)(pEvent);
+			return true;
+		} else {
+			return false;
+		}
+	}
+
 #endif
 
 protected:
@@ -188,222 +236,41 @@ protected:
 	void showEvent(QShowEvent *pShowEvent)
 	{
 		QWidget::showEvent(pShowEvent);
-		if (m_pPlugin)
-			(m_pPlugin->form())->toggleEditor(true);
+
+		if (m_pVstPlugin && m_pVstPlugin->isFormVisible())
+			(m_pVstPlugin->form())->toggleEditor(true);
 	}
 
 	void closeEvent(QCloseEvent *pCloseEvent)
 	{
-		if (m_pPlugin)
-			(m_pPlugin->form())->toggleEditor(false);
+		if (m_pVstPlugin && m_pVstPlugin->isFormVisible())
+			(m_pVstPlugin->form())->toggleEditor(false);
+
 		QWidget::closeEvent(pCloseEvent);
 	}
 
-#if defined(Q_WS_X11)
-
-	// Mouse events.
-	void enterEvent(QEvent *pEnterEvent)
+	void moveEvent(QMoveEvent *pMoveEvent)
 	{
-	//	qDebug("qtractorPluginWidget::enterEvent()\n");
-		QWidget::enterEvent(pEnterEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QPoint& globalPos = QCursor::pos();
-			const QPoint& pos = QWidget::mapFromGlobal(globalPos);
-			ev.xcrossing.display = m_pDisplay;
-			ev.xcrossing.type    = EnterNotify;
-			ev.xcrossing.window  = m_wEditor;
-			ev.xcrossing.root    = RootWindow(m_pDisplay, DefaultScreen(m_pDisplay));
-			ev.xcrossing.time    = CurrentTime;
-			ev.xcrossing.x       = pos.x();
-			ev.xcrossing.y       = pos.y();
-			ev.xcrossing.x_root  = globalPos.x();
-			ev.xcrossing.y_root  = globalPos.y();
-			ev.xcrossing.mode    = NotifyNormal;
-			ev.xcrossing.detail  = NotifyAncestor;
-			ev.xcrossing.state   = getXButtonState(QApplication::mouseButtons());
-			sendXEvent(&ev);
+		QWidget::moveEvent(pMoveEvent);
+	#if defined(Q_WS_X11)
+		if (m_wVstEditor) {
+			XMoveWindow(m_pDisplay, m_wVstEditor, 0, 0);
+		//	QWidget::update();
 		}
+	#endif
 	}
 
-	void mousePressEvent(QMouseEvent *pMouseEvent)
-	{
-	//	qDebug("qtractorPluginWidget::mousePressEvent()\n");
-		QWidget::mousePressEvent(pMouseEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QPoint& globalPos = pMouseEvent->globalPos();
-			const QPoint& pos = pMouseEvent->pos();
-			ev.xbutton.display = m_pDisplay;
-			ev.xbutton.type    = ButtonPress;
-			ev.xbutton.window  = m_wEditor;
-			ev.xbutton.root    = RootWindow(m_pDisplay, DefaultScreen(m_pDisplay));
-			ev.xbutton.time    = CurrentTime;
-			ev.xbutton.x       = pos.x();
-			ev.xbutton.y       = pos.y();
-			ev.xbutton.x_root  = globalPos.x();
-			ev.xbutton.y_root  = globalPos.y();
-			ev.xbutton.button  = getXButton(pMouseEvent->buttons());
-			ev.xbutton.state   = getXButtonState(pMouseEvent->buttons());
-			sendXEvent(&ev);
-		}
-	}
-
-	void mouseMoveEvent(QMouseEvent *pMouseEvent)
-	{
-	//	qDebug("qtractorPluginWidget::mouseMoveEvent()\n");
-		QWidget::mouseMoveEvent(pMouseEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QPoint& globalPos = pMouseEvent->globalPos();
-			const QPoint& pos = pMouseEvent->pos();
-			ev.xmotion.display = m_pDisplay;
-			ev.xmotion.type    = MotionNotify;
-			ev.xmotion.window  = m_wEditor;
-			ev.xmotion.root    = RootWindow(m_pDisplay, DefaultScreen(m_pDisplay));
-			ev.xmotion.time    = CurrentTime;
-			ev.xmotion.x       = pos.x();
-			ev.xmotion.y       = pos.y();
-			ev.xmotion.x_root  = globalPos.x();
-			ev.xmotion.y_root  = globalPos.y();
-			ev.xmotion.state   = getXButtonState(pMouseEvent->buttons());
-			sendXEvent(&ev);
-		}
-	}
-
-	void mouseReleaseEvent(QMouseEvent *pMouseEvent)
-	{
-	//	qDebug("qtractorPluginWidget::mouseReleaseEvent()\n");
-		QWidget::mouseReleaseEvent(pMouseEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QPoint& globalPos = pMouseEvent->globalPos();
-			const QPoint& pos = pMouseEvent->pos();
-			ev.xbutton.display = m_pDisplay;
-			ev.xbutton.type    = ButtonRelease;
-			ev.xbutton.window  = m_wEditor;
-			ev.xbutton.root    = RootWindow(m_pDisplay, DefaultScreen(m_pDisplay));
-			ev.xbutton.time    = CurrentTime;
-			ev.xbutton.x       = pos.x();
-			ev.xbutton.y       = pos.y();
-			ev.xbutton.x_root  = globalPos.x();
-			ev.xbutton.y_root  = globalPos.y();
-			ev.xbutton.button  = getXButton(pMouseEvent->buttons());
-			ev.xbutton.state   = getXButtonState(pMouseEvent->buttons());
-			sendXEvent(&ev);
-		}
-	}
-
-	void wheelEvent(QWheelEvent *pWheelEvent)
-	{
-	//	qDebug("qtractorPluginWidget::wheelEvent()\n");
-		QWidget::wheelEvent(pWheelEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QPoint& globalPos = pWheelEvent->globalPos();
-			const QPoint& pos = pWheelEvent->pos();
-			ev.xbutton.display = m_pDisplay;
-			ev.xbutton.type    = ButtonPress;
-			ev.xbutton.window  = m_wEditor;
-			ev.xbutton.root    = RootWindow(m_pDisplay, DefaultScreen(m_pDisplay));
-			ev.xbutton.time    = CurrentTime;
-			ev.xbutton.x       = pos.x();
-			ev.xbutton.y       = pos.y();
-			ev.xbutton.x_root  = globalPos.x();
-			ev.xbutton.y_root  = globalPos.y();
-			if (pWheelEvent->delta() > 0) {
-				ev.xbutton.button = Button4;
-				ev.xbutton.state |= Button4Mask;
-			} else {
-				ev.xbutton.button = Button5;
-				ev.xbutton.state |= Button5Mask;
-			}
-			sendXEvent(&ev);
-			// FIXME: delay here?
-			ev.xbutton.type = ButtonRelease;
-			sendXEvent(&ev);
-		}
-	}
-
-	void leaveEvent(QEvent *pLeaveEvent)
-	{
-	//	qDebug("qtractorPluginWidget::leaveEvent()\n");
-		QWidget::leaveEvent(pLeaveEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QPoint& globalPos = QCursor::pos();
-			const QPoint& pos = QWidget::mapFromGlobal(globalPos);
-			ev.xcrossing.display = m_pDisplay;
-			ev.xcrossing.type    = LeaveNotify;
-			ev.xcrossing.window  = m_wEditor;
-			ev.xcrossing.root    = RootWindow(m_pDisplay, DefaultScreen(m_pDisplay));
-			ev.xcrossing.time    = CurrentTime;
-			ev.xcrossing.x       = pos.x();
-			ev.xcrossing.y       = pos.y();
-			ev.xcrossing.x_root  = globalPos.x();
-			ev.xcrossing.y_root  = globalPos.y();
-			ev.xcrossing.mode    = NotifyNormal;
-			ev.xcrossing.detail  = NotifyAncestor;
-    		ev.xcrossing.focus   = QWidget::hasFocus();
-			ev.xcrossing.state   = getXButtonState(QApplication::mouseButtons());
-			sendXEvent(&ev);
-		}
-	}
-
-	// Composite paint event.
-	void paintEvent(QPaintEvent *pPaintEvent)
-	{
-		QWidget::paintEvent(pPaintEvent);
-		if (m_wEditor) {
-			XEvent ev;
-			::memset(&ev, 0, sizeof(ev));
-			const QRect& rect = QWidget::geometry();
-			ev.xexpose.type    = Expose;
-			ev.xexpose.display = m_pDisplay;
-			ev.xexpose.window  = m_wEditor;
-			ev.xexpose.x       = rect.x();
-			ev.xexpose.y       = rect.y();
-			ev.xexpose.width   = rect.width();
-			ev.xexpose.height  = rect.height();
-			sendXEvent(&ev);
-		}
-	}
-
-	// Send X11 event.
-	void sendXEvent(XEvent *pEvent)
-	{
-		if (m_eventProc) {
-			// If the plugin publish a event procedure, so it doesn't
-			// have a message thread running, pass the event directly.
-			(*m_eventProc)(pEvent);
-		}
-		else
-		if (m_wEditor) {
-			// If the plugin have a message thread running, then send events
-			// to that window: it will be caught by the message thread!
-			XSendEvent(m_pDisplay, m_wEditor, false, 0L, pEvent);
-			XFlush(m_pDisplay);
-		}
-	}
-
-#endif
-
 private:
 
 	// Instance variables...
 #if defined(Q_WS_X11)
 	Display   *m_pDisplay;
-	Window     m_wEditor;
-	XEventProc m_eventProc;
+	Window     m_wVstEditor;
+	XEventProc m_pVstEventProc;
+	bool       m_bButtonPress;
 #endif
 
-	qtractorPlugin *m_pPlugin;
+	qtractorVstPlugin *m_pVstPlugin;
 };
 
 
@@ -418,9 +285,6 @@ public:
 	// Constructor.
 	Effect(AEffect *pVstEffect = NULL) : m_pVstEffect(pVstEffect) {}
 
-	// Destructor.
-	~Effect() { close(); }
-
 	// Specific methods.
 	bool open(qtractorPluginFile *pFile);
 	void close();
@@ -443,18 +307,16 @@ private:
 bool qtractorVstPluginType::Effect::open ( qtractorPluginFile *pFile )
 {
 	// Do we have an effect descriptor already?
-	if (m_pVstEffect)
-		return true;
-
-	m_pVstEffect = qtractorVstPluginType::vst_effect(pFile);
+	if (m_pVstEffect == NULL)
+		m_pVstEffect = qtractorVstPluginType::vst_effect(pFile);
 	if (m_pVstEffect == NULL)
 		return false;
 
-	// Did VST plugin instantiated OK?
-	if (m_pVstEffect->magic != kEffectMagic)
-		return false;
+#ifdef CONFIG_DEBUG_0
+	qDebug("AEffect[%p]::open(%p)", m_pVstEffect, pFile);
+#endif
 
-	vst_dispatch(effIdentify, 0, 0, NULL, 0);
+//	vst_dispatch(effIdentify, 0, 0, NULL, 0);
 	vst_dispatch(effOpen,0, 0, NULL, 0.0f);
 //	vst_dispatch(effMainsChanged, 0, 0, NULL, 0.0f);
 
@@ -464,6 +326,14 @@ bool qtractorVstPluginType::Effect::open ( qtractorPluginFile *pFile )
 
 void qtractorVstPluginType::Effect::close (void)
 {
+	if (m_pVstEffect == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG_0
+	qDebug("AEffect[%p]::close()", m_pVstEffect);
+#endif
+
+//	vst_dispatch(effMainsChanged, 0, 0, NULL, 0.0f);
 	vst_dispatch(effClose, 0, 0, NULL, 0.0f);
 
 	m_pVstEffect = NULL;
@@ -494,16 +364,11 @@ int qtractorVstPluginType::Effect::vst_dispatch (
 bool qtractorVstPluginType::open (void)
 {
 	// Do we have an effect descriptor already?
-	if (m_pEffect == NULL) {
+	if (m_pEffect == NULL)
 		m_pEffect = new Effect();
-		if (!m_pEffect->open(file())) {
-			delete m_pEffect;
-			m_pEffect = NULL;
-			return false;
-		}
-	}
+	if (!m_pEffect->open(file()))
+		return false;
 
-	// We need the internal descriptor here...
 	AEffect *pVstEffect = m_pEffect->vst_effect();
 	if (pVstEffect == NULL)
 		return false;
@@ -514,12 +379,13 @@ bool qtractorVstPluginType::open (void)
 #endif
 
 	// Retrieve plugin type names.
-	char szName[256];
+	char szName[256]; ::memset(szName, 0, sizeof(szName));
 	if (vst_dispatch(effGetEffectName, 0, 0, (void *) szName, 0.0f))
 		m_sName = szName;
 	else
 		m_sName = QFileInfo(file()->filename()).baseName();
-	m_sLabel = m_sName;
+	// Sanitize plugin label.
+	m_sLabel = m_sName.simplified().replace(QRegExp("[\\s|\\.|\\-]+"), "_");
 
 	// Retrieve plugin unique identifier.
 	m_iUniqueID = pVstEffect->uniqueID;
@@ -536,7 +402,7 @@ bool qtractorVstPluginType::open (void)
 //	if (vst_canDo("plugAsChannelInsert")) m_iFlagsEx |= effFlagsExCanUseAsInsert;
 //	if (vst_canDo("plugAsSend"))          m_iFlagsEx |= effFlagsExCanUseAsSend;
 //	if (vst_canDo("mixDryWet"))           m_iFlagsEx |= effFlagsExCanMixDryWet;
-	if (vst_canDo("midiProgramNames"))    m_iFlagsEx |= effFlagsExCanMidiProgramNames;
+//	if (vst_canDo("midiProgramNames"))    m_iFlagsEx |= effFlagsExCanMidiProgramNames;
 
 	// Compute and cache port counts...
 	m_iControlIns  = pVstEffect->numParams;
@@ -548,8 +414,9 @@ bool qtractorVstPluginType::open (void)
 	m_iMidiOuts    = ((m_iFlagsEx & effFlagsExCanSendVstMidiEvents) ? 1 : 0);
 
 	// Cache flags.
-	m_bRealtime = true;
-	m_bEditor   = (pVstEffect->flags & effFlagsHasEditor);
+	m_bRealtime  = true;
+	m_bConfigure = (pVstEffect->flags & effFlagsProgramChunks);
+	m_bEditor    = (pVstEffect->flags & effFlagsHasEditor);
 
 	return true;
 }
@@ -557,10 +424,17 @@ bool qtractorVstPluginType::open (void)
 
 void qtractorVstPluginType::close (void)
 {
-	if (m_pEffect) {
-		delete m_pEffect;
-		m_pEffect = NULL;
-	}
+	if (m_pEffect == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorVstPluginType[%p]::close()", this);
+#endif
+
+	m_pEffect->close();
+
+	delete m_pEffect;
+	m_pEffect = NULL;
 }
 
 
@@ -592,7 +466,15 @@ AEffect *qtractorVstPluginType::vst_effect ( qtractorPluginFile *pFile )
 	if (pfnGetPluginInstance == NULL)
 		return NULL;
 
-	return (*pfnGetPluginInstance)(qtractorVstPlugin_HostCallback);
+	// Does the VST plugin instantiate OK?
+	AEffect *pVstEffect
+		= (*pfnGetPluginInstance)(qtractorVstPlugin_HostCallback);
+	if (pVstEffect == NULL)
+		return NULL;
+	if (pVstEffect->magic != kEffectMagic)
+		return NULL;
+
+	return pVstEffect;
 }
 
 
@@ -622,7 +504,7 @@ bool qtractorVstPluginType::vst_canDo ( const char *pszCanDo ) const
 //
 
 // Dynamic singleton list of VST plugins.
-QList<qtractorVstPlugin *> qtractorVstPlugin::g_vstPlugins;
+static QList<qtractorVstPlugin *> g_vstPlugins;
 
 // Constructors.
 qtractorVstPlugin::qtractorVstPlugin ( qtractorPluginList *pList,
@@ -697,9 +579,12 @@ void qtractorVstPlugin::setChannels ( unsigned short iChannels )
 	bool bActivated = isActivated();
 	setActivated(false);
 
+	// Close instances, all the way...
 	if (m_ppEffects) {
-		for (unsigned short i = 1; i < instances(); ++i)
+		for (unsigned short i = 1; i < instances(); ++i) {
+			m_ppEffects[i]->close();
 			delete m_ppEffects[i];
+		}
 		delete [] m_ppEffects;
 		m_ppEffects = NULL;
 	}
@@ -711,36 +596,40 @@ void qtractorVstPlugin::setChannels ( unsigned short iChannels )
 		return;
 	}
 
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorVstPlugin[%p]::setChannels(%u) instances=%u",
+		this, iChannels, iInstances);
+#endif
+
 	// Allocate new instances...
 	m_ppEffects = new  qtractorVstPluginType::Effect * [iInstances];
 	m_ppEffects[0] = pVstType->effect();
 	for (unsigned short i = 1; i < iInstances; ++i) {
 		m_ppEffects[i] = new qtractorVstPluginType::Effect();
 		m_ppEffects[i]->open(pVstType->file());
-		// Replicate the current parameter values...
-		if (pVstType && pVstType->effect()) {
-			AEffect *pVstEffect = (pVstType->effect())->vst_effect();
-			if (pVstEffect) {
-				QListIterator<qtractorPluginParam *> iter(params());
-				while (iter.hasNext()) {
-					qtractorPluginParam *pParam = iter.next();
-					pVstEffect->setParameter(pVstEffect, i, pParam->value());
-				}
-			}
-		}
 	}
 
+	// (Re)issue all configuration as needed...
+	realizeConfigs();
+	realizeValues();
+
+	// But won't need it anymore.
+	releaseConfigs();
+
 	// Setup all those instance alright...
 	for (unsigned short i = 0; i < iInstances; ++i) {
+		// And now all other things as well...
 		qtractorVstPluginType::Effect *pEffect = m_ppEffects[i];
-		pEffect->vst_dispatch(effSetProgram, 0, 0, NULL, 0.0f);
 		pEffect->vst_dispatch(effSetSampleRate, 0, 0, NULL, float(sampleRate()));
 		pEffect->vst_dispatch(effSetBlockSize,  0, bufferSize(), NULL, 0.0f);
+	//	pEffect->vst_dispatch(effSetProgram, 0, 0, NULL, 0.0f);
+	#if 0 // !VST_FORCE_DEPRECATED
 		unsigned short j;
 		for (j = 0; j < pVstType->audioIns(); ++j)
 			pEffect->vst_dispatch(effConnectInput, j, 1, NULL, 0.0f);
 		for (j = 0; j < pVstType->audioOuts(); ++j)
 			pEffect->vst_dispatch(effConnectOutput, j, 1, NULL, 0.0f);
+	#endif
 	}
 
 	// (Re)activate instance if necessary...
@@ -774,14 +663,17 @@ void qtractorVstPlugin::process (
 {
 	if (m_ppEffects == NULL)
 		return;
-	if (m_ppIBuffer == NULL || m_ppOBuffer == NULL)
-		return;
+
+	// To process MIDI events, if any...
+	qtractorMidiManager *pMidiManager = NULL;
+	if (type()->midiIns() > 0)
+		pMidiManager = list()->midiManager();
 
 	// We'll cross channels over instances...
 	unsigned short iInstances = instances();
 	unsigned short iChannels  = channels();
-	unsigned short iAudioIns  = audioInsCap();
-	unsigned short iAudioOuts = audioOutsCap();
+	unsigned short iAudioIns  = audioIns();
+	unsigned short iAudioOuts = audioOuts();
 	unsigned short iIChannel  = 0;
 	unsigned short iOChannel  = 0;
 	unsigned short i, j;
@@ -802,14 +694,22 @@ void qtractorVstPlugin::process (
 			if (++iOChannel >= iChannels)
 				iOChannel = 0;
 		}
-		// Make it run...
+		// Make it run MIDI,if applicable...
+		if (pMidiManager) {
+			pVstEffect->dispatcher(pVstEffect,
+				effProcessEvents, 0, 0, pMidiManager->vst_events(), 0.0f);
+		}
+		// Make it run audio...
 		if (pVstEffect->flags & effFlagsCanReplacing) {
 			pVstEffect->processReplacing(
 				pVstEffect, m_ppIBuffer, m_ppOBuffer, nframes);
-		} else {
+		}
+	#if 0 // !VST_FORCE_DEPRECATED
+		else {
 			pVstEffect->process(
 				pVstEffect, m_ppIBuffer, m_ppOBuffer, nframes);
 		}
+	#endif
 		// Wrap channels?...
 		if (iIChannel < iChannels - 1)
 			iIChannel++;
@@ -819,6 +719,138 @@ void qtractorVstPlugin::process (
 }
 
 
+// Parameter update method.
+void qtractorVstPlugin::updateParam (
+	qtractorPluginParam *pParam, float fValue )
+{
+	// Maybe we're not pretty instantiated yet...
+	for (unsigned short i = 0; i < instances(); ++i) {
+		AEffect *pVstEffect = vst_effect(i);
+		if (pVstEffect)
+			pVstEffect->setParameter(pVstEffect, pParam->index(), fValue);
+	}
+}
+
+
+// Bank/program selector.
+void qtractorVstPlugin::selectProgram ( int iBank, int iProg )
+{
+	if (iBank < 0 || iProg < 0)
+		return;
+
+	// HACK: We don't change program-preset when
+	// we're supposed to be multi-timbral...
+	if (list()->flags() == qtractorPluginList::MidiBus)
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorVstPlugin[%p]::selectprogram(%d, %d)", this, iBank, iProg);
+#endif
+
+	int iIndex = 0;
+	if (iBank >= 0 && iProg >= 0)
+		iIndex = (iBank << 7) + iProg;
+
+	for (unsigned short i = 0; i < instances(); ++i)
+		vst_dispatch(i, effSetProgram, 0, iIndex, NULL, 0.0f);
+
+	// Reset parameters default value...
+	AEffect *pVstEffect = vst_effect(0);
+	if (pVstEffect) {
+		QListIterator<qtractorPluginParam *> param(params());
+		while (param.hasNext()) {
+			qtractorPluginParam *pParam = param.next();
+			float *pfValue = pParam->data();
+			*pfValue = pVstEffect->getParameter(pVstEffect, pParam->index());
+			pParam->setDefaultValue(*pfValue);
+		}
+	}
+}
+
+
+// Provisional program/patch accessor.
+bool qtractorVstPlugin::getProgram ( int iIndex, Program& program ) const
+{
+	// Iteration sanity check...
+	AEffect *pVstEffect = vst_effect(0);
+	if (pVstEffect == NULL)
+		return false;
+	if (iIndex < 0 || iIndex >= pVstEffect->numPrograms)
+		return false;
+
+	char szName[256]; ::memset(szName, 0, sizeof(szName));
+	if (vst_dispatch(0, effGetProgramNameIndexed, iIndex, 0, (void *) szName, 0.0f) == 0) {
+		int iOldIndex = vst_dispatch(0, effGetProgram, 0, 0, NULL, 0.0f);
+		vst_dispatch(0, effSetProgram, 0, iIndex, NULL, 0.0f);
+		vst_dispatch(0, effGetProgramName, 0, 0, (void *) szName, 0.0f);
+		vst_dispatch(0, effSetProgram, 0, iOldIndex, NULL, 0.0f);
+	}
+
+	// Map this to that...
+	program.bank = 0;
+	program.prog = iIndex;
+	program.name = szName;
+
+	return true;
+}
+
+
+// Configuration (CLOB) stuff.
+void qtractorVstPlugin::configure ( const QString& sKey, const QString& sValue )
+{
+	if (sKey == "chunk") {
+		// Load the BLOB (base64 encoded)...
+		QByteArray data = qUncompress(QByteArray::fromBase64(sValue.toAscii()));
+		const char *pData = data.constData();
+		int iData = data.size();
+#ifdef CONFIG_DEBUG
+		qDebug("qtractorVstPlugin[%p]::configure() chunk.size=%d checksum=0x%04x",
+			this, iData, qChecksum(pData, iData));
+#endif
+		for (unsigned short i = 0; i < instances(); ++i)
+			vst_dispatch(i, effSetChunk, 0, iData, (void *) pData, 0.0f);
+	}
+}
+
+
+// Plugin configuration/state snapshot.
+void qtractorVstPlugin::freezeConfigs (void)
+{
+	if (!type()->isConfigure())
+		return;
+
+	AEffect *pVstEffect = vst_effect(0);
+	if (pVstEffect == NULL)
+		return;
+
+	// Save plugin state into chunk configuration...
+	char *pData = NULL;
+	int iData = vst_dispatch(0, effGetChunk, 0, 0, (void *) &pData, 0.0f);
+	if (iData < 1 || pData == NULL)
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorVstPlugin[%p]::freezeConfigs() chunk.size=%d checksum=0x%04x",
+		this, iData, qChecksum(pData, iData));
+#endif
+
+	// Set special plugin configuration item (base64 encoded)...
+	QByteArray data = qCompress(QByteArray(pData, iData)).toBase64();
+	for (int i = data.size() - (data.size() % 72); i >= 0; i -= 72)
+		data.insert(i, "\n       "); // Indentation.
+	setConfig("chunk", data.constData());
+}
+
+void qtractorVstPlugin::releaseConfigs (void)
+{
+#ifdef CONFIG_DEBUG
+	qDebug("qtractorVstPlugin[%p]::releaseConfigs()", this);
+#endif
+
+	clearConfigs();
+}
+
+
 // Specific accessors.
 AEffect *qtractorVstPlugin::vst_effect ( unsigned short iInstance ) const
 {
@@ -865,62 +897,41 @@ void qtractorVstPlugin::openEditor ( QWidget */*pParent*/ )
 		| Qt::WindowSystemMenuHint
 		| Qt::WindowMinMaxButtonsHint
 		| Qt::Tool;
+
 	m_pEditorWidget = new EditorWidget(NULL, wflags);
-
-	// Start the proper (child) editor...
-	long  value = 0;
-	void *ptr = (void *) m_pEditorWidget->winId();
-#if defined(Q_WS_X11)
-	value = (long) m_pEditorWidget->display();
-#endif
-	vst_dispatch(0, effEditOpen, 0, value, ptr, 0.0f);
-
-	// Make it the right size
-	struct ERect {
-		short top;
-		short left;
-		short bottom;
-		short right;
-	} *erect;
-
-	if (vst_dispatch(0, effEditGetRect, 0, 0, &erect, 0.0f)) {
-		m_pEditorWidget->setFixedSize(
-			erect->right - erect->left,
-			erect->bottom - erect->top);
-	}
-
-	// Final stabilization...
-	m_pEditorWidget->setPlugin(this);
-
-	setEditorVisible(true);
-	idleEditor();
+	m_pEditorWidget->open(this);
 }
 
 
 // Close editor.
 void qtractorVstPlugin::closeEditor (void)
 {
+	if (m_pEditorWidget == NULL)
+		return;
+
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorVstPlugin[%p]::closeEditor()", this);
 #endif
-	setEditorVisible(false);
-	vst_dispatch(0, effEditClose, 0, 0, NULL, 0.0f);
 
 	// Close the parent widget, if any.
-	if (m_pEditorWidget) {
-		delete m_pEditorWidget;
-		m_pEditorWidget = NULL;
-	}
+	delete m_pEditorWidget;
+	m_pEditorWidget = NULL;
 }
 
 
 // Idle editor.
 void qtractorVstPlugin::idleEditor (void)
 {
+	if (m_pEditorWidget == NULL)
+		return;
+
 #ifdef CONFIG_DEBUG
 	qDebug("qtractorVstPlugin[%p]::idleEditor()", this);
 #endif
+
 	vst_dispatch(0, effEditIdle, 0, 0, NULL, 0.0f);
+
+	m_pEditorWidget->update();
 }
 
 
@@ -950,14 +961,12 @@ void qtractorVstPlugin::setEditorTitle ( const QString& sTitle )
 void qtractorVstPlugin::idleEditorAll (void)
 {
 	QListIterator<qtractorVstPlugin *> iter(g_vstPlugins);
-	while (iter.hasNext()) {
-		qtractorVstPlugin *pVstPlugin = iter.next();
-		if (pVstPlugin->editorWidget())
-			pVstPlugin->idleEditor();
-	}
+	while (iter.hasNext())
+		iter.next()->idleEditor();
 }
 
 
+#if 0 // !VST_FORCE_DEPRECATED
 // Idle editor (static).
 void qtractorVstPlugin::idleTimerAll (void)
 {
@@ -980,6 +989,7 @@ void qtractorVstPlugin::idleTimerAll (void)
 		}
 	}
 }
+#endif
 
 
 // Our own editor widget accessor.
@@ -1005,6 +1015,24 @@ qtractorVstPlugin *qtractorVstPlugin::findPlugin ( AEffect *pVstEffect )
 }
 
 
+#if defined(Q_WS_X11)
+
+// Global X11 event filter.
+bool qtractorVstPlugin::x11EventFilter ( void *pvEvent )
+{
+	XEvent *pEvent = (XEvent *) pvEvent;
+
+	QListIterator<EditorWidget *> iter(g_vstEditors);
+	while (iter.hasNext()) {
+		if (iter.next()->x11EventFilter(pEvent))
+			return true;
+	}
+
+	return false;
+}
+
+#endif
+
 //----------------------------------------------------------------------------
 // qtractorVstPluginParam -- VST plugin control input port instance.
 //
@@ -1014,7 +1042,7 @@ qtractorVstPluginParam::qtractorVstPluginParam (
 	qtractorVstPlugin *pVstPlugin, unsigned long iIndex )
 	: qtractorPluginParam(pVstPlugin, iIndex)
 {
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 	qDebug("qtractorVstPluginParam[%p] pVstPlugin=%p iIndex=%lu", this, pVstPlugin, iIndex);
 #endif
 
@@ -1034,7 +1062,7 @@ qtractorVstPluginParam::qtractorVstPluginParam (
 
 	if (pVstType &&	pVstType->vst_dispatch(
 			effGetParameterProperties, iIndex, 0, (void *) &m_props, 0.0f)) {
-#ifdef CONFIG_DEBUG
+#ifdef CONFIG_DEBUG_0
 		qDebug("  VstParamProperties(%lu) {", iIndex);
 		qDebug("    .stepFloat               = %g", m_props.stepFloat);
 		qDebug("    .smallStepFloat          = %g", m_props.smallStepFloat);
@@ -1159,54 +1187,6 @@ QString qtractorVstPluginParam::display (void) const
 }
 
 
-// Current parameter value.
-void qtractorVstPluginParam::setValue ( float fValue )
-{
-	qtractorPluginParam::setValue(fValue);
-
-	qtractorVstPlugin *pVstPlugin
-		= static_cast<qtractorVstPlugin *> (plugin());
-	if (pVstPlugin == NULL)
-		return;
-
-	// Maybe we're not pretty instantiated yet...
-	unsigned short iInstances = pVstPlugin->instances();
-	if (iInstances > 0) {
-		for (unsigned short i = 0; i < iInstances; ++i) {
-			AEffect *pVstEffect = pVstPlugin->vst_effect(i);
-			if (pVstEffect) {
-				pVstEffect->setParameter(pVstEffect, index(), fValue);
-			}
-		}
-	} else {
-		qtractorVstPluginType *pVstType
-			= static_cast<qtractorVstPluginType *> (pVstPlugin->type());
-		if (pVstType && pVstType->effect()) {
-			AEffect *pVstEffect = (pVstType->effect())->vst_effect();
-			if (pVstEffect) {
-				pVstEffect->setParameter(pVstEffect, index(), fValue);
-			}
-		}
-	}
-}
-
-float qtractorVstPluginParam::value (void) const
-{
-	float fValue = qtractorPluginParam::value();
-#if 0
-	qtractorVstPluginType *pVstType = NULL;
-	if (plugin())
-		pVstType = static_cast<qtractorVstPluginType *> (plugin()->type());
-	if (pVstType && pVstType->effect()) {
-		AEffect *pVstEffect = (pVstType->effect())->vst_effect();
-		if (pVstEffect)
-			fValue = pVstEffect->getParameter(pVstEffect, index());
-	}
-#endif
-	return fValue;
-}
-
-
 //----------------------------------------------------------------------
 // VST host callback file selection helpers.
 
@@ -1227,24 +1207,22 @@ static VstIntPtr qtractorVstPlugin_openFileSelector (
 
     if (pvfs->command == kVstFileLoad || pvfs->command == kVstFileSave) {
 		QString sFilename;
-        QStringList filters;
-        for (int i = 0; i < pvfs->nbFileTypes; ++i) {
-            filters.append(QObject::tr("%1 (*.%2)")
+		QStringList filters;
+		for (int i = 0; i < pvfs->nbFileTypes; ++i) {
+			filters.append(QObject::tr("%1 (*.%2)")
 				.arg(pvfs->fileTypes[i].name).arg(pvfs->fileTypes[i].dosType));
 		}
-        filters.append(QObject::tr("All Files (*.*)"));
+        filters.append(QObject::tr("All files (*.*)"));
 		QString sFilter = filters.join(";;");
 		QString sDirectory = pvfs->initialPath;
 		QString sTitle = QString("%1 - %2")
 			.arg(pvfs->title).arg((pVstPlugin->type())->name());
 		if (pvfs->command == kVstFileLoad) {
 			sFilename = QFileDialog::getOpenFileName(
-				pVstPlugin->editorWidget(),
-				sTitle, sDirectory, sFilter);
+				pVstPlugin->editorWidget(), sTitle, sDirectory, sFilter);
 		} else {
 			sFilename = QFileDialog::getSaveFileName(
-				pVstPlugin->editorWidget(),
-				sTitle, sDirectory, sFilter);
+				pVstPlugin->editorWidget(), sTitle, sDirectory, sFilter);
 		}
 		if (!sFilename.isEmpty()) {
 			if (pvfs->returnPath == NULL) {
@@ -1261,8 +1239,7 @@ static VstIntPtr qtractorVstPlugin_openFileSelector (
 		QString sTitle = QString("%1 - %2")
 			.arg(pvfs->title).arg((pVstPlugin->type())->name());
 		sDirectory = QFileDialog::getExistingDirectory(
-			pVstPlugin->editorWidget(),
-			sTitle, sDirectory);
+			pVstPlugin->editorWidget(), sTitle, sDirectory);
 		if (!sDirectory.isEmpty()) {
 			if (pvfs->returnPath == NULL) {
 				pvfs->returnPath = new char [sDirectory.length() + 1];
@@ -1318,10 +1295,7 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 	VstIntPtr ret = 0;
 	qtractorVstPlugin *pVstPlugin = NULL;
 	static VstTimeInfo s_vstTimeInfo;
-	qtractorSession  *pSession  = NULL;
-	qtractorMainForm *pMainForm = qtractorMainForm::getInstance();
-	if (pMainForm)
-		pSession = pMainForm->session();
+	qtractorSession *pSession = qtractorSession::getInstance();
 
 	switch (opcode) {
 
@@ -1339,7 +1313,7 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 			qtractorPluginForm *pForm = pVstPlugin->form();
 			if (pForm)
 				pForm->updateParamValue(index, opt);
-			QApplication::processEvents();
+		//	QApplication::processEvents();
 		}
 		break;
 
@@ -1353,15 +1327,6 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		QApplication::processEvents();
 		break;
 
-	case audioMasterPinConnected:
-		VST_HC_DEBUG("audioMasterPinConnected");
-		break;
-
-	// VST 2.0 opcodes...
-	case audioMasterWantMidi:
-		VST_HC_DEBUG("audioMasterWantMidi");
-		break;
-
 	case audioMasterGetTime:
 		VST_HC_DEBUG("audioMasterGetTime");
 		if (pSession) {
@@ -1376,38 +1341,10 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		VST_HC_DEBUG("audioMasterProcessEvents");
 		break;
 
-	case audioMasterSetTime:
-		VST_HC_DEBUG("audioMasterSetTime");
-		break;
-
-	case audioMasterTempoAt:
-		VST_HC_DEBUG("audioMasterTempoAt");
-		if (pSession)
-			ret = (long) (pSession->tempo() * 10000.0f);
-		break;
-
-	case audioMasterGetNumAutomatableParameters:
-		VST_HC_DEBUG("audioMasterGetNumAutomatableParameters");
-		break;
-
-	case audioMasterGetParameterQuantization:
-		VST_HC_DEBUG("audioMasterGetParameterQuantization");
-		ret = 1; // full single float precision
-		break;
-
 	case audioMasterIOChanged:
 		VST_HC_DEBUG("audioMasterIOChanged");
 		break;
 
-	case audioMasterNeedIdle:
-		VST_HC_DEBUG("audioMasterNeedIdle");
-		pVstPlugin = qtractorVstPlugin::findPlugin(effect);
-		if (pVstPlugin && !pVstPlugin->isIdleTimer()) {
-			effect->dispatcher(effect, effIdle, 0, 0, NULL, 0.0f);
-			pVstPlugin->setIdleTimer(true);
-		}
-		break;
-
 	case audioMasterSizeWindow:
 		VST_HC_DEBUG("audioMasterSizeWindow");
 		break;
@@ -1438,19 +1375,6 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		VST_HC_DEBUG("audioMasterGetOutputLatency");
 		break;
 
-	case audioMasterGetPreviousPlug:
-		VST_HC_DEBUG("audioMasterGetPreviousPlug");
-		break;
-
-	case audioMasterGetNextPlug:
-		VST_HC_DEBUG("audioMasterGetNextPlug");
-		break;
-
-	case audioMasterWillReplaceOrAccumulate:
-		VST_HC_DEBUG("audioMasterWillReplaceOrAccumulate");
-		ret = 1;
-		break;
-
 	case audioMasterGetCurrentProcessLevel:
 		VST_HC_DEBUG("audioMasterGetCurrentProcessLevel");
 		break;
@@ -1460,10 +1384,6 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		ret = 1; // off
 		break;
 
-	case audioMasterSetOutputSampleRate:
-		VST_HC_DEBUG("audioMasterSetOutputSampleRate");
-		break;
-
 #if !defined(VST_2_3_EXTENSIONS) 
 	case audioMasterGetSpeakerArrangement:
 		VST_HC_DEBUG("audioMasterGetSpeakerArrangement");
@@ -1488,15 +1408,11 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		VST_HC_DEBUG("audioMasterVendorSpecific");
 		break;
 
-	case audioMasterSetIcon:
-		VST_HC_DEBUG("audioMasterSetIcon");
-		break;
-
 	case audioMasterCanDo:
 		VST_HC_DEBUGX("audioMasterCanDo");
-		if (::strcmp("sendVstMidiEvent",    (char *) ptr) == 0 ||
-			::strcmp("receiveVstMidiEvent", (char *) ptr) == 0 ||
-			::strcmp("midiProgramNames",    (char *) ptr) == 0 ||
+		if (::strcmp("receiveVstMidiEvent", (char *) ptr) == 0 ||
+		//	::strcmp("sendVstMidiEvent",    (char *) ptr) == 0 ||
+		//	::strcmp("midiProgramNames",    (char *) ptr) == 0 ||
 			::strcmp("openFileSelector",    (char *) ptr) == 0 ||
 			::strcmp("closeFileSelector",   (char *) ptr) == 0) {
 			ret = 1;
@@ -1508,6 +1424,65 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		ret = kVstLangEnglish;
 		break;
 
+#if 0 // !VST_FORCE_DEPRECATED
+	case audioMasterPinConnected:
+		VST_HC_DEBUG("audioMasterPinConnected");
+		break;
+
+	// VST 2.0 opcodes...
+	case audioMasterWantMidi:
+		VST_HC_DEBUG("audioMasterWantMidi");
+		break;
+
+	case audioMasterSetTime:
+		VST_HC_DEBUG("audioMasterSetTime");
+		break;
+
+	case audioMasterTempoAt:
+		VST_HC_DEBUG("audioMasterTempoAt");
+		if (pSession)
+			ret = (long) (pSession->tempo() * 10000.0f);
+		break;
+
+	case audioMasterGetNumAutomatableParameters:
+		VST_HC_DEBUG("audioMasterGetNumAutomatableParameters");
+		break;
+
+	case audioMasterGetParameterQuantization:
+		VST_HC_DEBUG("audioMasterGetParameterQuantization");
+		ret = 1; // full single float precision
+		break;
+
+	case audioMasterNeedIdle:
+		VST_HC_DEBUG("audioMasterNeedIdle");
+		pVstPlugin = qtractorVstPlugin::findPlugin(effect);
+		if (pVstPlugin && !pVstPlugin->isIdleTimer()) {
+			effect->dispatcher(effect, effIdle, 0, 0, NULL, 0.0f);
+			pVstPlugin->setIdleTimer(true);
+		}
+		break;
+
+	case audioMasterGetPreviousPlug:
+		VST_HC_DEBUG("audioMasterGetPreviousPlug");
+		break;
+
+	case audioMasterGetNextPlug:
+		VST_HC_DEBUG("audioMasterGetNextPlug");
+		break;
+
+	case audioMasterWillReplaceOrAccumulate:
+		VST_HC_DEBUG("audioMasterWillReplaceOrAccumulate");
+		ret = 1;
+		break;
+
+	case audioMasterSetOutputSampleRate:
+		VST_HC_DEBUG("audioMasterSetOutputSampleRate");
+		break;
+
+	case audioMasterSetIcon:
+		VST_HC_DEBUG("audioMasterSetIcon");
+		break;
+
 	case audioMasterOpenWindow:
 		VST_HC_DEBUG("audioMasterOpenWindow");
 		break;
@@ -1515,6 +1490,7 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 	case audioMasterCloseWindow:
 		VST_HC_DEBUG("audioMasterCloseWindow");
 		break;
+#endif
 
 	case audioMasterGetDirectory:
 		VST_HC_DEBUG("audioMasterGetDirectory");
@@ -1524,10 +1500,10 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 		VST_HC_DEBUG("audioMasterUpdateDisplay");
 		pVstPlugin = qtractorVstPlugin::findPlugin(effect);
 		if (pVstPlugin) {
-			qtractorPluginForm *pPluginForm = pVstPlugin->form();
-			if (pPluginForm)
-				pPluginForm->refresh();
-			QApplication::processEvents();
+			qtractorPluginForm *pForm = pVstPlugin->form();
+			if (pForm)
+				pForm->refresh();
+		//	QApplication::processEvents();
 		}
 		break;
 
@@ -1544,7 +1520,7 @@ static VstIntPtr VSTCALLBACK qtractorVstPlugin_HostCallback ( AEffect* effect,
 				pForm->updateParamValue(index,
 					effect->getParameter(effect, index));
 			}
-			QApplication::processEvents();
+		//	QApplication::processEvents();
 		}
 		break;
 
