@@ -1,6 +1,8 @@
 
 #define PACKAGE_NAME    "Qtractor"
-#define PACKAGE_VERSION "0.1.3"
+#define PACKAGE_VERSION "0.3.0"
+
+#define PACKAGE_STRING  PACKAGE_NAME " " PACKAGE_VERSION
 
 #define CONFIG_PREFIX   "."
 #define CONFIG_DEBUG    1
@@ -18,4 +20,6 @@
 
 #define CONFIG_FLOAT32
 
+#define CONFIG_GRADIENT
+
 #undef  CONFIG_XINITTHREADS
